@@ -10,25 +10,20 @@
 #define SHELL_Q '"'
 #define DL_DFLT_NAME "libpari.dll"
 
-#define PARIVERSION "GP/PARI CALCULATOR Version 2.3.5 (released)"
-#ifdef __cplusplus
-#  define PARIINFO_PRE "C++"
-#else
-#  define PARIINFO_PRE ""
-#endif
+#define PARIVERSION "GP/PARI CALCULATOR Version 2.5.0 (development svn-13228)"
 #if defined(__EMX__)
-#  define PARIINFO_POST "ix86 running EMX (ix86 kernel) 32-bit version"
+#  define PARIINFO "ix86 running EMX (ix86 kernel) 32-bit version"
 #endif
 #if defined(WINCE)
-#  define PARIINFO_POST "Windows CE 32-bit version"
+#  define PARIINFO "Windows CE 32-bit version"
 #endif
-#if !defined(PARIINFO_POST)
-#  define PARIINFO_POST "ix86 running Windows 3.2 (ix86 kernel) 32-bit version"
+#if !defined(PARIINFO)
+#  define PARIINFO "ix86 running Windows 3.2 (ix86 kernel) 32-bit version"
 #endif
-#define PARIINFO PARIINFO_PRE/**/PARIINFO_POST
-#define PARI_VERSION_CODE 131845
+#define PARI_VERSION_CODE 132352
 #define PARI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #define PARI_VERSION_SHIFT 8
+#define PARI_VCSVERSION "svn-13228"
 
 #define PARI_DOUBLE_FORMAT 1
 #ifdef _MSC_VER /* MSVC inline directive */
@@ -44,7 +39,6 @@
 #ifdef __EMX__
 #  define STACK_CHECK
 #  define READLINE
-#  define READLINE_LIBRARY
 #  define HAS_RL_MESSAGE
 #  define CPPFunction_defined
 /* in readline 1, no arguments for rl_refresh_line() */
@@ -58,6 +52,10 @@
 #  define CPPFunction_defined
 #endif
 
+#if defined(__EMX__) || defined(_WIN32)
+#  define HAS_ISATTY
+#endif
+
 /* No exp2, log2 in libc */
 #define NOEXP2
 
