@@ -1,4 +1,4 @@
-/* $Id: Qfb.c 9113 2007-10-29 09:35:40Z kb $
+/* $Id: Qfb.c 13201 2011-05-21 09:03:01Z kb $
 
 Copyright (C) 2000-2005  The PARI group.
 
@@ -21,22 +21,22 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*******************************************************************/
 
 void
-check_quaddisc(GEN x, long *s, long *r, char *f)
+check_quaddisc(GEN x, long *s, long *r, const char *f)
 {
   if (typ(x) != t_INT) pari_err(arither1);
-  *s = signe(x); if (!*s) pari_err(talker,"zero discriminant in %s", f);
+  *s = signe(x);
   if (Z_issquare(x)) pari_err(talker,"square discriminant in %s", f);
   *r = mod4(x); if (*s < 0 && *r) *r = 4 - *r;
   if (*r > 1) pari_err(talker, "discriminant not congruent to 0,1 mod 4 in %s", f);
 }
 void
-check_quaddisc_real(GEN x, long *r, char *f)
+check_quaddisc_real(GEN x, long *r, const char *f)
 {
   long sx; check_quaddisc(x, &sx, r, f);
   if (sx < 0) pari_err(talker, "negative discriminant in %s", f);
 }
 void
-check_quaddisc_imag(GEN x, long *r, char *f)
+check_quaddisc_imag(GEN x, long *r, const char *f)
 {
   long sx; check_quaddisc(x, &sx, r, f);
   if (sx > 0) pari_err(talker, "positive discriminant in %s", f);
@@ -67,11 +67,10 @@ Zquadpoly(GEN x, long v)
 GEN
 quadpoly0(GEN x, long v)
 {
-  long tx = typ(x);
-  if (is_matvec_t(tx))
+  if (is_matvec_t(typ(x)))
   {
-    long i, l = lg(x);
-    GEN y = cgetg(l, tx);
+    long i, l;
+    GEN y = cgetg_copy(x, &l);
     for (i=1; i<l; i++) gel(y,i) = quadpoly0(gel(x,i),v);
     return y;
   }
@@ -94,12 +93,6 @@ quadgen(GEN x)
 /**                      BINARY QUADRATIC FORMS                       **/
 /**                                                                   **/
 /***********************************************************************/
-
-static GEN
-qf_disc0(GEN x, GEN y, GEN z) { return subii(sqri(y), shifti(mulii(x,z),2)); }
-GEN
-qf_disc(GEN x) { return qf_disc0(gel(x,1), gel(x,2), gel(x,3)); }
-
 GEN
 qfi(GEN x, GEN y, GEN z)
 {
@@ -124,10 +117,12 @@ GEN
 Qfb0(GEN x, GEN y, GEN z, GEN d, long prec)
 {
   pari_sp av = avma;
+  GEN D;
   long s;
   if (typ(x)!=t_INT || typ(y)!=t_INT || typ(z)!=t_INT) pari_err(typeer,"Qfb");
-  s = signe(qf_disc0(x,y,z)); avma = av;
-  if (!s) pari_err(talker,"zero discriminant in Qfb");
+  D = qfb_disc3(x,y,z);
+  if (Z_issquare(D)) pari_err(talker,"square discriminant in Qfb");
+  s = signe(D); avma = av;
   if (s < 0) return qfi(x, y, z);
 
   d = d? gtofp(d,prec): real_0(prec);
@@ -151,7 +146,7 @@ qfb_sqr(GEN z, GEN x)
     v2 = mulii(v1, gcdii(d1,c)); /* = v1 iff x primitive */
     c = mulii(c, d1);
   }
-  setsigne(m, -signe(m));
+  togglesign(m);
   r = modii(m,v2);
   p1 = mulii(r, v1);
   c3 = addii(c, mulii(r,addii(gel(x,2),p1)));
@@ -160,7 +155,7 @@ qfb_sqr(GEN z, GEN x)
   gel(z,3) = diviiexact(c3,v2);
 }
 /* z <- x * y */
-void
+static void
 qfb_comp(GEN z, GEN x, GEN y)
 {
   GEN s, n, c, d, y1, v1, v2, c3, m, p1, r;
@@ -186,7 +181,7 @@ qfb_comp(GEN z, GEN x, GEN y)
     }
     m = addii(mulii(mulii(y1,y2),n), mulii(gel(y,3),x2));
   }
-  setsigne(m, -signe(m));
+  togglesign(m);
   r = modii(m, v1);
   p1 = mulii(r, v2);
   c3 = addii(c, mulii(r,addii(gel(y,2),p1)));
@@ -197,42 +192,47 @@ qfb_comp(GEN z, GEN x, GEN y)
 }
 
 static GEN
-compimag0(GEN x, GEN y, int raw)
+qficomp0(GEN x, GEN y, int raw)
 {
   pari_sp av = avma;
-  long tx = typ(x);
   GEN z = cgetg(4,t_QFI);
-  if (typ(y) != tx || tx != t_QFI) pari_err(typeer,"composition");
   if (absi_cmp(gel(x,1), gel(y,1)) > 0) swap(x, y);
   qfb_comp(z, x,y);
   if (raw) return gerepilecopy(av,z);
   return gerepileupto(av, redimag(z));
 }
 static GEN
-compreal0(GEN x, GEN y, int raw)
+qfrcomp0(GEN x, GEN y, int raw)
 {
   pari_sp av = avma;
-  long tx = typ(x);
   GEN z = cgetg(5,t_QFR);
-  if (typ(y) != tx || tx != t_QFR) pari_err(typeer,"composition");
   qfb_comp(z, x,y); gel(z,4) = addrr(gel(x,4),gel(y,4));
   if (raw) return gerepilecopy(av,z);
   return gerepileupto(av, redreal(z));
 }
 GEN
-compreal(GEN x, GEN y) { return compreal0(x,y,0); }
+qfrcomp(GEN x, GEN y) { return qfrcomp0(x,y,0); }
 GEN
-comprealraw(GEN x, GEN y) { return compreal0(x,y,1); }
+qfrcompraw(GEN x, GEN y) { return qfrcomp0(x,y,1); }
 GEN
-compimag(GEN x, GEN y) { return compimag0(x,y,0); }
+qficomp(GEN x, GEN y) { return qficomp0(x,y,0); }
 GEN
-compimagraw(GEN x, GEN y) { return compimag0(x,y,1); }
+qficompraw(GEN x, GEN y) { return qficomp0(x,y,1); }
 GEN
-compraw(GEN x, GEN y)
-{ return (typ(x)==t_QFI)? compimagraw(x,y): comprealraw(x,y); }
+qfbcompraw(GEN x, GEN y)
+{
+  long tx = typ(x);
+  if (typ(y) != tx) pari_err(typeer,"composition");
+  switch(tx) {
+    case t_QFI: return qficompraw(x,y);
+    case t_QFR: return qfrcompraw(x,y);
+  }
+  pari_err(typeer,"composition");
+  return NULL; /* not reached */
+}
 
 static GEN
-sqcompimag0(GEN x, long raw)
+qfisqr0(GEN x, long raw)
 {
   pari_sp av = avma;
   GEN z = cgetg(4,t_QFI);
@@ -243,7 +243,7 @@ sqcompimag0(GEN x, long raw)
   return gerepileupto(av, redimag(z));
 }
 static GEN
-sqcompreal0(GEN x, long raw)
+qfrsqr0(GEN x, long raw)
 {
   pari_sp av = avma;
   GEN z = cgetg(5,t_QFR);
@@ -254,22 +254,22 @@ sqcompreal0(GEN x, long raw)
   return gerepileupto(av, redreal(z));
 }
 GEN
-sqcompreal(GEN x) { return sqcompreal0(x,0); }
+qfrsqr(GEN x) { return qfrsqr0(x,0); }
 GEN
-sqcomprealraw(GEN x) { return sqcompreal0(x,1); }
+qfrsqrraw(GEN x) { return qfrsqr0(x,1); }
 GEN
-sqcompimag(GEN x) { return sqcompimag0(x,0); }
+qfisqr(GEN x) { return qfisqr0(x,0); }
 GEN
-sqcompimagraw(GEN x) { return sqcompimag0(x,1); }
+qfisqrraw(GEN x) { return qfisqr0(x,1); }
 
 static GEN
-qfr_unit_by_disc(GEN D, long prec)
+qfr_1_by_disc(GEN D, long prec)
 {
   GEN y = cgetg(5,t_QFR), isqrtD;
   pari_sp av = avma;
   long r;
 
-  check_quaddisc_real(D, /*junk*/&r, "qfr_unit_by_disc");
+  check_quaddisc_real(D, &r, "qfr_1_by_disc");
   gel(y,1) = gen_1; isqrtD = sqrti(D);
   if ((r & 1) != mod2(isqrtD)) /* we know isqrtD > 0 */
     isqrtD = gerepileuptoint(av, addsi(-1,isqrtD));
@@ -278,22 +278,48 @@ qfr_unit_by_disc(GEN D, long prec)
   gel(y,4) = real_0(prec); return y;
 }
 GEN
-qfr_unit(GEN x)
+qfr_1(GEN x)
 {
   long prec;
-  if (typ(x) != t_QFR) pari_err(typeer,"qfr_unit");
+  if (typ(x) != t_QFR) pari_err(typeer,"qfr_1");
   prec = precision(gel(x,4));
   if (!prec) pari_err(talker,"not a t_REAL in 4th component of a t_QFR");
-  return qfr_unit_by_disc(qf_disc(x), prec);
+  return qfr_1_by_disc(qfb_disc(x), prec);
+}
+
+static void
+qfr_1_fill(GEN y, struct qfr_data *S)
+{
+  pari_sp av = avma;
+  GEN y2 = S->isqrtD;
+  gel(y,1) = gen_1;
+  if (mod2(S->D) != mod2(y2)) y2 = addsi(-1,y2);
+  gel(y,2) = y2; av = avma;
+  gel(y,3) = gerepileuptoint(av, shifti(subii(sqri(y2), S->D),-2));
+}
+static GEN
+qfr5_1(struct qfr_data *S, long prec)
+{
+  GEN y = cgetg(6, t_VEC);
+  qfr_1_fill(y, S);
+  gel(y,4) = gen_0;
+  gel(y,5) = real_1(prec); return y;
+}
+static GEN
+qfr3_1(struct qfr_data *S)
+{
+  GEN y = cgetg(4, t_VEC);
+  qfr_1_fill(y, S);
+  return y;
 }
 
 static GEN
-qfi_unit_by_disc(GEN D)
+qfi_1_by_disc(GEN D)
 {
   GEN y = cgetg(4,t_QFI);
   long r;
 
-  check_quaddisc_imag(D, &r, "qfi_unit_by_disc");
+  check_quaddisc_imag(D, &r, "qfi_1_by_disc");
   gel(y,1) = gen_1;
   gel(y,2) = r? gen_1: gen_0;
   /* upon return, y[3] = (1-D) / 4 or -D / 4, whichever is an integer */
@@ -306,68 +332,67 @@ qfi_unit_by_disc(GEN D)
   setsigne(y[3], 1); return y;
 }
 GEN
-qfi_unit(GEN x)
+qfi_1(GEN x)
 {
-  if (typ(x) != t_QFI) pari_err(typeer,"qfi_unit");
-  return qfi_unit_by_disc(qf_disc(x));
+  if (typ(x) != t_QFI) pari_err(typeer,"qfi_1");
+  return qfi_1_by_disc(qfb_disc(x));
 }
 
 static GEN
 invraw(GEN x)
 {
   GEN y = gcopy(x);
-  setsigne(y[2], -signe(y[2]));
-  if (typ(y) == t_QFR) setsigne(y[4], -signe(y[4]));
-  return y;
+  if (typ(y) == t_QFR) togglesign(gel(y,4));
+  togglesign(gel(y,2)); return y;
 }
 GEN
-powrealraw(GEN x, long n)
+qfrpowraw(GEN x, long n)
 {
   pari_sp av = avma;
   long m;
   GEN y;
 
-  if (typ(x) != t_QFR) pari_err(talker,"not a t_QFR in powrealraw");
-  if (!n) return qfr_unit(x);
+  if (typ(x) != t_QFR) pari_err(talker,"not a t_QFR in qfrpowraw");
+  if (!n) return qfr_1(x);
   if (n== 1) return gcopy(x);
   if (n==-1) return invraw(x);
 
   y = NULL; m = labs(n);
   for (; m>1; m>>=1)
   {
-    if (m&1) y = y? comprealraw(y,x): x;
-    x = sqcomprealraw(x);
+    if (m&1) y = y? qfrcompraw(y,x): x;
+    x = qfrsqrraw(x);
   }
-  y = y? comprealraw(y,x): x;
+  y = y? qfrcompraw(y,x): x;
   if (n < 0) y = invraw(y);
   return gerepileupto(av,y);
 }
 GEN
-powimagraw(GEN x, long n)
+qfipowraw(GEN x, long n)
 {
   pari_sp av = avma;
   long m;
   GEN y;
 
-  if (typ(x) != t_QFI) pari_err(talker,"not a t_QFI in powimag");
-  if (!n) return qfi_unit(x);
+  if (typ(x) != t_QFI) pari_err(talker,"not a t_QFI in qfipow");
+  if (!n) return qfi_1(x);
   if (n== 1) return gcopy(x);
   if (n==-1) return invraw(x);
 
   y = NULL; m = labs(n);
   for (; m>1; m>>=1)
   {
-    if (m&1) y = y? compimagraw(y,x): x;
-    x = sqcompimagraw(x);
+    if (m&1) y = y? qficompraw(y,x): x;
+    x = qfisqrraw(x);
   }
-  y = y? compimagraw(y,x): x;
+  y = y? qficompraw(y,x): x;
   if (n < 0) y = invraw(y);
   return gerepileupto(av,y);
 }
 
 GEN
-powraw(GEN x, long n)
-{ return (typ(x)==t_QFI)? powimagraw(x,n): powrealraw(x,n); }
+qfbpowraw(GEN x, long n)
+{ return (typ(x)==t_QFI)? qfipowraw(x,n): qfrpowraw(x,n); }
 
 static long
 parteucl(GEN L, GEN *d, GEN *v3, GEN *v, GEN *v2)
@@ -504,55 +529,77 @@ nupow(GEN x, GEN n)
   GEN y, l;
 
   if (typ(n) != t_INT) pari_err(talker,"not an integer exponent in nupow");
-  if (gcmp1(n)) return gcopy(x);
-  av = avma; y = qfi_unit(x);
+  if (gequal1(n)) return gcopy(x);
+  av = avma; y = qfi_1(x);
   if (!signe(n)) return y;
 
   l = sqrti(shifti(sqrti(gel(y,3)),1));
-  y = leftright_pow(x, n, (void*)l, &mul_nudupl, &mul_nucomp);
+  y = gen_pow(x, n, (void*)l, &mul_nudupl, &mul_nucomp);
   if (signe(n) < 0
   && !absi_equal(gel(y,1),gel(y,2))
-  && !absi_equal(gel(y,1),gel(y,3))) setsigne(y[2],-signe(y[2]));
+  && !absi_equal(gel(y,1),gel(y,3))) togglesign(gel(y,2));
   return gerepileupto(av, y);
 }
 
 /* Reduction */
 
+static GEN
+dvmdii_round(GEN b, GEN a, GEN *r)
+{
+  GEN a2 = shifti(a, 1), q = dvmdii(b, a2, r);
+  if (signe(b) >= 0) {
+    if (absi_cmp(*r, a) > 0) { q = addis(q,  1); *r = subii(*r, a2); }
+  } else { /* r <= 0 */
+    if (absi_cmp(*r, a) >= 0){ q = addis(q, -1); *r = addii(*r, a2); }
+  }
+  return q;
+}
 /* reduce b mod 2*a. Update b,c */
-#define REDB_i(a,b,c)\
-  GEN r, a2 = shifti(a, 1), q = dvmdii(b, a2, &r);\
-  if (signe(b) >= 0) {\
-    if (absi_cmp(r, a) > 0) { q = addis(q,  1); r = subii(r, a2); }\
-  } else { /* r <= 0 */\
-    if (absi_cmp(r, a) >= 0){ q = addis(q, -1); r = addii(r, a2); }\
-  }\
-  c = subii(c, mulii(q, shifti(addii(b, r),-1)));\
-  b = r;
-
-#define REDBU(a,b,c, u1,u2) { REDB_i(a,b,c); u2 = subii(u2, mulii(q, u1)); }
-#define REDB(a,b,c) { REDB_i(a,b,c); }
+static void
+REDB(GEN a, GEN *b, GEN *c)
+{
+  GEN r, q = dvmdii_round(*b, a, &r);
+  *c = subii(*c, mulii(q, shifti(addii(*b, r),-1)));
+  *b = r;
+}
+static void
+REDBU(GEN a, GEN *b, GEN *c, GEN u1, GEN *u2)
+{ /* REDB(a,b,c) */
+  GEN r, q = dvmdii_round(*b, a, &r);
+  *c = subii(*c, mulii(q, shifti(addii(*b, r),-1)));
+  *b = r;
+  *u2 = subii(*u2, mulii(q, u1));
+}
 
+/* q t_QFI, return reduced representative and set base change U in Sl2(Z) */
 GEN
 redimagsl2(GEN q, GEN *U)
 {
   GEN Q = cgetg(4, t_QFI);
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma, av2, lim = stack_lim(av, 1);
   GEN z, u1,u2,v1,v2, a = gel(q,1), b = gel(q,2), c = gel(q,3);
   long cmp;
   /* upper bound for size of final (a,b,c) */
-  (void)new_chunk(lgefint(a) + lgefint(b) + lgefint(c) + 3);
-  u1 = gen_1;
-  u2 = gen_0; cmp = absi_cmp(a, b);
-  if (cmp <= 0 && (cmp || signe(b) < 0)) REDBU(a,b,c, u1,u2);
+  (void)new_chunk(2*(lgefint(a) + lgefint(b) + lgefint(c) + 3));
+  av2 = avma;
+  u1 = gen_1; u2 = gen_0;
+  cmp = absi_cmp(a, b);
+  if (cmp < 0)
+    REDBU(a,&b,&c, u1,&u2);
+  else if (cmp == 0 && signe(b) < 0)
+  { /* b = -a */
+    b = negi(b);
+    u2 = gen_1;
+  }
   for(;;)
   {
     cmp = absi_cmp(a, c); if (cmp <= 0) break;
     swap(a,c); b = negi(b);
     z = u1; u1 = u2; u2 = negi(z);
-    REDBU(a,b,c, u1,u2);
+    REDBU(a,&b,&c, u1,&u2);
     if (low_stack(lim, stack_lim(av, 1))) {
       if (DEBUGMEM>1) pari_warn(warnmem, "redimagsl2");
-      gerepileall(av, 5, &a,&b,&c, &u1,&u2);
+      gerepileall(av2, 5, &a,&b,&c, &u1,&u2);
     }
   }
   if (cmp == 0 && signe(b) < 0)
@@ -576,30 +623,32 @@ redimagsl2(GEN q, GEN *U)
   avma = av;
   v1 = icopy(v1);
   v2 = icopy(v2);
-  *U = cgetg(3, t_MAT);
-  gel(*U,1) = mkcol2(u1,v1);
-  gel(*U,2) = mkcol2(u2,v2); return Q;
+  *U = mkmat2(mkcol2(u1,v1), mkcol2(u2,v2)); return Q;
 }
 
 GEN
 redimag(GEN q)
 {
   GEN Q = cgetg(4, t_QFI);
-  pari_sp av = avma, lim = stack_lim(av, 1);
+  pari_sp av = avma, av2, lim = stack_lim(av, 1);
   GEN a = gel(q,1), b = gel(q,2), c = gel(q,3);
   long cmp;
   /* upper bound for size of final (a,b,c) */
   (void)new_chunk(lgefint(a) + lgefint(b) + lgefint(c) + 3);
+  av2 = avma;
   cmp = absi_cmp(a, b);
-  if (cmp <= 0 && (cmp || signe(b) < 0)) REDB(a,b,c);
+  if (cmp < 0)
+    REDB(a,&b,&c);
+  else if (cmp == 0 && signe(b) < 0)
+    b = negi(b);
   for(;;)
   {
     cmp = absi_cmp(a, c); if (cmp <= 0) break;
     swap(a,c); b = negi(b); /* apply rho */
-    REDB(a,b,c);
+    REDB(a,&b,&c);
     if (low_stack(lim, stack_lim(av, 1))) {
       if (DEBUGMEM>1) pari_warn(warnmem, "redimag");
-      gerepileall(av, 3, &a,&b,&c);
+      gerepileall(av2, 3, &a,&b,&c);
     }
   }
   if (cmp == 0 && signe(b) < 0) b = negi(b);
@@ -626,7 +675,7 @@ rhoimag(GEN x)
   x = cgetg(4, t_QFI);
   (void)new_chunk(lgefint(a) + lgefint(b) + lgefint(c) + 3);
   swap(a,c); b = negi(b);
-  REDB(a, b, c); avma = (pari_sp)x;
+  REDB(a, &b, &c); avma = (pari_sp)x;
   gel(x,1) = icopy(a);
   gel(x,2) = icopy(b);
   gel(x,3) = icopy(c); return x;
@@ -642,8 +691,6 @@ rhoimag(GEN x)
  * (resp. 5). A qfr3 [a,b,c] contains the form coeffs, in a qfr5 [a,b,c, e,d]
  * the t_INT e is a binary exponent, d a t_REAL, coding the distance in
  * multiplicative form: the true distance is obtained from qfr5_dist.
- * D, sqrtD, isqrtD are included in the function's arguments [sqrtD is only
- * used for distance computations].
  * All other qfr routines are obsolete (inefficient) wrappers */
 
 /* static functions are not stack-clean. Unless mentionned otherwise, public
@@ -653,7 +700,7 @@ rhoimag(GEN x)
 static void
 fix_expo(GEN x)
 {
-  long d = expo(x[5]) - (1 << EMAX);
+  long d = expo(x[5]) - (1L << EMAX);
   if (d >= 0) {
     gel(x,4) = addsi(1, gel(x,4));
     setexpo(x[5], d);
@@ -673,26 +720,26 @@ qfr5_dist(GEN e, GEN d, long prec)
 }
 
 static void
-rho_get_BC(GEN *B, GEN *C, GEN b, GEN c, GEN D, GEN isqrtD)
+rho_get_BC(GEN *B, GEN *C, GEN b, GEN c, struct qfr_data *S)
 {
   GEN t, u;
   u = shifti(c,1); if (u == gen_0) pari_err(talker, "reducible form in qfr_rho");
-  t = (absi_cmp(isqrtD,c) >= 0)? isqrtD: c;
+  t = (absi_cmp(S->isqrtD,c) >= 0)? S->isqrtD: c;
   u = remii(addii_sign(t,1, b,signe(b)), u);
   *B = addii_sign(t, 1, u, -signe(u)); /* |t| - (|t|+b) % |2c| */
   if (*B == gen_0)
-  { u = shifti(D, -2); setsigne(u, -1); }
+  { u = shifti(S->D, -2); setsigne(u, -1); }
   else
-    u = shifti(addii_sign(sqri(*B),1, D,-1), -2);
+    u = shifti(addii_sign(sqri(*B),1, S->D,-1), -2);
   *C = diviiexact(u, c); /* = (B^2-D)/4c */
 }
 /* Not stack-clean */
 GEN
-qfr3_rho(GEN x, GEN D, GEN isqrtD)
+qfr3_rho(GEN x, struct qfr_data *S)
 {
   GEN B, C, y, b = gel(x,2), c = gel(x,3);
 
-  rho_get_BC(&B, &C, b, c, D, isqrtD);
+  rho_get_BC(&B, &C, b, c, S);
   y = cgetg(4, t_VEC);
   gel(y,1) = c;
   gel(y,2) = B;
@@ -700,24 +747,19 @@ qfr3_rho(GEN x, GEN D, GEN isqrtD)
 }
 /* Not stack-clean */
 GEN
-qfr5_rho(GEN x, GEN D, GEN sqrtD, GEN isqrtD)
+qfr5_rho(GEN x, struct qfr_data *S)
 {
   GEN B, C, y, b = gel(x,2), c = gel(x,3);
   long sb = signe(b);
 
-  rho_get_BC(&B, &C, b, c, D, isqrtD);
-  y = cgetg(6, t_VEC);
-  gel(y,1) = c;
-  gel(y,2) = B;
-  gel(y,3) = C;
-  y[4] = x[4];
-  y[5] = x[5];
+  rho_get_BC(&B, &C, b, c, S);
+  y = mkvec5(c,B,C, gel(x,4), gel(x,5));
   if (sb) {
-    GEN t = subii(sqri(b), D);
+    GEN t = subii(sqri(b), S->D);
     if (sb < 0)
-      t = divir(t, gsqr(subir(b,sqrtD)));
+      t = divir(t, sqrr(subir(b,S->sqrtD)));
     else
-      t = divri(gsqr(addir(b,sqrtD)), t);
+      t = divri(sqrr(addir(b,S->sqrtD)), t);
     /* t = (b + sqrt(D)) / (b - sqrt(D)), evaluated stably */
     gel(y,5) = mulrr(t, gel(y,5)); fix_expo(y);
   }
@@ -727,17 +769,10 @@ qfr5_rho(GEN x, GEN D, GEN sqrtD, GEN isqrtD)
 /* Not stack-clean */
 GEN
 qfr_to_qfr5(GEN x, long prec)
-{
-  GEN y = cgetg(6,t_VEC);
-  y[1] = x[1];
-  y[2] = x[2];
-  y[3] = x[3];
-  gel(y,4) = gen_0;
-  gel(y,5) = real_1(prec); return y;
-}
+{ return mkvec5(gel(x,1),gel(x,2),gel(x,3),gen_0,real_1(prec)); }
 
 /* d0 = initial distance, x = [a,b,c, expo(d), d], d = exp(2*distance) */
-static GEN
+GEN
 qfr5_to_qfr(GEN x, GEN d0)
 {
   GEN y;
@@ -748,16 +783,21 @@ qfr5_to_qfr(GEN x, GEN d0)
     {
       n = addis(shifti(n, EMAX), expo(d));
       setexpo(d, 0); d = logr_abs(d);
-      d = mpadd(d, mulir(n, mplog2(lg(d0))));
+      if (signe(n)) d = addrr(d, mulir(n, mplog2(lg(d0))));
+      setexpo(d, expo(d)-1);
+      d0 = addrr(d0, d);
+    }
+    else if (!gequal1(d)) /* avoid loss of precision */
+    {
+      d = logr_abs(d);
+      setexpo(d, expo(d)-1);
+      d0 = addrr(d0, d);
     }
-    else
-      d = gcmp1(d)? NULL: logr_abs(d); /* avoid loss of precision */
-    if (d) d0 = addrr(d0, shiftr(d,-1));
   }
   y = cgetg(5, t_QFR);
-  y[1] = x[1];
-  y[2] = x[2];
-  y[3] = x[3];
+  gel(y,1) = gel(x,1);
+  gel(y,2) = gel(x,2);
+  gel(y,3) = gel(x,3);
   gel(y,4) = d0; return y;
 }
 
@@ -766,14 +806,14 @@ GEN
 qfr3_to_qfr(GEN x, GEN d)
 {
   GEN z = cgetg(5, t_QFR);
-  z[1] = x[1];
-  z[2] = x[2];
-  z[3] = x[3];
+  gel(z,1) = gel(x,1);
+  gel(z,2) = gel(x,2);
+  gel(z,3) = gel(x,3);
   gel(z,4) = d; return z;
 }
 
 static int
-abi_isreduced(GEN a, GEN b, GEN isqrtD)
+ab_isreduced(GEN a, GEN b, GEN isqrtD)
 {
   if (signe(b) > 0 && absi_cmp(b, isqrtD) <= 0)
   {
@@ -787,32 +827,40 @@ abi_isreduced(GEN a, GEN b, GEN isqrtD)
 INLINE int
 qfr_isreduced(GEN x, GEN isqrtD)
 {
-  return abi_isreduced(gel(x,1),gel(x,2),isqrtD);
+  return ab_isreduced(gel(x,1),gel(x,2),isqrtD);
 }
 
 /* Not stack-clean */
 GEN
-qfr5_red(GEN x, GEN D, GEN sqrtD, GEN isqrtD) {
-  while (!qfr_isreduced(x,isqrtD)) x = qfr5_rho(x,D,sqrtD,isqrtD);
+qfr5_red(GEN x, struct qfr_data *S) {
+  while (!qfr_isreduced(x,S->isqrtD)) x = qfr5_rho(x,S);
   return x;
 }
 /* Not stack-clean */
 GEN
-qfr3_red(GEN x, GEN D, GEN isqrtD) {
-  while (!qfr_isreduced(x,isqrtD)) x = qfr3_rho(x,D,isqrtD);
+qfr3_red(GEN x, struct qfr_data *S) {
+  while (!qfr_isreduced(x, S->isqrtD)) x = qfr3_rho(x, S);
   return x;
 }
 
 static void
-get_disc(GEN x, GEN *D)
+get_disc(GEN x, struct qfr_data *S)
 {
-  if (!*D) *D = qf_disc(x);
-  else if (typ(*D) != t_INT) pari_err(arither1);
-  if (!signe(*D)) pari_err(talker,"reducible form in qfr_init");
+  if (!S->D) S->D = qfb_disc(x);
+  else if (typ(S->D) != t_INT) pari_err(arither1);
+  if (!signe(S->D)) pari_err(talker,"reducible form in qfr_init");
+}
+
+void
+qfr_data_init(GEN D, long prec, struct qfr_data *S)
+{
+  S->D = D;
+  S->sqrtD = sqrtr(itor(S->D,prec));
+  S->isqrtD = truncr(S->sqrtD);
 }
 
 static GEN
-qfr5_init(GEN x, GEN *D, GEN *isqrtD, GEN *sqrtD)
+qfr5_init(GEN x, struct qfr_data *S)
 {
   GEN d = gel(x,4);
   long prec = lg(d), l = nbits2prec(-expo(d));
@@ -820,21 +868,26 @@ qfr5_init(GEN x, GEN *D, GEN *isqrtD, GEN *sqrtD)
   if (prec < 3) prec = 3;
   x = qfr_to_qfr5(x,prec);
 
-  get_disc(x, D);
-  if (!*sqrtD) *sqrtD = sqrtr(itor(*D,prec));
-  else if (typ(*sqrtD) != t_REAL) pari_err(arither1);
+  get_disc(x, S);
+  if (!S->sqrtD) S->sqrtD = sqrtr(itor(S->D,prec));
+  else if (typ(S->sqrtD) != t_REAL) pari_err(arither1);
 
-  if (!*isqrtD) *isqrtD = truncr(*sqrtD);
-  else if (typ(*isqrtD) != t_INT) pari_err(arither1);
+  if (!S->isqrtD)
+  {
+    pari_sp av=avma;
+    long e;
+    S->isqrtD = grndtoi(S->sqrtD,&e);
+    if (e>-2) { avma = av; S->isqrtD = sqrti(S->D); }
+  }
+  else if (typ(S->isqrtD) != t_INT) pari_err(arither1);
   return x;
 }
 static GEN
-qfr3_init(GEN x, GEN *D, GEN *isqrtD)
+qfr3_init(GEN x, struct qfr_data *S)
 {
-  get_disc(x, D);
-
-  if (!*isqrtD) *isqrtD = sqrti(*D);
-  else if (typ(*isqrtD) != t_INT) pari_err(arither1);
+  get_disc(x, S);
+  if (!S->isqrtD) S->isqrtD = sqrti(S->D);
+  else if (typ(S->isqrtD) != t_INT) pari_err(arither1);
   return x;
 }
 
@@ -845,15 +898,19 @@ static GEN
 redreal0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD)
 {
   pari_sp av = avma;
-  GEN d = gel(x,4);
+  struct qfr_data S;
+  GEN d;
   if (typ(x) != t_QFR) pari_err(talker,"not a real quadratic form in redreal");
-  x = (flag & qf_NOD)? qfr3_init(x, &D,&isqrtD)
-                     : qfr5_init(x, &D,&isqrtD,&sqrtD);
+  d = gel(x,4);
+  S.D = D;
+  S.sqrtD = sqrtD;
+  S.isqrtD = isqrtD;
+  x = (flag & qf_NOD)? qfr3_init(x, &S): qfr5_init(x, &S);
   switch(flag) {
-    case 0:              x = qfr5_red(x,D,sqrtD,isqrtD); break;
-    case qf_NOD:         x = qfr3_red(x,D,isqrtD); break;
-    case qf_STEP:        x = qfr5_rho(x,D,sqrtD,isqrtD); break;
-    case qf_STEP|qf_NOD: x = qfr3_rho(x,D,isqrtD); break;
+    case 0:              x = qfr5_red(x,&S); break;
+    case qf_NOD:         x = qfr3_red(x,&S); break;
+    case qf_STEP:        x = qfr5_rho(x,&S); break;
+    case qf_STEP|qf_NOD: x = qfr3_rho(x,&S); break;
     default: pari_err(flagerr,"qfbred");
   }
   return gerepilecopy(av, qfr5_to_qfr(x,d));
@@ -879,94 +936,97 @@ qfbred0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD)
 }
 
 GEN
-qfr5_comp(GEN x, GEN y, GEN D, GEN sqrtD, GEN isqrtD)
+qfr5_comp(GEN x, GEN y, struct qfr_data *S)
 {
   pari_sp av = avma;
   GEN z = cgetg(6,t_VEC); qfb_comp(z,x,y);
   if (x == y)
   {
     gel(z,4) = shifti(gel(x,4),1);
-    gel(z,5) = gsqr(gel(x,5));
+    gel(z,5) = sqrr(gel(x,5));
   }
   else
   {
     gel(z,4) = addii(gel(x,4),gel(y,4));
     gel(z,5) = mulrr(gel(x,5),gel(y,5));
   }
-  fix_expo(z); z = qfr5_red(z,D,sqrtD,isqrtD);
+  fix_expo(z); z = qfr5_red(z,S);
   return gerepilecopy(av,z);
 }
 /* Not stack-clean */
 GEN
-qfr3_comp(GEN x, GEN y, GEN D, GEN isqrtD)
+qfr3_comp(GEN x, GEN y, struct qfr_data *S)
 {
   GEN z = cgetg(4,t_VEC); qfb_comp(z,x,y);
-  return qfr3_red(z, D, isqrtD);
-}
-
-/* assume n != 0, return x^|n|. Not stack-clean */
-GEN
-qfr5_pow(GEN x, GEN n, GEN D, GEN sqrtD, GEN isqrtD)
-{
-  GEN y = NULL;
-  long i, m;
-  for (i=lgefint(n)-1; i>1; i--)
-  {
-    m = n[i];
-    for (; m; m>>=1)
-    {
-      if (m&1) y = y? qfr5_comp(y,x,D,sqrtD,isqrtD): x;
-      if (m == 1 && i == 2) break;
-      x = qfr5_comp(x,x,D,sqrtD,isqrtD);
-    }
-  }
-  return y;
-}
-/* assume n != 0, return x^|n|. Not stack-clean */
-GEN
-qfr3_pow(GEN x, GEN n, GEN D, GEN isqrtD)
-{
-  GEN y = NULL;
-  long i, m;
-  for (i=lgefint(n)-1; i>1; i--)
-  {
-    m = n[i];
-    for (; m; m>>=1)
-    {
-      if (m&1) y = y? qfr3_comp(y,x,D,isqrtD): x;
-      if (m == 1 && i == 2) break;
-      x = qfr3_comp(x,x,D,isqrtD);
-    }
-  }
-  return y;
+  return qfr3_red(z, S);
 }
 
+/* valid for t_QFR, qfr3, qfr5 */
 static GEN
 qfr_inv(GEN x) {
-  GEN z = cgetg(5, t_QFR);
-  z[1] = x[1];
-  gel(z,2) = negi(gel(x,2));
-  z[3] = x[3];
-  z[4] = x[4]; return z;
+  GEN z = shallowcopy(x);
+  gel(z,2) = negi(gel(z,2));
+  return z;
 }
-/* assume n != 0 */
-GEN
-qfr_pow(GEN x, GEN n)
-{
-  pari_sp av = avma;
-  GEN D, sqrtD, isqrtD, d0;
 
-  if (is_pm1(n)) return signe(n) > 0? gcopy(x): ginv(x);
-  if (signe(n) < 0) x = qfr_inv(x);
+/* return x^n. Not stack-clean */
+GEN
+qfr5_pow(GEN x, GEN n, struct qfr_data *S)
+{
+  GEN y = NULL;
+  long i, m, s = signe(n);
+  if (!s) return qfr5_1(S, lg(gel(x,5)));
+  for (i=lgefint(n)-1; i>1; i--)
+  {
+    m = n[i];
+    for (; m; m>>=1)
+    {
+      if (m&1) y = y? qfr5_comp(y,x,S): x;
+      if (m == 1 && i == 2) break;
+      x = qfr5_comp(x,x,S);
+    }
+  }
+  return y;
+}
+/* return x^n. Not stack-clean */
+GEN
+qfr3_pow(GEN x, GEN n, struct qfr_data *S)
+{
+  GEN y = NULL;
+  long i, m, s = signe(n);
+  if (!s) return qfr3_1(S);
+  if (s < 0) x = qfr_inv(x);
+  for (i=lgefint(n)-1; i>1; i--)
+  {
+    m = n[i];
+    for (; m; m>>=1)
+    {
+      if (m&1) y = y? qfr3_comp(y,x,S): x;
+      if (m == 1 && i == 2) break;
+      x = qfr3_comp(x,x,S);
+    }
+  }
+  return y;
+}
+GEN
+qfrpow(GEN x, GEN n)
+{
+  struct qfr_data S = { NULL, NULL, NULL };
+  long s = signe(n);
+  pari_sp av = avma;
+  GEN d0;
+
+  if (!s) return qfr_1(x);
+  if (is_pm1(n)) return s > 0? redreal(x): ginv(x);
+  if (s < 0) x = qfr_inv(x);
   d0 = gel(x,4);
-  D = sqrtD = isqrtD = NULL;
   if (!signe(d0)) {
-    x = qfr3_init(x, &D,&isqrtD);
-    x = qfr3_pow(x, n, D,isqrtD);
+    x = qfr3_init(x, &S);
+    x = qfr3_pow(x, n, &S);
     x = qfr3_to_qfr(x, d0);
   } else {
-    x = qfr5_init(x, &D,&isqrtD,&sqrtD);
-    x = qfr5_pow(qfr_to_qfr5(x, lg(sqrtD)), n, D,sqrtD,isqrtD);
+    x = qfr5_init(x, &S);
+    x = qfr5_pow(qfr_to_qfr5(x, lg(S.sqrtD)), n, &S);
     x = qfr5_to_qfr(x, mulri(d0,n));
   }
   return gerepilecopy(av, x);
@@ -982,7 +1042,8 @@ primeform_u(GEN x, ulong p)
 {
   GEN c, y = cgetg(4, t_QFI);
   pari_sp av = avma;
-  ulong b, s;
+  ulong b;
+  long s;
 
   s = mod8(x); if (signe(x) < 0 && s) s = 8-s;
   /* 2 or 3 mod 4 */
@@ -998,7 +1059,7 @@ primeform_u(GEN x, ulong p)
   } else {
     b = Fl_sqrt(umodiu(x,p), p); if (b == ~0UL) pari_err(sqrter5);
     /* mod(b) != mod2(x) ? */
-    if ((b & 1) != (s & 1)) b = p - b;
+    if ((b ^ s) & 1) b = p - b;
     c = diviuexact(shifti(subii(sqru(b), x), -2), p);
   }
   gel(y,3) = gerepileuptoint(av, c);
@@ -1016,23 +1077,29 @@ primeform(GEN x, GEN p, long prec)
 
   if (typ(x) != t_INT || !sx) pari_err(arither1);
   if (typ(p) != t_INT || !sp) pari_err(arither1);
-  if (is_pm1(p)) {
-    if (sx < 0) return qfi_unit_by_disc(x);
-    y = qfr_unit_by_disc(x,prec);
-    if (sp < 0) { gel(y,1) = negi(gel(y,1)); gel(y,3) = negi(gel(y,3)); }
-    return y;
-  }
-  if (sp < 0 && sx < 0) pari_err(impl,"negative definite t_QFI");
   if (lgefint(p) == 3)
-  { 
+  {
+    if (p[2] == 1) {
+      if (sx < 0) {
+        if (sp < 0) pari_err(impl,"negative definite t_QFI");
+        return qfi_1_by_disc(x);
+      }
+      y = qfr_1_by_disc(x,prec);
+      if (sp < 0) { gel(y,1) = gen_m1; togglesign(gel(y,3)); }
+      return y;
+    }
     y = primeform_u(x, p[2]);
-    if (sx < 0) return y;
-    if (sp < 0) { gel(y,1) = negi(gel(y,1)); gel(y,3) = negi(gel(y,3)); }
+    if (sx < 0) {
+      if (sp < 0) pari_err(impl,"negative definite t_QFI");
+      return y;
+    }
+    if (sp < 0) { togglesign(gel(y,1)); togglesign(gel(y,3)); }
     return gcopy( qfr3_to_qfr(y, real_0(prec)) );
   }
   s = mod8(x);
   if (sx < 0)
   {
+    if (sp < 0) pari_err(impl,"negative definite t_QFI");
     if (s) s = 8-s;
     y = cgetg(4, t_QFI);
   }
@@ -1052,14 +1119,14 @@ primeform(GEN x, GEN p, long prec)
   av = avma;
   gel(y,3) = gerepileuptoint(av, diviiexact(shifti(subii(sqri(b), x), -2), p));
   gel(y,2) = b;
-  gel(y,1) = gcopy(p);
+  gel(y,1) = icopy(p);
   return y;
 }
 
 /* Let M and N in SL_2(Z), return (N*M^-1)[,1]
- * 
+ *
 */
-static GEN 
+static GEN
 SL2_div_mul_e1(GEN N, GEN M)
 {
   GEN b = gcoeff(M, 2, 1);
@@ -1072,9 +1139,9 @@ SL2_div_mul_e1(GEN N, GEN M)
   return p2;
 }
 /* Let M and N in SL_2(Z), return (N*[1,0;0,-1]*M^-1)[,1]
- * 
+ *
 */
-static GEN 
+static GEN
 SL2_swap_div_mul_e1(GEN N, GEN M)
 {
   GEN b = gcoeff(M, 2, 1);
@@ -1108,7 +1175,7 @@ qfbsolve_cornacchia(GEN c, GEN p, int swap)
 }
 
 GEN
-qfbimagsolvep(GEN Q, GEN p)
+qfisolvep(GEN Q, GEN p)
 {
   GEN M, N, x,y, a,b,c, d;
   pari_sp av = avma;
@@ -1116,10 +1183,10 @@ qfbimagsolvep(GEN Q, GEN p)
   {
     a = gel(Q,1);
     c = gel(Q,3); /* if principal form, use faster cornacchia */
-    if (gcmp1(a)) return qfbsolve_cornacchia(c, p, 0);
-    if (gcmp1(c)) return qfbsolve_cornacchia(a, p, 1);
+    if (gequal1(a)) return qfbsolve_cornacchia(c, p, 0);
+    if (gequal1(c)) return qfbsolve_cornacchia(a, p, 1);
   }
-  d = qf_disc(Q); if (kronecker(d,p) < 0) return gen_0;
+  d = qfb_disc(Q); if (kronecker(d,p) < 0) return gen_0;
   a = redimagsl2(Q, &N);
   if (is_pm1(gel(a,1))) /* principal form */
   {
@@ -1128,12 +1195,16 @@ qfbimagsolvep(GEN Q, GEN p)
     {
       a = qfbsolve_cornacchia(gel(a,3), p, 0);
       if (a == gen_0) { avma = av; return gen_0; }
-      return gerepileupto(av, gmul(a, shallowtrans(N)));
+      a = ZM_ZC_mul(N, a);
+      a[0] = evaltyp(t_VEC) | _evallg(3); /* transpose */
+      return gerepileupto(av, a);
     }
     /* x^2 + xy + ((1-d)/4)y^2 = p <==> (2x + y)^2 - d y^2 = 4p */
     if (!cornacchia2(negi(d), p, &x, &y)) { avma = av; return gen_0; }
     x = divis_rem(subii(x,y), 2, &r); if (r) { avma = av; return gen_0; }
-    return gerepileupto(av, gmul(mkvec2(x,y), shallowtrans(N)));
+    a = ZM_ZC_mul(N, mkvec2(x,y));
+    a[0] = evaltyp(t_VEC) | _evallg(3); /* transpose */
+    return gerepileupto(av, a);
   }
   b = redimagsl2(primeform(d, p, 0), &M);
   if (!GL2_qfb_equal(a,b)) { avma = av; return gen_0; }
@@ -1149,13 +1220,13 @@ redrealsl2step(GEN A)
 {
   pari_sp ltop = avma;
   GEN N;
-  GEN V = gel(A,1); 
+  GEN V = gel(A,1);
   GEN M = gel(A,2);
-  GEN a = gel(V,1); 
+  GEN a = gel(V,1);
   GEN b = gel(V,2);
   GEN c = gel(V,3);
-  GEN d = qf_disc0(a,b,c);
-  GEN rd = sqrti(d); 
+  GEN d = qfb_disc3(a,b,c);
+  GEN rd = sqrti(d);
   GEN ac = mpabs(c);
   GEN r = addii(b, gmax(rd, ac));
   GEN q = truedvmdii(r, shifti(ac, 1), NULL);
@@ -1178,28 +1249,25 @@ redrealsl2(GEN V)
   GEN a = gel(V,1);
   GEN b = gel(V,2);
   GEN c = gel(V,3);
-  GEN d = qf_disc0(a,b,c);
+  GEN d = qfb_disc3(a,b,c);
   GEN rd = sqrti(d);
   btop = avma; st_lim = stack_lim(btop, 1);
-  u1 = v2 = gen_1; v1 = u2 = gen_0; 
-  while (!abi_isreduced(a,b,rd))
+  u1 = v2 = gen_1; v1 = u2 = gen_0;
+  while (!ab_isreduced(a,b,rd))
   {
     GEN ac = mpabs(c);
     GEN r = addii(b, gmax(rd,ac));
-    GEN q = truedvmdii(r, mulsi(2, ac), NULL);
-    r = subii(mulii(mulis(q, 2), ac), b);
+    GEN q = truedvmdii(r, shifti(ac, 1), NULL);
+    r = subii(mulii(shifti(q, 1), ac), b);
     a = c; b = r;
-    c = truedvmdii(subii(sqri(r), d), mulsi(4, c), NULL);
-    q = mulis(q, signe(a));
+    c = truedvmdii(subii(sqri(r), d), shifti(c, 2), NULL);
+    if (signe(a) < 0) togglesign(q);
     r = u1; u1 = v1; v1 = subii(mulii(q, v1), r);
     r = u2; u2 = v2; v2 = subii(mulii(q, v2), r);
     if (low_stack(st_lim, stack_lim(btop, 1)))
     {
-      GEN *bptr[7];
-      bptr[0]=&a; bptr[1]=&b; bptr[2]=&c;
-      bptr[3]=&u1; bptr[4]=&u2;
-      bptr[5]=&v1; bptr[6]=&v2;
-      gerepilemany(ltop, bptr, 7);
+      if (DEBUGMEM>1) pari_warn(warnmem,"redrealsl2");
+      gerepileall(btop, 7, &a,&b,&c,&u1,&u2,&v1,&v2);
     }
   }
   M = mkmat2(mkcol2(u1,u2), mkcol2(v1,v2));
@@ -1207,27 +1275,25 @@ redrealsl2(GEN V)
 }
 
 GEN
-qfbrealsolvep(GEN Q, GEN p)
+qfrsolvep(GEN Q, GEN p)
 {
   pari_sp ltop = avma, btop, st_lim;
-  GEN N, P, P1, P2, M, d = qf_disc(Q);
+  GEN N, P, P1, P2, M, d = qfb_disc(Q);
   if (kronecker(d, p) < 0) { avma = ltop; return gen_0; }
   M = N = redrealsl2(Q);
   P = primeform(d, p, DEFAULTPREC);
   P1 = redrealsl2(P);
-  gel(P,2) = negi(gel(P,2));
+  togglesign( gel(P,2) );
   P2 = redrealsl2(P);
   btop = avma; st_lim = stack_lim(btop, 1);
-  while (!gequal(gel(M,1), gel(P1,1)) && !gequal(gel(M,1), gel(P2,1)))
+  while (!ZV_equal(gel(M,1), gel(P1,1)) && !ZV_equal(gel(M,1), gel(P2,1)))
   {
     M = redrealsl2step(M);
-    if (gequal(gel(M,1), gel(N,1))) { avma = ltop; return gen_0; }
+    if (ZV_equal(gel(M,1), gel(N,1))) { avma = ltop; return gen_0; }
     if (low_stack(st_lim, stack_lim(btop, 1))) M = gerepileupto(btop, M);
   }
-  if (gequal(gel(M,1),gel(P1,1)))
-    return gerepilecopy(ltop, SL2_div_mul_e1(gel(M,2),gel(P1,2)));
-  else
-    return gerepilecopy(ltop, SL2_div_mul_e1(gel(M,2),gel(P2,2)));
+  N = ZV_equal(gel(M,1),gel(P1,1))? gel(P1,2): gel(P2,2);
+  return gerepilecopy(ltop, SL2_div_mul_e1(gel(M,2),N));
 }
 
 GEN
@@ -1236,8 +1302,8 @@ qfbsolve(GEN Q,GEN n)
   if (typ(n)!=t_INT) pari_err(typeer,"qfbsolve");
   switch(typ(Q))
   {
-  case t_QFI: return qfbimagsolvep(Q,n);
-  case t_QFR: return qfbrealsolvep(Q,n);
+  case t_QFI: return qfisolvep(Q,n);
+  case t_QFR: return qfrsolvep(Q,n);
   default:
     pari_err(typeer,"qfbsolve");
     return NULL; /* NOT REACHED */
@@ -1272,7 +1338,7 @@ cornacchia(GEN d, GEN p, GEN *px, GEN *py)
   }
   a = subii(p, sqri(b));
   c = dvmdii(a, d, &r);
-  if (r != gen_0 || !Z_issquarerem(c, &c)) { avma = av; return 0; }
+  if (r != gen_0 || !Z_issquareall(c, &c)) { avma = av; return 0; }
   avma = av;
   *px = icopy(b);
   *py = icopy(c); return 1;
@@ -1301,7 +1367,7 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
       default: return 0;
     }
     *py = gen_1; return 1;
-  } 
+  }
   b = Fp_sqrt(negi(d), p);
   if (!b) { avma = av; return 0; }
   if (!signe(b)) { /* d = p,2p,3p,4p */
@@ -1323,7 +1389,7 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
   }
   a = subii(px4, sqri(b));
   c = dvmdii(a, d, &r);
-  if (r != gen_0 || !Z_issquarerem(c, &c)) { avma = av; return 0; }
+  if (r != gen_0 || !Z_issquareall(c, &c)) { avma = av; return 0; }
   avma = av;
   *px = icopy(b);
   *py = icopy(c); return 1;
