@@ -1,4 +1,4 @@
-/* $Id: perm.c 7821 2006-04-04 16:50:47Z kb $
+/* $Id: perm.c 13025 2011-03-31 09:33:27Z bill $
 
 Copyright (C) 2000-2003  The PARI group.
 
@@ -21,158 +21,40 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /**                   Routines for handling VEC/COL                     **/
 /**                                                                     **/
 /*************************************************************************/
-
-/* The functions below are shallow*/
-
-GEN
-vec_setconst(GEN v, GEN x)
-{
-  long i, l = lg(v);
-  for (i = 1; i < l; i++) gel(v,i) = x;
-  return v;
-}
-
-GEN
-const_vec(long n, GEN x)
-{
-  GEN v = cgetg(n+1, t_VEC);
-  long i;
-  for (i = 1; i <= n; i++) gel(v,i) = x;
-  return v;
-}
-
 int
 vec_isconst(GEN v)
 {
-  long i, l=lg(v);
+  long i, l = lg(v);
+  GEN w;
   if (l==1) return 1;
+  w = gel(v,1);
   for(i=2;i<l;i++)
-    if (!gequal(gel(v,i), gel(v,1)))
-      return 0;
+    if (!gequal(gel(v,i), w)) return 0;
   return 1;
 }
 
 /* Check if all the elements of v are different.
- * Use a quadratic algorithm.
- * It could be done in n*log(n) by sorting.
- */
-
+ * Use a quadratic algorithm. Could be done in n*log(n) by sorting. */
 int
 vec_is1to1(GEN v)
 {
-  long i,j;
-  long l=lg(v);
+  long i, j, l = lg(v);
   for (i=1; i<l; i++)
+  {
+    GEN w = gel(v,i);
     for(j=i+1; j<l; j++)
-      if (gequal(gel(v,i), gel(v,j)))
-	return 0;
+      if (gequal(gel(v,j), w)) return 0;
+  }
   return 1;
 }
 
-GEN
-const_col(long n, GEN x)
-{
-  GEN v = cgetg(n+1, t_COL);
-  long i;
-  for (i = 1; i <= n; i++) gel(v,i) = x;
-  return v;
-}
-
-GEN
-vec_shorten(GEN v, long n)
-{
-  long i;
-  GEN V = cgetg(n+1,t_VEC);
-  for(i=1;i<=n;i++) V[i] = v[i];
-  return V;
-}
-
-GEN
-vec_lengthen(GEN v, long n)
-{
-  long i;
-  long l=lg(v);
-  GEN V = cgetg(n+1,t_VEC);
-  for(i=1;i<l;i++) V[i] = v[i];
-  return V;
-}
-
-GEN
-vec_to_vecsmall(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l, t_VECSMALL);
-  for (i=1; i<l; i++) x[i] = itos(gel(z,i));
-  return x;
-}
-
 /*************************************************************************/
 /**                                                                     **/
 /**                   Routines for handling VECSMALL                    **/
 /**                                                                     **/
 /*************************************************************************/
-
-GEN
-vecsmall_to_vec(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_VEC);
-  for (i=1; i<l; i++) gel(x,i) = stoi(z[i]);
-  return x;
-}
-
-GEN
-vecsmall_to_col(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_COL);
-  for (i=1; i<l; i++) gel(x,i) = stoi(z[i]);
-  return x;
-}
-
-GEN
-vecsmall_copy(GEN x)
-{
-  long i, l = lg(x);
-  GEN z = cgetg(l, t_VECSMALL);
-  if (typ(x)!=t_VECSMALL) pari_err(typeer,"vecsmall_copy");
-  for (i=1; i<l; i++) z[i] = x[i];
-  return z;
-}
-
-GEN
-const_vecsmall(long n, long c)
-{
-  long i;
-  GEN V = cgetg(n+1,t_VECSMALL);
-  for(i=1;i<=n;i++) V[i] = c;
-  return V;
-}
-
-GEN
-vecsmall_shorten(GEN v, long n)
-{
-  long i;
-  GEN V = cgetg(n+1,t_VECSMALL);
-  for(i=1;i<=n;i++) V[i] = v[i];
-  return V;
- 
-}
-
-GEN
-vecsmall_lengthen(GEN v, long n)
-{
-  long i;
-  long l=lg(v);
-  GEN V = cgetg(n+1,t_VECSMALL);
-  for(i=1;i<l;i++) V[i] = v[i];
-  return V;
-}
-
-/* Sort v[0]...v[n-1] and put result in w[0]...w[n-1]. 
- * We accept v==w. w must be allocated.
- */
-
+/* Sort v[0]...v[n-1] and put result in w[0]...w[n-1].
+ * We accept v==w. w must be allocated. */
 static void
 vecsmall_sortspec(GEN v, long n, GEN w)
 {
@@ -182,13 +64,13 @@ vecsmall_sortspec(GEN v, long n, GEN w)
   GEN x, y;
   if (n<=2)
   {
-    if (n==1) 
+    if (n==1)
       w[0]=v[0];
     else if (n==2)
     {
       long v0=v[0], v1=v[1];
       if (v0<=v1) { w[0]=v0; w[1]=v1; }
-      else { w[0]=v1; w[1]=v0; }
+      else        { w[0]=v1; w[1]=v0; }
     }
     return;
   }
@@ -200,10 +82,8 @@ vecsmall_sortspec(GEN v, long n, GEN w)
       w[m++]=x[ix++];
     else
       w[m++]=y[iy++];
-  for(;ix<nx;)
-    w[m++]=x[ix++];
-  for(;iy<ny;)
-    w[m++]=y[iy++];
+  for(;ix<nx;) w[m++]=x[ix++];
+  for(;iy<ny;) w[m++]=y[iy++];
   avma=ltop;
 }
 
@@ -211,40 +91,44 @@ vecsmall_sortspec(GEN v, long n, GEN w)
 void
 vecsmall_sort(GEN V)
 {
-  long l=lg(V)-1;
+  long l = lg(V)-1;
   if (l<=1) return;
   vecsmall_sortspec(V+1,l,V+1);
 }
 
+/* cf gen_sortspec */
 static GEN
 vecsmall_indexsortspec(GEN v, long n)
 {
-  long nx=n>>1, ny=n-nx;
-  long m, ix, iy;
-  GEN x, y;
-  GEN w=cgetg(n+1,t_VECSMALL);
-  if (n<=2)
+  long nx, ny, m, ix, iy;
+  GEN x, y, w;
+  switch(n)
   {
-    if (n==1)
-      w[1]=1;
-    else if (n==2)
-    {
-      if (v[1]<=v[2]) { w[1]=1; w[2]=2; }
-      else { w[1]=2; w[2]=1; }
-    }
-    return w;
+    case 1: return mkvecsmall(1);
+    case 2: return (v[1] <= v[2])? mkvecsmall2(1,2): mkvecsmall2(2,1);
+    case 3:
+      if (v[1] <= v[2]) {
+        if (v[2] <= v[3]) return mkvecsmall3(1,2,3);
+        return (v[1] <= v[3])? mkvecsmall3(1,3,2)
+                             : mkvecsmall3(3,1,2);
+      } else {
+        if (v[1] <= v[3]) return mkvecsmall3(2,1,3);
+        return (v[2] <= v[3])? mkvecsmall3(2,3,1)
+                             : mkvecsmall3(3,2,1);
+      }
   }
-  x=vecsmall_indexsortspec(v,nx);
-  y=vecsmall_indexsortspec(v+nx,ny);
+  nx = n>>1; ny = n-nx;
+  w = cgetg(n+1,t_VECSMALL);
+  x = vecsmall_indexsortspec(v,nx);
+  y = vecsmall_indexsortspec(v+nx,ny);
   for (m=1, ix=1, iy=1; ix<=nx && iy<=ny; )
     if (v[x[ix]] <= v[y[iy]+nx])
       w[m++] = x[ix++];
     else
       w[m++] = y[iy++]+nx;
-  for(;ix<=nx;) w[m++]=x[ix++];
-  for(;iy<=ny;) w[m++]=y[iy++]+nx;
-  avma = (pari_sp) w;
-  return w;
+  for(;ix<=nx;) w[m++] = x[ix++];
+  for(;iy<=ny;) w[m++] = y[iy++]+nx;
+  avma = (pari_sp)w; return w;
 }
 
 /*indirect sort.*/
@@ -252,13 +136,13 @@ GEN
 vecsmall_indexsort(GEN V)
 {
   long l=lg(V)-1;
-  if (l==0) return V;
+  if (l==0) return cgetg(1, t_VECSMALL);
   return vecsmall_indexsortspec(V,l);
 }
 
 /* assume V sorted */
 GEN
-vecsmall_uniq(GEN V)
+vecsmall_uniq_sorted(GEN V)
 {
   GEN W;
   long i,j, l = lg(V);
@@ -271,130 +155,36 @@ vecsmall_uniq(GEN V)
   setlg(W, j); return W;
 }
 
-int
-vecsmall_lexcmp(GEN x, GEN y)
+GEN
+vecsmall_uniq(GEN V)
 {
-  long lx,ly,l,i;
-  lx = lg(x);
-  ly = lg(y); l = min(lx,ly);
-  for (i=1; i<l; i++)
-    if (x[i] != y[i]) return x[i]<y[i]? -1: 1;
-  if (lx == ly) return 0;
-  return (lx < ly)? -1 : 1;
+  pari_sp av = avma;
+  V = zv_copy(V); vecsmall_sort(V);
+  return gerepileuptoleaf(av, vecsmall_uniq_sorted(V));
 }
 
-int
-vecsmall_prefixcmp(GEN x, GEN y)
+/* assume x sorted */
+long
+vecsmall_duplicate_sorted(GEN x)
 {
-  long i, lx = lg(x), ly = lg(y), l = min(lx,ly);
-  for (i=1; i<l; i++)
-    if (x[i] != y[i]) return x[i]<y[i]? -1: 1;
+  long i,k,l=lg(x);
+  if (l==1) return 0;
+  for (k=x[1],i=2; i<l; k=x[i++])
+    if (x[i] == k) return i;
   return 0;
 }
 
-/*Can be used on t_VEC, but coeffs not gcopy-ed*/
-GEN
-vecsmall_prepend(GEN V, long s)
-{
-  long i, l2 = lg(V);
-  GEN res = cgetg(l2+1, typ(V));
-  res[1] = s;
-  for (i = 2; i <= l2; ++i) res[i] = V[i - 1];
-  return res;
-}
-
-/*Can be used on t_VEC, but coeffs not gcopy-ed*/
-GEN
-vecsmall_append(GEN V, long s)
-{
-  long i, l2 = lg(V);
-  GEN res = cgetg(l2+1, typ(V));
-  for (i = 1; i < l2; ++i) res[i] = V[i];
-  res[l2] = s; return res;
-}
-
-GEN
-vecsmall_concat(GEN u, GEN v)
-{
-  long i, l1 = lg(u)-1, l2 = lg(v)-1;
-  GEN res = cgetg(l1+l2+1, t_VECSMALL);
-  for (i = 1; i <= l1; ++i) res[i]    = u[i];
-  for (i = 1; i <= l2; ++i) res[i+l1] = v[i];
-  return res;
-}
-
-/* return the number of indices where u and v are equal */
 long
-vecsmall_coincidence(GEN u, GEN v)
+vecsmall_duplicate(GEN x)
 {
-  long i, s = 0, l = min(lg(u),lg(v));
-  for(i=1; i<l; i++)
-    if(u[i] == v[i]) s++;
-  return s;
-}
-
-/* returns the first index i<=n such that x=v[i] if it exits, 0 otherwise */
-long
-vecsmall_isin(GEN v, long x)
-{
-  long i, l = lg(v);
-  for (i = 1; i < l; i++)
-    if (v[i] == x) return i;
-  return 0;
-}
-
-
-long
-vecsmall_pack(GEN V, long base, long mod)
-{
-  long i, s = 0;
-  for(i=1; i<lg(V); i++) s = (base*s + V[i]) % mod;
-  return s;
-}
-
-/*************************************************************************/
-/**                                                                     **/
-/**               Routines for handling bit vector                      **/
-/**                                                                     **/
-/*************************************************************************/
-
-GEN
-bitvec_alloc(long n)
-{
-  long l = 1 + (n>>TWOPOTBITS_IN_LONG);
-  return const_vecsmall(l,0);
-}
-
-
-GEN
-bitvec_shorten(GEN bitvec, long n)
-{
-  long l = 1 + (n>>TWOPOTBITS_IN_LONG);
-  return vecsmall_shorten(bitvec,l);
-}
-
-long
-bitvec_test(GEN bitvec, long b)
-{
-  long q = b>>TWOPOTBITS_IN_LONG;
-  long r = b&(BITS_IN_LONG-1);
-  return (bitvec[1+q]>>r) & 1L;
-}
-
-void
-bitvec_set(GEN bitvec, long b)
-{
-  long q = b>>TWOPOTBITS_IN_LONG;
-  long r = b&(BITS_IN_LONG-1);
-  bitvec[1+q] |= 1L<<r;
-}
-
-void
-bitvec_clear(GEN bitvec, long b)
-{
-  long q = b>>TWOPOTBITS_IN_LONG;
-  long r = b&(BITS_IN_LONG-1);
-  bitvec[1+q] &= ~(1L<<r);
+  pari_sp av=avma;
+  GEN p=vecsmall_indexsort(x);
+  long k,i,r=0,l=lg(x);
+  if (l==1) return 0;
+  for (k=x[p[1]],i=2; i<l; k=x[p[i++]])
+    if (x[p[i]] == k) { r=p[i]; break; }
+  avma=av;
+  return r;
 }
 
 /*************************************************************************/
@@ -406,19 +196,19 @@ bitvec_clear(GEN bitvec, long b)
 GEN
 vecvecsmall_sort(GEN x)
 {
-  return gen_sort(x, 0, vecsmall_lexcmp);
+  return gen_sort(x, (void*)&vecsmall_lexcmp, cmp_nodata);
 }
 
 GEN
 vecvecsmall_indexsort(GEN x)
 {
-  return gen_sort(x, cmp_IND | cmp_C, vecsmall_lexcmp);
+  return gen_indexsort(x, (void*)&vecsmall_lexcmp, cmp_nodata);
 }
 
 long
 vecvecsmall_search(GEN x, GEN y, long flag)
 {
-  return gen_search(x,y,flag,vecsmall_prefixcmp);
+  return gen_search(x,y,flag,(void*)vecsmall_prefixcmp, cmp_nodata);
 }
 
 /*************************************************************************/
@@ -431,101 +221,44 @@ vecvecsmall_search(GEN x, GEN y, long flag)
  * perm (VECSMALL): a bijection from 1...n to 1...n i-->perm[i]
  * cyc (VEC of VECSMALL): a product of disjoint cycles. */
 
-/* Identity permutation. Not a good name : l is not a perm */
-GEN
-perm_identity(long l)
-{
-  GEN perm = cgetg(l + 1, t_VECSMALL);
-  long i;
-  for (i = 1; i <= l; i++) perm[i] = i;
-  return perm;
-} 
-
-GEN
-cyclicperm(long l, long d)
-{
-  GEN perm = cgetg(l + 1, t_VECSMALL);
-  long i;
-  for (i = 1;     i <= l-d; i++) perm[i] = i+d;
-  for (i = l-d+1; i <= l; i++)   perm[i] = i-l+d;
-  return perm;
-}
-
-/* Multiply (compose) two permutations.
- * Can be used if s is a t_VEC but no copy */
-GEN
-perm_mul(GEN s, GEN t)
-{
-  GEN u;
-  long i, l = lg(t);
-  u = cgetg(l, typ(s));
-  for (i = 1; i < l; i++) u[i] = s[ t[i] ];
-  return u;
-}
-
 /* Multiply (compose) two permutations, putting the result in the second one. */
-static GEN
-perm_mul_inplace(GEN s, GEN t)
-{
-  long i;
-  for (i = 1; i < lg(s); i++) t[i] = s[t[i]];
-  return t;
-}
-/* Compute the inverse (reciprocal) of a permutation. */
-GEN
-perm_inv(GEN x)
-{
-  long i, lx = lg(x);
-  GEN y = cgetg(lx,t_VECSMALL);
-  for (i=1; i<lx; i++) y[ x[i] ] = i;
-  return y;
-}
-
-/* Return s*t*s^-1 */
-GEN 
-perm_conj(GEN s, GEN t)
+static void
+perm_mul_inplace2(GEN s, GEN t)
 {
   long i, l = lg(s);
-  GEN v = cgetg(l, t_VECSMALL);
-  for (i = 1; i < l; i++) v[ s[i] ] = s[ t[i] ];
-  return v;
+  for (i = 1; i < l; i++) t[i] = s[t[i]];
 }
 
 /* Orbits of the subgroup generated by v on {1,..,n} */
 static GEN
 vecperm_orbits_i(GEN v, long n)
 {
-  long mj = 1, j, k, l, m, o, p, flag;
-  GEN bit, cycle, cy;
-  cycle = cgetg(n+1, t_VEC);
-  bit = bitvec_alloc(n);
+  long mj = 1, k, l, m;
+  GEN cy, cycle = cgetg(n+1, t_VEC), bit = zero_F2v(n);
   for (k = 1, l = 1; k <= n;)
   {
-    for (  ; bitvec_test(bit,mj); mj++) /*empty*/;
+    for (  ; F2v_coeff(bit,mj); mj++) /*empty*/;
     cy = cgetg(n+1, t_VECSMALL);
-    m = 1;
-    k++;
-    cy[m++] = mj;
-    bitvec_set(bit, mj++);
-    do
+    m = 1; k++; cy[m++] = mj;
+    F2v_set(bit, mj++);
+    for(;;)
     {
-      flag = 0;
+      long o, mold = m;
       for (o = 1; o < lg(v); o++)
-	for (p = 1; p < m; p++)	/* m increases! */
-	{
-	  j = mael(v,o,cy[p]);
-	  if (!bitvec_test(bit,j))
-	  {
-	    flag = 1;
-	    bitvec_set(bit,j);
-	    k++;
-	    cy[m++] = j;
-	  }
-	}
+      {
+        GEN vo = gel(v,o);
+        long p;
+        for (p = 1; p < m; p++) /* m increases! */
+        {
+          long j = vo[ cy[p] ];
+          if (!F2v_coeff(bit,j)) cy[m++] = j;
+          F2v_set(bit,j);
+        }
+      }
+      if (m == mold) break;
+      k += m - mold;
     }
-    while (flag);
-    setlg(cy, m);
-    gel(cycle,l++) = cy;
+    setlg(cy, m); gel(cycle,l++) = cy;
   }
   setlg(cycle, l); return cycle;
 }
@@ -550,9 +283,8 @@ long
 perm_order(GEN v)
 {
   pari_sp ltop = avma;
-  GEN c;
+  GEN c = vecperm_orbits_i(mkvec(v), lg(v)-1);
   long i, d;
-  c = vecperm_orbits_i(mkvec(v), lg(v)-1);
   for(i=1, d=1; i<lg(c); i++) d = clcm(d, lg(c[i])-1);
   avma = ltop; return d;
 }
@@ -571,7 +303,7 @@ cyc_pow(GEN cyc, long exp)
   for (r = j = 1; j < lg(cyc); j++)
   {
     GEN v = gel(cyc,j);
-    long n = lg(v) - 1, e = smodss(exp,n), g = cgcd(n, e), m = n / g;
+    long n = lg(v) - 1, e = smodss(exp,n), g = (long)ugcd(n, e), m = n / g;
     for (i = 0; i < g; i++)
     {
       GEN p = cgetg(m+1, t_VECSMALL);
@@ -616,7 +348,7 @@ perm_pow(GEN perm, long exp)
   return cyc_pow_perm(perm_cycles(perm), exp);
 }
 
-GEN 
+GEN
 perm_to_GAP(GEN p)
 {
   pari_sp ltop=avma;
@@ -629,7 +361,7 @@ perm_to_GAP(GEN p)
   long lp=lg(p)-1;
   if (typ(p) != t_VECSMALL)  pari_err(typeer, "perm_to_GAP");
   x = perm_cycles(p);
-  sz = (long) ((bfffo(lp)+1) * L2SL10 + 1);
+  sz = (long) ((bfffo(lp)+1) * LOG10_2 + 1);
   /*Dry run*/
   for (i = 1, nb = 1; i < lg(x); ++i)
   {
@@ -662,16 +394,17 @@ perm_to_GAP(GEN p)
     }
   }
   if (!c) { s[c++]='('; s[c++]=')'; }
-  s[c++] = 0;
+  s[c] = '\0';
   return gerepileupto(ltop,gap);
 }
 
 int
-perm_commute(GEN p, GEN q)
+perm_commute(GEN s, GEN t)
 {
-  pari_sp ltop = avma;
-  int r = gequal(perm_mul(p,q), perm_mul(q,p));
-  avma = ltop; return r;
+  long i, l = lg(t);
+  for (i = 1; i < l; i++)
+    if (t[ s[i] ] != s[ t[i] ]) return 0;
+  return 1;
 }
 
 /*************************************************************************/
@@ -679,39 +412,41 @@ perm_commute(GEN p, GEN q)
 /**                  Routines for handling groups                       **/
 /**                                                                     **/
 /*************************************************************************/
-
-/* Groups are vectors [gen,orders]
- * gen (vecvecsmall): list of generators given by permutations 
+/* A Group is a t_VEC [gen,orders]
+ * gen (vecvecsmall): list of generators given by permutations
  * orders (vecsmall): relatives orders of generators. */
+INLINE GEN grp_get_gen(GEN G) { return gel(G,1); }
+INLINE GEN grp_get_ord(GEN G) { return gel(G,2); }
+
+/* A Quotient Group is a t_VEC [gen,coset]
+ * gen (vecvecsmall): coset generators
+ * coset (vecsmall): gen[coset[p[1]]] generate the p-coset.
+ */
+INLINE GEN quo_get_gen(GEN C) { return gel(C,1); }
+INLINE GEN quo_get_coset(GEN C) { return gel(C,2); }
 
 static GEN
 trivialsubgroups(void)
-{
-  GEN p2 = cgetg(2, t_VEC);
-  gel(p2,1) = mkvec2(cgetg(1,t_VEC), cgetg(1,t_VECSMALL));
-  return p2;
-}
+{ GEN L = cgetg(2, t_VEC); gel(L,1) = trivialgroup(); return L; }
 
-/* Compute the orders of p modulo the group S given by a set */
+/* Compute the order of p modulo the group given by a set */
 long
-perm_relorder(GEN p, GEN S)
+perm_relorder(GEN p, GEN set)
 {
   pari_sp ltop = avma;
   long n = 1;
-  GEN  q = p;
-  while (!vecvecsmall_search(S, q, 0)) { q = perm_mul(q, p); n++; }
+  long q = p[1];
+  while (!F2v_coeff(set,q)) { q = p[q]; n++; }
   avma = ltop; return n;
 }
 
 GEN
 perm_generate(GEN S, GEN H, long o)
 {
-  long i, k, n = lg(H)-1;
-  GEN L = cgetg(1+n*o, t_VEC);
-  for(i=1; i<=n; i++)
-    gel(L,i) = vecsmall_copy(gel(H,i));
-  for(k=n+1; k <= n*o; ++k)
-    gel(L,k) = perm_mul(gel(L,k-n), S);
+  long i, n = lg(H)-1;
+  GEN L = cgetg(n*o + 1, t_VEC);
+  for(i=1; i<=n;     i++) gel(L,i) = vecsmall_copy(gel(H,i));
+  for(   ; i <= n*o; i++) gel(L,i) = perm_mul(gel(L,i-n), S);
   return L;
 }
 
@@ -719,149 +454,191 @@ perm_generate(GEN S, GEN H, long o)
 long
 group_order(GEN G)
 {
-  GEN ord = gel(G,2);
-  long i, card = 1;
-  for (i = 1; i < lg(ord); i++) card *= ord[i];
-  return card;
+  return zv_prod(grp_get_ord(G));
 }
 
 /* G being a subgroup of S_n, output n */
 long
 group_domain(GEN G)
 {
-  if (lg(G[1]) < 2) pari_err(talker,"empty group in group_domain");
-  return lg(gmael(G,1,1)) - 1;
+  GEN gen = grp_get_gen(G);
+  if (lg(gen) < 2) pari_err(talker,"empty group in group_domain");
+  return lg(gel(gen,1)) - 1;
 }
 
-/*Compute the left coset of g mod G: gG*/
+/*Left coset of g mod G: gG*/
 GEN
 group_leftcoset(GEN G, GEN g)
 {
-  GEN res;
+  GEN gen = grp_get_gen(G), ord = grp_get_ord(G);
+  GEN res = cgetg(group_order(G)+1, t_VEC);
   long i, j, k;
-  GEN gen = gel(G,1);
-  GEN ord = gel(G,2);
-  res = cgetg(group_order(G)+1, t_VEC);
   gel(res,1) = vecsmall_copy(g);
   k = 1;
   for (i = 1; i < lg(gen); i++)
   {
     long c = k * (ord[i] - 1);
-    for (j = 1; j <= c; j++)
-      gel(res,++k) = perm_mul(gel(res,j), gel(gen,i));
+    for (j = 1; j <= c; j++) gel(res,++k) = perm_mul(gel(res,j), gel(gen,i));
   }
   return res;
 }
-
-/*Compute the right coset of g mod G: Gg*/
+/*Right coset of g mod G: Gg*/
 GEN
 group_rightcoset(GEN G, GEN g)
 {
-  GEN res;
+  GEN gen = grp_get_gen(G), ord = grp_get_ord(G);
+  GEN res = cgetg(group_order(G)+1, t_VEC);
   long i, j, k;
-  GEN gen = gel(G,1);
-  GEN ord = gel(G,2);
-  res = cgetg(group_order(G)+1, t_VEC);
   gel(res,1) = vecsmall_copy(g);
   k = 1;
   for (i = 1; i < lg(gen); i++)
   {
     long c = k * (ord[i] - 1);
-    for (j = 1; j <= c; j++)
-      gel(res,++k) = perm_mul(gel(gen,i), gel(res,j));
+    for (j = 1; j <= c; j++) gel(res,++k) = perm_mul(gel(gen,i), gel(res,j));
+  }
+  return res;
+}
+/*Elements of a group from the generators, cf group_leftcoset*/
+GEN
+group_elts(GEN G, long n)
+{
+  GEN gen = grp_get_gen(G), ord = grp_get_ord(G);
+  GEN res = cgetg(group_order(G)+1, t_VEC);
+  long i, j, k;
+  gel(res,1) = identity_perm(n);
+  k = 1;
+  for (i = 1; i < lg(gen); i++)
+  {
+    long c = k * (ord[i] - 1);
+    /* j = 1, use res[1] = identity */
+    gel(res,++k) = vecsmall_copy(gel(gen,i));
+    for (j = 2; j <= c; j++) gel(res,++k) = perm_mul(gel(res,j), gel(gen,i));
   }
   return res;
 }
 
-/*Compute the elements of a group from the generators*/
-/*Not stack clean!*/
 GEN
-group_elts(GEN G, long n)
+groupelts_set(GEN elts, long n)
 {
-  return group_leftcoset(G, perm_identity(n));
+  GEN res = zero_F2v(n);
+  long i, l = lg(elts);
+  for(i=1; i<l; i++)
+    F2v_set(res,mael(elts,i,1));
+  return res;
 }
 
+/*Elements of a group from the generators, returned as a set (bitmap)*/
+GEN
+group_set(GEN G, long n)
+{
+  GEN res = zero_F2v(n);
+  pari_sp av = avma;
+  GEN elts = group_elts(G, n);
+  long i, l = lg(elts);
+  for(i=1; i<l; i++)
+    F2v_set(res,mael(elts,i,1));
+  avma = av;
+  return res;
+}
+
+static int
+sgcmp(GEN a, GEN b) { return vecsmall_lexcmp(gel(a,1),gel(b,1)); }
+
+GEN
+subgroups_tableset(GEN S, long n)
+{
+  long i, l = lg(S);
+  GEN  v = cgetg(l, t_VEC);
+  for(i=1; i<l; i++)
+    gel(v,i) = mkvec2(group_set(gel(S,i), n), mkvecsmall(i));
+  gen_sort_inplace(v,(void*)sgcmp,cmp_nodata, NULL);
+  return v;
+}
+
+long
+tableset_find_index(GEN tbl, GEN set)
+{
+  long i = tablesearch(tbl,mkvec2(set,mkvecsmall(0)),sgcmp);
+  if (!i) return 0;
+  return mael3(tbl,i,2,1);
+}
+
+GEN
+trivialgroup(void)
+{
+  GEN G = cgetg(3, t_VEC);
+  gel(G,1) = cgetg(1,t_VEC);
+  gel(G,2) = cgetg(1,t_VECSMALL); return G;
+}
 /*Cyclic group generated by g of order s*/
 GEN
 cyclicgroup(GEN g, long s)
 {
-  GEN p2 = cgetg(3, t_VEC);
-  gel(p2,1) = mkvec( vecsmall_copy(g) );
-  gel(p2,2) = mkvecsmall(s); return p2;
+  GEN G = cgetg(3, t_VEC);
+  gel(G,1) = mkvec( vecsmall_copy(g) );
+  gel(G,2) = mkvecsmall(s); return G;
 }
-
-/*Return the group generated by g1,g2 of rel orders s1,s2*/
-
+/*Return the group generated by g1,g2 of relative orders s1,s2*/
 GEN
 dicyclicgroup(GEN g1, GEN g2, long s1, long s2)
 {
-  GEN H = cgetg(3, t_VEC);
-  GEN p3,p4;
-  p3 = cgetg(3, t_VEC);
-  gel(p3,1) = vecsmall_copy((GEN)g1);
-  gel(p3,2) = vecsmall_copy((GEN)g2);
-  p4 = cgetg(3,t_VECSMALL);
-  p4[1] = s1;
-  p4[2] = s2;
-  gel(H,1) = p3;
-  gel(H,2) = p4;
-  return H;
+  GEN G = cgetg(3, t_VEC);
+  gel(G,1) = mkvec2(vecsmall_copy(g1), vecsmall_copy(g2));
+  gel(G,2) = mkvecsmall2(s1, s2);
+  return G;
 }
 
 /* return the quotient map G --> G/H */
 /*The ouput is [gen,hash]*/
 /* gen (vecvecsmall): coset generators
- * hash (vecvecsmall): sorted vecsmall of concat(perm,coset number) */
+ * coset (vecsmall): vecsmall of coset number) */
 GEN
 group_quotient(GEN G, GEN H)
 {
   pari_sp ltop = avma;
-  GEN p1, p2, p3;
-  long i, j, k, a = 1;
+  GEN  p2, p3;
+  long i, j, a = 1;
   long n = group_domain(G), o = group_order(H);
-  GEN elt = vecvecsmall_sort(group_elts(G,n));
+  GEN  elt = group_elts(G,n), el;
   long le = lg(elt)-1;
-  GEN used = bitvec_alloc(le+1);
+  GEN used = zero_F2v(le+1);
   long l = le/o;
   p2 = cgetg(l+1, t_VEC);
-  p3 = cgetg(le+1, t_VEC);
-  for (i = 1, k = 1; i <= l; ++i)
+  p3 = const_vecsmall(n, 0);
+  el = const_vecsmall(n, 0);
+  for (i = 1; i<=le; i++)
+    el[mael(elt,i,1)]=i;
+  for (i = 1; i <= l; ++i)
   {
     GEN V;
-    while(bitvec_test(used,a)) a++;
+    while(F2v_coeff(used,a)) a++;
     V = group_leftcoset(H,gel(elt,a));
-    p2[i] = V[1];
-    for(j=1;j<lg(V);j++) 
+    gel(p2,i) = gel(V,1);
+    for(j=1;j<lg(V);j++)
     {
-      long b=vecvecsmall_search(elt,gel(V,j),0);
-      bitvec_set(used,b);
+      long b = el[mael(V,j,1)];
+      if (b==0) pari_err(talker, "not a WSS group");
+      F2v_set(used,b);
     }
     for (j = 1; j <= o; j++)
-      gel(p3,k++) = vecsmall_append(gel(V,j),i);
+      p3[mael(V, j, 1)] = i;
   }
-  p1 = cgetg(3,t_VEC);
-  gel(p1,1) = gcopy(p2);
-  gel(p1,2) = vecvecsmall_sort(p3);
-  return gerepileupto(ltop,p1);
-}
-
-/*Find in which coset a perm lie.*/
-long
-cosets_perm_search(GEN C, GEN p)
-{
-  long n = gen_search(gel(C,2),p,0,vecsmall_prefixcmp);
-  if (!n) pari_err(talker, "coset not found in cosets_perm_search");
-  return mael3(C,2,n,lg(p));
+  return gerepilecopy(ltop,mkvec2(p2,p3));
 }
 
 /*Compute the image of a permutation by a quotient map.*/
 GEN
 quotient_perm(GEN C, GEN p)
 {
-  long j, l2 = lg(C[1]);
-  GEN p3 = cgetg(l2, t_VECSMALL);
-  for (j = 1; j < l2; ++j)
-    p3[j] = cosets_perm_search(C, perm_mul(p, gmael(C,1,j)));
+  GEN gen = quo_get_gen(C);
+  GEN coset = quo_get_coset(C);
+  long j, l = lg(gen);
+  GEN p3 = cgetg(l, t_VECSMALL);
+  for (j = 1; j < l; ++j)
+  {
+    p3[j] = coset[p[mael(gen,j,1)]];
+    if (p3[j]==0) pari_err(talker,"not a WSS group");
+  }
   return p3;
 }
 
@@ -871,77 +648,83 @@ quotient_perm(GEN C, GEN p)
 GEN
 quotient_subgroup_lift(GEN C, GEN H, GEN S)
 {
-  long l1 = lg(H[1])-1;
-  long l2 = lg(S[1])-1, j;
+  GEN genH = grp_get_gen(H);
+  GEN genS = grp_get_gen(S);
+  GEN genC = quo_get_gen(C);
+  long l1 = lg(genH)-1;
+  long l2 = lg(genS)-1, j;
   GEN p1 = cgetg(3, t_VEC), L = cgetg(l1+l2+1, t_VEC);
-  for (j = 1; j <= l1; ++j) gel(L,j) = gmael(H,1,j);
-  for (j = 1; j <= l2; ++j) gel(L,l1+j) = gmael(C,1,mael3(S,1,j,1));
+  for (j = 1; j <= l1; ++j) gel(L,j) = gel(genH,j);
+  for (j = 1; j <= l2; ++j) gel(L,l1+j) = gel(genC, mael(genS,j,1));
   gel(p1,1) = L;
-  gel(p1,2) = vecsmall_concat(gel(H,2),gel(S,2));
+  gel(p1,2) = vecsmall_concat(grp_get_ord(H), grp_get_ord(S));
   return p1;
 }
 
-/* Let G a group and H a quotient map G --> G/H
+/* Let G a group and C a quotient map G --> G/H
  * Assume H is normal, return the group G/H */
 GEN
 quotient_group(GEN C, GEN G)
 {
   pari_sp ltop = avma;
-  GEN Qgen, Qord, Qelt, Q;
-  long i,j, n = lg(C[1])-1, l = lg(G[1]);
+  GEN Qgen, Qord, Qelt, Qset, Q;
+  GEN Cgen = quo_get_gen(C);
+  GEN Ggen = grp_get_gen(G);
+  long i,j, n = lg(Cgen)-1, l = lg(Ggen);
   Qord = cgetg(l, t_VECSMALL);
   Qgen = cgetg(l, t_VEC);
-  Qelt = mkvec(perm_identity(n));
+  Qelt = mkvec(identity_perm(n));
+  Qset = groupelts_set(Qelt, n);
   for (i = 1, j = 1; i < l; ++i)
   {
-    gel(Qgen,j) = quotient_perm(C, gmael(G,1,i));
-    Qord[j] = perm_relorder(gel(Qgen,j), vecvecsmall_sort(Qelt));
-    if (Qord[j] != 1)
+    GEN  g = quotient_perm(C, gel(Ggen,i));
+    long o = perm_relorder(g, Qset);
+    gel(Qgen,j) = g;
+    Qord[j] = o;
+    if (o != 1)
     {
-      Qelt = perm_generate(gel(Qgen,j), Qelt, Qord[j]);
+      Qelt = perm_generate(g, Qelt, o);
+      Qset = groupelts_set(Qelt, n);
       j++;
     }
   }
   setlg(Qgen,j);
   setlg(Qord,j); Q = mkvec2(Qgen, Qord);
-  if (group_order(Q) != n) pari_err(talker,"galoissubgroup: not a WSS group");
   return gerepilecopy(ltop,Q);
 }
 
-/* Test if g normalize N*/
+/* Return 1 if g normalizes N, 0 otherwise */
 long
 group_perm_normalize(GEN N, GEN g)
 {
   pari_sp ltop = avma;
   long r = gequal(vecvecsmall_sort(group_leftcoset(N, g)),
-                vecvecsmall_sort(group_rightcoset(N, g)));
+                  vecvecsmall_sort(group_rightcoset(N, g)));
   avma = ltop; return r;
 }
 
-/* L is a list of subgroups, C is a coset and r a rel. order.*/
+/* L is a list of subgroups, C is a coset and r a relative order.*/
 static GEN
 liftlistsubgroups(GEN L, GEN C, long r)
 {
   pari_sp ltop = avma;
+  long c = lg(C)-1, l = lg(L)-1, n = lg(gel(C,1))-1, i, k;
   GEN R;
-  long c = lg(C)-1;
-  long l = lg(L)-1, n = lg(C[1])-1, i, k;
-  if ( !l )
-    return cgetg(1,t_VEC);
+  if (!l) return cgetg(1,t_VEC);
   R = cgetg(l*c+1, t_VEC);
   for (i = 1, k = 1; i <= l; ++i)
   {
-    GEN S = gel(L,i), Selt = vecvecsmall_sort(group_elts(S,n));
+    GEN S = gel(L,i), Selt = group_set(S,n);
+    GEN gen = grp_get_gen(S);
+    GEN ord = grp_get_ord(S);
     long j;
     for (j = 1; j <= c; ++j)
-      if ((perm_relorder(gel(C,j), Selt) == r)
-        && group_perm_normalize(S, gel(C,j)))
-      {
-        GEN g = cgetg(3, t_VEC);
-        gel(g,1) = vecsmall_append(gel(S,1), C[j]);
-        gel(g,2) = vecsmall_append(gel(S,2), r);
-        gel(R,k++) = g;
-      }
+    {
+      GEN p = gel(C,j);
+      if (perm_relorder(p, Selt) == r && group_perm_normalize(S, p))
+        gel(R,k++) = mkvec2(vecsmall_append(gen, (long)p),
+                            vecsmall_append(ord, r));
+    }
   }
   setlg(R, k);
   return gerepilecopy(ltop, R);
@@ -956,11 +739,14 @@ liftsubgroup(GEN C, GEN H, GEN S)
 {
   pari_sp ltop = avma;
   GEN V = trivialsubgroups();
-  long n = lg(S[1]), i;
+  GEN Sgen = grp_get_gen(S);
+  GEN Sord = grp_get_ord(S);
+  GEN Cgen = quo_get_gen(C);
+  long n = lg(Sgen), i;
   for (i = 1; i < n; ++i)
   { /*loop over generators of S*/
-    GEN W = group_leftcoset(H, gmael(C, 1, mael3(S, 1, i, 1)));
-    V = liftlistsubgroups(V, W, mael(S, 2, i));
+    GEN W = group_leftcoset(H, gel(Cgen, mael(Sgen, i, 1)));
+    V = liftlistsubgroups(V, W, Sord[i]);
   }
   return gerepilecopy(ltop,V);
 }
@@ -969,8 +755,8 @@ liftsubgroup(GEN C, GEN H, GEN S)
 long
 group_isA4S4(GEN G)
 {
-  GEN elt = gel(G,1);
-  GEN ord = gel(G,2);
+  GEN elt = grp_get_gen(G);
+  GEN ord = grp_get_ord(G);
   long n = lg(ord);
   if (n != 4 && n != 5) return 0;
   if (ord[1]!=2 || ord[2]!=2 || ord[3]!=3) return 0;
@@ -982,38 +768,40 @@ group_isA4S4(GEN G)
 }
 /* compute all the subgroups of a group G */
 GEN
-group_subgroups(GEN G) 
+group_subgroups(GEN G)
 {
   pari_sp ltop = avma;
-  GEN p1, H, C, Q, M, sg1, sg2, sg3, gen = gel(G,1), ord = gel(G,2);
+  GEN p1, H, C, Q, M, sg1, sg2, sg3;
+  GEN gen = grp_get_gen(G);
+  GEN ord = grp_get_ord(G);
   long lM, i, j, n = lg(gen);
   if (n == 1) return trivialsubgroups();
   if (group_isA4S4(G))
   {
-    GEN u = perm_mul(gel(gen,1),gel(gen,2));
-    H = dicyclicgroup(gel(gen,1),gel(gen,2),2,2);
+    GEN s = gel(gen,1);       /*s = (1,2)(3,4) */
+    GEN t = gel(gen,2);       /*t = (1,3)(2,4) */
+    GEN st = perm_mul(s, t); /*st = (1,4)(2,3) */
+    H = dicyclicgroup(s, t, 2, 2);
     /* sg3 is the list of subgroups intersecting only partially with H*/
     sg3 = cgetg((n==4)?4: 10, t_VEC);
-    gel(sg3,1) = cyclicgroup(gel(gen,1), 2);
-    gel(sg3,2) = cyclicgroup(gel(gen,2), 2);
-    gel(sg3,3) = cyclicgroup(u, 2);
+    gel(sg3,1) = cyclicgroup(s, 2);
+    gel(sg3,2) = cyclicgroup(t, 2);
+    gel(sg3,3) = cyclicgroup(st, 2);
     if (n==5)
     {
-      GEN s = gel(gen,1); /*s=(1,2)(3,4)*/
-      GEN t = gel(gen,2); /*t=(1,3)(2,4)*/
       GEN u = gel(gen,3);
-      GEN v = gel(gen,4), st = perm_mul(s,t), w, u2;
-      if (gequal(perm_conj(u,s), t)) /*u=(2,3,4)*/
-        u2 = perm_mul(u,u); 
-      else 
-      { 
-        u2 = u; 
-        u = perm_mul(u,u); 
+      GEN v = gel(gen,4), w, u2;
+      if (zv_equal(perm_conj(u,s), t)) /*u=(2,3,4)*/
+        u2 = perm_mul(u,u);
+      else
+      {
+        u2 = u;
+        u = perm_mul(u,u);
       }
       if (perm_order(v)==2)
       {
         if (!perm_commute(s,v)) /*v=(1,2)*/
-        { 
+        {
           v = perm_conj(u,v);
           if (!perm_commute(s,v)) v = perm_conj(u,v);
         }
@@ -1022,10 +810,10 @@ group_subgroups(GEN G)
       else
       {
         w = v;
-        if (!gequal(perm_mul(w,w), s)) /*w=(1,4,2,3)*/
+        if (!zv_equal(perm_mul(w,w), s)) /*w=(1,4,2,3)*/
         {
           w = perm_conj(u,w);
-          if (!gequal(perm_mul(w,w), s)) w = perm_conj(u,w);
+          if (!zv_equal(perm_mul(w,w), s)) w = perm_conj(u,w);
         }
         v = perm_mul(w,t); /*v=(1,2)*/
       }
@@ -1046,31 +834,27 @@ group_subgroups(GEN G)
   }
   C = group_quotient(G,H);
   Q = quotient_group(C,G);
-  M = group_subgroups(Q);
-  lM = lg(M);
+  M = group_subgroups(Q); lM = lg(M);
   /* sg1 is the list of subgroups containing H*/
   sg1 = cgetg(lM, t_VEC);
-  for (i = 1; i < lM; ++i)
-    gel(sg1,i) = quotient_subgroup_lift(C,H,gel(M,i));
+  for (i = 1; i < lM; ++i) gel(sg1,i) = quotient_subgroup_lift(C,H,gel(M,i));
   /*sg2 is a list of lists of subgroups not intersecting with H*/
   sg2 = cgetg(lM, t_VEC);
   /* Loop over all subgroups of G/H */
-  for (j = 1; j < lM; ++j)
-    gel(sg2,j) = liftsubgroup(C, H, gel(M,j));
-  p1 = concat(sg1, concat(sg2,NULL));
+  for (j = 1; j < lM; ++j) gel(sg2,j) = liftsubgroup(C, H, gel(M,j));
+  p1 = concat(sg1, shallowconcat1(sg2));
   if (sg3)
   {
     p1 = concat(p1, sg3);
-    /*Fixup to avoid the D4 subgroups of S4 to be in non supersolvable format*/
-    if (n==5)
+    if (n==5) /*ensure that the D4 subgroups of S4 are in supersolvable format*/
       for(j = 3; j <= 5; j++)
       {
         GEN c = gmael(p1,j,1);
         if (!perm_commute(gel(c,1),gel(c,3)))
         {
-          if (perm_commute(gel(c,2),gel(c,3))) { lswap(c[1], c[2]); }
+          if (perm_commute(gel(c,2),gel(c,3))) { swap(gel(c,1), gel(c,2)); }
           else
-            (void)perm_mul_inplace(gel(c,2), gel(c,1));
+            perm_mul_inplace2(gel(c,2), gel(c,1));
         }
       }
   }
@@ -1078,22 +862,22 @@ group_subgroups(GEN G)
 }
 
 /*return 1 if G is abelian, else 0*/
-long 
+long
 group_isabelian(GEN G)
 {
-  long i, j;
-  GEN g = gel(G,1);
-  for(i=2; i<lg(g); i++)
+  GEN g = grp_get_gen(G);
+  long i, j, n = lg(g);
+  for(i=2; i<n; i++)
     for(j=1; j<i; j++)
       if (!perm_commute(gel(g,i), gel(g,j))) return 0;
-  return 1;  
+  return 1;
 }
 
 /*If G is abelian, return its HNF matrix*/
 GEN
 group_abelianHNF(GEN G, GEN S)
 {
-  GEN M, g = gel(G,1), o = gel(G,2);
+  GEN M, g = grp_get_gen(G), o = grp_get_ord(G);
   long i, j, k, n = lg(g);
   if (!group_isabelian(G)) return NULL;
   if (n==1) return cgetg(1,t_MAT);
@@ -1106,7 +890,7 @@ group_abelianHNF(GEN G, GEN S)
     gel(M,i) = C;
     P = perm_pow(gel(g,i), o[i]);
     for(j=1; j<lg(S); j++)
-      if (gequal(P, gel(S,j))) break;
+      if (zv_equal(P, gel(S,j))) break;
     avma = av;
     if (j==lg(S)) pari_err(talker,"wrong argument in galoisisabelian");
     j--;
@@ -1115,7 +899,7 @@ group_abelianHNF(GEN G, GEN S)
       long q = j / o[k];
       gel(C,k) = stoi(j - q*o[k]);
       j = q;
-    }  
+    }
     gel(C,k) = stoi(o[i]);
     for (k++; k<n; k++) gel(C,k) = gen_0;
   }
@@ -1129,22 +913,22 @@ group_abelianSNF(GEN G, GEN L)
   pari_sp ltop = avma;
   GEN H = group_abelianHNF(G,L);
   if (!H) return NULL;
-  return gerepileupto(ltop, smithclean( smith(H) ));
+  return gerepileupto(ltop, smithclean( ZM_snf(H) ));
 }
 
 GEN
 abelian_group(GEN v)
 {
-  GEN G = cgetg(3,t_VEC);
-  long card, i, d = 1;
-  gel(G,1) = cgetg(lg(v),t_VEC);
+  long card, i, d = 1, l = lg(v);
+  GEN G = cgetg(3,t_VEC), gen = cgetg(l,t_VEC);
+  gel(G,1) = gen;
   gel(G,2) = vecsmall_copy(v);
   card = group_order(G);
-  for(i=1; i<lg(v); i++)
+  for(i=1; i<l; i++)
   {
     GEN p = cgetg(card+1, t_VECSMALL);
     long o = v[i], u = d*(o-1), j, k, l;
-    gmael(G,1,i) = p;
+    gel(gen, i) = p;
     /* The following loop is over-optimized. Remember that I wrote it for
      * testpermutation. Something has survived... BA */
     for(j=1;j<=card;)
@@ -1158,89 +942,106 @@ abelian_group(GEN v)
   return G;
 }
 
+/*return 1 if G is abelian, else 0*/
+long
+group_subgroup_isnormal(GEN G, GEN H)
+{
+  GEN g = grp_get_gen(G);
+  long i, n = lg(g);
+  if (lg(grp_get_gen(H)) > 1 && group_domain(G) != group_domain(H))
+    pari_err(talker,"not a subgroup in group_subgroup_isnormal");
+  for(i=1; i<n; i++)
+    if (!group_perm_normalize(H, gel(g,i))) return 0;
+  return 1;
+}
+
 GEN
 groupelts_center(GEN S)
 {
   pari_sp ltop = avma;
   long i, j, n = lg(S)-1, l = n;
-  GEN V, elts = bitvec_alloc(n+1);
+  GEN V, elts = zero_F2v(n+1);
   for(i=1; i<=n; i++)
   {
-    if (bitvec_test(elts,i)) { l--;  continue; }
+    if (F2v_coeff(elts,i)) { l--;  continue; }
     for(j=1; j<=n; j++)
       if (!perm_commute(gel(S,i),gel(S,j)))
       {
-        bitvec_set(elts,i);
-        bitvec_set(elts,j); l--; break;
+        F2v_set(elts,i);
+        F2v_set(elts,j); l--; break;
       }
   }
   V = cgetg(l+1,t_VEC);
   for (i=1, j=1; i<=n ;i++)
-    if (!bitvec_test(elts,i)) gel(V,j++) = vecsmall_copy(gel(S,i));
+    if (!F2v_coeff(elts,i)) gel(V,j++) = vecsmall_copy(gel(S,i));
   return gerepileupto(ltop,V);
 }
 
+/* S a list of generators */
 GEN
 groupelts_abelian_group(GEN S)
 {
   pari_sp ltop = avma;
   GEN Qgen, Qord, Qelt;
-  long i, j, n = lg(S[1])-1, l = lg(S);
+  long i, j, n = lg(gel(S,1))-1, l = lg(S);
   Qord = cgetg(l, t_VECSMALL);
   Qgen = cgetg(l, t_VEC);
-  Qelt = mkvec(perm_identity(n));
+  Qelt = mkvec(identity_perm(n));
   for (i = 1, j = 1; i < l; ++i)
   {
-    Qgen[j] = S[i];
-    Qord[j] = perm_relorder(gel(Qgen,j), vecvecsmall_sort(Qelt));
-    if (Qord[j] != 1)
-    {
-      Qelt = perm_generate(gel(Qgen,j), Qelt, Qord[j]);
-      j++;
-    }
+    GEN  g = gel(S,i);
+    long o = perm_relorder(g, groupelts_set(Qelt, n));
+    gel(Qgen,j) = g;
+    Qord[j] = o;
+    if (o != 1) { Qelt = perm_generate(g, Qelt, o); j++; }
   }
   setlg(Qgen,j);
-  setlg(Qord,j); 
+  setlg(Qord,j);
   return gerepilecopy(ltop, mkvec2(Qgen, Qord));
 }
- 
-GEN 
+
+GEN
 group_export_GAP(GEN G)
 {
-  pari_sp ltop = avma;
-  GEN s, g = gel(G,1);
-  long i, l = lg(g);
+  pari_sp av = avma;
+  GEN s, comma, g = grp_get_gen(G);
+  long i, k, l = lg(g);
   if (l == 1) return strtoGENstr("Group(())");
-  s = strtoGENstr("Group(");
+  s = cgetg(2*l, t_VEC); k = 2;
+  comma = strtoGENstr(", ");
+  gel(s,1) = strtoGENstr("Group(");
   for (i = 1; i < l; ++i)
   {
-    if (i > 1) s = shallowconcat(s, strtoGENstr(", "));
-    s = shallowconcat(s, perm_to_GAP(gel(g,i)));
+    if (i > 1) gel(s,k++) = comma;
+    gel(s,k++) = perm_to_GAP(gel(g,i));
   }
-  s = concat(s, strtoGENstr(")"));
-  return gerepileupto(ltop,s);
-}  
+  gel(s,k++) = strtoGENstr(")");
+  return gerepilecopy(av, shallowconcat1(s));
+}
 
-GEN 
+GEN
 group_export_MAGMA(GEN G)
 {
-  pari_sp ltop=avma;
-  GEN s, g = gel(G,1);
-  long i, l = lg(g);
+  pari_sp av = avma;
+  GEN s, comma, g = grp_get_gen(G);
+  long i, k, l = lg(g);
   if (l == 1) return strtoGENstr("PermutationGroup<1|>");
-  s = strtoGENstr("PermutationGroup<");
-  s = shallowconcat(s, stoi(group_domain(G)));
-  s = shallowconcat(s, strtoGENstr("|"));
+  s = cgetg(2*l+2, t_VEC); k = 1;
+  gel(s,k++) = strtoGENstr("PermutationGroup<");
+  gel(s,k++) = strtoGENstr( itostr( stoi(group_domain(G)) ) );
+  gel(s,k++) = strtoGENstr("|"); comma = strtoGENstr(", ");
   for (i = 1; i < l; ++i)
   {
-    if (i > 1) s = shallowconcat(s, strtoGENstr(", "));
-    s = shallowconcat(s, vecsmall_to_vec(gel(g,i)));
+    char *t = GENtostr( vecsmall_to_vec(gel(g,i)) );
+    if (i > 1) gel(s,k++) = comma;
+    gel(s,k++) = strtoGENstr(t);
+    pari_free(t);
   }
-  s = concat(s, strtoGENstr(">"));
-  return gerepileupto(ltop,s);
-}  
+  gel(s,k++) = strtoGENstr(">");
+  return gerepilecopy(av, shallowconcat1(s));
+}
 
-GEN 
+GEN
 group_export(GEN G, long format)
 {
   switch(format)
