@@ -1,4 +1,4 @@
-/* $Id: gp_rl.c 12098 2010-01-29 13:40:22Z bill $
+/* $Id: gp_rl.c 13215 2011-06-08 08:41:49Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -23,7 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #ifdef READLINE
 
 #include "paripriv.h"
-#include "../language/anal.h"
 #include "gp.h"
 
 typedef int (*RLCI)(int, int); /* rl_complete and rl_insert functions */
@@ -36,16 +35,9 @@ BEGINEXTERN
 #  define PREFER_STDARG
 #endif
 
-#ifdef READLINE_LIBRARY
-#  include <readline.h>
-#  ifdef HAS_HISTORY_H
-#    include <history.h>
-#  endif
-#else
-#  include <readline/readline.h>
-#  ifdef HAS_HISTORY_H
-#    include <readline/history.h>
-#  endif
+#include <readline/readline.h>
+#ifdef HAS_HISTORY_H
+#  include <readline/history.h>
 #endif
 
 #ifndef HAS_HISTORY_H
@@ -93,12 +85,16 @@ ENDEXTERN
 #endif
 /**************************************************************************/
 
+enum { DO_MATCHED_INSERT = 2, DO_ARGS_COMPLETE = 4 };
+static ulong readline_state = DO_ARGS_COMPLETE;
+static char *current_histfile = NULL;
+
 static int pari_rl_back;
 static int did_init_matched = 0;
 static entree *current_ep = NULL;
 
 static int
-change_state(char *msg, ulong flag, int count)
+change_state(const char *msg, ulong flag, int count)
 {
   int c = (readline_state & flag) != 0;
   ulong o_readline_state = readline_state;
@@ -114,15 +110,10 @@ change_state(char *msg, ulong flag, int count)
   else {
     readline_state &= ~flag;
     if (!readline_state && o_readline_state)
-	readline_state = 1;
+        readline_state = 1;
   }
   SAVE_PROMPT();
-#if 0				/* Does not work well... */
-  rl_message("%s[%s: %s] %s", term_get_color(c_PROMPT),
-	     msg, c? "on": "off", term_get_color(c_INPUT));
-#else
   rl_message("[%s: %s] ", msg, c? "on": "off");
-#endif
   c = rl_read_key();
   RESTORE_PROMPT();
   rl_clear_message();
@@ -184,7 +175,7 @@ pari_rl_matched_insert(int count, int key)
   if (count <= 0)
     return change_state("electric parens", DO_MATCHED_INSERT, count);
   while (paropen[i] && paropen[i] != key) i++;
-  if (!paropen[i] || !(readline_state & DO_MATCHED_INSERT) || GP_DATA->flags & EMACS)
+  if (!paropen[i] || !(readline_state & DO_MATCHED_INSERT) || GP_DATA->flags & gpd_EMACS)
     return ((RLCI)rl_insert)(count,key);
   rl_begin_undo_group();
   ((RLCI)rl_insert)(count,key);
@@ -217,8 +208,8 @@ pari_rl_forward_sexp(int count, int key)
   }
   while (count || deep)
   {
-    move_point = 1;	/* Need to move point if moving left. */
-    lfail = 0;		/* Do not need to fail left movement yet. */
+    move_point = 1;        /* Need to move point if moving left. */
+    lfail = 0;                /* Do not need to fail left movement yet. */
     while ( !is_keyword_char(rl_line_buffer[rl_point])
             && !strchr("\"([{}])",rl_line_buffer[rl_point])
             && !( (dir == 1)
@@ -261,7 +252,7 @@ pari_rl_forward_sexp(int count, int key)
                   || (move_point = 0)) )
         rl_point += dir;
       if (bad) goto fail;
-      rl_point += dir;	/* Skip the other delimiter */
+      rl_point += dir;        /* Skip the other delimiter */
       if (!deep) count--;
     }
     else
@@ -284,12 +275,12 @@ static int
 add_paren(int end)
 {
   entree *ep;
-  char *s;
+  const char *s;
 
   if (end < 0 || rl_line_buffer[end] == '(')
     return 0; /* not from command_generator or already there */
   ep = do_alias(current_ep); /* current_ep set in command_generator */
-  if (EpVALENCE(ep) < EpUSER)
+  if (EpVALENCE(ep) < EpNEW)
   { /* is it a constant masked as a function (e.g Pi)? */
     s = ep->help; if (!s) return 1;
     while (is_keyword_char(*s)) s++;
@@ -297,16 +288,17 @@ add_paren(int end)
   }
   switch(EpVALENCE(ep))
   {
-    case EpUSER:
+    case EpVAR: return typ(ep->value) == t_CLOSURE;
     case EpINSTALL: return 1;
   }
   return 0;
 }
 
 static void
-match_concat(char **matches, char *s)
+match_concat(char **matches, const char *s)
 {
-  matches[0] = gprealloc(matches[0], strlen(matches[0])+strlen(s)+1);
+  matches[0] = (char*)pari_realloc((void*)matches[0],
+                                strlen(matches[0])+strlen(s)+1);
   strcat(matches[0],s);
 }
 
@@ -346,12 +338,12 @@ matches_for_emacs(const char *text, char **matches)
    /* we don't want readline to do anything, but insert some junk
     * which will be erased by emacs.
     */
-    for (i=0; matches[i]; i++) free(matches[i]);
-    free(matches);
+    for (i=0; matches[i]; i++) pari_free(matches[i]);
+    pari_free(matches);
   }
-  matches = (char **) gpmalloc(2*sizeof(char *));
-  matches[0] = gpmalloc(2); sprintf(matches[0],"_");
-  matches[1] = NULL; printf("@E_N_D"); pariflush();
+  matches = (char **) pari_malloc(2*sizeof(char *));
+  matches[0] = (char*)pari_malloc(2); sprintf(matches[0],"_");
+  matches[1] = NULL; printf("@E_N_D"); pari_flush();
   return matches;
 }
 
@@ -363,37 +355,24 @@ get_matches(int code, const char *text, GF f)
 {
   char **matches = COMPLETION_MATCHES(text, f);
   if (matches && !matches[1]) treat_single(code, matches);
-  if (GP_DATA->flags & EMACS) matches = matches_for_emacs(text,matches);
+  if (GP_DATA->flags & gpd_EMACS) matches = matches_for_emacs(text,matches);
   return matches;
 }
 
-#define DFLT 0
-#define ENTREE 1
 static char *
-generator(void *list, const char *text, int *nn, int len, int typ)
+generator(void *list, const char *text, int *nn, int len)
 {
-  char *def = NULL, *name;
+  const char *def = NULL;
   int n = *nn;
 
   /* Return the next name which partially matches from list.*/
-  switch(typ)
-  {
-    case DFLT :
-      do
-	def = (((default_type *) list)[n++]).name;
-      while (def && strncmp(def,text,len));
-      break;
-
-    case ENTREE :
-      do
-	def = (((entree *) list)[n++]).name;
-      while (def && strncmp(def,text,len));
-  }
-
+  do
+    def = (((entree *) list)[n++]).name;
+  while (def && strncmp(def,text,len));
   *nn = n;
   if (def)
   {
-    name = strcpy(gpmalloc(strlen(def)+1), def);
+    char *name = strcpy((char*)pari_malloc(strlen(def)+1), def);
     return name;
   }
   return NULL; /* no names matched */
@@ -404,28 +383,19 @@ old_generator(const char *text,int state)
   static int n,len;
   static char *res;
 
-  if (!state) { res = "a"; n=0; len=strlen(text); }
+  if (!state) { res = (char*)"a"; n=0; len=strlen(text); }
   if (res)
   {
-    res = generator((void *)oldfonctions,text,&n,len,ENTREE);
+    res = generator((void *)oldfonctions,text,&n,len);
     if (res) return res;
     n=0;
   }
-  return generator((void *)functions_oldgp,text,&n,len,ENTREE);
+  return generator((void *)functions_oldgp,text,&n,len);
 }
 static char *
-default_generator(const char *text,int state)
+add_prefix(const char *name, const char *text, long junk)
 {
-  static int n,len;
-
-  if (!state) { n=0; len=strlen(text); }
-  return generator(gp_default_list,text,&n,len,DFLT);
-}
-
-static char *
-add_prefix(char *name, const char *text, long junk)
-{
-  char *s = strncpy(gpmalloc(strlen(name)+1+junk),text,junk);
+  char *s = strncpy((char*)pari_malloc(strlen(name)+1+junk),text,junk);
   strcpy(s+junk,name); return s;
 }
 static void
@@ -438,6 +408,9 @@ init_prefix(const char *text, int *len, int *junk, char **TEXT)
   *junk = j;
   *len  = l - j;
 }
+static int
+is_internal(entree *ep) { return ep->menu >= 13 && ep->menu <= 15; }
+
 /* Generator function for command completion.  STATE lets us know whether
  * to start from scratch; without any state (i.e. STATE == 0), then we
  * start at the top of the list. */
@@ -467,19 +440,57 @@ hashtable_generator(const char *text, int state, entree **hash)
       if (++hashpos >= functions_tblsz) return NULL; /* no names matched */
       ep = hash[hashpos];
     }
-    else if (strncmp(ep->name,TEXT,len))
+    else if (is_internal(ep) || strncmp(ep->name,TEXT,len))
       ep = ep->next;
     else
       break;
   current_ep = ep; ep = ep->next;
   return add_prefix(current_ep->name,text,junk);
 }
+/* Generator function for member completion.  STATE lets us know whether
+ * to start from scratch; without any state (i.e. STATE == 0), then we
+ * start at the top of the list. */
+static char *
+member_generator(const char *text, int state)
+{
+  static int hashpos, len, junk;
+  static entree* ep;
+  static char *TEXT;
+  entree **hash=functions_hash;
+
+ /* If this is a new word to complete, initialize now:
+  *  + indexes hashpos (GP hash list) and n (keywords specific to long help).
+  *  + file completion and keyword completion use different word boundaries,
+  *    have TEXT point to the keyword start.
+  *  + save the length of TEXT for efficiency.
+  */
+  if (!state)
+  {
+    hashpos = 0; ep = hash[hashpos];
+    init_prefix(text, &len, &junk, &TEXT);
+  }
+
+  /* Return the next name which partially matches from the command list. */
+  for(;;)
+    if (!ep)
+    {
+      if (++hashpos >= functions_tblsz) return NULL; /* no names matched */
+      ep = hash[hashpos];
+    }
+    else if (ep->name[0]=='_' && ep->name[1]=='.'
+             && !strncmp(ep->name+2,TEXT,len))
+        break;
+    else
+        ep = ep->next;
+  current_ep = ep; ep = ep->next;
+  return add_prefix(current_ep->name+2,text,junk);
+}
 static char *
 command_generator(const char *text, int state)
 { return hashtable_generator(text,state, functions_hash); }
 static char *
-member_generator(const char *text, int state)
-{ return hashtable_generator(text,state, members_hash); }
+default_generator(const char *text,int state)
+{ return hashtable_generator(text,state, defaults_hash); }
 
 static char *
 ext_help_generator(const char *text, int state)
@@ -545,8 +556,8 @@ add_space(int start)
 #  ifdef HAS_COMPLETION_APPEND_CHAR
   rl_completion_append_character = '\0';
 #  endif
-  m = (char**)gpmalloc(2 * sizeof(char*));
-  m[0] = gpmalloc(1); *(m[0]) = 0;
+  m = (char**)pari_malloc(2 * sizeof(char*));
+  m[0] = (char*)pari_malloc(1); *(m[0]) = 0;
   m[1] = NULL; return m;
 #endif
 }
@@ -617,10 +628,10 @@ pari_completion(char *text, int START, int END)
     /* If we are in empty parens, insert the default arguments */
     if ((readline_state & DO_ARGS_COMPLETE) && k == j
          && (rl_line_buffer[j] == ')' || !rl_line_buffer[j])
-	 && (iend - i < (long)sizeof(buf))
-	 && ( strncpy(buf, rl_line_buffer + i, iend - i),
-	      buf[iend - i] = 0, 1)
-	 && (ep = is_entry(buf)) && ep->help)
+         && (iend - i < (long)sizeof(buf))
+         && ( strncpy(buf, rl_line_buffer + i, iend - i),
+              buf[iend - i] = 0, 1)
+         && (ep = is_entry(buf)) && ep->help)
     {
 #if 0 /* duplicate F1 */
       rl_print_aide(buf,h_RL);
@@ -629,19 +640,19 @@ pari_completion(char *text, int START, int END)
 #  endif
       return NULL;
 #else
-      char *s = ep->help;
+      const char *s = ep->help;
       while (is_keyword_char(*s)) s++;
       if (*s++ == '(')
       { /* function call: insert arguments */
-        char *e = s; 
+        const char *e = s;
         while (*e && *e != ')' && *e != '(') e++;
         if (*e == ')')
         { /* we just skipped over the arguments in short help text */
-          char *str = strncpy(gpmalloc(e-s + 1), s, e-s);
-          char **ret = (char**)gpmalloc(sizeof(char*)*2);
+          char *str = strncpy((char*)pari_malloc(e-s + 1), s, e-s);
+          char **ret = (char**)pari_malloc(sizeof(char*)*2);
           str[e-s] = 0;
           ret[0] = str; ret[1] = NULL;
-          if (GP_DATA->flags & EMACS) ret = matches_for_emacs("",ret);
+          if (GP_DATA->flags & gpd_EMACS) ret = matches_for_emacs("",ret);
           return ret;
         }
       }
@@ -662,40 +673,28 @@ pari_completion(char *text, int START, int END)
 static int
 rl_short_help(int count, int key)
 {
-  int flag = (count < 0 || rl_last_func == rl_short_help)? h_RL|h_LONG: h_RL;
-  int off = rl_point;
+  int flag = h_RL;
+  char *s = rl_line_buffer + rl_point;
 
   (void)key;
-  /* func() with cursor on ')' following completion */
-  if (off && rl_line_buffer[off-1] == '('
-          && !is_keyword_char(rl_line_buffer[off])) off--;
+  /* func() with cursor on ')', e.g. following completion */
+  if (s > rl_line_buffer && *s == ')' && s[-1] == '(') s--;
+  while (s > rl_line_buffer && is_keyword_char(s[-1])) s--;
+  /* check for '\c' */
+  if (s > rl_line_buffer && s[-1] == '\\') s--;
 
-  while (off && is_keyword_char(rl_line_buffer[off-1])) off--;
-
-  /* Check for \c type situation.  Could check for leading whitespace too... */
-  if (off == 1 && rl_line_buffer[off-1] == '\\') off--;
-  if (off >= 8) { /* Check for default(whatever) */
-    int t = off - 1;
-
-    while (t >= 7 && isspace((int)rl_line_buffer[t])) t--;
-    if (rl_line_buffer[t--] == '(') {
-      while (t >= 6 && isspace((int)rl_line_buffer[t])) t--;
-      t -= 6;
-      if (t >= 0
-	  && strncmp(rl_line_buffer + t, "default", 7) == 0
-	  && (t == 0 || !is_keyword_char(rl_line_buffer[t-1])))
-	off = t;
-    }
-  }
-  rl_print_aide(rl_line_buffer + off, flag);
-  return 0;
+  if (count < 0 || rl_last_func == rl_short_help) flag |= h_LONG;
+  rl_print_aide(s, flag); return 0;
 }
 
 static int
-rl_long_help(int count, int key)
+rl_long_help(int count, int key) { (void)count; return rl_short_help(-1,key); }
+
+static void
+init_histfile(void)
 {
-  (void)count;
-  return rl_short_help(-1,key);
+  if (current_histfile && read_history(current_histfile))
+    write_history(current_histfile);
 }
 
 void
@@ -704,13 +703,15 @@ init_readline(void)
   static int init_done = 0;
 
   if (init_done) return;
+  if (! GP_DATA->use_readline) readline_state = 0;
   init_done = 1;
+  init_histfile();
 
   /* Allow conditional parsing of the ~/.inputrc file. */
   rl_readline_name = "Pari-GP";
 
   /* added ~, ? and , */
-  rl_basic_word_break_characters = " \t\n\"\\'`@$><=;|&{(?,~";
+  rl_basic_word_break_characters = " \t\n\"\\'`@$><=;|&{(?~";
   rl_special_prefixes = "~";
 
   /* custom completer */
@@ -724,7 +725,7 @@ init_readline(void)
 
   /* we always want the whole list of completions under emacs */
 #ifdef HAS_RL_COMPLETION_QUERY_ITEMS
-  if (GP_DATA->flags & EMACS) rl_completion_query_items = 0x8fff;
+  if (GP_DATA->flags & gpd_EMACS) rl_completion_query_items = 0x8fff;
 #endif
 #ifdef HAS_RL_BIND_KEY_IN_MAP
 #  ifdef _RL_FUNCTION_TYPEDEF
@@ -753,8 +754,6 @@ init_readline(void)
 
   Bind('h', rl_short_help, emacs_meta_keymap);
   Bind('H', rl_long_help,  emacs_meta_keymap);
-  Bind('h', rl_short_help, vi_movement_keymap);
-  Bind('H', rl_long_help,  vi_movement_keymap);
 #  ifdef HAS_RL_GENERIC_BIND
 #define KSbind(s,f,k) rl_generic_bind(ISFUNC, (s), (char*)(f), (k))
 
@@ -762,8 +761,9 @@ init_readline(void)
   KSbind("[11~", rl_short_help,  emacs_meta_keymap); /* f1, xterm */
   KSbind("OP",   rl_short_help,  vi_movement_keymap); /* f1, vt100 */
   KSbind("[11~", rl_short_help,  vi_movement_keymap); /* f1, xterm */
-  /* XTerm may signal start/end of paste by eming F200/F201 */
-  /* XXXX For vi mode something more intelligent is needed - to switch to the
+  /* XTerm may signal start/end of paste by emitting F200/F201
+   * TODO: check to what extent this patch has been applied */
+  /* FIXME: For vi mode something more intelligent is needed - to switch to the
      insert mode - and back when restoring. */
   KSbind("[200~", pari_rl_matched_insert_suspend,  emacs_meta_keymap);  /* pre-paste xterm */
   KSbind("[200~", pari_rl_matched_insert_suspend,  vi_movement_keymap); /* pre-paste xterm */
@@ -783,11 +783,40 @@ init_readline(void)
 #endif
 }
 
+/* readline-specific defaults */
+GEN
+sd_readline(const char *v, long flag)
+{
+  const char *msg[] = {
+    "(bits 0x2/0x4 control matched-insert/arg-complete)",
+    "(bits 0x2/0x4 control matched-insert/arg-complete)", NULL};
+  ulong o_readline_state = readline_state;
+  GEN res = sd_ulong(v,flag,"readline", &readline_state, 0, 7, msg);
+
+  if (o_readline_state != readline_state)
+    (void)sd_toggle(readline_state? "1": "0", d_SILENT, "readline", &(GP_DATA->use_readline));
+  return res;
+}
+GEN
+sd_histfile(const char *v, long flag)
+{
+  char *old = current_histfile;
+  GEN r = sd_string(v, flag, "histfile", &current_histfile);
+  if (v && !*v)
+  {
+    free(current_histfile);
+    current_histfile = NULL;
+  }
+  else if (current_histfile != old && (!old || strcmp(old,current_histfile)))
+    init_histfile();
+  return r;
+}
+
 static void
 print_escape_string(char *s)
 {
   long l = strlen(s);
-  char *t, *t0 = gpmalloc(l * 3 + 3);
+  char *t, *t0 = (char*)pari_malloc(l * 3 + 3);
 
   t = t0; *t++ = '"';
   for ( ;*s; *t++ = *s++)
@@ -801,7 +830,7 @@ print_escape_string(char *s)
       case '"': *t++ = '\\'; continue;
     }
   *t++ = '"';
-  *t = '\0'; puts(t0); free(t0);
+  *t = '\0'; puts(t0); pari_free(t0);
 }
 
 static char *
@@ -837,11 +866,11 @@ completion_word(long end)
 
 /* completion required, cursor on s + pos. Complete wrt strict left prefix */
 void
-texmacs_completion(char *s, long pos)
+texmacs_completion(const char *s, long pos)
 {
   char **matches, *text;
 
-  if (rl_line_buffer) free(rl_line_buffer);
+  if (rl_line_buffer) pari_free(rl_line_buffer);
   rl_line_buffer = pari_strdup(s);
   text = completion_word(pos);
   /* text = start of expression we complete */
@@ -852,17 +881,17 @@ texmacs_completion(char *s, long pos)
   if (matches)
   {
     long i, prelen = (rl_line_buffer+pos) - text;
-    char *t = gpmalloc(prelen+1);
+    char *t = (char*)pari_malloc(prelen+1);
     strncpy(t, text, prelen); t[prelen] = 0; /* prefix */
     printf(" ");
-    print_escape_string(t); free(t);
+    print_escape_string(t); pari_free(t);
     for (i = matches[1]? 1: 0; matches[i]; i++)
     {
       printf(" ");
       print_escape_string(matches[i] + prelen);
-      free(matches[i]);
+      pari_free(matches[i]);
     }
-    free(matches);
+    pari_free(matches);
   }
   printf(")%c", DATA_END);
   fflush(stdout);
@@ -882,7 +911,7 @@ history_is_new(char *s)
 static void
 gp_add_history(char *s)
 {
-  if (history_is_new(s)) add_history(s);
+  if (history_is_new(s)) { add_history(s); append_history(1,current_histfile); }
 }
 
 /* Read line; returns a malloc()ed string of the user input or NULL on EOF.
@@ -890,19 +919,22 @@ gp_add_history(char *s)
 static char *
 gprl_input(char **endp, int first, input_method *IM, filtre_t *F)
 {
+  char buf[MAX_PROMPT_LEN + 24];
   Buffer *b = F->buf;
   ulong used = *endp - b->buf;
   ulong left = b->len - used, l;
-  char *s, *t, *prompt = expand_prompt(first? IM->prompt: IM->prompt_cont, F);
+  const char *prompt = first? IM->prompt
+                            : do_prompt(buf, IM->prompt_cont, F);
+  char *s, *t;
 
-  if (! (s = readline(color_prompt(prompt))) ) return NULL; /* EOF */
+  if (! (s = readline(prompt)) ) return NULL; /* EOF */
   gp_add_history(s); /* Makes a copy */
   l = strlen(s) + 1;
   /* put back \n that readline stripped. This is needed for
    * { print("a
    *   b"); }
    * and conforms with the other input methods anyway. */
-  t = gpmalloc(l + 1);
+  t = (char*)pari_malloc(l + 1);
   strncpy(t, s, l-1);
   t[l-1] = '\n';
   t[l]   = 0; /* equivalent to sprintf(t,"%s\n", s) */
@@ -918,9 +950,9 @@ gprl_input(char **endp, int first, input_method *IM, filtre_t *F)
 
 /* request one line interactively.
  * Return 0: EOF
- *        1: got one line from readline or infile */
+ *        1: got one line from readline or pari_infile */
 int
-get_line_from_readline(char *prompt, char *prompt_cont, filtre_t *F)
+get_line_from_readline(const char *prompt, const char *prompt_cont, filtre_t *F)
 {
   const int index = history_length;
   char *s;
@@ -930,7 +962,7 @@ get_line_from_readline(char *prompt, char *prompt_cont, filtre_t *F)
   IM.prompt_cont = prompt_cont;
   IM.getline = &gprl_input;
   IM.free = 1;
-  if (! input_loop(F,&IM)) { pariputs("\n"); return 0; }
+  if (! input_loop(F,&IM)) { pari_puts("\n"); return 0; }
 
   s = F->buf->buf;
   if (*s)
@@ -940,12 +972,11 @@ get_line_from_readline(char *prompt, char *prompt_cont, filtre_t *F)
       int i = history_length;
       while (i > index) {
         HIST_ENTRY *e = remove_history(--i);
-        free(e->line); free(e);
+        pari_free(e->line); pari_free(e);
       }
       gp_add_history(s);
     }
-
-    if (logfile) update_logfile(expand_prompt(IM.prompt, F), s);
+    echo_and_log(prompt, s);
   }
   return 1;
 }
