@@ -1,4 +1,4 @@
-/* $Id: plotQt.c 7523 2005-12-09 19:34:24Z kb $
+/* $Id: plotQt.c 12049 2009-12-22 23:16:30Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -26,7 +26,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /////////////////////////////////////////////////////////////////////////////
 extern "C" {
 #include "pari.h"
+#include "paripriv.h"
 #undef grem
+#undef swap
 #include "rect.h"
 }
 
@@ -59,9 +61,9 @@ protected:
 
 public:
     Plotter( long *w, long *x, long *y, long lw,
-	     QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
+             QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
     void save( const QString& s = *plotFile + ".xpm",//QString("pariplot.xpm"),
-	       const QString& f = QString( "XPM"));
+               const QString& f = QString( "XPM"));
 
 protected:
     void paintEvent( QPaintEvent *);
@@ -75,7 +77,8 @@ private:
     long *x;                           // x, y: array of x,y-coorinates of the
     long *y;                           //   top left corners of the rectwindows
     long lw;                           // lw: number of rectwindows
-    QColor color[MAX_COLORS];
+    long numcolors;
+    QColor *color;
     QFont font;
     static QString *plotFile;
     void draw(QPainter *p);
@@ -89,25 +92,26 @@ QString *Plotter::plotFile = new QString( "pariplot");
 
 
 Plotter::Plotter( long *w, long *x, long *y, long lw,
-		  QWidget* parent,  const char* name, WFlags fl)
+                  QWidget* parent,  const char* name, WFlags fl)
     : QWidget( parent, name, fl), font( "lucida", 9) {
 
+    long i;
+
     this->w=w; this->x=x; this->y=y; this->lw=lw;
 #ifndef __FANCY_WIN__
     this->resize( pari_plot.width, pari_plot.height);
     this->setCaption( "Pari QtPlot");
 #endif
-    this->setBackgroundColor( white);
     this->setFont( font);
-    // defaults as in plotX.c (cf. rgb.txt in a X11 system)
-    color[0]         = white;
-    color[BLACK]     = black;
-    color[BLUE]      = blue;
-    color[VIOLET]    = QColor( 208, 32, 144) ;
-    color[RED]       = red;
-    color[GREEN]     = green;
-    color[GREY]      = gray;
-    color[GAINSBORO] = QColor( 220, 220, 220);
+    numcolors = lg(pari_colormap)-1;
+    color = (QColor*)pari_malloc(numcolors*sizeof(QColor));
+    for (i = 1; i < lg(pari_colormap); i++)
+    {
+      int r, g, b;
+      color_to_rgb(gel(pari_colormap,i), &r, &g, &b);
+      color[i-1] = QColor(r, g, b);
+    }
+    this->setBackgroundColor( color[0]);
 }
 
 // void Plotter::setPlotFile( const char *s) {
@@ -119,12 +123,14 @@ Plotter::Plotter( long *w, long *x, long *y, long lw,
 struct data_qt
 {
   QPainter *p;
+  long numcolors;
   QColor *color;
 };
 
 static void SetForeground(void *data, long col)
 {
    struct data_qt *d = (struct data_qt *) data;
+   if (col >= d->numcolors) col = d->numcolors-1;
    d->p->setPen(d->color[col]);
 }
 
@@ -176,6 +182,7 @@ void Plotter::draw(QPainter *p){
   struct plot_eng plotQt;
   struct data_qt d;
   d.p= p;
+  d.numcolors = numcolors;
   d.color=color;
   plotQt.sc=&SetForeground;
   plotQt.pt=&DrawPoint;
@@ -196,7 +203,7 @@ void Plotter::save( const QString& s, const QString& f){
     QPainter p;
 
     p.begin( &pm, this);
-    p.fillRect( 0, 0, pm.width(), pm.height(), white);
+    p.fillRect( 0, 0, pm.width(), pm.height(), color[0]);
     this->draw(&p);
     p.end();
 
@@ -219,8 +226,8 @@ void Plotter::keyPressEvent( QKeyEvent *e) {
 
     switch ( tolower( e->ascii())) {
         case 's':
-	    save();
-	    this->setCaption( "Pari QtPlot: " + *plotFile);
+            save();
+            this->setCaption( "Pari QtPlot: " + *plotFile);
             break;
     }
 }
@@ -248,8 +255,8 @@ void Plotter::mouseReleaseEvent( QMouseEvent*) {
 #include <qtoolbar.h>
 #include <qaction.h>
 #include <qfiledialog.h>
-#include <qmessagebox.h> 
-#include <qfile.h> 
+#include <qmessagebox.h>
+#include <qfile.h>
 #include <qstatusbar.h>
 #include <qimage.h>
 #include <qstrlist.h>
@@ -261,8 +268,8 @@ void Plotter::mouseReleaseEvent( QMouseEvent*) {
 /* XPM */
 static const char * const fullscreen_xpm[] = {
 "14 14 2 1",
-" 	c None",
-".	c #000000",
+"         c None",
+".        c #000000",
 "..............",
 ".     ..     .",
 ".     ..     .",
@@ -276,7 +283,7 @@ static const char * const fullscreen_xpm[] = {
 ".    ....    .",
 ".     ..     .",
 ".     ..     .",
-".............."}; 
+".............."};
 
 
 class SaveAsDialog: public
@@ -291,8 +298,8 @@ QFileDialog
 
 public:
     SaveAsDialog( const QString & c = QString::null,
-		  const QString & s = QString::null, int w = 0, int h = 0,
-		  QWidget *parent = 0, const char *name = 0, WFlags f = 0);
+                  const QString & s = QString::null, int w = 0, int h = 0,
+                  QWidget *parent = 0, const char *name = 0, WFlags f = 0);
     ~SaveAsDialog();
 #ifdef __QPE__
     QString selectedFile() { return nameW->text();}
@@ -308,7 +315,7 @@ private:
 
 
 SaveAsDialog::SaveAsDialog( const QString & c, const QString & s, int w, int h,
-			    QWidget *parent, const char *name, WFlags f)
+                            QWidget *parent, const char *name, WFlags f)
 #ifdef __QPE__
     // simplistic dialog in case of QPE ( fancy alternative: class FileSelector)
 
@@ -381,7 +388,7 @@ class PlotWindow: public QMainWindow {
 
 public:
     PlotWindow( long *w, long *x, long *y, long lw,
-		QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
+                QWidget* parent = 0, const char* name = 0, WFlags fl = 0);
     ~PlotWindow();
 
 #ifndef __QPE__
@@ -410,7 +417,7 @@ const QStrList PlotWindow::file_formats = QImage::outputFormats();
 
 
 PlotWindow::PlotWindow( long *w, long *x, long *y, long lw,
-			QWidget* parent, const char* name, WFlags fl)
+                        QWidget* parent, const char* name, WFlags fl)
     : QMainWindow( parent, name, fl),
       saveFileName( "pariplot"), saveFileFormat( 0) {
 
@@ -429,10 +436,10 @@ PlotWindow::PlotWindow( long *w, long *x, long *y, long lw,
     // Setting up the File and View menu
     QPopupMenu *format = new QPopupMenu( this);
     for( uint i = 0; i < file_formats.count(); i++) {
-	format->insertItem( QString( QStrList(file_formats).at(i)) + " ...",
-			    this, SLOT( save( int)), 0, i);
-	if( 0 == QString( QStrList(file_formats).at(i)).compare( "PNG"))
-	    format->setItemEnabled( i, FALSE); // PNG seems to be broken
+        format->insertItem( QString( QStrList(file_formats).at(i)) + " ...",
+                            this, SLOT( save( int)), 0, i);
+        if( 0 == QString( QStrList(file_formats).at(i)).compare( "PNG"))
+            format->setItemEnabled( i, FALSE); // PNG seems to be broken
     }
     QPopupMenu *file = new QPopupMenu( this);
     CHECK_PTR( file );
@@ -448,8 +455,8 @@ PlotWindow::PlotWindow( long *w, long *x, long *y, long lw,
 #endif
     // Setting up the Fullscreen action
     QAction *a = new QAction( "use full screen",
-			      QPixmap( (const char ** )fullscreen_xpm),
-			      "&Fullscreen", CTRL+Key_F, this);
+                              QPixmap( (const char ** )fullscreen_xpm),
+                              "&Fullscreen", CTRL+Key_F, this);
     connect( a, SIGNAL( activated()), this, SLOT( fullScreen()));
 #ifdef __QPE__
     a->addTo( toolBar);
@@ -464,7 +471,7 @@ PlotWindow::PlotWindow( long *w, long *x, long *y, long lw,
 
 #ifndef __QPE__
     this->resize( pari_plot.width,
-		  pari_plot.height + 25);
+                  pari_plot.height + 25);
     res = new QLabel( statusBar());
     statusBar()->addWidget( res);
 #endif
@@ -477,11 +484,11 @@ PlotWindow::~PlotWindow() {
 
 #ifndef __QPE__
 void PlotWindow::resizeEvent( QResizeEvent *e) {
-    
+
     QMainWindow::resizeEvent( e);
     res->setText( QString( "Resolution: ") +
-		  QString::number( plr->width()) + "x" +
-		  QString::number( plr->height()));
+                  QString::number( plr->width()) + "x" +
+                  QString::number( plr->height()));
     res->setFixedSize( res->sizeHint());
 }
 #endif
@@ -490,10 +497,10 @@ void PlotWindow::resizeEvent( QResizeEvent *e) {
 void PlotWindow::fullScreen() {
 
     if ( plr->parentWidget()) {
-	plr->reparent( 0, WStyle_Tool | WStyle_Customize | WStyle_StaysOnTop,
-		      QPoint( 0, 0), FALSE);
-	plr->resize( qApp->desktop()->width(), qApp->desktop()->height());
-	plr->show();
+        plr->reparent( 0, WStyle_Tool | WStyle_Customize | WStyle_StaysOnTop,
+                      QPoint( 0, 0), FALSE);
+        plr->resize( qApp->desktop()->width(), qApp->desktop()->height());
+        plr->show();
     }
 }
 
@@ -501,9 +508,9 @@ void PlotWindow::fullScreen() {
 void PlotWindow::normalView() {
 
     if ( !plr->parentWidget()) {
-	plr->reparent( this, 0, QPoint(0,0), FALSE);
-	this->setCentralWidget( plr);
-	plr->show();
+        plr->reparent( this, 0, QPoint(0,0), FALSE);
+        this->setCentralWidget( plr);
+        plr->show();
     }
 }
 
@@ -529,30 +536,30 @@ void PlotWindow::save( int id) {
     QString s( ff + " (*." + ff.lower() +");;All (*)");
 #endif
     SaveAsDialog d( s, saveFileName + "." + ff.lower(),
-		    plr->width(), plr->height());
+                    plr->width(), plr->height());
     if( QDialog::Rejected == d.exec()) return;
     QString fn = d.selectedFile();
     if ( !fn.isEmpty()) {
-	if( QFile( fn).exists() &&
-	    QMessageBox::warning(
-		this, this->caption(),
-		QString( "A file named\n\"") + fn
-		+ QString( "\"\nalready exists\n"
-			   "Should this file be overwritten ?\n\n"),
-		"&Overwrite", "&Cancel"))  return;
-	saveFileName = fn;
-	int p;
-	if( (p = saveFileName.findRev( "." + ff, -1, FALSE)) >=0)
-	    saveFileName.truncate( p);
-	saveFileFormat = id;
-	int old_w = plr->width(), old_h = plr->height();
-	int w = d.picWidth(), h = d.picHeight();
-	if( w != old_w ||  h != old_h) {
-	    plr->resize( w, h);
-	    save();
-	    plr->resize( old_w, old_h);
-	} else
-	    save();
+        if( QFile( fn).exists() &&
+            QMessageBox::warning(
+                this, this->caption(),
+                QString( "A file named\n\"") + fn
+                + QString( "\"\nalready exists\n"
+                           "Should this file be overwritten ?\n\n"),
+                "&Overwrite", "&Cancel"))  return;
+        saveFileName = fn;
+        int p;
+        if( (p = saveFileName.findRev( "." + ff, -1, FALSE)) >=0)
+            saveFileName.truncate( p);
+        saveFileFormat = id;
+        int old_w = plr->width(), old_h = plr->height();
+        int w = d.picWidth(), h = d.picHeight();
+        if( w != old_w ||  h != old_h) {
+            plr->resize( w, h);
+            save();
+            plr->resize( old_w, old_h);
+        } else
+            save();
     }
 }
 
@@ -571,7 +578,7 @@ void PlotWindow::save( int id) {
 void
 rectdraw0(long *w, long *x, long *y, long lw)
 {
-    if (fork()) return;  // parent process returns
+    if (pari_daemon()) return;  // parent process returns
 
     pari_close();
     PARI_get_plot(1);
@@ -584,7 +591,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
 #else
     QApplication
 #endif
-	a( argc, argv);
+        a( argc, argv);
 #ifdef __FANCY_WIN__
     PlotWindow *win = new PlotWindow(w, x, y, lw);
 #else
@@ -613,7 +620,7 @@ PARI_get_plot(long f)
     pari_plot.width   = 400;         // width and
     pari_plot.height  = 300;         //  height of plot window
 #endif
-    pari_plot.hunit   = 3;           // 
+    pari_plot.hunit   = 3;           //
     pari_plot.vunit   = 3;           //
     pari_plot.fwidth  = 6;           // font width
     pari_plot.fheight = 9;           //   and height
