@@ -1,4 +1,4 @@
-/* $Id: plotX.c 7523 2005-12-09 19:34:24Z kb $
+/* $Id: plotX.c 13145 2011-04-20 21:56:40Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -20,8 +20,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*******************************************************************/
 
 #include "pari.h"
+#include "paripriv.h"
 #include "rect.h"
-#include "../language/anal.h"
 
 #ifdef HPPA
 #  ifndef __GNUC__
@@ -33,22 +33,30 @@ BEGINEXTERN
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
+#ifndef XK_c
+#  include <X11/keysym.h>
+#endif
 ENDEXTERN
 
 static Colormap PARI_Colormap;
 static XColor  *PARI_Colors;
-static XColor  *PARI_ExactColors;
 
 struct data_x
 {
   Display *display;
   Window win;
+  int numcolors;
   GC gc;
 };
 
 static void SetForeground(void *data, long col)
 {
   struct data_x *dx = (struct data_x *) data;
+  if (col >= dx->numcolors)
+  {
+    pari_warn(warner,"non-existent color: %ld", col);
+    col = dx->numcolors-1;
+  }
   XSetForeground(dx->display,dx->gc, PARI_Colors[col].pixel);
 }
 
@@ -73,7 +81,7 @@ static void DrawRectangle(void *data, long x, long y, long w, long h)
 static void DrawPoints(void *data, long nb, struct plot_points *p)
 {
   struct data_x *dx = (struct data_x *) data;
-  XPoint *xp=(XPoint*)gpmalloc(sizeof(xp)*nb);
+  XPoint *xp=(XPoint*)pari_malloc(sizeof(xp)*nb);
   long i;
   for (i=0;i<nb;i++)
   {
@@ -81,13 +89,13 @@ static void DrawPoints(void *data, long nb, struct plot_points *p)
     xp[i].y=p[i].y;
   }
   XDrawPoints(dx->display,dx->win,dx->gc, xp, nb, 0);
-  free(xp);
+  pari_free(xp);
 }
 
 static void DrawLines(void *data, long nb, struct plot_points *p)
 {
   struct data_x *dx = (struct data_x *) data;
-  XPoint *xp=(XPoint*)gpmalloc(sizeof(xp)*nb);
+  XPoint *xp=(XPoint*)pari_malloc(sizeof(xp)*nb);
   long i;
   for (i=0;i<nb;i++)
   {
@@ -95,7 +103,7 @@ static void DrawLines(void *data, long nb, struct plot_points *p)
     xp[i].y=p[i].y;
   }
   XDrawLines(dx->display,dx->win,dx->gc, xp, nb, 0);
-  free(xp);
+  pari_free(xp);
 }
 
 static void DrawString(void *data, long x, long y, char *text, long numtext)
@@ -104,41 +112,36 @@ static void DrawString(void *data, long x, long y, char *text, long numtext)
   XDrawString(dx->display,dx->win,dx->gc, x,y, text, numtext);
 }
 
-static char *PARI_DefaultColors[MAX_COLORS] =
-{
-  " ",
-  "black",    /* Default */
-  "blue",     /* Axes */
-  "violetred",   /* Odd numbered curves in ploth */
-  "red",      /* Curves, or Even numbered curves in ploth */
-  "green",
-  "grey",
-  "gainsboro",
-};
-
 static void
-PARI_ColorSetUp(Display *display, char **Colors, int n)
+PARI_ColorSetUp(Display *display, GEN colors)
 {
   static int init_done = 0;
-  int i;
+  long i, n = lg(colors)-1;
 
   if (init_done) return;
   init_done=1;
 
   PARI_Colormap = DefaultColormap(display, 0);
-  PARI_Colors = (XColor *) gpmalloc((n+1) * sizeof(XColor));
-  PARI_ExactColors = (XColor *) gpmalloc((n+1) * sizeof(XColor));
-  for (i=1; i<n; i++)
-    XAllocNamedColor(display, PARI_Colormap, Colors[i],
-		     &PARI_ExactColors[i], &PARI_Colors[i]);
+  PARI_Colors = (XColor *) pari_malloc((n+1) * sizeof(XColor));
+  for (i=0; i<n; i++)
+  {
+    int r, g, b;
+    color_to_rgb(gel(pari_colormap,i+1), &r, &g, &b);
+    PARI_Colors[i].red   = r*65535/255;
+    PARI_Colors[i].green = g*65535/255;
+    PARI_Colors[i].blue  = b*65535/255;
+    PARI_Colors[i].flags = DoRed | DoGreen | DoBlue;
+    if (!XAllocColor(display, PARI_Colormap, &PARI_Colors[i]))
+      pari_err(talker, "cannot allocate color %ld", i);
+  }
 }
 
 /* after fork(), we don't want the child to recover but to exit */
 static void
-exiterr(char *str)
+exiterr(const char *str)
 {
   term_color(c_ERR);
-  fprintferr("\n  *** X fatal error: %s\n",str);
+  err_printf("\n  *** X fatal error: %s\n",str);
   term_color(c_NONE); exit(1);
 }
 
@@ -165,7 +168,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
   struct plot_eng plotX;
   struct data_x dx;
   double xs = 1, ys = 1;
-  int screen;
+  int screen, keystate;
   Display *display;
   GC gc;
   Window win;
@@ -175,7 +178,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
   XSetWindowAttributes attrib;
   Atom wm_delete_window, wm_protocols;
 
-  if (fork()) return;  /* parent process returns */
+  if (pari_daemon()) return;  /* parent process returns */
 
   pari_close();
   PARI_get_plot(1);
@@ -185,13 +188,13 @@ rectdraw0(long *w, long *x, long *y, long lw)
   if (!font_info) exiterr("cannot open 9x15 font");
   XSetErrorHandler(Xerror);
   XSetIOErrorHandler(IOerror);
-  PARI_ColorSetUp(display,PARI_DefaultColors,MAX_COLORS);
+  PARI_ColorSetUp(display,pari_colormap);
 
   screen = DefaultScreen(display);
   win = XCreateSimpleWindow
     (display, RootWindow(display, screen), 0, 0,
      pari_plot.width, pari_plot.height, 4,
-     BlackPixel(display, screen), WhitePixel(display, screen));
+     PARI_Colors[1].pixel, PARI_Colors[0].pixel);
 
   size_hints.flags = PPosition | PSize;
   size_hints.x = 0;
@@ -206,7 +209,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
   XSetWMProtocols(display,win,&wm_delete_window, 1);
 
   XSelectInput (display, win,
-    ExposureMask | ButtonPressMask | StructureNotifyMask);
+    ExposureMask | ButtonPressMask | KeyReleaseMask | StructureNotifyMask);
 
   /* enable backing-store */
   attrib.backing_store = Always;
@@ -222,6 +225,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
   oldheight = pari_plot.height;
   dx.display= display;
   dx.win = win;
+  dx.numcolors = lg(pari_colormap)-1;
   dx.gc = gc;
   plotX.sc = &SetForeground;
   plotX.pt = &DrawPoint;
@@ -242,9 +246,26 @@ rectdraw0(long *w, long *x, long *y, long lw)
             (Atom)event.xclient.data.l[0] != wm_delete_window) break;
       case ButtonPress:
       case DestroyNotify:
-	XUnloadFont(display,font_info->fid); XFreeGC(display,gc);
-	free_graph();
-	XCloseDisplay(display); exit(0);
+EXIT:
+        XUnloadFont(display,font_info->fid);
+        XFreeGC(display,gc);
+        XCloseDisplay(display); exit(0);
+
+      case KeyRelease:
+        /* Mod4 == Super on "std" Linux */
+        keystate = event.xkey.state & (ShiftMask|ControlMask|Mod1Mask|Mod4Mask);
+        switch (XKeycodeToKeysym(display, event.xkey.keycode, 0))
+        {
+        case XK_q:
+          if (!keystate || keystate == ControlMask)
+            goto EXIT;
+          break;
+        case XK_c:
+          if (keystate == ControlMask)
+            goto EXIT;
+          break;
+        }
+        break;
 
       case ConfigureNotify:
       {
@@ -253,13 +274,13 @@ rectdraw0(long *w, long *x, long *y, long lw)
 
         if (width == oldwidth && height == oldheight) break;
         oldwidth  = width;
-        oldheight = height; 
+        oldheight = height;
 
         /* recompute scale */
-	xs = ((double)width)/pari_plot.width;
+        xs = ((double)width)/pari_plot.width;
         ys = ((double)height)/pari_plot.height;
       }
-      case Expose: 
+      case Expose:
         gen_rectdraw0(&plotX, (void *)&dx, w, x, y,lw,xs,ys);
     }
   }
