@@ -1,4 +1,4 @@
-/* $Id: rect.h 7473 2005-11-26 16:07:52Z bill $
+/* $Id: rect.h 11744 2009-06-03 12:39:15Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -16,10 +16,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 BEGINEXTERN
 
 #define PLOT_NAME_LEN 20
-#define NUMRECT 18
-
-#define DTOL(t) ((long)(t + 0.5))
-
 typedef struct PARI_plot {
   long width;
   long height;
@@ -34,13 +30,6 @@ typedef struct PARI_plot {
 extern PARI_plot pari_plot, pari_psplot;
 extern PARI_plot *pari_plot_engine;
 
-#define w_height (pari_plot_engine->height)
-#define w_width (pari_plot_engine->width)
-#define f_height (pari_plot_engine->fheight)
-#define f_width (pari_plot_engine->fwidth)
-#define h_unit (pari_plot_engine->hunit)
-#define v_unit (pari_plot_engine->vunit)
-
 typedef struct dblPointList{
   double *d;                   /* data */
   long nb;                     /* number of elements */
@@ -58,8 +47,8 @@ typedef struct PariRect {
   double cursorx,cursory;
   double xscale,yscale;
   double xshift,yshift;
-  long has_graph;	   /* xy-ranges of this rectangle should be used
-			      for interactive operations.  */
+  long has_graph; /* xy-ranges of this rectangle should be used
+                     for interactive operations.  */
 } PariRect;
 
 /* The structures below are "subclasses" of RectObj. */
@@ -118,31 +107,19 @@ struct plot_eng {
   void (*st)(void *data, long x, long y, char *s, long l);
 };
 
+#define ROt_MV 0                        /* Move */
+#define ROt_PT 1                        /* Point */
+#define ROt_LN 2                        /* Line */
+#define ROt_BX 3                        /* Box */
+#define ROt_MP 4                        /* Multiple point */
+#define ROt_ML 5                        /* Multiple lines */
+#define ROt_ST 6                        /* String */
+#define ROt_PTT 7                        /* Point type change */
+#define ROt_LNT 8                        /* Line type change */
+#define ROt_PTS 9                        /* Point size change */
+#define ROt_NULL 10                /* To be the start of the chain */
 
-#define BLACK      1 /* Default */
-#define BLUE       2 /* Axes */
-#define VIOLET     3 /* Odd numbered curves in ploth */
-#define RED        4 /* Curves, or Even numbered curves in ploth */
-#define GREEN      5
-#define GREY       6
-#define GAINSBORO  7
-
-#define MAX_COLORS 8
-#define DEFAULT_COLOR BLACK
-
-#define ROt_MV 0			/* Move */
-#define ROt_PT 1			/* Point */
-#define ROt_LN 2			/* Line */
-#define ROt_BX 3			/* Box */
-#define ROt_MP 4			/* Multiple point */
-#define ROt_ML 5			/* Multiple lines */
-#define ROt_ST 6			/* String */
-#define ROt_PTT 7			/* Point type change */
-#define ROt_LNT 8			/* Line type change */
-#define ROt_PTS 9			/* Point size change */
-#define ROt_NULL 10		/* To be the start of the chain */
-
-#define ROt_MAX 10		/* Maximal type */
+#define ROt_MAX 10                /* Maximal type */
 
 /* Pointer conversion. */
 
@@ -202,18 +179,18 @@ struct plot_eng {
 #define RoSTy(rop) (RoST(rop)->y)
 #define RoSTdir(rop) (RoST(rop)->dir)
 
-#define RoSTdirLEFT	  0x00
-#define RoSTdirCENTER	  0x01
-#define RoSTdirRIGHT	  0x02
+#define RoSTdirLEFT          0x00
+#define RoSTdirCENTER          0x01
+#define RoSTdirRIGHT          0x02
 #define RoSTdirHPOS_mask  0x03
 
-#define RoSTdirBOTTOM	  0x00
-#define RoSTdirVCENTER	  0x04
-#define RoSTdirTOP	  0x08
+#define RoSTdirBOTTOM          0x00
+#define RoSTdirVCENTER          0x04
+#define RoSTdirTOP          0x08
 #define RoSTdirVPOS_mask  0x0c
 
-#define RoSTdirHGAP	  0x10
-#define RoSTdirVGAP	  0x20
+#define RoSTdirHGAP          0x10
+#define RoSTdirVGAP          0x20
 
 
 #define RoPTTpen(rop) (RoPTT(rop)->pen)
@@ -221,8 +198,6 @@ struct plot_eng {
 #define RoPTSsize(rop) (RoPTS(rop)->size)
 
 #define PL_POINTS 1
-#define GOODRECT(r) (0 <= r && r < NUMRECT)
-#define GOODCOLOR(c) (1 <= c && c < MAX_COLORS)
 
 #define PLOT_PARAMETRIC   0x00001
 #define PLOT_RECURSIVE    0x00002
@@ -236,6 +211,7 @@ struct plot_eng {
 #define PLOT_NO_TICK_X    0x00200
 #define PLOT_NO_TICK_Y    0x00400
 #define PLOT_NODOUBLETICK 0x00800
+#define PLOT_COMPLEX      0x01000
 
 #define PLOT_POSTSCRIPT   0x80000
 
@@ -245,38 +221,39 @@ struct plot_eng {
 #define RECT_CP_SE        0x4
 #define RECT_CP_NE        0x6
 
-#define TICKS_CLOCKW	1		/* Draw in clockwise direction */
-#define TICKS_ACLOCKW	2		/* Draw in anticlockwise direction */
-#define TICKS_ENDSTOO	4		/* Draw at endspoints if needed */
-#define TICKS_NODOUBLE	8		/* Do not draw double-length ticks */
+#define TICKS_CLOCKW        1        /* Draw in clockwise direction */
+#define TICKS_ACLOCKW        2        /* Draw in anticlockwise direction */
+#define TICKS_ENDSTOO        4        /* Draw at endspoints if needed */
+#define TICKS_NODOUBLE        8        /* Do not draw double-length ticks */
 
 /* Not implemented yet */
-#define TICKS_COORD	16		/* Output [x,y,l,isdbl] for each tick */
-#define TICKS_RELATIVE	32		/* x,y-coordinates are relative */
+#define TICKS_COORD        16        /* Output [x,y,l,isdbl] for each tick */
+#define TICKS_RELATIVE        32        /* x,y-coordinates are relative */
 
-extern PariRect  **rectgraph;
 extern long  rectpoint_itype;
 extern long  rectline_itype;
 
 /* plotport.c */
-typedef long col_counter[MAX_COLORS][ROt_MAX];
+typedef long (*col_counter)[ROt_MAX];
 
+void  color_to_rgb(GEN c, int *r, int *g, int *b);
 void  initrect(long ne, long x, long y);
 void  initrect_gen(long ne, GEN x, GEN y, long flag);
 void  killrect(long ne);
 void  plot_count(long *w, long lw, col_counter rcolcnt);
-void  plot(entree *ep, GEN a, GEN b, char *ch, GEN ysmlu, GEN ybigu, long prec);
-GEN   ploth(entree *ep, GEN a, GEN b, char *ch, long prec, long flag, long numpoints);
-GEN   ploth2(entree *ep, GEN a, GEN b, char *ch, long prec);
-GEN   plothmult(entree *ep, GEN a, GEN b, char *ch, long prec);
+void  plot(GEN a, GEN b, GEN code, GEN ysmlu, GEN ybigu, long prec);
+GEN   ploth(GEN a, GEN b, GEN code, long prec, long flag, long numpoints);
+GEN   ploth2(GEN a, GEN b, GEN code, long prec);
+GEN   plothmult(GEN a, GEN b, GEN code, long prec);
 GEN   plothraw(GEN listx, GEN listy, long flag);
 GEN   plothsizes(void);
 GEN   plothsizes_flag(long flag);
 void  postdraw(GEN list);
 void  postdraw_flag(GEN list, long flag);
-GEN   postploth(entree *ep,GEN a,GEN b,char *ch,long prec,long flag,long numpoints);
-GEN   postploth2(entree *ep,GEN a,GEN b,char *ch,long prec,long numpoints);
+GEN   postploth(GEN a,GEN b,GEN code,long prec,long flag,long numpoints);
+GEN   postploth2(GEN a,GEN b,GEN code,long prec,long numpoints);
 GEN   postplothraw(GEN listx, GEN listy, long flag);
+void  Printx(dblPointList *f);
 void  rectbox(long ne, GEN gx2, GEN gy2);
 void  rectcolor(long ne, long color);
 void  rectcopy(long source, long dest, long xoff, long yoff);
@@ -288,7 +265,7 @@ void  rectline(long ne, GEN gx2, GEN gy2);
 void  rectlines(long ne, GEN listx, GEN listy, long flag);
 void  rectlinetype(long ne, long t);
 void  rectmove(long ne, GEN x, GEN y);
-GEN   rectploth(long drawrect,entree *ep, GEN a, GEN b, char *ch, long prec, ulong flags, long testpoints);
+GEN   rectploth(long drawrect,GEN a, GEN b, GEN code, long prec, ulong flags, long testpoints);
 GEN   rectplothraw(long drawrect, GEN data, long flags);
 void  rectpoint(long ne, GEN x, GEN y);
 void  rectpoints(long ne, GEN listx, GEN listy);
@@ -303,9 +280,6 @@ void  rectstring(long ne, char *x);
 void  rectstring3(long ne, char *x, long dir);
 void  rectclip(long rect);
 
-void  init_graph(void);
-void  free_graph(void);
-
 void gen_rectdraw0(struct plot_eng *eng, void *data, long *w, long *x, long *y, long lw, double xs, double ys);
 
 /* architecture-dependent plot file (plotX.c ...) */
