@@ -1,4 +1,4 @@
-/* $Id: parierr.h,v 1.31 2004/06/23 17:02:01 kb Exp $
+/* $Id: parierr.h 12748 2010-11-20 18:58:33Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -14,61 +14,37 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 enum {
-  cant_deflate = 1,	/* Force errors into non-0 */
+/* Force errors into non-0 */
+  syntaxer = 1, bugparier, /* untrappable errors */
 
-/* Always catched up to this point */
+  alarmer, openfiler,
 
-  caracer1,
-  paramer1, varer1, obsoler, openfiler, talker2,
+  talker, flagerr, impl, archer, notfuncer,
 
-/* NO CONTEXT now */
+  precer, typeer, consister, user,
 
-  talker, flagerr, warner, warnprec, warnfile,
-  accurer, bugparier, impl, archer, warnmem, precer,
-
-  siginter, typeer, consister, user,
-
-/* mp.s ou mp.c */
-
-  affer2, affer3,
-  
-  errpile, errlg, errexpo, errvalp, rtodber,
+  errpile, overflower,
 
 /* alglin.c */
-
   matinv1, mattype1,
 
-/* anal.c   */
-
-  valencer1,
-
 /* arith.c  */
-
-  arither1, primer1, primer2, invmoder,
+  arither1, primer1, invmoder,
 
 /* base.c   */
   constpoler, notpoler, redpoler, zeropoler,
 
-/* bibli.c  */
-
-  intger2, lllger3,
-
-/* elliptic.c */
-
-  elliper1,
-
 /* gen.c */
-  operi, operf, gdiver, overwriter,
+  operi, operf, gdiver,
 
 /* init.c */
-
   memer,
 
 /* trans.c */
+  negexper, sqrter5,
 
-  infprecer, negexper, sqrter5,
-
-/* PAS D'ERREUR */
-
+/* NO ERROR */
   noer
 };
+
+enum { warner, warnprec, warnfile, warnmem };
