@@ -1,4 +1,4 @@
-/* $Id: parigen.h,v 1.27.2.2 2007-10-29 10:16:34 kb Exp $
+/* $Id: parigen.h 11744 2009-06-03 12:39:15Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -19,25 +19,30 @@ typedef long *GEN;
 typedef unsigned long pari_ulong;
 #define ulong pari_ulong
 
-#ifdef LONG_IS_64BIT
-#  define TWOPOTBYTES_IN_LONG  3
-#else
-#  define TWOPOTBYTES_IN_LONG  2
-#endif
+#undef ULONG_MAX
+#undef LONG_MAX
 
-#define DEFAULTPREC    2 + (8>>TWOPOTBYTES_IN_LONG)
-#define MEDDEFAULTPREC 2 + (16>>TWOPOTBYTES_IN_LONG)
-#define BIGDEFAULTPREC 2 + (24>>TWOPOTBYTES_IN_LONG)
-#define TWOPOTBITS_IN_LONG (TWOPOTBYTES_IN_LONG+3)
-#define BYTES_IN_LONG (1L<<TWOPOTBYTES_IN_LONG)
-#define BITS_IN_LONG  (1L<<TWOPOTBITS_IN_LONG)
+#ifdef LONG_IS_64BIT
+#  define BITS_IN_LONG 64
+#  define TWOPOTBITS_IN_LONG 6
+#  define LONG_MAX (9223372036854775807L) /* 2^63-1 */
+#  define SMALL_ULONG(p) ((ulong)p <= 3037000493UL)
+#else
+#  define BITS_IN_LONG 32
+#  define TWOPOTBITS_IN_LONG 5
+#  define LONG_MAX (2147483647L) /* 2^31-1 */
+#  define SMALL_ULONG(p) ((ulong)p <= 46337UL) /* 2p^2 < 2^BITS_IN_LONG */
+#endif
+#define ULONG_MAX (~0x0UL)
+
+#define DEFAULTPREC    (2 + (long)(8/sizeof(long)))
+#define MEDDEFAULTPREC (2 + (long)(16/sizeof(long)))
+#define BIGDEFAULTPREC (2 + (long)(24/sizeof(long)))
 #define HIGHBIT (1UL << (BITS_IN_LONG-1))
 #define BITS_IN_HALFULONG (BITS_IN_LONG>>1)
-#define MAXULONG (~0x0UL)
-#define MAXHALFULONG ((1UL<<BITS_IN_HALFULONG) - 1)
-#define LOWMASK  (MAXHALFULONG)
+
+#define LOWMASK ((1UL<<BITS_IN_HALFULONG) - 1)
 #define HIGHMASK (~LOWMASK)
-#define SMALL_MASK (HIGHBIT>>1)
 
 #define HIGHWORD(a) ((a) >> BITS_IN_HALFULONG)
 #define LOWWORD(a) ((a) & LOWMASK)
@@ -75,6 +80,8 @@ typedef unsigned long pari_ulong;
 #define VALPBITS    ((1UL<<VALPnumBITS)-1)
 #define VARNBITS    (MAXVARN<<VARNSHIFT)
 #define MAXVARN     ((1UL<<VARNnumBITS)-1)
+#define NO_VARIABLE (2147483647) /* > MAXVARN */
+
 #define HIGHEXPOBIT (1UL<<(EXPOnumBITS-1))
 #define HIGHVALPBIT (1UL<<(VALPnumBITS-1))
 #define CLONEBIT    (1UL<<LGnumBITS)
@@ -104,10 +111,6 @@ typedef unsigned long pari_ulong;
 #define signe(x)      ((((long*)(x))[1]) >> SIGNSHIFT)
 #define setsigne(x,s) (((ulong*)(x))[1]=\
                         (((ulong*)(x))[1]&(~SIGNBITS)) | (ulong)evalsigne(s))
-#define togglesign(x) (void)((((GEN)(x))[1] & SIGNBITS) && (((GEN)(x))[1] ^= HIGHBIT))
-
-#define lgeflist(x)      (((long*)(x))[1])
-#define setlgeflist(x,l) (((ulong*)(x))[1]=(ulong)(l))
 
 #define lgefint(x)      ((long)(((ulong*)(x))[1] & LGBITS))
 #define setlgefint(x,s) (((ulong*)(x))[1]=\
@@ -115,19 +118,48 @@ typedef unsigned long pari_ulong;
 
 #define expo(x)       ((long) ((((ulong*)(x))[1] & EXPOBITS) - HIGHEXPOBIT))
 #define setexpo(x,s)  (((ulong*)(x))[1]=\
-		       (((ulong*)(x))[1]&(~EXPOBITS)) | (ulong)evalexpo(s))
+                       (((ulong*)(x))[1]&(~EXPOBITS)) | (ulong)evalexpo(s))
 
 #define valp(x)       ((long) ((((ulong*)(x))[1] & VALPBITS) - HIGHVALPBIT))
 #define setvalp(x,s)  (((ulong*)(x))[1]=\
-		       (((ulong*)(x))[1]&(~VALPBITS)) | (ulong)evalvalp(s))
+                       (((ulong*)(x))[1]&(~VALPBITS)) | (ulong)evalvalp(s))
 
 #define precp(x)      ((long) (((ulong*)(x))[1] >> PRECPSHIFT))
 #define setprecp(x,s) (((ulong*)(x))[1]=\
-		       (((ulong*)(x))[1]&(~PRECPBITS)) | (ulong)evalprecp(s))
+                       (((ulong*)(x))[1]&(~PRECPBITS)) | (ulong)evalprecp(s))
 
 #define varn(x)       ((long)((((ulong*)(x))[1]&VARNBITS) >> VARNSHIFT))
 #define setvarn(x,s)  (((ulong*)(x))[1]=\
-		       (((ulong*)(x))[1]&(~VARNBITS)) | (ulong)evalvarn(s))
+                       (((ulong*)(x))[1]&(~VARNBITS)) | (ulong)evalvarn(s))
 
 #define varncmp(x,y)  ((x)-(y))
 
+/* t_LIST */
+#define list_nmax(x) ((GEN)x)[1]
+#define list_data(x) ((GEN*)x)[2]
+
+/* DO NOT REORDER THESE
+ * actual values can be changed. Adapt lontyp in gen2.c */
+enum {
+  t_INT    =  1,
+  t_REAL   =  2,
+  t_INTMOD =  3,
+  t_FRAC   =  4,
+  t_FFELT  =  5,
+  t_COMPLEX=  6,
+  t_PADIC  =  7,
+  t_QUAD   =  8,
+  t_POLMOD =  9,
+  t_POL    =  10,
+  t_SER    =  11,
+  t_RFRAC  =  13,
+  t_QFR    =  15,
+  t_QFI    =  16,
+  t_VEC    =  17,
+  t_COL    =  18,
+  t_MAT    =  19,
+  t_LIST   =  20,
+  t_STR    =  21,
+  t_VECSMALL= 22,
+  t_CLOSURE = 23
+};
