@@ -1,4 +1,4 @@
-/* $Id: parisys.h,v 1.14 2006/04/14 15:56:31 kb Exp $
+/* $Id: parisys.h 12331 2010-05-26 18:49:56Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -63,6 +63,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #ifndef INLINE
 #  define INLINE static
 #endif
+#ifdef ENABLE_TLS
+#  define THREAD __thread
+#else
+#  define THREAD
+#endif
 
 #if defined(_WIN32) || defined(__CYGWIN32__)
 /* ANSI C does not allow to longjmp() out of a signal handler, in particular,
@@ -71,6 +76,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
  * Instead, we check whether win32ctrlc is set in new_chunk(). */
 BEGINEXTERN
   extern int win32ctrlc;
-  void dowin32ctrlc();
+  void dowin32ctrlc(void);
 ENDEXTERN
+#define CHECK_CTRLC if (win32ctrlc) dowin32ctrlc();
+#else
+#define CHECK_CTRLC
 #endif
