@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/gmp/int.h"
-/* $Id: int.h 5645 2004-06-21 13:40:10Z bill $
+/* $Id: int.h 11189 2008-10-24 09:00:13Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -17,7 +17,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #define int_MSW(x) ((x)+lgefint((x))-1)
 /*x being a t_INT, return a pointer to the most significant word of x.*/
 
-#define int_LSW(x) (x+2)
+#define int_LSW(x) ((x)+2)
 /*x being a t_INT, return a pointer to the least significant word of x.*/
 
 #define int_precW(x) ((x)-1)
@@ -31,5 +31,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #define int_W(x,l) ((x)+2+(l))
 /*x being a t_INT, return a pointer to the l-th least significant word of x.*/
 
+#define int_W_lg(x,l,lx) ((x)+2+(l))
+/*x being a t_INT, return a pointer to the l-th least significant word of x,
+ * assuming lgefint(x) = lx.*/
+
 #define PARI_KERNEL_GMP
 /*This macro should not be used in libpari itself.*/
