@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/gmp/mp.c"
-/* $Id: mp.c 10295 2008-06-10 16:03:56Z kb $
+/* $Id: mp.c 13082 2011-04-12 08:15:19Z kb $
 
 Copyright (C) 2002-2003  The PARI group.
 
@@ -15,8 +15,8 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 /***********************************************************************/
-/**								      **/
-/**		                GMP KERNEL                     	      **/
+/**                                                                   **/
+/**                               GMP KERNEL                          **/
 /** BA2002Sep24                                                       **/
 /***********************************************************************/
 /* GMP t_INT as just like normal t_INT, just the mantissa is the other way
@@ -31,9 +31,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
  *   only you know it may be quite different on beyond.  Oh, Kitty!
  *   how nice it would be if we could only get through into Looking-
  *   glass House!  I'm sure it's got, oh! such beautiful things in it!
- *                                                                             
+ *
  *  Through the Looking Glass,  Lewis Carrol
- *  
+ *
  *  (pityful attempt to beat GN code/comments rate)
  *  */
 
@@ -45,20 +45,24 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*We need PARI invmod renamed to invmod_pari*/
 #define INVMOD_PARI
 
-static void *wrap_gprealloc(void *ptr, size_t old_size, size_t new_size) {
-  (void)old_size; return (void *) gprealloc(ptr,new_size);
+static void *gmp_realloc(void *ptr, size_t old_size, size_t new_size) {
+  (void)old_size; return (void *) pari_realloc(ptr,new_size);
+}
+
+static void gmp_free(void *ptr, size_t old_size){
+  (void)old_size; pari_free(ptr);
 }
 
 int pari_kernel_init(void)
 {
-  /* Use gpmalloc instead of malloc */
-  mp_set_memory_functions((void *(*)(size_t)) gpmalloc, wrap_gprealloc, NULL);
+  /* Use pari_malloc instead of malloc */
+  mp_set_memory_functions((void *(*)(size_t)) pari_malloc, gmp_realloc, gmp_free);
   return 0;
 }
 
 #define LIMBS(x)  ((mp_limb_t *)((x)+2))
 #define NLIMBS(x) (lgefint(x)-2)
-/*This one is for t_REAL to emphasize there are not t_INT*/
+/*This one is for t_REALs to emphasize they are not t_INTs*/
 #define RLIMBS(x)  ((mp_limb_t *)((x)+2))
 #define RNLIMBS(x) (lg(x)-2)
 
@@ -112,16 +116,17 @@ icopy_ef(GEN x, long l)
  */
 
 /***********************************************************************/
-/**								      **/
-/**		         ADDITION / SUBTRACTION          	      **/
+/**                                                                   **/
+/**                     ADDITION / SUBTRACTION                        **/
 /**                                                                   **/
 /***********************************************************************/
 
 GEN
 setloop(GEN a)
 {
-  GEN z0 = (GEN)avma; (void)cgetg(lgefint(a) + 3, t_VECSMALL);
-  return icopy_av(a, z0 - 2); /* two cells of extra space after a */
+  pari_sp av = avma - 2 * sizeof(long);
+  (void)cgetg(lgefint(a) + 3, t_VECSMALL);
+  return icopy_avma(a, av); /* two cells of extra space after a */
 }
 
 /* we had a = setloop(?), then some incloops. Reset a to b */
@@ -182,11 +187,12 @@ incloop(GEN a)
 }
 
 INLINE GEN
-addsispec(long s, GEN x, long nx)
+adduispec(ulong s, GEN x, long nx)
 {
   GEN  zd;
   long lz;
 
+  if (nx == 1) return adduu((ulong)x[0], s);
   lz = nx+3; zd = cgeti(lz);
   if (mpn_add_1(LIMBS(zd),(mp_limb_t *)x,nx,s))
     zd[lz-1]=1;
@@ -196,6 +202,15 @@ addsispec(long s, GEN x, long nx)
   return zd;
 }
 
+GEN
+adduispec_offset(ulong s, GEN x, long offset, long nx)
+{
+  GEN xd=x+2+offset;
+  while (nx && *(xd+nx-1)==0) nx--;
+  if (!nx) return utoi(s);
+  return adduispec(s,xd,nx);
+}
+
 INLINE GEN
 addiispec(GEN x, GEN y, long nx, long ny)
 {
@@ -203,7 +218,7 @@ addiispec(GEN x, GEN y, long nx, long ny)
   long lz;
 
   if (nx < ny) swapspec(x,y, nx,ny);
-  if (ny == 1) return addsispec(*y,x,nx);
+  if (ny == 1) return adduispec(*y,x,nx);
   lz = nx+3; zd = cgeti(lz);
 
   if (mpn_add(LIMBS(zd),(mp_limb_t *)x,nx,(mp_limb_t *)y,ny))
@@ -217,12 +232,14 @@ addiispec(GEN x, GEN y, long nx, long ny)
 
 /* assume x >= y */
 INLINE GEN
-subisspec(GEN x, long s, long nx)
+subiuspec(GEN x, ulong s, long nx)
 {
   GEN zd;
   long lz;
+
+  if (nx == 1) return utoi(x[0] - s);
+
   lz = nx + 2; zd = cgeti(lz);
-  
   mpn_sub_1 (LIMBS(zd), (mp_limb_t *)x, nx, s);
   if (! zd[lz - 1]) { --lz; }
 
@@ -236,12 +253,12 @@ subiispec(GEN x, GEN y, long nx, long ny)
 {
   GEN zd;
   long lz;
-  if (ny==1) return subisspec(x,*y,nx);
+  if (ny==1) return subiuspec(x,*y,nx);
   lz = nx+2; zd = cgeti(lz);
-  
+
   mpn_sub (LIMBS(zd), (mp_limb_t *)x, nx, (mp_limb_t *)y, ny);
   while (lz >= 3 && zd[lz - 1] == 0) { lz--; }
-  
+
   zd[1] = evalsigne(1) | evallgefint(lz);
   return zd;
 }
@@ -310,41 +327,37 @@ shifti(GEN x, long n)
   lx = lgefint(x);
   if (n > 0)
   {
-    long d = n>>TWOPOTBITS_IN_LONG;
+    long d = dvmdsBIL(n, &m);
     long i;
 
-    m = n & (BITS_IN_LONG-1);
-
-    lz = lx + d + (m!=0);  
-    z = cgeti(lz); 
+    lz = lx + d + (m!=0);
+    z = cgeti(lz);
     for (i=0; i<d; i++) LIMBS(z)[i] = 0;
 
-    if (!m) xmpn_copy(LIMBS(z)+d, LIMBS(x), NLIMBS(x)); 
+    if (!m) xmpn_copy(LIMBS(z)+d, LIMBS(x), NLIMBS(x));
     else
     {
-      ulong carry = mpn_lshift(LIMBS(z)+d, LIMBS(x), NLIMBS(x), m); 
-      if (carry) z[lz - 1] = carry; 
-      else lz--; 
+      ulong carry = mpn_lshift(LIMBS(z)+d, LIMBS(x), NLIMBS(x), m);
+      if (carry) z[lz - 1] = carry;
+      else lz--;
     }
   }
   else
   {
-    long d = (-n)>>TWOPOTBITS_IN_LONG;
+    long d = dvmdsBIL(-n, &m);
 
-    n = -n;
     lz = lx - d;
     if (lz<3) return gen_0;
     z = cgeti(lz);
-    m = n & (BITS_IN_LONG-1);
 
     if (!m) xmpn_copy(LIMBS(z), LIMBS(x) + d, NLIMBS(x) - d);
     else
     {
-      mpn_rshift(LIMBS(z), LIMBS(x) + d, NLIMBS(x) - d, m); 
+      mpn_rshift(LIMBS(z), LIMBS(x) + d, NLIMBS(x) - d, m);
       if (z[lz - 1] == 0)
       {
         if (lz == 3) { avma = (pari_sp)(z+3); return gen_0; }
-        lz--; 
+        lz--;
       }
     }
   }
@@ -353,12 +366,12 @@ shifti(GEN x, long n)
 }
 
 GEN
-ishiftr_lg(GEN x, long lx, long n)
+trunc2nr_lg(GEN x, long lx, long n)
 {
   long ly, i, m, s = signe(x);
   GEN y;
   if (!s) return gen_0;
-  if (!n) 
+  if (!n)
   {
     y = cgeti(lx);
     y[1] = evalsigne(s) | evallgefint(lx);
@@ -368,16 +381,15 @@ ishiftr_lg(GEN x, long lx, long n)
   if (n > 0)
   {
     GEN z = (GEN)avma;
-    long d = n>>TWOPOTBITS_IN_LONG;
+    long d = dvmdsBIL(n, &m);
 
     ly = lx+d; y = new_chunk(ly);
     for ( ; d; d--) *--z = 0;
-    m = n & (BITS_IN_LONG-1);
     if (!m) for (i=2; i<lx; i++) y[i]=x[i];
     else
     {
       register const ulong sh = BITS_IN_LONG - m;
-      shift_left2(y,x, 2,lx-1, 0,m,sh);
+      shift_left(y,x, 2,lx-1, 0,m);
       i = ((ulong)x[2]) >> sh;
       /* Extend y on the left? */
       if (i) { ly++; y = new_chunk(1); y[2] = i; }
@@ -385,11 +397,9 @@ ishiftr_lg(GEN x, long lx, long n)
   }
   else
   {
-    n = -n;
-    ly = lx - (n>>TWOPOTBITS_IN_LONG);
+    ly = lx - dvmdsBIL(-n, &m);
     if (ly<3) return gen_0;
     y = new_chunk(ly);
-    m = n & (BITS_IN_LONG-1);
     if (m) {
       shift_right(y,x, 2,ly, 0,m);
       if (y[2] == 0)
@@ -412,8 +422,7 @@ truncr(GEN x)
   long s, e, d, m, i;
   GEN y;
   if ((s=signe(x)) == 0 || (e=expo(x)) < 0) return gen_0;
-  d = (e>>TWOPOTBITS_IN_LONG) + 3;
-  m = e & (BITS_IN_LONG-1);
+  d = nbits2prec(e+1); m = remsBIL(e);
   if (d > lg(x)) pari_err(precer, "truncr (precision loss in truncation)");
 
   y=cgeti(d); y[1] = evalsigne(s) | evallgefint(d);
@@ -437,8 +446,7 @@ floorr(GEN x)
   GEN y;
   if (signe(x) >= 0) return truncr(x);
   if ((e=expo(x)) < 0) return gen_m1;
-  d = (e>>TWOPOTBITS_IN_LONG) + 3;
-  m = e & (BITS_IN_LONG-1);
+  d = nbits2prec(e+1); m = remsBIL(e);
   lx=lg(x); if (d>lx) pari_err(precer, "floorr (precision loss in truncation)");
   y = cgeti(d+1);
   if (++m == BITS_IN_LONG)
@@ -459,7 +467,7 @@ floorr(GEN x)
     }
   }
   if (mpn_add_1(LIMBS(y),LIMBS(y),d-2,1))
-    y[d++]=1; 
+    y[d++]=1;
 END:
   y[1] = evalsigne(-1) | evallgefint(d);
   return y;
@@ -478,58 +486,25 @@ absi_equal_lg(GEN x, GEN y, long l)
 }
 
 /***********************************************************************/
-/**								      **/
-/**		          MULTIPLICATION                 	      **/
+/**                                                                   **/
+/**                          MULTIPLICATION                           **/
 /**                                                                   **/
 /***********************************************************************/
-GEN
-mulss(long x, long y)
-{
-  long s,p1;
-  GEN z;
-  LOCAL_HIREMAINDER;
-
-  if (!x || !y) return gen_0;
-  if (x<0) { s = -1; x = -x; } else s=1;
-  if (y<0) { s = -s; y = -y; }
-  p1 = mulll(x,y);
-  if (hiremainder)
-  {
-    z=cgeti(4); z[1] = evalsigne(s) | evallgefint(4);
-    z[3]=hiremainder; z[2]=p1; return z;
-  }
-  z=cgeti(3); z[1] = evalsigne(s) | evallgefint(3);
-  z[2]=p1; return z;
-}
-
-GEN
-muluu(ulong x, ulong y)
-{
-  long p1;
-  GEN z;
-  LOCAL_HIREMAINDER;
-
-  if (!x || !y) return gen_0;
-  p1 = mulll(x,y);
-  if (hiremainder)
-  {
-    z=cgeti(4); z[1] = evalsigne(1) | evallgefint(4);
-    z[3]=hiremainder; z[2]=p1; return z;
-  }
-  z=cgeti(3); z[1] = evalsigne(1) | evallgefint(3);
-  z[2]=p1; return z;
-}
-
 /* assume ny > 0 */
 INLINE GEN
 muluispec(ulong x, GEN y, long ny)
 {
-  long lz = ny+3;
-  GEN z=cgeti(lz);
-  ulong hi = mpn_mul_1 (LIMBS(z), (mp_limb_t *)y, ny, x);
-  if (hi) { z[lz - 1] = hi; } else lz--;
-  z[1] = evalsigne(1) | evallgefint(lz);
-  return z;
+  if (ny == 1)
+    return muluu(x, *y);
+  else
+  {
+    long lz = ny+3;
+    GEN z = cgeti(lz);
+    ulong hi = mpn_mul_1 (LIMBS(z), (mp_limb_t *)y, ny, x);
+    if (hi) { z[lz - 1] = hi; } else lz--;
+    z[1] = evalsigne(1) | evallgefint(lz);
+    return z;
+  }
 }
 
 /* a + b*|y| */
@@ -574,11 +549,9 @@ muliispec_mirror(GEN x, GEN y, long nx, long ny)
   return z;
 }
 
-/*#define KARAMULR_VARIANT*/
-
 /***********************************************************************/
-/**								      **/
-/**		          DIVISION                       	      **/
+/**                                                                   **/
+/**                          DIVISION                                 **/
 /**                                                                   **/
 /***********************************************************************/
 
@@ -607,7 +580,7 @@ diviu_rem(GEN y, ulong x, ulong *rem)
   ly = lgefint(y);
   if (ly == 3 && (ulong)x > (ulong)y[2]) { *rem = (ulong)y[2]; return gen_0; }
 
-  z = cgeti(ly); 
+  z = cgeti(ly);
   *rem = mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
   if (z [ly - 1] == 0) ly--;
   z[1] = evallgefint(ly) | evalsigne(1);
@@ -627,7 +600,7 @@ divis_rem(GEN y, long x, long *rem)
   ly = lgefint(y);
   if (ly == 3 && (ulong)x > (ulong)y[2]) { *rem = itos(y); return gen_0; }
 
-  z = cgeti(ly); 
+  z = cgeti(ly);
   *rem = mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
   if (sy<0) *rem = -  *rem;
   if (z[ly - 1] == 0) ly--;
@@ -648,7 +621,7 @@ divis(GEN y, long x)
   ly = lgefint(y);
   if (ly == 3 && (ulong)x > (ulong)y[2]) return gen_0;
 
-  z = cgeti(ly); 
+  z = cgeti(ly);
   (void)mpn_divrem_1(LIMBS(z), 0, LIMBS(y), NLIMBS(y), x);
   if (z[ly - 1] == 0) ly--;
   z[1] = evallgefint(ly) | evalsigne(s);
@@ -660,11 +633,11 @@ static GEN
 divrr_with_gmp(GEN x, GEN y)
 {
   long lx=RNLIMBS(x),ly=RNLIMBS(y);
-  long lw=min(lx,ly);
-  long lly=min(lw+1,ly);
+  long lw=minss(lx,ly);
+  long lly=minss(lw+1,ly);
   GEN  w=cgetr(lw+2);
   long lu=lw+lly;
-  long llx=min(lu,lx);
+  long llx=minss(lu,lx);
   mp_limb_t *u=(mp_limb_t *)new_chunk(lu);
   mp_limb_t *z=(mp_limb_t *)new_chunk(lly);
   mp_limb_t *q,*r;
@@ -677,11 +650,11 @@ divrr_with_gmp(GEN x, GEN y)
   r = (mp_limb_t *)new_chunk(lly);
 
   mpn_tdiv_qr(q,r,0,u,lu,z,lly);
-  
+
   /*Round up: This is not exactly correct we should test 2*r>z*/
   if ((ulong)r[lly-1] > ((ulong)z[lly-1]>>1))
     mpn_add_1(q,q,lw+1,1);
-  
+
   xmpn_mirrorcopy(RLIMBS(w),q,lw);
 
   if (q[lw] == 0) e--;
@@ -698,7 +671,7 @@ static GEN
 divri_with_gmp(GEN x, GEN y)
 {
   long llx=RNLIMBS(x),ly=NLIMBS(y);
-  long lly=min(llx+1,ly);
+  long lly=minss(llx+1,ly);
   GEN  w=cgetr(llx+2);
   long lu=llx+lly, ld=ly-lly;
   mp_limb_t *u=(mp_limb_t *)new_chunk(lu);
@@ -715,11 +688,11 @@ divri_with_gmp(GEN x, GEN y)
   r = (mp_limb_t *)new_chunk(lly);
 
   mpn_tdiv_qr(q,r,0,u,lu,z,lly);
-  
+
   /*Round up: This is not exactly correct we should test 2*r>z*/
   if ((ulong)r[lly-1] > ((ulong)z[lly-1]>>1))
     mpn_add_1(q,q,llx+1,1);
-  
+
   xmpn_mirrorcopy(RLIMBS(w),q,llx);
 
   if (q[llx] == 0) e--;
@@ -738,7 +711,11 @@ divri(GEN x, GEN y)
 
   if (!s) pari_err(gdiver);
   if (!signe(x)) return real_0_bit(expo(x) - expi(y));
-  if (!is_bigint(y)) return divrs(x, s>0? y[2]: -y[2]);
+  if (!is_bigint(y)) {
+    GEN z = divru(x, y[2]);
+    if (s < 0) togglesign(z);
+    return z;
+  }
   return divri_with_gmp(x,y);
 }
 
@@ -753,7 +730,7 @@ divrr(GEN x, GEN y)
   e = expo(x) - expo(y);
   if (!sx) return real_0_bit(e);
   if (sy<0) sx = -sx;
-    
+
   lx=lg(x); ly=lg(y);
   if (ly==3)
   {
@@ -772,20 +749,20 @@ divrr(GEN x, GEN y)
   if (ly>=DIVRR_GMP_LIMIT)
     return divrr_with_gmp(x,y);
 
-  lr = min(lx,ly); r = new_chunk(lr);
+  lr = minss(lx,ly); r = new_chunk(lr);
   r1 = r-1;
   r1[1] = 0; for (i=2; i<lr; i++) r1[i]=x[i];
   r1[lr] = (lx>ly)? x[lr]: 0;
   y0 = y[2]; y1 = y[3];
   for (i=0; i<lr-1; i++)
-  { /* r1 = r + (i-1) */
-    ulong k,qp;
+  { /* r1 = r + (i-1), OK up to r1[2] (accesses at most r[lr]) */
+    ulong k, qp;
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
-    
+
     if ((ulong)r1[1] == y0)
     {
-      qp = MAXULONG; k=addll(y0,r1[2]);
+      qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
@@ -793,20 +770,27 @@ divrr(GEN x, GEN y)
       {
         GEN y1 = y+1;
         j = lr-i; r1[j] = subll(r1[j],y1[j]);
-	for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
-	j=i; do r[--j]++; while (j && !r[j]);
+        for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
+        j=i; do r[--j]++; while (j && !r[j]);
       }
       hiremainder = r1[1]; overflow = 0;
       qp = divll(r1[2],y0); k = hiremainder;
     }
+    j = lr-i+1;
     if (!overflow)
     {
-      long k3 = subll(mulll(qp,y1), r1[3]);
-      long k4 = subllx(hiremainder,k);
+      long k3, k4;
+      k3 = mulll(qp,y1);
+      if (j == 3) /* i = lr - 2 maximal, r1[3] undefined -> 0 */
+        k4 = subll(hiremainder,k);
+      else
+      {
+        k3 = subll(k3, r1[3]);
+        k4 = subllx(hiremainder,k);
+      }
       while (!overflow && k4) { qp--; k3=subll(k3,y1); k4=subllx(k4,y0); }
     }
-    j = lr-i+1;
-    if (j<ly) (void)mulll(qp,y[j]); else { hiremainder = 0 ; j = ly; } 
+    if (j<ly) (void)mulll(qp,y[j]); else { hiremainder = 0 ; j = ly; }
     for (j--; j>1; j--)
     {
       r1[j] = subll(r1[j], addmul(qp,y[j]));
@@ -822,14 +806,14 @@ divrr(GEN x, GEN y)
       }
       else
       {
-	r1[1] -= hiremainder;
-	while (r1[1])
-	{
-	  qp++; if (!qp) { j=i; do r[--j]++; while (j && !r[j]); }
+        r1[1] -= hiremainder;
+        while (r1[1])
+        {
+          qp++; if (!qp) { j=i; do r[--j]++; while (j && !r[j]); }
           j = lr-i-(lr-i>=ly); r1[j] = subll(r1[j],y[j]);
           for (j--; j>1; j--) r1[j] = subllx(r1[j],y[j]);
-	  r1[1] -= overflow;
-	}
+          r1[1] -= overflow;
+        }
       }
     }
     *++r1 = qp;
@@ -844,7 +828,7 @@ divrr(GEN x, GEN y)
   if (r[0] == 0) e--;
   else if (r[0] == 1) { shift_right(r,r, 2,lr, 1,1); }
   else { /* possible only when rounding up to 0x2 0x0 ... */
-    r[2] = HIGHBIT; e++; 
+    r[2] = (long)HIGHBIT; e++;
   }
   r[0] = evaltyp(t_REAL)|evallg(lr);
   r[1] = evalsigne(sx) | evalexpo(e);
@@ -881,7 +865,7 @@ dvmdii(GEN x, GEN y, GEN *z)
     {
       long s=mpn_cmp(LIMBS(x),LIMBS(y),NLIMBS(x));
       if (s>0) goto DIVIDE;
-      if (s==0) 
+      if (s==0)
       {
         if (z == ONLY_REM) return gen_0;
         if (z) *z = gen_0;
@@ -929,7 +913,7 @@ DIVIDE: /* quotient is non-zero */
       if (lr == 2) {avma=av; return gen_0;} /* exact division */
     }
     r[1] = evalsigne(sx) | evallgefint(lr);
-    avma = (pari_sp) r; return r; 
+    avma = (pari_sp) r; return r;
   }
   else
   {
@@ -947,18 +931,10 @@ DIVIDE: /* quotient is non-zero */
       if (lr == 2) {avma=(pari_sp) q; *z=gen_0; return q;} /* exact division */
     }
     r[1] = evalsigne(sx) | evallgefint(lr);
-    avma = (pari_sp) r; *z = r; return q; 
+    avma = (pari_sp) r; *z = r; return q;
   }
 }
 
-/* assume y > x > 0. return y mod x */
-
-static ulong
-resiu(GEN y, ulong x)
-{
-  return mpn_mod_1(LIMBS(y), NLIMBS(y), x);
-}
-
 /* Montgomery reduction.
  * N has k words, assume T >= 0 has less than 2k.
  * Return res := T / B^k mod N, where B = 2^BIL
@@ -968,91 +944,158 @@ red_montgomery(GEN T, GEN N, ulong inv)
 {
   pari_sp av;
   GEN Te, Td, Ne, Nd, scratch;
-  ulong i, j, m, t, d, k = lgefint(N)-2;
+  ulong i, j, m, t, d, k = NLIMBS(N);
   int carry;
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
   if (k == 0) return gen_0;
-  d = lgefint(T)-2; /* <= 2*k */
+  d = NLIMBS(T); /* <= 2*k */
+  if (d == 0) return gen_0;
 #ifdef DEBUG
   if (d > 2*k) pari_err(bugparier,"red_montgomery");
 #endif
   if (k == 1)
   { /* as below, special cased for efficiency */
     ulong n = (ulong)N[2];
-    t = (ulong)T[d+1];
-    m = t * inv;
-    (void)addll(mulll(m, n), t); /* = 0 */
-    t = hiremainder + overflow;
-    if (d == 2)
-    {
-      t = addll((ulong)T[2], t);
+    if (d == 1) {
+      hiremainder = (ulong)T[2];
+      m = hiremainder * inv;
+      (void)addmul(m, n); /* t + m*n = 0 */
+      return utoi(hiremainder);
+    } else { /* d = 2 */
+      hiremainder = (ulong)T[2];
+      m = hiremainder * inv;
+      (void)addmul(m, n); /* t + m*n = 0 */
+      t = addll(hiremainder, (ulong)T[3]);
       if (overflow) t -= n; /* t > n doesn't fit in 1 word */
+      return utoi(t);
     }
-    return utoi(t);
   }
   /* assume k >= 2 */
   av = avma; scratch = new_chunk(k<<1); /* >= k + 2: result fits */
 
   /* copy T to scratch space (pad with zeroes to 2k words) */
-  Td = (GEN)av;
-  Te = T + (d+2);
-  for (i=0; i < d     ; i++) *--Td = *--Te;
-  for (   ; i < (k<<1); i++) *--Td = 0;
+  Td = scratch;
+  Te = T + 2;
+  for (i=0; i < d     ; i++) *Td++ = *Te++;
+  for (   ; i < (k<<1); i++) *Td++ = 0;
 
-  Te = (GEN)av; /* 1 beyond end of T mantissa */
-  Ne = N + k+2; /* 1 beyond end of N mantissa */
+  Te = scratch - 1; /* 1 beyond end of current T mantissa (in scratch) */
+  Ne = N + 1;       /* 1 beyond end of N mantissa */
 
   carry = 0;
   for (i=0; i<k; i++) /* set T := T/B nod N, k times */
   {
     Td = Te; /* one beyond end of (new) T mantissa */
     Nd = Ne;
-    m = *--Td * inv; /* solve T + m N = O(B) */
+    hiremainder = *++Td;
+    m = hiremainder * inv; /* solve T + m N = O(B) */
 
     /* set T := (T + mN) / B */
     Te = Td;
-    (void)addll(mulll(m, *--Nd), *Td); /* = 0 */
+    (void)addmul(m, *++Nd); /* = 0 */
     for (j=1; j<k; j++)
     {
+      t = addll(addmul(m, *++Nd), *++Td);
+      *Td = t;
       hiremainder += overflow;
-      t = addll(addmul(m, *--Nd), *--Td); *Td = t;
     }
-    overflow += hiremainder;
-    t = addll(overflow, *--Td); *Td = t + carry;
+    t = addll(hiremainder, *++Td); *Td = t + carry;
     carry = (overflow || (carry && *Td == 0));
   }
   if (carry)
   { /* Td > N overflows (k+1 words), set Td := Td - N */
     Td = Te;
     Nd = Ne;
-    t = subll(*--Td, *--Nd); *Td = t;
-    while (Td > scratch) { t = subllx(*--Td, *--Nd); *Td = t; }
+    t = subll(*++Td, *++Nd); *Td = t;
+    while (Td < (GEN)av) { t = subllx(*++Td, *++Nd); *Td = t; }
   }
 
   /* copy result */
-  Td = (GEN)av;
-  while (! *scratch) scratch++; /* strip leading zeroes */
-  while (Te > scratch) *--Td = *--Te;
-  k = ((GEN)av - Td) + 2;
-  *--Td = evalsigne(1) | evallgefint(k);
-  *--Td = evaltyp(t_INT) | evallg(k);
+  Td = (GEN)av - 1; /* *Td = high word of final result */
+  while (*Td == 0 && Te < Td) Td--; /* strip leading 0s */
+  k = Td - Te; if (!k) { avma = av; return gen_0; }
+  Td = (GEN)av - k; /* will write mantissa there */
+  (void)memmove(Td, Te+1, k*sizeof(long));
+  Td -= 2;
+  Td[0] = evaltyp(t_INT) | evallg(k+2);
+  Td[1] = evalsigne(1) | evallgefint(k+2);
 #ifdef DEBUG
-  {
-    long l = lgefint(N)-2, s = BITS_IN_LONG*l;
-    GEN R = int2n(s);
-    GEN res = remii(mulii(T, Fp_inv(R, N)), N);
-    if (k > lgefint(N)
-        || !equalii(remii(Td,N),res)
-        || cmpii(Td, addii(shifti(T, -s), N)) >= 0) pari_err(bugparier,"red_montgomery");
-  }
+{
+  long l = NLIMBS(N), s = BITS_IN_LONG*l;
+  GEN R = int2n(s);
+  GEN res = remii(mulii(T, Fp_inv(R, N)), N);
+  if (k > lgefint(N)
+    || !equalii(remii(Td,N),res)
+    || cmpii(Td, addii(shifti(T, -s), N)) >= 0) pari_err(bugparier,"red_montgomery");
+}
 #endif
   avma = (pari_sp)Td; return Td;
 }
 
 /* EXACT INTEGER DIVISION */
 
+#if 1 /* use undocumented GMP interface */
+static void
+GEN2mpz(mpz_t X, GEN x)
+{
+  long l = lgefint(x)-2;
+  X->_mp_alloc = l;
+  X->_mp_size = signe(x) > 0? l: -l;
+  X->_mp_d = LIMBS(x);
+}
+static void
+mpz2GEN(GEN z, mpz_t Z)
+{
+  long l = Z->_mp_size;
+  z[1] = evalsigne(l > 0? 1: -1) | evallgefint(labs(l)+2);
+}
+
+/* assume y != 0 and the division is exact */
+GEN
+diviuexact(GEN x, ulong y)
+{
+  if (!signe(x)) return gen_0;
+  {
+    long l = lgefint(x);
+    mpz_t X, Z;
+    GEN z = cgeti(l);
+    GEN2mpz(X, x);
+    Z->_mp_alloc = l-2;
+    Z->_mp_size  = l-2;
+    Z->_mp_d = LIMBS(z);
+    mpz_divexact_ui(Z, X, y);
+    mpz2GEN(z, Z); return z;
+  }
+}
+
+/* Find z such that x=y*z, knowing that y | x (unchecked) */
+GEN
+diviiexact(GEN x, GEN y)
+{
+  if (!signe(y)) pari_err(gdiver);
+  if (lgefint(y) == 3)
+  {
+    GEN z = diviuexact(x, y[2]);
+    if (signe(y) < 0) togglesign(z);
+    return z;
+  }
+  if (!signe(x)) return gen_0;
+  {
+    long l = lgefint(x);
+    mpz_t X, Y, Z;
+    GEN z = cgeti(l);
+    GEN2mpz(X, x);
+    GEN2mpz(Y, y);
+    Z->_mp_alloc = l-2;
+    Z->_mp_size  = l-2;
+    Z->_mp_d = LIMBS(z);
+    mpz_divexact(Z, X, Y);
+    mpz2GEN(z, Z); return z;
+  }
+}
+#else
 /* assume y != 0 and the division is exact */
 GEN
 diviuexact(GEN x, ulong y)
@@ -1070,6 +1113,8 @@ diviiexact(GEN x, GEN y)
   /*TODO: use mpn_bdivmod instead*/
   return divii(x,y);
 }
+#endif
+
 
 /********************************************************************/
 /**                                                                **/
@@ -1088,7 +1133,7 @@ muliispec(GEN x, GEN y, long nx, long ny)
   if (nx < ny) swapspec(x,y, nx,ny);
   if (!ny) return gen_0;
   if (ny == 1) return muluispec((ulong)*y, x, nx);
-    
+
   lz = nx+ny+2;
   zd = cgeti(lz);
   hi = mpn_mul(LIMBS(zd), (mp_limb_t *)x, nx, (mp_limb_t *)y, ny);
@@ -1106,8 +1151,8 @@ sqrispec(GEN x, long nx)
   long lz;
 
   if (!nx) return gen_0;
-  if (nx==1) return muluu(*x,*x);
-    
+  if (nx==1) return sqru(*x);
+
   lz = (nx<<1)+2;
   zd = cgeti(lz);
   mpn_mul_n(LIMBS(zd), (mp_limb_t *)x, (mp_limb_t *)x, nx);
@@ -1119,16 +1164,15 @@ sqrispec(GEN x, long nx)
 
 /* x % (2^n), assuming x, n >= 0 */
 GEN
-resmod2n(GEN x, long n)
+remi2n(GEN x, long n)
 {
   ulong hi;
-  long l,k,lx,ly;
+  long l, k, lx, ly, sx = signe(x);
   GEN z, xd, zd;
 
-  if (!signe(x) || !n) return gen_0;
+  if (!sx || !n) return gen_0;
 
-  l = n & (BITS_IN_LONG-1);    /* n % BITS_IN_LONG */
-  k = n >> TWOPOTBITS_IN_LONG; /* n / BITS_IN_LONG */
+  k = dvmdsBIL(n, &l);
   lx = lgefint(x);
   if (lx < k+3) return icopy(x);
 
@@ -1146,7 +1190,7 @@ resmod2n(GEN x, long n)
     ly = k+3;
 
   zd = z = cgeti(ly);
-  *++zd = evalsigne(1) | evallgefint(ly);
+  *++zd = evalsigne(sx) | evallgefint(ly);
   xd = x+1; for ( ;k; k--) *++zd = *++xd;
   if (hi) *++zd = hi;
   return z;
@@ -1159,7 +1203,7 @@ resmod2n(GEN x, long n)
 /********************************************************************/
 
 /* Return S (and set R) s.t S^2 + R = N, 0 <= R <= 2S.
- * As for dvmdii, R is last on stack and guaranteed to be gen_0 in case the 
+ * As for dvmdii, R is last on stack and guaranteed to be gen_0 in case the
  * remainder is 0. R = NULL is allowed. */
 GEN
 sqrtremi(GEN a, GEN *r)
@@ -1172,7 +1216,7 @@ sqrtremi(GEN a, GEN *r)
     return gen_0;
   }
   l = (na + 5) >> 1; /* 2 + ceil(na/2) */
-  S = cgeti(l); S[1] = evalsigne(1) | evallgefint(l);
+  S = cgetipos(l);
   if (r) {
     GEN R = cgeti(2 + na);
     nr = mpn_sqrtrem(LIMBS(S), LIMBS(R), LIMBS(a), na);
@@ -1207,14 +1251,14 @@ sqrtr_abs(GEN a)
   else
   {
     ulong u;
-    b = (mp_limb_t *) ishiftr_lg(a,l+2,-1);
+    b = (mp_limb_t *) trunc2nr_lg(a,l+2,-1);
     b[1] = ((ulong)a[l+1])<<(BITS_IN_LONG-1);
     b = (mp_limb_t *) new_chunk(l);
     xmpn_zero(b,l+1); /* overwrites the former b[0] */
     c = (mp_limb_t *) new_chunk(l + 1);
     n = mpn_sqrtrem(c,b,b,(l<<1)+2); /* c <- sqrt; b <- rem */
     u = (ulong)*c++;
-    if ( u&HIGHBIT || (u == ~HIGHBIT && 
+    if ( u&HIGHBIT || (u == ~HIGHBIT &&
              (n>l || (n==l && mpn_cmp(b,c,l)>0))))
       mpn_add_1(c,c,l,1);
   }
@@ -1222,7 +1266,7 @@ sqrtr_abs(GEN a)
   avma = (pari_sp)res; return res;
 }
 
-/* Normalize a non-negative integer.  */
+/* Normalize a non-negative integer */
 GEN
 int_normalize(GEN x, long known_zero_words)
 {
@@ -1231,3 +1275,45 @@ int_normalize(GEN x, long known_zero_words)
     if (x[i]) { setlgefint(x, i+1); return x; }
   x[1] = evalsigne(0) | evallgefint(2); return x;
 }
+
+/********************************************************************
+ **                                                                **
+ **                           Base Conversion                      **
+ **                                                                **
+ ********************************************************************/
+
+ulong *
+convi(GEN x, long *l)
+{
+  long n = nchar2nlong(2 + (long)(NLIMBS(x) * (BITS_IN_LONG * LOG10_2)));
+  GEN str = cgetg(n+1, t_VECSMALL);
+  unsigned char *res = (unsigned char*) GSTR(str);
+  long llz = mpn_get_str(res, 10, LIMBS(icopy(x)), NLIMBS(x));
+  long lz;
+  ulong *z;
+  long i, j;
+  unsigned char *t;
+  while (!*res) {res++; llz--;} /*Strip leading zeros*/
+  lz  = (8+llz)/9;
+  z = (ulong*)new_chunk(1+lz);
+  t=res+llz+9;
+  for(i=0;i<llz-8;i+=9)
+  {
+    ulong s;
+    t-=18;
+    s=*t++;
+    for (j=1; j<9;j++)
+      s=10*s+*t++;
+    *z++=s;
+  }
+  if (i<llz)
+  {
+    unsigned char *t = res;
+    ulong s=*t++;
+    for (j=i+1; j<llz;j++)
+      s=10*s+*t++;
+    *z++=s;
+  }
+  *l = lz;
+  return z;
+}
