@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/add.c"
-/* $Id: add.c 8602 2007-04-18 20:06:59Z kb $
+/* $Id: add.c 11744 2009-06-03 12:39:15Z bill $
 
 Copyright (C) 2002-2003  The PARI group.
 
@@ -14,22 +14,6 @@ Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-/* prototype of positive small ints */
-static long pos_s[] = {
-  evaltyp(t_INT) | _evallg(3), evalsigne(1) | evallgefint(3), 0 };
-
-/* prototype of negative small ints */
-static long neg_s[] = {
-  evaltyp(t_INT) | _evallg(3), (long)evalsigne(-1) | evallgefint(3), 0 };
-
-GEN
-addss(long x, long y)
-{
-  if (!x) return stoi(y);
-  if (x>0) { pos_s[2] = x; return addsi(y,pos_s); }
-  neg_s[2] = -x; return addsi(y,neg_s);
-}
-
 INLINE GEN
 icopy_sign(GEN x, long sx)
 {
@@ -49,7 +33,7 @@ addsi_sign(long x, GEN y, long sy)
   if (x<0) { sx=-1; x=-x; } else sx=1;
   if (sx==sy)
   {
-    z = addsispec(x,y+2, lgefint(y)-2);
+    z = adduispec(x,y+2, lgefint(y)-2);
     setsigne(z,sy); return z;
   }
   ly=lgefint(y);
@@ -68,10 +52,11 @@ addsi_sign(long x, GEN y, long sy)
     }
     return z;
   }
-  z = subisspec(y+2,x, ly-2);
+  z = subiuspec(y+2,x, ly-2);
   setsigne(z,sy); return z;
 }
 
+/* return gen_0 when the sign is 0 */
 GEN
 addii_sign(GEN x, long sx, GEN y, long sy)
 {
@@ -104,46 +89,65 @@ rcopy_sign(GEN x, long sx) { GEN y = rcopy(x); setsigne(y,sx); return y; }
 GEN
 addir_sign(GEN x, long sx, GEN y, long sy)
 {
-  long e,l,ly;
+  long e, l, ly;
   GEN z;
 
   if (!sx) return rcopy_sign(y, sy);
   e = expo(y) - expi(x);
   if (!sy)
   {
-    if (e > 0) return rcopy_sign(y, sy);
-    z = itor(x, 3 + ((-e)>>TWOPOTBITS_IN_LONG));
+    if (e >= 0) return rcopy_sign(y, sy);
+    z = itor(x, nbits2prec(-e));
     setsigne(z, sx); return z;
   }
 
   ly = lg(y);
   if (e > 0)
   {
-    l = ly - (e>>TWOPOTBITS_IN_LONG);
+    l = ly - divsBIL(e);
     if (l < 3) return rcopy_sign(y, sy);
   }
-  else l = ly + ((-e)>>TWOPOTBITS_IN_LONG)+1;
+  else l = ly + nbits2nlong(-e);
   z = (GEN)avma;
   y = addrr_sign(itor(x,l), sx, y, sy);
   ly = lg(y); while (ly--) *--z = y[ly];
   avma = (pari_sp)z; return z;
 }
 
-GEN
-addsr(long x, GEN y)
+static GEN
+addsr_sign(long x, GEN y, long sy)
 {
-  if (!x) return rcopy(y);
-  if (x>0) { pos_s[2]=x; return addir_sign(pos_s, 1, y, signe(y)); }
-  neg_s[2] = -x; return addir_sign(neg_s, -1, y, signe(y));
+  long e, l, ly, sx;
+  GEN z;
+
+  if (!x) return rcopy_sign(y, sy);
+  if (x < 0) { sx = -1; x = -x; } else sx = 1;
+  e = expo(y) - expu(x);
+  if (!sy)
+  {
+    if (e >= 0) return rcopy_sign(y, sy);
+    if (sx == -1) x = -x;
+    return stor(x, nbits2prec(-e));
+  }
+
+  ly = lg(y);
+  if (e > 0)
+  {
+    l = ly - divsBIL(e);
+    if (l < 3) return rcopy_sign(y, sy);
+  }
+  else l = ly + nbits2nlong(-e);
+  z = (GEN)avma;
+  y = addrr_sign(stor(x,l), sx, y, sy);
+  ly = lg(y); while (ly--) *--z = y[ly];
+  avma = (pari_sp)z; return z;
 }
 
 GEN
-subsr(long x, GEN y)
-{
-  if (!x) return rcopy_sign(y, -signe(y));
-  if (x>0) { pos_s[2]=x; return addir_sign(pos_s, 1, y, -signe(y)); }
-  neg_s[2] = -x; return addir_sign(neg_s, -1, y, -signe(y));
-}
+addsr(long x, GEN y) { return addsr_sign(x, y, signe(y)); }
+
+GEN
+subsr(long x, GEN y) { return addsr_sign(x, y, -signe(y)); }
 
 /* return x + 1, assuming x > 0 is a normalized t_REAL of exponent 0 */
 GEN
@@ -151,29 +155,32 @@ addrex01(GEN x)
 {
   long l = lg(x);
   GEN y = cgetr(l);
-  y[1] = evalsigne(1) | evalexpo(1);
+  y[1] = evalsigne(1) | _evalexpo(1);
   y[2] = HIGHBIT | (((ulong)x[2] & ~HIGHBIT) >> 1);
-  shift_right(y, x, 3,l, x[2], 1); 
+  shift_right(y, x, 3,l, x[2], 1);
   return y;
 }
-/* return x - 1 to same accuracy as x, assuming x > 1 is a normalized t_REAL
- * of exponent 0 */
+/* return subrs(x,1) to the min of (prec(x), prec(x-1) + 1),
+ * assuming x > 1 is a normalized t_REAL of exponent 0
+ * [ goal: avoid the loss of significant bits form subrs ]*/
 GEN
 subrex01(GEN x)
 {
-  long i, sh, k, l = lg(x);
+  long i, sh, k, ly, lx = lg(x);
   ulong u;
-  GEN y = cgetr(l);
+  GEN y;
   k = 2;
   u = (ulong)x[2] & (~HIGHBIT);
   while (!u) u = x[++k]; /* terminates: x not a power of 2 */
+  ly = (k == 2)? lx: lx - k+3; /* NB: +3, not +2: 1 extra word */
+  y = cgetr(ly);
   sh = bfffo(u);
   if (sh)
-  { shift_left(y+2, x+k, 0, l-k-1, 0, sh); }
+    shift_left(y+2, x+k, 0, lx-k-1, 0, sh);
   else
-  { for (i = 2; i < l-k+2; i++) y[i] = x[k-2 + i]; }
-  for (i = l-k+2; i < l; i++) y[i] = 0;
-  y[1] = evalsigne(1) | evalexpo(- ((k-2)*BITS_IN_LONG + sh));
+  { for (i = 2; i < lx-k+2; i++) y[i] = x[k-2 + i]; }
+  for (i = lx-k+2; i < ly; i++) y[i] = 0;
+  y[1] = evalsigne(1) | evalexpo(- (bit_accuracy(k) + sh));
   return y;
 }
 
@@ -194,22 +201,22 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
       if (e > 0) ex = ey;
       return real_0_bit(ex);
     }
-    if (e > 0) return real_0_bit(ey);
-    lz = 3 + ((-e)>>TWOPOTBITS_IN_LONG);
+    if (e >= 0) return real_0_bit(ey);
+    lz = nbits2prec(-e);
     lx = lg(x); if (lz > lx) lz = lx;
     z = cgetr(lz); while(--lz) z[lz] = x[lz];
     setsigne(z,sx); return z;
   }
   if (!sx)
   {
-    if (e < 0) return real_0_bit(ex);
-    lz = 3 + (e>>TWOPOTBITS_IN_LONG);
+    if (e <= 0) return real_0_bit(ex);
+    lz = nbits2prec(e);
     ly = lg(y); if (lz > ly) lz = ly;
     z = cgetr(lz); while (--lz) z[lz] = y[lz];
     setsigne(z,sy); return z;
   }
 
-  if (e < 0) { z=x; x=y; y=z; ey=ex; i=sx; sx=sy; sy=i; e=-e; }
+  if (e < 0) { swap(x,y); lswap(sx,sy); ey=ex; e=-e; }
   /* now ey >= ex */
   lx = lg(x);
   ly = lg(y);
@@ -219,9 +226,8 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
   extend = 0;
   if (e)
   {
-    long d = e >> TWOPOTBITS_IN_LONG, l = ly-d;
+    long d = dvmdsBIL(e, &m), l = ly-d;
     if (l <= 2) return rcopy_sign(y, sy);
-    m = e & (BITS_IN_LONG-1);
     if (l > lx) { lz = lx + d + 1; extend = 1; }
     else        { lz = ly; lx = l; }
     if (m)
@@ -229,7 +235,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
       const pari_sp av = avma;
       const ulong sh = BITS_IN_LONG-m;
       GEN p1 = x; x = new_chunk(lx + lz + 1);
-      shift_right2(x,p1,2,lx, 0,m,sh);
+      shift_right(x,p1,2,lx, 0,m);
       if (extend) x[lx] = p1[lx-1] << sh;
       avma = av; /* HACK: cgetr(lz) will not overwrite x */
     }
@@ -281,7 +287,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
   else
   {
     i = 2; while (i < lx && x[i] == y[i]) i++;
-    if (i==lx) return real_0_bit(ey - bit_accuracy(lx));
+    if (i==lx) return real_0_bit(ey+1 - bit_accuracy(lx));
     f2 = ((ulong)y[i] > (ulong)x[i]);
   }
   /* result is non-zero. f2 = (y > x) */
@@ -307,7 +313,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
   x = z+2; i = 0; while (!x[i]) i++;
   lz -= i; z += i;
   j = bfffo(z[2]); /* need to shift left by j bits to normalize mantissa */
-  ez = ey - (j | (i<<TWOPOTBITS_IN_LONG));
+  ez = ey - (j | (i * BITS_IN_LONG));
   if (extend)
   { /* z was extended by d+1 words [should be e bits = d words + m bits] */
     /* not worth keeping extra word if less than 5 significant bits in there */
@@ -329,7 +335,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
       { /* round up */
         i = lz-1;
         while (++z[i] == 0 && i > 1) i--;
-        if (i == 1) { ez++; z[2] = HIGHBIT; }
+        if (i == 1) { ez++; z[2] = (long)HIGHBIT; }
       }
     }
     else if (j) shift_left(z,z,2,lz-1, 0,j);
