@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/addll.h"
-/* $Id: addll.h 7867 2006-04-14 15:26:51Z kb $
+/* $Id: addll.h 12466 2010-06-24 14:53:41Z kb $
 
 Copyright (C) 2003  The PARI group.
 
@@ -35,13 +35,15 @@ extern long subllx(ulong x, ulong y);
 #define LOCAL_OVERFLOW register ulong overflow
 
 #define addll(a, b)                                             \
-({ ulong __arg1 = (a), __arg2 = (b), __value = __arg1 + __arg2; \
+__extension__ ({                                                \
+   ulong __arg1 = (a), __arg2 = (b), __value = __arg1 + __arg2; \
    overflow = (__value < __arg1);                               \
    __value;                                                     \
 })
 
 #define addllx(a, b)                                          \
-({ ulong __arg1 = (a), __arg2 = (b), __value, __tmp = __arg1 + overflow;\
+__extension__ ({                                              \
+   ulong __arg1 = (a), __arg2 = (b), __value, __tmp = __arg1 + overflow;\
    overflow = (__tmp < __arg1);                               \
    __value = __tmp + __arg2;                                  \
    overflow |= (__value < __tmp);                             \
@@ -49,13 +51,15 @@ extern long subllx(ulong x, ulong y);
 })
 
 #define subll(a, b)                                           \
-({ ulong __arg1 = (a), __arg2 = (b);                          \
+__extension__ ({                                              \
+   ulong __arg1 = (a), __arg2 = (b);                          \
    overflow = (__arg2 > __arg1);                              \
    __arg1 - __arg2;                                           \
 })
 
 #define subllx(a, b)                                  \
-({ ulong __arg1 = (a), __arg2 = (b), __value, __tmp = __arg1 - overflow;\
+__extension__ ({                                      \
+   ulong __arg1 = (a), __arg2 = (b), __value, __tmp = __arg1 - overflow;\
    overflow = (__arg1 < overflow);                    \
    __value = __tmp - __arg2;                          \
    overflow |= (__arg2 > __tmp);                      \
