@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/cmp.c"
-/* $Id: cmp.c 6490 2005-01-13 21:50:15Z kb $
+/* $Id: cmp.c 10982 2008-09-18 20:47:17Z kb $
 
 Copyright (C) 2002-2003  The PARI group.
 
@@ -23,70 +23,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 /*They depend on absi_cmp_lg and absi_equal_lg in mp.c*/
 
-#define MASK(x) (((ulong)(x)) & (LGBITS | SIGNBITS))
 int
 equalii(GEN x, GEN y)
 {
-  if (MASK(x[1]) != MASK(y[1])) return 0;
+  if ((x[1] & (LGBITS|SIGNBITS)) != (y[1] & (LGBITS|SIGNBITS))) return 0;
   return absi_equal_lg(x, y, lgefint(x));
 }
-#undef MASK
-
-/* x == y ? */
-int
-equalsi(long x, GEN y)
-{
-  if (!x) return !signe(y);
-  if (x > 0)
-  {
-    if (signe(y) <= 0 || lgefint(y) != 3) return 0;
-    return ((ulong)y[2] == (ulong)x);
-  }
-  if (signe(y) >= 0 || lgefint(y) != 3) return 0;
-  return ((ulong)y[2] == (ulong)-x);
-}
-/* x == |y| ? */
-int
-equalui(ulong x, GEN y)
-{
-  if (!x) return !signe(y);
-  return (lgefint(y) == 3 && (ulong)y[2] == x);
-}
-
-int
-cmpsi(long x, GEN y)
-{
-  ulong p;
-
-  if (!x) return -signe(y);
-
-  if (x>0)
-  {
-    if (signe(y)<=0) return 1;
-    if (lgefint(y)>3) return -1;
-    p=y[2]; if (p == (ulong)x) return 0;
-    return p < (ulong)x ? 1 : -1;
-  }
-
-  if (signe(y)>=0) return -1;
-  if (lgefint(y)>3) return 1;
-  p=y[2]; if (p == (ulong)-x) return 0;
-  return p < (ulong)(-x) ? -1 : 1;
-}
-
-/* compare x and |y| */
-int
-cmpui(ulong x, GEN y)
-{
-  long l = lgefint(y);
-  ulong p;
-
-  if (!x) return (l > 2)? -1: 0;
-  if (l == 2) return 1;
-  if (l > 3) return -1;
-  p = y[2]; if (p == x) return 0;
-  return p < x ? 1 : -1;
-}
 
 int
 cmpii(GEN x, GEN y)
@@ -107,6 +49,31 @@ cmpii(GEN x, GEN y)
     return -absi_cmp_lg(x, y, lx);
 }
 
+int
+equalrr(GEN x, GEN y)
+{
+  long lx, ly, i;
+
+  if (!signe(x)) return signe(y) == 0; /* all zeroes are equal */
+  if (x[1] != y[1]) return 0; /* includes signe(y) = 0 */
+
+  lx = lg(x);
+  ly = lg(y);
+  if (lx < ly)
+  {
+    i=2; while (i<lx && x[i]==y[i]) i++;
+    if (i<lx) return 0;
+    for (; i < ly; i++) if (y[i]) return 0;
+  }
+  else
+  {
+    i=2; while (i<ly && x[i]==y[i]) i++;
+    if (i<ly) return 0;
+    for (; i < lx; i++) if (x[i]) return 0;
+  }
+  return 1;
+}
+
 int
 cmprr(GEN x, GEN y)
 {
