@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/divll.h"
-/* $Id: divll.h 7867 2006-04-14 15:26:51Z kb $
+/* $Id: divll.h 12466 2010-06-24 14:53:41Z kb $
 
 Copyright (C) 2003  The PARI group.
 
@@ -39,9 +39,9 @@ extern ulong hiremainder;
 #undef LOCAL_HIREMAINDER
 #define LOCAL_HIREMAINDER register ulong hiremainder
 
-#define divll(n0, d) 							\
-({ 									\
-  ulong __d1, __d0, __q1, __q0, __r1, __r0, __m, __n1, __n0;	        \
+#define divll(n0, d)                                                    \
+__extension__ ({                                                        \
+  ulong __d1, __d0, __q1, __q0, __r1, __r0, __m, __n1, __n0;            \
   ulong __k, __d;                                                       \
                                                                         \
   __n1 = hiremainder; __n0 = n0; __d = d;                               \
@@ -71,34 +71,34 @@ extern ulong hiremainder;
       __d = __d << __k; __SPLIT(__d, __d1, __d0);                       \
     }                                                                   \
     __LDIV(__n1, __d1, __q1, __r1);                                     \
-    __m =  __q1 * __d0; 					        \
-    __r1 = __GLUE(__r1, HIGHWORD(__n0));  				\
-    if (__r1 < __m)							\
-    {									\
-      __q1--, __r1 += __d;						\
+    __m =  __q1 * __d0;                                                 \
+    __r1 = __GLUE(__r1, HIGHWORD(__n0));                                  \
+    if (__r1 < __m)                                                        \
+    {                                                                        \
+      __q1--, __r1 += __d;                                                \
       if (__r1 >= __d) /* we didn't get carry when adding to __r1 */    \
-        if (__r1 < __m)	__q1--, __r1 += __d;				\
-    }									\
-    __r1 -= __m;							\
+        if (__r1 < __m)        __q1--, __r1 += __d;                                \
+    }                                                                        \
+    __r1 -= __m;                                                        \
     __LDIV(__r1, __d1, __q0, __r0);                                     \
-    __m =  __q0 * __d0;  					        \
-    __r0 = __GLUE(__r0, LOWWORD(__n0));   				\
-    if (__r0 < __m)							\
-    {									\
-      __q0--, __r0 += __d;						\
-      if (__r0 >= __d)			        			\
-        if (__r0 < __m)	__q0--, __r0 += __d;				\
-    }									\
-    hiremainder = (__r0 - __m) >> __k;		                        \
-    __q1 = __GLUE(__q1, __q0);                 				\
-  }                                   				        \
-  __q1;					                                \
+    __m =  __q0 * __d0;                                                  \
+    __r0 = __GLUE(__r0, LOWWORD(__n0));                                   \
+    if (__r0 < __m)                                                        \
+    {                                                                        \
+      __q0--, __r0 += __d;                                                \
+      if (__r0 >= __d)                                                        \
+        if (__r0 < __m)        __q0--, __r0 += __d;                                \
+    }                                                                        \
+    hiremainder = (__r0 - __m) >> __k;                                        \
+    __q1 = __GLUE(__q1, __q0);                                                 \
+  }                                                                           \
+  __q1;                                                                        \
 })
 
 #else /* __GNUC__ */
 
 INLINE long
-divll(ulong n0, ulong d) 							
+divll(ulong n0, ulong d)
 {
   ulong __d1, __d0, __q1, __q0, __r1, __r0, __m, __n1, __n0;
   ulong __k, __d;
