@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/gcdll.c"
-/* $Id: gcdll.c 7835 2006-04-08 11:56:51Z kb $
+/* $Id: gcdll.c 13145 2011-04-20 21:56:40Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -15,19 +15,16 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 /***********************************************************************/
-/**								      **/
-/**		          GCD                            	      **/
+/**                                                                   **/
+/**                          GCD                                      **/
 /**                                                                   **/
 /***********************************************************************/
-
-/* Ultra-fast private ulong gcd for trial divisions.  Called with y odd;
-   x can be arbitrary (but will most of the time be smaller than y).
-   Will also be used from inside ifactor2.c, so it's `semi-private' really.
-   --GN */
+/* Fast ulong gcd.  Called with y odd; x can be arbitrary (but will most of
+ * the time be smaller than y). */
 
 /* Gotos are Harmful, and Programming is a Science.  E.W.Dijkstra. */
 ulong
-ugcd(ulong x, ulong y)         /* assume y&1==1, y > 1 */
+gcduodd(ulong x, ulong y)         /* assume y&1==1, y > 1 */
 {
   if (!x) return y;
   /* fix up x */
@@ -61,6 +58,28 @@ ugcd(ulong x, ulong y)         /* assume y&1==1, y > 1 */
 /* Gotos are useful, and Programming is an Art.  D.E.Knuth. */
 /* PS: Of course written with Dijkstra's lessons firmly in mind... --GN */
 
+/* at least one of a or b is odd, return gcd(a,b) */
+INLINE ulong
+mygcduodd(ulong a, ulong b)
+{
+  ulong c;
+  if (b&1)
+  {
+    if (a==1 || b==1)
+      c = 1;
+    else
+     c = gcduodd(a, b);
+  }
+  else
+  {
+    if (a==1)
+      c = 1;
+    else
+      c = gcduodd(b, a);
+  }
+  return c;
+}
+
 /* modified right shift binary algorithm with at most one division */
 long
 cgcd(long a,long b)
@@ -69,42 +88,39 @@ cgcd(long a,long b)
   a=labs(a); if (!b) return a;
   b=labs(b); if (!a) return b;
   if (a>b) { a %= b; if (!a) return b; }
-  else { b %= a; if (!b) return a; }
-  v=vals(a|b); a>>=v; b>>=v;
-  if (a==1 || b==1) return 1L<<v;
-  if (b&1)
-    return ((long)ugcd((ulong)a, (ulong)b)) << v;
-  else
-    return ((long)ugcd((ulong)b, (ulong)a)) << v;
+  else     { b %= a; if (!b) return a; }
+  v = vals(a|b);
+  return (long)(mygcduodd((ulong)(a>>v), (ulong)(b>>v)) << v);
+}
+ulong
+ugcd(ulong a,ulong b)
+{
+  long v;
+  if (!b) return a;
+  if (!a) return b;
+  if (a>b) { a %= b; if (!a) return b; }
+  else     { b %= a; if (!b) return a; }
+  v = vals(a|b);
+  return mygcduodd(a>>v, b>>v) << v;
 }
 
-/*Warning: will overflow silently if lcm does not fit*/
+/* For gcdii(): assume a>b>0, return gcd(a,b) as a GEN */
+static GEN
+igcduu(ulong a, ulong b)
+{
+  long v;
+  a %= b; if (!a) return utoipos(b);
+  v = vals(a|b);
+  return utoipos( mygcduodd(a>>v, b>>v) << v );
+}
+
+/*Warning: overflows silently if lcm does not fit*/
 long
 clcm(long a,long b)
 {
-  long d;
-  if(!a) return 0;
-  d=cgcd(a,b);
-  if(d!=1) return a*(b/d);
-  return a*b;
-}
-
-/* assume a>b>0, return gcd(a,b) as a GEN (for gcdii) */
-static GEN
-gcduu(ulong a, ulong b)
-{
-  GEN r = cgeti(3);
-  long v;
-
-  r[1] = evalsigne(1)|evallgefint(3);
-  a %= b; if (!a) { r[2]=(long)b; return r; }
-  v=vals(a|b); a>>=v; b>>=v;
-  if (a==1 || b==1) { r[2]=(long)(1UL<<v); return r; }
-  if (b&1)
-    r[2] = (long)(ugcd((ulong)a, (ulong)b) << v);
-  else
-    r[2] = (long)(ugcd((ulong)b, (ulong)a) << v);
-  return r;
+  long d = cgcd(a,b);
+  if (!d) return 0;
+  return d == 1? a*b: a*(b/d);
 }
 
 /********************************************************************/
@@ -194,7 +210,7 @@ gcduu(ulong a, ulong b)
  * rgcduu() is a variant of xxgcduu() which does not have f  (the effect is
  * that of f=0),  but instead has a ulong vmax parameter, for use in rational
  * reconstruction below.  It returns when v1 exceeds vmax;  v will never
- * exceed vmax.  (vmax=0 is taken as a synonym of MAXULONG i.e. unlimited,
+ * exceed vmax.  (vmax=0 is taken as a synonym of ULONG_MAX i.e. unlimited,
  * in which case rgcduu behaves exactly like xxgcduu with f=0.)  The return
  * value of rgcduu() is typically meaningless;  the interesting part is the
  * matrix.
@@ -227,7 +243,7 @@ xgcduu(ulong d, ulong d1, int f, ulong* v, ulong* v1, long *s)
   xv = 0UL; xv1 = 1UL;
   while (d1 > 1UL)
   {
-    d -= d1;			/* no need to use subll */
+    d -= d1;                        /* no need to use subll */
     if (d >= d1)
     {
       hiremainder = 0; q = 1 + divll(d,d1); d = hiremainder;
@@ -235,9 +251,9 @@ xgcduu(ulong d, ulong d1, int f, ulong* v, ulong* v1, long *s)
     }
     else
       xv += xv1;
-				/* possible loop exit */
+                                /* possible loop exit */
     if (d <= 1UL) { xs=1; break; }
-				/* repeat with inverted roles */
+                                /* repeat with inverted roles */
     d1 -= d;
     if (d1 >= d)
     {
@@ -248,7 +264,7 @@ xgcduu(ulong d, ulong d1, int f, ulong* v, ulong* v1, long *s)
       xv1 += xv;
   } /* while */
 
-  if (!(f&1))			/* division by 1 postprocessing if needed */
+  if (!(f&1))                        /* division by 1 postprocessing if needed */
   {
     if (xs && d==1)
     { xv1 += d1 * xv; xs = 0; res = 1UL; }
@@ -271,7 +287,7 @@ xgcduu(ulong d, ulong d1, int f, ulong* v, ulong* v1, long *s)
 
 ulong
 xxgcduu(ulong d, ulong d1, int f,
-	ulong* u, ulong* u1, ulong* v, ulong* v1, long *s)
+        ulong* u, ulong* u1, ulong* v, ulong* v1, long *s)
 {
   ulong xu,xu1, xv,xv1, xs, q,res;
   LOCAL_HIREMAINDER;
@@ -281,7 +297,7 @@ xxgcduu(ulong d, ulong d1, int f,
   xu1 = xv = 0UL;
   while (d1 > 1UL)
   {
-    d -= d1;			/* no need to use subll */
+    d -= d1;                        /* no need to use subll */
     if (d >= d1)
     {
       hiremainder = 0; q = 1 + divll(d,d1); d = hiremainder;
@@ -290,9 +306,9 @@ xxgcduu(ulong d, ulong d1, int f,
     }
     else
     { xv += xv1; xu += xu1; }
-				/* possible loop exit */
+                                /* possible loop exit */
     if (d <= 1UL) { xs=1; break; }
-				/* repeat with inverted roles */
+                                /* repeat with inverted roles */
     d1 -= d;
     if (d1 >= d)
     {
@@ -304,7 +320,7 @@ xxgcduu(ulong d, ulong d1, int f,
     { xv1 += xv; xu1 += xu; }
   } /* while */
 
-  if (!(f&1))			/* division by 1 postprocessing if needed */
+  if (!(f&1))                        /* division by 1 postprocessing if needed */
   {
     if (xs && d==1)
     {
@@ -340,13 +356,13 @@ rgcduu(ulong d, ulong d1, ulong vmax,
   int f = 0;
   LOCAL_HIREMAINDER;
 
-  if (vmax == 0) vmax = MAXULONG;
+  if (vmax == 0) vmax = ULONG_MAX;
   xs = res = 0;
   xu = xv1 = 1UL;
   xu1 = xv = 0UL;
   while (d1 > 1UL)
   {
-    d -= d1;			/* no need to use subll */
+    d -= d1;                        /* no need to use subll */
     if (d >= d1)
     {
       hiremainder = 0; q = 1 + divll(d,d1); d = hiremainder;
@@ -355,10 +371,10 @@ rgcduu(ulong d, ulong d1, ulong vmax,
     }
     else
     { xv += xv1; xu += xu1; }
-				/* possible loop exit */
+                                /* possible loop exit */
     if (xv > vmax) { f=xs=1; break; }
     if (d <= 1UL) { xs=1; break; }
-				/* repeat with inverted roles */
+                                /* repeat with inverted roles */
     d1 -= d;
     if (d1 >= d)
     {
@@ -368,11 +384,11 @@ rgcduu(ulong d, ulong d1, ulong vmax,
     }
     else
     { xv1 += xv; xu1 += xu; }
-				/* possible loop exit */
+                                /* possible loop exit */
     if (xv1 > vmax) { f=1; break; }
   } /* while */
 
-  if (!(f&1))			/* division by 1 postprocessing if needed */
+  if (!(f&1))                        /* division by 1 postprocessing if needed */
   {
     if (xs && d==1)
     {
@@ -422,7 +438,7 @@ cbezout(long a,long b,long *uu,long *vv)
   ulong r,u,u1,v,v1;
 
 #ifdef DEBUG_CBEZOUT
-  fprintferr("> cbezout(%ld,%ld,%p,%p)\n", a, b, (void *)uu, (void *)vv);
+  err_printf("> cbezout(%ld,%ld,%p,%p)\n", a, b, (void *)uu, (void *)vv);
 #endif
   if (!b)
   {
@@ -431,13 +447,13 @@ cbezout(long a,long b,long *uu,long *vv)
     {
       *uu=1L;
 #ifdef DEBUG_CBEZOUT
-      fprintferr("< %ld (%ld, %ld)\n", 1L, *uu, *vv);
+      err_printf("< %ld (%ld, %ld)\n", 1L, *uu, *vv);
 #endif
       return 0L;
     }
     *uu = a < 0 ? -1L : 1L;
 #ifdef DEBUG_CBEZOUT
-    fprintferr("< %ld (%ld, %ld)\n", (long)d, *uu, *vv);
+    err_printf("< %ld (%ld, %ld)\n", (long)d, *uu, *vv);
 #endif
     return (long)d;
   }
@@ -445,15 +461,15 @@ cbezout(long a,long b,long *uu,long *vv)
   {
     *uu = 0L; *vv = b < 0 ? -1L : 1L;
 #ifdef DEBUG_CBEZOUT
-    fprintferr("< %ld (%ld, %ld)\n", (long)d1, *uu, *vv);
+    err_printf("< %ld (%ld, %ld)\n", (long)d1, *uu, *vv);
 #endif
     return (long)d1;
   }
-  else if (d == 1)		/* frequently used by nfinit */
+  else if (d == 1)                /* frequently used by nfinit */
   {
     *uu = a; *vv = 0L;
 #ifdef DEBUG_CBEZOUT
-    fprintferr("< %ld (%ld, %ld)\n", 1L, *uu, *vv);
+    err_printf("< %ld (%ld, %ld)\n", 1L, *uu, *vv);
 #endif
     return 1L;
   }
@@ -461,11 +477,11 @@ cbezout(long a,long b,long *uu,long *vv)
   {
 /* bug in gcc-2.95.3:
  * s = a; a = b; b = s; produces wrong result a = b. This is OK:  */
-    { long _x = a; a = b; b = _x; }	/* in order to keep the right signs */
+    { long _x = a; a = b; b = _x; }        /* in order to keep the right signs */
     r = d; d = d1; d1 = r;
     t = uu; uu = vv; vv = t;
 #ifdef DEBUG_CBEZOUT
-    fprintferr("  swapping\n");
+    err_printf("  swapping\n");
 #endif
   }
   /* d > d1 > 0 */
@@ -481,7 +497,7 @@ cbezout(long a,long b,long *uu,long *vv)
     *vv = b < 0 ? (long)v : -(long)v;
   }
 #ifdef DEBUG_CBEZOUT
-  fprintferr("< %ld (%ld, %ld)\n", (long)r, *uu, *vv);
+  err_printf("< %ld (%ld, %ld)\n", (long)r, *uu, *vv);
 #endif
   return (long)r;
 }
@@ -571,11 +587,11 @@ lgcdii(ulong* d, ulong* d1,
    * possibly overflow.)
    */
   ulong dd,dd1,ddlo,dd1lo, sh,shc;        /* `digits', shift count */
-  ulong xu,xu1, xv,xv1, q,res;	/* recurrences, partial quotient, count */
+  ulong xu,xu1, xv,xv1, q,res;        /* recurrences, partial quotient, count */
   ulong tmp0,tmp1,tmp2,tmpd,tmpu,tmpv; /* temps */
   ulong dm1,dm2,d1m1,d1m2;
-  long ld, ld1, lz;		/* t_INT effective lengths */
-  int skip = 0;			/* a boolean flag */
+  long ld, ld1, lz;                /* t_INT effective lengths */
+  int skip = 0;                        /* a boolean flag */
   LOCAL_OVERFLOW;
   LOCAL_HIREMAINDER;
 
@@ -583,21 +599,21 @@ lgcdii(ulong* d, ulong* d1,
   voir(d, -1); voir(d1, -1);
 #endif
   /* following is just for convenience: vmax==0 means no bound */
-  if (vmax == 0) vmax = MAXULONG;
+  if (vmax == 0) vmax = ULONG_MAX;
   ld = lgefint(d); ld1 = lgefint(d1); lz = ld - ld1; /* >= 0 */
-  if (lz > 1) return 0;		/* rare, quick and desperate exit */
+  if (lz > 1) return 0;                /* rare, quick and desperate exit */
 
-  d = int_MSW(d); d1 = int_MSW(d1);		/* point at the leading `digits' */
-  dm1 = *int_precW(d); d1m1 = *int_precW(d1);	
+  d = int_MSW(d); d1 = int_MSW(d1);                /* point at the leading `digits' */
+  dm1 = *int_precW(d); d1m1 = *int_precW(d1);
   dm2 = *int_precW(int_precW(d)); d1m2 = *int_precW(int_precW(d1));
-  dd1lo = 0;		        /* unless we find something better */
-  sh = bfffo(*d);		/* obtain dividend left shift count */
+  dd1lo = 0;                        /* unless we find something better */
+  sh = bfffo(*d);                /* obtain dividend left shift count */
 
   if (sh)
-  {				/* do the shifting */
+  {                                /* do the shifting */
     shc = BITS_IN_LONG - sh;
     if (lz)
-    {				/* dividend longer than divisor */
+    {                                /* dividend longer than divisor */
       dd1 = (*d1 >> shc);
       if (!(HIGHMASK & dd1)) return 0;  /* overflow detected */
       if (ld1 > 3)
@@ -606,7 +622,7 @@ lgcdii(ulong* d, ulong* d1,
         dd1lo = (*d1 << sh);
     }
     else
-    {				/* dividend and divisor have the same length */
+    {                                /* dividend and divisor have the same length */
       dd1 = (*d1 << sh);
       if (!(HIGHMASK & dd1)) return 0;
       if (ld1 > 3)
@@ -626,8 +642,8 @@ lgcdii(ulong* d, ulong* d1,
       ddlo = (dm1 << sh);
   }
   else
-  {				/* no shift needed */
-    if (lz) return 0;		/* div'd longer than div'r: o'flow automatic */
+  {                                /* no shift needed */
+    if (lz) return 0;                /* div'd longer than div'r: o'flow automatic */
     dd1 = *d1;
     if (!(HIGHMASK & dd1)) return 0;
     if(ld1 > 3) dd1lo = d1m1;
@@ -635,7 +651,7 @@ lgcdii(ulong* d, ulong* d1,
     dd = *d; ddlo = dm1;
   }
 #ifdef DEBUG_LEHMER
-  fprintferr("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
+  err_printf("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
 #endif
 
   /* First subtraction/division stage.  (If a subtraction initially suffices,
@@ -646,28 +662,28 @@ lgcdii(ulong* d, ulong* d1,
    */
   dd -= dd1;
   if (dd < dd1)
-  {				/* first quotient known to be == 1 */
+  {                                /* first quotient known to be == 1 */
     xv1 = 1UL;
-    if (!dd)			/* !(Jebelean condition), extraspecial case */
-    {				/* note this can actually happen...  Now
-    				 * q==1 is known, but we underflow already.
-				 * OTOH we've just shortened d by a whole word.
-				 * Thus we feel pleased with ourselves and
-				 * return.  (The re-shift code below would
-				 * do so anyway.) */
+    if (!dd)                        /* !(Jebelean condition), extraspecial case */
+    {                                /* note this can actually happen...  Now
+                                     * q==1 is known, but we underflow already.
+                                 * OTOH we've just shortened d by a whole word.
+                                 * Thus we feel pleased with ourselves and
+                                 * return.  (The re-shift code below would
+                                 * do so anyway.) */
       *u = 0; *v = *u1 = *v1 = 1UL;
-      return -1;		/* Next step will be a full division. */
+      return -1;                /* Next step will be a full division. */
     } /* if !(Jebelean) then */
   }
   else
-  {				/* division indicated */
+  {                                /* division indicated */
     hiremainder = 0;
-    xv1 = 1 + divll(dd, dd1);	/* xv1: alternative spelling of `q', here ;) */
+    xv1 = 1 + divll(dd, dd1);        /* xv1: alternative spelling of `q', here ;) */
     dd = hiremainder;
-    if (dd < xv1)		/* !(Jebelean cond'), non-extra special case */
-    {				/* Attempt to complete the division using the
-				 * less significant bits, before skipping right
-				 * past the 1st loop to the reshift stage. */
+    if (dd < xv1)                /* !(Jebelean cond'), non-extra special case */
+    {                                /* Attempt to complete the division using the
+                                 * less significant bits, before skipping right
+                                 * past the 1st loop to the reshift stage. */
       ddlo = subll(ddlo, mulll(xv1, dd1lo));
       dd = subllx(dd, hiremainder);
 
@@ -677,10 +693,10 @@ lgcdii(ulong* d, ulong* d1,
        * quotient is in fact q-1.  Adjust our data accordingly. */
       if (overflow)
       {
-	xv1--;
-	ddlo = addll(ddlo,dd1lo);
-	dd = addllx(dd,dd1);	/* overflows again which cancels the borrow */
-	/* ...and fall through to skip=1 below */
+        xv1--;
+        ddlo = addll(ddlo,dd1lo);
+        dd = addllx(dd,dd1);        /* overflows again which cancels the borrow */
+        /* ...and fall through to skip=1 below */
       }
       else
       /* Test Jebelean condition anew, at this point using _all_ the extracted
@@ -688,7 +704,7 @@ lgcdii(ulong* d, ulong* d1,
        * even chance of succeeding this time.  Note that if we fail, we really
        * do not know whether the correct quotient would have been q or some
        * smaller value. */
-	if (!dd && ddlo < xv1) return 0;
+        if (!dd && ddlo < xv1) return 0;
 
       /* Otherwise, we now know that q is correct, but we cannot go into the
        * 1st loop.  Raise a flag so we'll remember to skip past the loop.
@@ -698,10 +714,10 @@ lgcdii(ulong* d, ulong* d1,
   }
   res = 1;
 #ifdef DEBUG_LEHMER
-  fprintferr("  q = %ld, %lx, %lx\n", xv1, dd1, dd);
+  err_printf("  q = %ld, %lx, %lx\n", xv1, dd1, dd);
 #endif
   if (xv1 > vmax)
-  {				/* gone past the bound already */
+  {                                /* gone past the bound already */
     *u = 0UL; *u1 = 1UL; *v = 1UL; *v1 = xv1;
     return res;
   }
@@ -743,79 +759,79 @@ lgcdii(ulong* d, ulong* d1,
        * entries) when successful. */
       tmpd = dd1 - dd;
       if (tmpd < dd)
-      {				/* quotient suspected to be 1 */
+      {                                /* quotient suspected to be 1 */
 #ifdef DEBUG_LEHMER
-	q = 1;
+        q = 1;
 #endif
-	tmpu = xu + xu1;	/* cannot overflow -- everything bounded by
-				 * the original dd during first loop */
-	tmpv = xv + xv1;
+        tmpu = xu + xu1;        /* cannot overflow -- everything bounded by
+                                 * the original dd during first loop */
+        tmpv = xv + xv1;
       }
       else
-      {				/* division indicated */
-	hiremainder = 0;
-	q = 1 + divll(tmpd, dd);
-	tmpd = hiremainder;
-	tmpu = xu + q*xu1;	/* can't overflow, but may need to be undone */
-	tmpv = xv + q*xv1;
+      {                                /* division indicated */
+        hiremainder = 0;
+        q = 1 + divll(tmpd, dd);
+        tmpd = hiremainder;
+        tmpu = xu + q*xu1;        /* can't overflow, but may need to be undone */
+        tmpv = xv + q*xv1;
       }
 
       tmp0 = addll(tmpv, xv1);
       if ((tmpd < tmpu) || overflow ||
-	  (dd - tmpd < tmp0))	/* !(Jebelean cond.) */
-	break;			/* skip ahead to reshift stage */
+          (dd - tmpd < tmp0))        /* !(Jebelean cond.) */
+        break;                        /* skip ahead to reshift stage */
       else
-      {				/* commit dd1, xu, xv */
-	res++;
-	dd1 = tmpd; xu = tmpu; xv = tmpv;
+      {                                /* commit dd1, xu, xv */
+        res++;
+        dd1 = tmpd; xu = tmpu; xv = tmpv;
 #ifdef DEBUG_LEHMER
-	fprintferr("  q = %ld, %lx, %lx [%lu,%lu;%lu,%lu]\n",
-		   q, dd, dd1, xu1, xu, xv1, xv);
+        err_printf("  q = %ld, %lx, %lx [%lu,%lu;%lu,%lu]\n",
+                   q, dd, dd1, xu1, xu, xv1, xv);
 #endif
-	if (xv > vmax)
-	{			/* time to return */
-	  *u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
-	  return res;
-	}
+        if (xv > vmax)
+        {                        /* time to return */
+          *u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
+          return res;
+        }
       }
 
       /* Second half of loop divides dd1 into dd, and the matrix returns to its
        * normal arrangement. */
       tmpd = dd - dd1;
       if (tmpd < dd1)
-      {				/* quotient suspected to be 1 */
+      {                                /* quotient suspected to be 1 */
 #ifdef DEBUG_LEHMER
-	q = 1;
+        q = 1;
 #endif
-	tmpu = xu1 + xu;	/* cannot overflow */
-	tmpv = xv1 + xv;
+        tmpu = xu1 + xu;        /* cannot overflow */
+        tmpv = xv1 + xv;
       }
       else
-      {				/* division indicated */
-	hiremainder = 0;
-	q = 1 + divll(tmpd, dd1);
-	tmpd = hiremainder;
-	tmpu = xu1 + q*xu;
-	tmpv = xv1 + q*xv;
+      {                                /* division indicated */
+        hiremainder = 0;
+        q = 1 + divll(tmpd, dd1);
+        tmpd = hiremainder;
+        tmpu = xu1 + q*xu;
+        tmpv = xv1 + q*xv;
       }
 
       tmp0 = addll(tmpu, xu);
       if ((tmpd < tmpv) || overflow ||
-	  (dd1 - tmpd < tmp0))	/* !(Jebelean cond.) */
-	break;			/* skip ahead to reshift stage */
+          (dd1 - tmpd < tmp0))        /* !(Jebelean cond.) */
+        break;                        /* skip ahead to reshift stage */
       else
-      {				/* commit dd, xu1, xv1 */
-	res++;
-	dd = tmpd; xu1 = tmpu; xv1 = tmpv;
+      {                                /* commit dd, xu1, xv1 */
+        res++;
+        dd = tmpd; xu1 = tmpu; xv1 = tmpv;
 #ifdef DEBUG_LEHMER
-	fprintferr("  q = %ld, %lx, %lx [%lu,%lu;%lu,%lu]\n",
-		q, dd1, dd, xu, xu1, xv, xv1);
+        err_printf("  q = %ld, %lx, %lx [%lu,%lu;%lu,%lu]\n",
+                q, dd1, dd, xu, xu1, xv, xv1);
 #endif
-	if (xv1 > vmax)
-	{			/* time to return */
-	  *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
-	  return res;
-	}
+        if (xv1 > vmax)
+        {                        /* time to return */
+          *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
+          return res;
+        }
       }
 
     } /* end of first loop */
@@ -855,7 +871,7 @@ lgcdii(ulong* d, ulong* d1,
       ddlo = tmp1;
     }
 #ifdef DEBUG_LEHMER
-    fprintferr("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
+    err_printf("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
 #endif
   } /* end of skip-pable section:  get here also, with res==1, when there
      * was a problem immediately after the very first division. */
@@ -893,35 +909,35 @@ lgcdii(ulong* d, ulong* d1,
    * to return.)
    */
 #ifdef DEBUG_LEHMER
-  fprintferr("(sh)");
+  err_printf("(sh)");
 #endif
 
   if (res&1)
-  {				/* after odd number of division(s) */
+  {                                /* after odd number of division(s) */
     if (dd1 && (sh = bfffo(dd1)))
     {
       shc = BITS_IN_LONG - sh;
       dd = (ddlo >> shc) + (dd << sh);
       if (!(HIGHMASK & dd))
       {
-	*u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
-	return -res;		/* full division asked for */
+        *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
+        return -res;                /* full division asked for */
       }
       dd1 = (dd1lo >> shc) + (dd1 << sh);
     }
     else
-    {				/* time to return: <= 1 word left, or sh==0 */
+    {                                /* time to return: <= 1 word left, or sh==0 */
       *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
       return res;
     }
   }
   else
-  {				/* after even number of divisions */
+  {                                /* after even number of divisions */
     if (dd)
     {
-      sh = bfffo(dd);		/* Known to be positive. */
+      sh = bfffo(dd);                /* Known to be positive. */
       shc = BITS_IN_LONG - sh;
-				/* dd:ddlo will become the new dd1, and v.v. */
+                                /* dd:ddlo will become the new dd1, and v.v. */
       tmpd = (ddlo >> shc) + (dd << sh);
       dd = (dd1lo >> shc) + (dd1 << sh);
       dd1 = tmpd;
@@ -930,26 +946,26 @@ lgcdii(ulong* d, ulong* d1,
        * most annoying bug. The Management. */
       if (HIGHMASK & dd)
       {
-	/* recurrence matrix is the wrong way round;  swap it. */
-	tmp0 = xu; xu = xu1; xu1 = tmp0;
-	tmp0 = xv; xv = xv1; xv1 = tmp0;
+        /* recurrence matrix is the wrong way round;  swap it. */
+        tmp0 = xu; xu = xu1; xu1 = tmp0;
+        tmp0 = xv; xv = xv1; xv1 = tmp0;
       }
       else
       {
-	/* recurrence matrix is the wrong way round;  fix this. */
-	*u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
-	return -res;		/* full division asked for */
+        /* recurrence matrix is the wrong way round;  fix this. */
+        *u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
+        return -res;                /* full division asked for */
       }
     }
     else
-    {				/* time to return: <= 1 word left */
+    {                                /* time to return: <= 1 word left */
       *u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
       return res;
     }
   } /* end reshift */
 
 #ifdef DEBUG_LEHMER
-  fprintferr("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
+  err_printf("  %lx:%lx, %lx:%lx\n", dd, ddlo, dd1, dd1lo);
 #endif
 
   /* The Second Loop.  Rip-off of the first, but we now check for overflow
@@ -964,16 +980,16 @@ lgcdii(ulong* d, ulong* d1,
      * entries) when successful. */
     tmpd = dd1 - dd;
     if (tmpd < dd)
-    {				/* quotient suspected to be 1 */
+    {                                /* quotient suspected to be 1 */
 #ifdef DEBUG_LEHMER
       q = 1;
 #endif
       tmpu = xu + xu1;
-      tmpv = addll(xv, xv1);	/* xv,xv1 will overflow first */
+      tmpv = addll(xv, xv1);        /* xv,xv1 will overflow first */
       tmp1 = overflow;
     }
     else
-    {				/* division indicated */
+    {                                /* division indicated */
       hiremainder = 0;
       q = 1 + divll(tmpd, dd);
       tmpd = hiremainder;
@@ -984,7 +1000,7 @@ lgcdii(ulong* d, ulong* d1,
 
     tmp0 = addll(tmpv, xv1);
     if ((tmpd < tmpu) || overflow || tmp1 ||
-	(dd - tmpd < tmp0))	/* !(Jebelean cond.) */
+        (dd - tmpd < tmp0))        /* !(Jebelean cond.) */
     {
       /* The recurrence matrix has not yet been warped... */
       *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
@@ -994,10 +1010,10 @@ lgcdii(ulong* d, ulong* d1,
     res++;
     dd1 = tmpd; xu = tmpu; xv = tmpv;
 #ifdef DEBUG_LEHMER
-    fprintferr("  q = %ld, %lx, %lx\n", q, dd, dd1);
+    err_printf("  q = %ld, %lx, %lx\n", q, dd, dd1);
 #endif
     if (xv > vmax)
-    {				/* time to return */
+    {                                /* time to return */
       *u = xu1; *u1 = xu; *v = xv1; *v1 = xv;
       return res;
     }
@@ -1006,7 +1022,7 @@ lgcdii(ulong* d, ulong* d1,
      * normal arrangement. */
     tmpd = dd - dd1;
     if (tmpd < dd1)
-    {				/* quotient suspected to be 1 */
+    {                                /* quotient suspected to be 1 */
 #ifdef DEBUG_LEHMER
       q = 1;
 #endif
@@ -1015,7 +1031,7 @@ lgcdii(ulong* d, ulong* d1,
       tmp1 = overflow;
     }
     else
-    {				/* division indicated */
+    {                                /* division indicated */
       hiremainder = 0;
       q = 1 + divll(tmpd, dd1);
       tmpd = hiremainder;
@@ -1026,7 +1042,7 @@ lgcdii(ulong* d, ulong* d1,
 
     tmp0 = addll(tmpu, xu);
     if ((tmpd < tmpv) || overflow || tmp1 ||
-	(dd1 - tmpd < tmp0))	/* !(Jebelean cond.) */
+        (dd1 - tmpd < tmp0))        /* !(Jebelean cond.) */
     {
       /* The recurrence matrix has not yet been unwarped, so it is
        * the wrong way round;  fix this. */
@@ -1037,10 +1053,10 @@ lgcdii(ulong* d, ulong* d1,
     res++; /* commit dd, xu1, xv1 */
     dd = tmpd; xu1 = tmpu; xv1 = tmpv;
 #ifdef DEBUG_LEHMER
-    fprintferr("  q = %ld, %lx, %lx\n", q, dd1, dd);
+    err_printf("  q = %ld, %lx, %lx\n", q, dd1, dd);
 #endif
     if (xv1 > vmax)
-    {				/* time to return */
+    {                                /* time to return */
       *u = xu; *u1 = xu1; *v = xv; *v1 = xv1;
       return res;
     }
@@ -1055,7 +1071,7 @@ Fl_inv(ulong x, ulong p)
 {
   long s;
   ulong xv, xv1, g = xgcduu(p, x, 1, &xv, &xv1, &s);
-  if (g != 1UL) pari_err(invmoder, "%Z", mkintmod(utoi(x), utoi(p)));
+  if (g != 1UL) pari_err(invmoder, mkintmod(utoi(x), utoi(p)));
   xv = xv1 % p; if (s < 0) xv = p - xv;
   return xv;
 }
