@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/invmod.c"
-/* $Id: invmod.c 7534 2005-12-12 08:58:13Z kb $
+/* $Id: invmod.c 13145 2011-04-20 21:56:40Z kb $
 
 Copyright (C) 2003  The PARI group.
 
@@ -37,8 +37,8 @@ invmod(GEN a, GEN b, GEN *res)
   pari_sp av, av1, lim;
   long s;
   ulong g;
-  ulong xu,xu1,xv,xv1;		/* Lehmer stage recurrence matrix */
-  int lhmres;			/* Lehmer stage return value */
+  ulong xu,xu1,xv,xv1;                /* Lehmer stage recurrence matrix */
+  int lhmres;                        /* Lehmer stage return value */
 
   if (typ(a) != t_INT || typ(b) != t_INT) pari_err(arither1);
   if (!signe(b)) { *res=absi(a); return 0; }
@@ -55,8 +55,8 @@ invmod(GEN a, GEN b, GEN *res)
     }
     g = xgcduu((ulong)(b[2]), d1, 1, &xv, &xv1, &s);
 #ifdef DEBUG_LEHMER
-    fprintferr(" <- %lu,%lu\n", (ulong)(b[2]), (ulong)(d1[2]));
-    fprintferr(" -> %lu,%ld,%lu; %lx\n", g,s,xv1,avma);
+    err_printf(" <- %lu,%lu\n", (ulong)(b[2]), (ulong)(d1[2]));
+    err_printf(" -> %lu,%ld,%lu; %lx\n", g,s,xv1,avma);
 #endif
     avma = av;
     if (g != 1UL) { *res = utoipos(g); return 0; }
@@ -67,9 +67,9 @@ invmod(GEN a, GEN b, GEN *res)
   (void)new_chunk(lgefint(b));
   d = absi(b); d1 = modii(a,d);
 
-  v=gen_0; v1=gen_1;	/* general case */
+  v=gen_0; v1=gen_1;        /* general case */
 #ifdef DEBUG_LEHMER
-  fprintferr("INVERT: -------------------------\n");
+  err_printf("INVERT: -------------------------\n");
   output(d1);
 #endif
   av1 = avma; lim = stack_lim(av,1);
@@ -77,49 +77,41 @@ invmod(GEN a, GEN b, GEN *res)
   while (lgefint(d) > 3 && signe(d1))
   {
 #ifdef DEBUG_LEHMER
-    fprintferr("Calling Lehmer:\n");
+    err_printf("Calling Lehmer:\n");
 #endif
-    lhmres = lgcdii((ulong*)d, (ulong*)d1, &xu, &xu1, &xv, &xv1, MAXULONG);
-    if (lhmres != 0)		/* check progress */
-    {				/* apply matrix */
+    lhmres = lgcdii((ulong*)d, (ulong*)d1, &xu, &xu1, &xv, &xv1, ULONG_MAX);
+    if (lhmres != 0)                /* check progress */
+    {                                /* apply matrix */
 #ifdef DEBUG_LEHMER
-      fprintferr("Lehmer returned %d [%lu,%lu;%lu,%lu].\n",
-	      lhmres, xu, xu1, xv, xv1);
+      err_printf("Lehmer returned %d [%lu,%lu;%lu,%lu].\n",
+              lhmres, xu, xu1, xv, xv1);
 #endif
       if ((lhmres == 1) || (lhmres == -1))
       {
-	if (xv1 == 1)
-	{
-	  r = subii(d,d1); d=d1; d1=r;
-	  a = subii(v,v1); v=v1; v1=a;
-	}
-	else
-	{
-	  r = subii(d, mului(xv1,d1)); d=d1; d1=r;
-	  a = subii(v, mului(xv1,v1)); v=v1; v1=a;
-	}
+        if (xv1 == 1)
+        {
+          r = subii(d,d1); d=d1; d1=r;
+          a = subii(v,v1); v=v1; v1=a;
+        }
+        else
+        {
+          r = subii(d, mului(xv1,d1)); d=d1; d1=r;
+          a = subii(v, mului(xv1,v1)); v=v1; v1=a;
+        }
       }
       else
       {
-	r  = subii(muliu(d,xu),  muliu(d1,xv));
-	a  = subii(muliu(v,xu),  muliu(v1,xv));
-	d1 = subii(muliu(d,xu1), muliu(d1,xv1)); d = r;
-	v1 = subii(muliu(v,xu1), muliu(v1,xv1)); v = a;
-        if (lhmres&1)
-	{
-          setsigne(d,-signe(d));
-          setsigne(v,-signe(v));
-        }
-        else
-	{
-          if (signe(d1)) { setsigne(d1,-signe(d1)); }
-          setsigne(v1,-signe(v1));
-        }
+        r  = subii(muliu(d,xu),  muliu(d1,xv));
+        a  = subii(muliu(v,xu),  muliu(v1,xv));
+        d1 = subii(muliu(d,xu1), muliu(d1,xv1)); d = r;
+        v1 = subii(muliu(v,xu1), muliu(v1,xv1)); v = a;
+        if (lhmres&1) { togglesign(d);  togglesign(v); }
+        else          { togglesign(d1); togglesign(v1); }
       }
     }
 #ifdef DEBUG_LEHMER
     else
-      fprintferr("Lehmer returned 0.\n");
+      err_printf("Lehmer returned 0.\n");
     output(d); output(d1); output(v); output(v1);
     sleep(1);
 #endif
@@ -128,7 +120,7 @@ invmod(GEN a, GEN b, GEN *res)
     {
       q = dvmdii(d,d1,&r);
 #ifdef DEBUG_LEHMER
-      fprintferr("Full division:\n");
+      err_printf("Full division:\n");
       printf("  q = "); output(q); sleep (1);
 #endif
       a = subii(v,mulii(q,v1));
@@ -137,9 +129,8 @@ invmod(GEN a, GEN b, GEN *res)
     }
     if (low_stack(lim, stack_lim(av,1)))
     {
-      GEN *gptr[4]; gptr[0]=&d; gptr[1]=&d1; gptr[2]=&v; gptr[3]=&v1;
       if(DEBUGMEM>1) pari_warn(warnmem,"invmod");
-      gerepilemany(av1,gptr,4);
+      gerepileall(av1, 4, &d,&d1,&v,&v1);
     }
   } /* end while */
 
@@ -152,8 +143,8 @@ invmod(GEN a, GEN b, GEN *res)
     g = xxgcduu((ulong)d[2], (ulong)d1[2], 1, &xu, &xu1, &xv, &xv1, &s);
 #ifdef DEBUG_LEHMER
     output(d);output(d1);output(v);output(v1);
-    fprintferr(" <- %lu,%lu\n", (ulong)d[2], (ulong)d1[2]);
-    fprintferr(" -> %lu,%ld,%lu; %lx\n", g,s,xv1,avma);
+    err_printf(" <- %lu,%lu\n", (ulong)d[2], (ulong)d1[2]);
+    err_printf(" -> %lu,%ld,%lu; %lx\n", g,s,xv1,avma);
 #endif
     if (g != 1UL) { avma = av; *res = utoipos(g); return 0; }
     /* (From the xgcduu() blurb:)
@@ -177,7 +168,7 @@ invmod(GEN a, GEN b, GEN *res)
   if (!equalii(d,gen_1)) { *res = icopy(d); return 0; }
   *res = modii(v,b);
 #ifdef DEBUG_LEHMER
-  output(*res); fprintferr("============================Done.\n");
+  output(*res); err_printf("============================Done.\n");
   sleep(1);
 #endif
   return 1;
