@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp.c"
-/* $Id: mp.c 8000 2006-08-24 21:51:50Z kb $
+/* $Id: mp.c 13142 2011-04-20 15:17:30Z bill $
 
 Copyright (C) 2000-2003 The PARI group.
 
@@ -15,8 +15,8 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 /***********************************************************************/
-/**								      **/
-/**		         MULTIPRECISION KERNEL           	      **/
+/**                                                                   **/
+/**                       MULTIPRECISION KERNEL                       **/
 /**                                                                   **/
 /***********************************************************************/
 #include "pari.h"
@@ -30,41 +30,41 @@ int pari_kernel_init(void) { return 0; } /*nothing to do*/
  * BITS_IN_LONG) : a[0], ..., a[na-1]. [ In ordre to facilitate splitting: no
  * need to reintroduce codewords ] */
 
+#define LIMBS(x)  ((x)+2)
+#define NLIMBS(x) (lgefint(x)-2)
+
 /* Normalize a non-negative integer */
 GEN
 int_normalize(GEN x, long known_zero_words)
 {
-  long lx = lgefint(x);
-  long i = 2 + known_zero_words;
-  for ( ; i < lx; i++)
-    if (x[i])
-    {
-      if (i != 2)
-      {
-        GEN x0 = x;
-        i -= 2; x += i;
-        if (x0 == (GEN)avma) avma = (pari_sp)x;
-        else stackdummy((pari_sp)(x0+i), (pari_sp)x0);
-        lx -= i;
-        x[0] = evaltyp(t_INT) | evallg(lx);
-        x[1] = evalsigne(1) | evallgefint(lx);
-      }
-      return x;
-    }
-  x[1] = evalsigne(0) | evallgefint(2); return x;
+  long i, lx = lgefint(x);
+  GEN x0;
+  if (lx == 2) { x[1] = evalsigne(0) | evallgefint(2); return x; }
+  if (!known_zero_words && x[2]) return x;
+  for (i = 2+known_zero_words; i < lx; i++)
+    if (x[i]) break;
+  x0 = x; i -= 2; x += i;
+  if (x0 == (GEN)avma) avma = (pari_sp)x;
+  else stackdummy((pari_sp)(x0+i), (pari_sp)x0);
+  lx -= i;
+  x[0] = evaltyp(t_INT) | evallg(lx);
+  if (lx == 2) x[1] = evalsigne(0) | evallgefint(lx);
+  else         x[1] = evalsigne(1) | evallgefint(lx);
+  return x;
 }
 
 /***********************************************************************/
-/**								      **/
-/**		         ADDITION / SUBTRACTION          	      **/
+/**                                                                   **/
+/**                      ADDITION / SUBTRACTION                       **/
 /**                                                                   **/
 /***********************************************************************/
 
 GEN
 setloop(GEN a)
 {
-  GEN z0 = (GEN)avma; (void)cgetg(lgefint(a) + 3, t_VECSMALL);
-  return icopy_av(a, z0); /* two cells of extra space before a */
+  pari_sp av = avma;
+  (void)cgetg(lgefint(a) + 3, t_VECSMALL);
+  return icopy_avma(a, av); /* two cells of extra space before a */
 }
 
 /* we had a = setloop(?), then some incloops. Reset a to b */
@@ -128,15 +128,16 @@ incloop(GEN a)
 }
 
 INLINE GEN
-addsispec(long s, GEN x, long nx)
+adduispec(ulong s, GEN x, long nx)
 {
   GEN xd, zd = (GEN)avma;
   long lz;
 
+  if (nx == 1) return adduu(s, (ulong)x[0]);
   lz = nx+3; (void)new_chunk(lz);
   xd = x + nx;
-  *--zd = *--xd + s;
-  if ((ulong)*zd < (ulong)s)
+  *--zd = (ulong)*--xd + s;
+  if ((ulong)*zd < s)
     for(;;)
     {
       if (xd == x) { *--zd = 1; break; } /* enlarge z */
@@ -150,43 +151,56 @@ addsispec(long s, GEN x, long nx)
   avma=(pari_sp)zd; return zd;
 }
 
+GEN
+adduispec_offset(ulong s, GEN x, long offset, long nx)
+{
+  GEN xd = x+lgefint(x)-nx-offset;
+  while (nx && *xd==0) {xd++; nx--;}
+  if (!nx) return utoi(s);
+  return adduispec(s,xd,nx);
+}
+
 static GEN
 addiispec(GEN x, GEN y, long nx, long ny)
 {
-  GEN xd,yd,zd;
-  long lz;
+  GEN xd, yd, zd;
+  long lz, i;
   LOCAL_OVERFLOW;
 
   if (nx < ny) swapspec(x,y, nx,ny);
-  if (ny == 1) return addsispec(*y,x,nx);
+  if (ny == 1) return adduispec(*y,x,nx);
   zd = (GEN)avma;
   lz = nx+3; (void)new_chunk(lz);
   xd = x + nx;
   yd = y + ny;
-  *--zd = addll(*--xd, *--yd);
-  while (yd > y) *--zd = addllx(*--xd, *--yd);
+  zd[-1] = addll(xd[-1], yd[-1]);
+  for (i = -2; i >= -ny; i--) zd[i] = addllx(xd[i], yd[i]);
   if (overflow)
     for(;;)
     {
-      if (xd == x) { *--zd = 1; break; } /* enlarge z */
-      *--zd = ((ulong)*--xd) + 1;
-      if (*zd) { lz--; break; }
+      if (i < -nx) { zd[i] = 1; i--; break; } /* enlarge z */
+      zd[i] = (ulong)xd[i] + 1;
+      if (zd[i]) { i--; lz--; break; }
+      i--;
     }
   else lz--;
-  while (xd > x) *--zd = *--xd;
+  for (; i >= -nx; i--) zd[i] = xd[i];
+  zd += i+1;
   *--zd = evalsigne(1) | evallgefint(lz);
   *--zd = evaltyp(t_INT) | evallg(lz);
   avma=(pari_sp)zd; return zd;
 }
 
-/* assume x >= y */
+/* assume x >= s */
 INLINE GEN
-subisspec(GEN x, long s, long nx)
+subiuspec(GEN x, ulong s, long nx)
 {
   GEN xd, zd = (GEN)avma;
   long lz;
   LOCAL_OVERFLOW;
 
+  if (nx == 1) return utoi(x[0] - s);
+
   lz = nx+2; (void)new_chunk(lz);
   xd = x + nx;
   *--zd = subll(*--xd, s);
@@ -213,7 +227,7 @@ subiispec(GEN x, GEN y, long nx, long ny)
   long lz;
   LOCAL_OVERFLOW;
 
-  if (ny==1) return subisspec(x,*y,nx);
+  if (ny==1) return subiuspec(x,*y,nx);
   zd = (GEN)avma;
   lz = nx+2; (void)new_chunk(lz);
   xd = x + nx;
@@ -242,7 +256,7 @@ roundr_up_ip(GEN x, long l)
   for(;;)
   {
     if (++x[--i]) break;
-    if (i == 2) { x[2] = HIGHBIT; setexpo(x, expo(x)+1); break; }
+    if (i == 2) { x[2] = (long)HIGHBIT; setexpo(x, expo(x)+1); break; }
   }
 }
 
@@ -300,16 +314,15 @@ shifti_spec(GEN x, long lx, long n)
   if (n > 0)
   {
     GEN z = (GEN)avma;
-    long d = n>>TWOPOTBITS_IN_LONG;
+    long d = dvmdsBIL(n, &m);
 
     ly = lx+d; y = new_chunk(ly);
     for ( ; d; d--) *--z = 0;
-    m = n & (BITS_IN_LONG-1);
     if (!m) for (i=2; i<lx; i++) y[i]=x[i];
     else
     {
       register const ulong sh = BITS_IN_LONG - m;
-      shift_left2(y,x, 2,lx-1, 0,m,sh);
+      shift_left(y,x, 2,lx-1, 0,m);
       i = ((ulong)x[2]) >> sh;
       /* Extend y on the left? */
       if (i) { ly++; y = new_chunk(1); y[2] = i; }
@@ -317,11 +330,9 @@ shifti_spec(GEN x, long lx, long n)
   }
   else
   {
-    n = -n;
-    ly = lx - (n>>TWOPOTBITS_IN_LONG);
+    ly = lx - dvmdsBIL(-n, &m);
     if (ly<3) return gen_0;
     y = new_chunk(ly);
-    m = n & (BITS_IN_LONG-1);
     if (m) {
       shift_right(y,x, 2,ly, 0,m);
       if (y[2] == 0)
@@ -344,7 +355,7 @@ shifti(GEN x, long n)
 }
 
 GEN
-ishiftr_lg(GEN x, long lx, long n)
+trunc2nr_lg(GEN x, long lx, long n)
 { /*This is a kludge since x is not an integer*/
   return shifti_spec(x, lx, n);
 }
@@ -356,18 +367,14 @@ truncr(GEN x)
   GEN y;
 
   if ((s=signe(x)) == 0 || (e=expo(x)) < 0) return gen_0;
-  d = (e>>TWOPOTBITS_IN_LONG) + 3;
-  m = e & (BITS_IN_LONG-1);
+  d = nbits2prec(e+1); m = remsBIL(e);
   if (d > lg(x)) pari_err(precer, "truncr (precision loss in truncation)");
 
   y=cgeti(d); y[1] = evalsigne(s) | evallgefint(d);
   if (++m == BITS_IN_LONG)
     for (i=2; i<d; i++) y[i]=x[i];
   else
-  {
-    register const ulong sh = BITS_IN_LONG - m;
-    shift_right2(y,x, 2,d,0, sh,m);
-  }
+    shift_right(y,x, 2,d,0, BITS_IN_LONG - m);
   return y;
 }
 
@@ -380,8 +387,7 @@ floorr(GEN x)
 
   if (signe(x) >= 0) return truncr(x);
   if ((e=expo(x)) < 0) return gen_m1;
-  d = (e>>TWOPOTBITS_IN_LONG) + 3;
-  m = e & (BITS_IN_LONG-1);
+  d = nbits2prec(e+1); m = remsBIL(e);
   lx=lg(x); if (d>lx) pari_err(precer, "floorr (precision loss in truncation)");
   y = new_chunk(d);
   if (++m == BITS_IN_LONG)
@@ -392,8 +398,7 @@ floorr(GEN x)
   }
   else
   {
-    register const ulong sh = BITS_IN_LONG - m;
-    shift_right2(y,x, 2,d,0, sh,m);
+    shift_right(y,x, 2,d,0, BITS_IN_LONG - m);
     if (x[d-1]<<m == 0)
     {
       i=d; while (i<lx && !x[i]) i++;
@@ -425,48 +430,10 @@ absi_equal_lg(GEN x, GEN y, long l)
 }
 
 /***********************************************************************/
-/**								      **/
-/**		          MULTIPLICATION                 	      **/
+/**                                                                   **/
+/**                          MULTIPLICATION                           **/
 /**                                                                   **/
 /***********************************************************************/
-GEN
-mulss(long x, long y)
-{
-  long s,p1;
-  GEN z;
-  LOCAL_HIREMAINDER;
-
-  if (!x || !y) return gen_0;
-  if (x<0) { s = -1; x = -x; } else s=1;
-  if (y<0) { s = -s; y = -y; }
-  p1 = mulll(x,y);
-  if (hiremainder)
-  {
-    z=cgeti(4); z[1] = evalsigne(s) | evallgefint(4);
-    z[2]=hiremainder; z[3]=p1; return z;
-  }
-  z=cgeti(3); z[1] = evalsigne(s) | evallgefint(3);
-  z[2]=p1; return z;
-}
-
-GEN
-muluu(ulong x, ulong y)
-{
-  long p1;
-  GEN z;
-  LOCAL_HIREMAINDER;
-
-  if (!x || !y) return gen_0;
-  p1 = mulll(x,y);
-  if (hiremainder)
-  {
-    z=cgeti(4); z[1] = evalsigne(1) | evallgefint(4);
-    z[2]=hiremainder; z[3]=p1; return z;
-  }
-  z=cgeti(3); z[1] = evalsigne(1) | evallgefint(3);
-  z[2]=p1; return z;
-}
-
 /* assume ny > 0 */
 INLINE GEN
 muluispec(ulong x, GEN y, long ny)
@@ -495,8 +462,8 @@ addumului(ulong a, ulong b, GEN Y)
 
   if (!signe(Y)) return utoi(a);
 
-  y = Y+2; z = (GEN)avma;
-  ny = lgefint(Y)-2;
+  y = LIMBS(Y); z = (GEN)avma;
+  ny = NLIMBS(Y);
   lz = ny+3;
 
   (void)new_chunk(lz);
@@ -514,8 +481,8 @@ GEN muliispec(GEN a, GEN b, long na, long nb);
 #define muliispec_mirror muliispec
 
 /***********************************************************************/
-/**								      **/
-/**		          DIVISION                       	      **/
+/**                                                                   **/
+/**                          DIVISION                                 **/
 /**                                                                   **/
 /***********************************************************************/
 
@@ -557,7 +524,7 @@ diviu_rem(GEN y, ulong x, ulong *rem)
     if (ly==3) { *rem = (ulong)y[2]; return gen_0; }
     hiremainder=y[2]; ly--; y++;
   }
-  z = cgeti(ly); z[1] = evallgefint(ly) | evalsigne(1);
+  z = cgetipos(ly);
   for (i=2; i<ly; i++) z[i]=divll(y[i],x);
   *rem = hiremainder; return z;
 }
@@ -636,20 +603,20 @@ divrr(GEN x, GEN y)
     hiremainder=k; r[2]=divll(l,y[2]); return r;
   }
 
-  lr = min(lx,ly); r = new_chunk(lr);
+  lr = minss(lx,ly); r = new_chunk(lr);
   r1 = r-1;
   r1[1] = 0; for (i=2; i<lr; i++) r1[i]=x[i];
   r1[lr] = (lx>ly)? x[lr]: 0;
   y0 = y[2]; y1 = y[3];
   for (i=0; i<lr-1; i++)
-  { /* r1 = r + (i-1) */
+  { /* r1 = r + (i-1), OK up to r1[2] (accesses at most r[lr]) */
     ulong k, qp;
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
 
     if ((ulong)r1[1] == y0)
     {
-      qp = MAXULONG; k = addll(y0,r1[2]);
+      qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
@@ -657,19 +624,26 @@ divrr(GEN x, GEN y)
       {
         GEN y1 = y+1;
         j = lr-i; r1[j] = subll(r1[j],y1[j]);
-	for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
-	j=i; do r[--j]++; while (j && !r[j]);
+        for (j--; j>0; j--) r1[j] = subllx(r1[j],y1[j]);
+        j=i; do r[--j]++; while (j && !r[j]);
       }
       hiremainder = r1[1]; overflow = 0;
       qp = divll(r1[2],y0); k = hiremainder;
     }
+    j = lr-i+1;
     if (!overflow)
     {
-      long k3 = subll(mulll(qp,y1), r1[3]);
-      long k4 = subllx(hiremainder,k);
+      long k3, k4;
+      k3 = mulll(qp,y1);
+      if (j == 3) /* i = lr - 2 maximal, r1[3] undefined -> 0 */
+        k4 = subll(hiremainder,k);
+      else
+      {
+        k3 = subll(k3, r1[3]);
+        k4 = subllx(hiremainder,k);
+      }
       while (!overflow && k4) { qp--; k3 = subll(k3,y1); k4 = subllx(k4,y0); }
     }
-    j = lr-i+1;
     if (j<ly) (void)mulll(qp,y[j]); else { hiremainder = 0 ; j = ly; }
     for (j--; j>1; j--)
     {
@@ -686,14 +660,14 @@ divrr(GEN x, GEN y)
       }
       else
       {
-	r1[1] -= hiremainder;
-	while (r1[1])
-	{
-	  qp++; if (!qp) { j=i; do r[--j]++; while (j && !r[j]); }
+        r1[1] -= hiremainder;
+        while (r1[1])
+        {
+          qp++; if (!qp) { j=i; do r[--j]++; while (j && !r[j]); }
           j = lr-i-(lr-i>=ly); r1[j] = subll(r1[j],y[j]);
           for (j--; j>1; j--) r1[j] = subllx(r1[j],y[j]);
-	  r1[1] -= overflow;
-	}
+          r1[1] -= overflow;
+        }
       }
     }
     *++r1 = qp;
@@ -708,7 +682,7 @@ divrr(GEN x, GEN y)
   if (r[0] == 0) e--;
   else if (r[0] == 1) { shift_right(r,r, 2,lr, 1,1); }
   else { /* possible only when rounding up to 0x2 0x0 ... */
-    r[2] = HIGHBIT; e++;
+    r[2] = (long)HIGHBIT; e++;
   }
   r[0] = evaltyp(t_REAL)|evallg(lr);
   r[1] = evalsigne(sx) | evalexpo(e);
@@ -724,8 +698,11 @@ divri(GEN x, GEN y)
 
   if (!s) pari_err(gdiver);
   if (!signe(x)) return real_0_bit(expo(x) - expi(y));
-  if (!is_bigint(y)) return divrs(x, s>0? y[2]: -y[2]);
-
+  if (!is_bigint(y)) {
+    GEN z = divru(x, y[2]);
+    if (s < 0) togglesign(z);
+    return z;
+  }
   lx = lg(x); z = cgetr(lx); av = avma;
   affrr(divrr(x, itor(y, lx+1)), z);
   avma = av; return z;
@@ -808,8 +785,8 @@ DIVIDE: /* quotient is non-zero */
   { /* normalize so that highbit(y) = 1 (shift left x and y by sh bits)*/
     register const ulong m = BITS_IN_LONG - sh;
     r = new_chunk(ly);
-    shift_left2(r, y,2,ly-1, 0,sh,m); y = r;
-    shift_left2(r1,x,2,lx-1, 0,sh,m);
+    shift_left(r, y,2,ly-1, 0,sh); y = r;
+    shift_left(r1,x,2,lx-1, 0,sh);
     r1[1] = ((ulong)x[2]) >> m;
   }
   else
@@ -826,7 +803,7 @@ DIVIDE: /* quotient is non-zero */
 
     if ((ulong)r1[1] == y0)
     {
-      qp = MAXULONG; k = addll(y0,r1[2]);
+      qp = ULONG_MAX; k = addll(y0,r1[2]);
     }
     else
     {
@@ -945,29 +922,33 @@ red_montgomery(GEN T, GEN N, ulong inv)
 {
   pari_sp av;
   GEN Te, Td, Ne, Nd, scratch;
-  ulong i, j, m, t, d, k = lgefint(N)-2;
+  ulong i, j, m, t, d, k = NLIMBS(N);
   int carry;
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
   if (k == 0) return gen_0;
-  d = lgefint(T)-2; /* <= 2*k */
+  d = NLIMBS(T); /* <= 2*k */
+  if (d == 0) return gen_0;
 #ifdef DEBUG
   if (d > 2*k) pari_err(bugparier,"red_montgomery");
 #endif
   if (k == 1)
   { /* as below, special cased for efficiency */
     ulong n = (ulong)N[2];
-    t = (ulong)T[d+1];
-    m = t * inv;
-    (void)addll(mulll(m, n), t); /* = 0 */
-    t = hiremainder + overflow;
-    if (d == 2)
-    {
-      t = addll((ulong)T[2], t);
+    if (d == 1) {
+      hiremainder = (ulong)T[2];
+      m = hiremainder * inv;
+      (void)addmul(m, n); /* t + m*n = 0 */
+      return utoi(hiremainder);
+    } else { /* d = 2 */
+      hiremainder = (ulong)T[3];
+      m = hiremainder * inv;
+      (void)addmul(m, n); /* t + m*n = 0 */
+      t = addll(hiremainder, (ulong)T[2]);
       if (overflow) t -= n; /* t > n doesn't fit in 1 word */
+      return utoi(t);
     }
-    return utoi(t);
   }
   /* assume k >= 2 */
   av = avma; scratch = new_chunk(k<<1); /* >= k + 2: result fits */
@@ -978,7 +959,7 @@ red_montgomery(GEN T, GEN N, ulong inv)
   for (i=0; i < d     ; i++) *--Td = *--Te;
   for (   ; i < (k<<1); i++) *--Td = 0;
 
-  Te = (GEN)av; /* 1 beyond end of T mantissa */
+  Te = (GEN)av; /* 1 beyond end of current T mantissa (in scratch) */
   Ne = N + k+2; /* 1 beyond end of N mantissa */
 
   carry = 0;
@@ -986,18 +967,19 @@ red_montgomery(GEN T, GEN N, ulong inv)
   {
     Td = Te; /* one beyond end of (new) T mantissa */
     Nd = Ne;
-    m = *--Td * inv; /* solve T + m N = O(B) */
+    hiremainder = *--Td;
+    m = hiremainder * inv; /* solve T + m N = O(B) */
 
     /* set T := (T + mN) / B */
     Te = Td;
-    (void)addll(mulll(m, *--Nd), *Td); /* = 0 */
+    (void)addmul(m, *--Nd); /* = 0 */
     for (j=1; j<k; j++)
     {
+      t = addll(addmul(m, *--Nd), *--Td);
+      *Td = t;
       hiremainder += overflow;
-      t = addll(addmul(m, *--Nd), *--Td); *Td = t;
     }
-    overflow += hiremainder;
-    t = addll(overflow, *--Td); *Td = t + carry;
+    t = addll(hiremainder, *--Td); *Td = t + carry;
     carry = (overflow || (carry && *Td == 0));
   }
   if (carry)
@@ -1010,15 +992,15 @@ red_montgomery(GEN T, GEN N, ulong inv)
 
   /* copy result */
   Td = (GEN)av;
-  while (! *scratch && Te > scratch) scratch++; /* strip leading 0s */
+  while (*scratch == 0 && Te > scratch) scratch++; /* strip leading 0s */
   while (Te > scratch) *--Td = *--Te;
-  k = (GEN)av - Td; if (!k) return gen_0;
+  k = (GEN)av - Td; if (!k) { avma = av; return gen_0; }
   k += 2;
   *--Td = evalsigne(1) | evallgefint(k);
   *--Td = evaltyp(t_INT) | evallg(k);
 #ifdef DEBUG
 {
-  long l = lgefint(N)-2, s = BITS_IN_LONG*l;
+  long l = NLIMBS(N), s = BITS_IN_LONG*l;
   GEN R = int2n(s);
   GEN res = remii(mulii(T, Fp_inv(R, N)), N);
   if (k > lgefint(N)
@@ -1042,7 +1024,7 @@ diviuexact_i(GEN x, ulong y)
   if (y == 1) return icopy(x);
   lx = lgefint(x);
   if (lx == 3) return utoipos((ulong)x[2] / y);
-  yinv = invrev(y);
+  yinv = invmod2BIL(y);
   lz = (y <= (ulong)x[2]) ? lx : lx-1;
   z = new_chunk(lz);
   z0 = z + lz;
@@ -1062,7 +1044,7 @@ diviuexact_i(GEN x, ulong y)
         if ((ulong)*x1 < hiremainder)
         {
           *x1 -= hiremainder;
-          do (*--x1)--; while ((ulong)*x1 == MAXULONG);
+          do (*--x1)--; while ((ulong)*x1 == ULONG_MAX);
         }
         else
           *x1 -= hiremainder;
@@ -1083,7 +1065,7 @@ diviuexact(GEN x, ulong y)
   pari_sp av;
   long lx, vy, s = signe(x);
   GEN z;
-  
+
   if (!s) return gen_0;
   if (y == 1) return icopy(x);
   lx = lgefint(x);
@@ -1092,7 +1074,7 @@ diviuexact(GEN x, ulong y)
     return (s > 0)? utoipos(q): utoineg(q);
   }
   av = avma; (void)new_chunk(lx); vy = vals(y);
-  if (vy) { 
+  if (vy) {
     y >>= vy;
     if (y == 1) { avma = av; return shifti(x, -vy); }
     x = shifti(x, -vy);
@@ -1141,7 +1123,7 @@ diviiexact(GEN x, GEN y)
     x = diviuexact_i(x,(ulong)y[2]); /* x != 0 */
     setsigne(x, (sx+sy)? 1: -1); return x;
   }
-  y0inv = invrev(y[ly-1]);
+  y0inv = invmod2BIL(y[ly-1]);
   i=2; while (i<ly && y[i]==x[i]) i++;
   lz = (i==ly || (ulong)y[i] < (ulong)x[i]) ? lx-ly+3 : lx-ly+2;
   z = new_chunk(lz);
@@ -1157,7 +1139,7 @@ diviiexact(GEN x, GEN y)
     if (!q) continue;
 
     /* x := x - q * y */
-    (void)mulll(q,y[0]); limj = max(lx - lz, ii+3-ly);
+    (void)mulll(q,y[0]); limj = maxss(lx - lz, ii+3-ly);
     { /* update neither lowest word (could set it to 0) nor highest ones */
       register GEN x0 = x + (ii - 1), y0 = y - 1, xlim = x + limj;
       for (; x0 >= xlim; x0--, y0--)
@@ -1170,7 +1152,7 @@ diviiexact(GEN x, GEN y)
         if ((ulong)*x0 < hiremainder)
         {
           *x0 -= hiremainder;
-          do (*--x0)--; while ((ulong)*x0 == MAXULONG);
+          do (*--x0)--; while ((ulong)*x0 == ULONG_MAX);
         }
         else
           *x0 -= hiremainder;
@@ -1187,7 +1169,7 @@ diviiexact(GEN x, GEN y)
 
 /********************************************************************/
 /**                                                                **/
-/**               INTEGER MULTIPLICATION (KARATSUBA)               **/
+/**               INTEGER MULTIPLICATION (BASECASE)                **/
 /**                                                                **/
 /********************************************************************/
 /* nx >= ny = num. of digits of x, y (not GEN, see mulii) */
@@ -1198,7 +1180,8 @@ muliispec_basecase(GEN x, GEN y, long nx, long ny)
   long p1,lz;
   LOCAL_HIREMAINDER;
 
-  if (!ny) return gen_0;
+  if (ny == 1) return muluispec((ulong)*y, x, nx);
+  if (ny == 0) return gen_0;
   zd = (GEN)avma; lz = nx+ny+2;
   (void)new_chunk(lz);
   xd = x + nx;
@@ -1237,7 +1220,8 @@ sqrispec_basecase(GEN x, long nx)
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
-  if (!nx) return gen_0;
+  if (nx == 1) return sqru((ulong)*x);
+  if (nx == 0) return gen_0;
   zd = (GEN)avma; lz = (nx+1) << 1;
   z0 = new_chunk(lz);
   if (nx == 1)
@@ -1291,6 +1275,286 @@ END:
   avma=(pari_sp)zd; return zd;
 }
 
+/********************************************************************/
+/**                                                                **/
+/**               INTEGER MULTIPLICATION (FFT)                     **/
+/**                                                                **/
+/********************************************************************/
+
+/*
+ Compute parameters for FFT:
+   len: result length
+   k: FFT depth.
+   n: number of blocks (2^k)
+   bs: block size
+   mod: Modulus is M=2^(BIL*mod)+1
+   ord: order of 2 in Z/MZ.
+ We must have:
+   bs*n >= l
+   2^(BIL*mod) > nb*2^(2*BIL*bs)
+   2^k | 2*BIL*mod
+*/
+static void
+mulliifft_params(long len, long *k, long *mod, long *bs, long *n, ulong *ord)
+{
+  long r;
+  *k = expu((3*len)>>2)-3;
+  do {
+    (*k)--;
+    r  = *k-(TWOPOTBITS_IN_LONG+2);
+    *n = 1L<<*k;
+    *bs = (len+*n-1)>>*k;
+    *mod= 2**bs+1;
+    if (r>0)
+      *mod=((*mod+(1L<<r)-1)>>r)<<r;
+  } while(*mod>=3**bs);
+  *ord= 4**mod*BITS_IN_LONG;
+}
+
+/* Zf_: arithmetic in ring Z/MZ where M= 2^(BITS_IN_LONG*mod)+1
+ * for some mod.
+ * Do not garbage collect.
+ */
+
+static GEN
+Zf_add(GEN a, GEN b, GEN M)
+{
+  GEN y, z = addii(a,b);
+  long mod = lgefint(M)-3;
+  long l = NLIMBS(z);
+  if (l<=mod) return z;
+  y = subis(z, 1);
+  if (NLIMBS(y)<=mod) return z;
+  return int_normalize(y,1);
+}
+
+static GEN
+Zf_sub(GEN a, GEN b, GEN M)
+{
+  GEN z = subii(a,b);
+  return signe(z)>=0? z: addii(M,z);
+}
+
+/* destroy z */
+static GEN
+Zf_red_destroy(GEN z, GEN M)
+{
+  long mod = lgefint(M)-3;
+  long l = NLIMBS(z);
+  GEN y;
+  if (l<=mod) return z;
+  y = shifti(z, -mod*BITS_IN_LONG);
+  z = int_normalize(z, NLIMBS(y));
+  y = Zf_red_destroy(y, M);
+  z = subii(z, y);
+  if (signe(z)<0) z = addii(z, M);
+  return z;
+}
+
+INLINE GEN
+Zf_shift(GEN a, ulong s, GEN M) { return Zf_red_destroy(shifti(a, s), M); }
+
+/*
+ Multiply by sqrt(2)^s
+ We use the formula sqrt(2)=z_8*(1-z_4)) && z_8=2^(ord/16) [2^(ord/4)+1]
+*/
+
+static GEN
+Zf_mulsqrt2(GEN a, ulong s, ulong ord, GEN M)
+{
+  ulong hord = ord>>1;
+  if (!signe(a)) return gen_0;
+  if (odd(s)) /* Multiply by 2^(s/2) */
+  {
+    GEN az8  = Zf_shift(a,   ord>>4, M);
+    GEN az83 = Zf_shift(az8, ord>>3, M);
+    a = Zf_sub(az8, az83, M);
+    s--;
+  }
+  if (s < hord)
+    return Zf_shift(a, s>>1, M);
+  else
+    return subii(M,Zf_shift(a, (s-hord)>>1, M));
+}
+
+INLINE GEN
+Zf_sqr(GEN a, GEN M) { return Zf_red_destroy(sqri(a), M); }
+
+INLINE GEN
+Zf_mul(GEN a, GEN b, GEN M) { return Zf_red_destroy(mulii(a,b), M); }
+
+/* In place, bit reversing FFT */
+static void
+muliifft_dit(ulong o, ulong ord, GEN M, GEN FFT, long d, long step)
+{
+  pari_sp av = avma;
+  long i;
+  ulong j, no = (o<<1)%ord;
+  long hstep=step>>1;
+  for (i = d+1, j = 0; i <= d+hstep; ++i, j =(j+o)%ord)
+  {
+    GEN a = Zf_add(gel(FFT,i), gel(FFT,i+hstep), M);
+    GEN b = Zf_mulsqrt2(Zf_sub(gel(FFT,i), gel(FFT,i+hstep), M), j, ord, M);
+    affii(a,gel(FFT,i));
+    affii(b,gel(FFT,i+hstep));
+    avma = av;
+  }
+  if (hstep>1)
+  {
+    muliifft_dit(no, ord, M, FFT, d, hstep);
+    muliifft_dit(no, ord, M, FFT, d+hstep, hstep);
+  }
+}
+
+/* In place, bit reversed FFT, inverse of muliifft_dit */
+static void
+muliifft_dis(ulong o, ulong ord, GEN M, GEN FFT, long d, long step)
+{
+  pari_sp av = avma;
+  long i;
+  ulong j, no = (o<<1)%ord;
+  long hstep=step>>1;
+  if (hstep>1)
+  {
+    muliifft_dis(no, ord, M, FFT, d, hstep);
+    muliifft_dis(no, ord, M, FFT, d+hstep, hstep);
+  }
+  for (i = d+1, j = 0; i <= d+hstep; ++i, j =(j+o)%ord)
+  {
+    GEN z = Zf_mulsqrt2(gel(FFT,i+hstep), j, ord, M);
+    GEN a = Zf_add(gel(FFT,i), z, M);
+    GEN b = Zf_sub(gel(FFT,i), z, M);
+    affii(a,gel(FFT,i));
+    affii(b,gel(FFT,i+hstep));
+    avma = av;
+  }
+}
+
+static GEN
+muliifft_spliti(GEN a, long na, long bs, long n, long mod)
+{
+  GEN ap = a+na-1;
+  GEN c  = cgetg(n+1, t_VEC);
+  long i,j;
+  for(i=1;i<=n;i++)
+  {
+    GEN z = cgeti(mod+3);
+    if (na)
+    {
+      long m = minss(bs, na), v=0;
+      GEN zp, aa=ap-m+1;
+      while (!*aa && v<m) {aa++; v++;}
+      zp = z+m-v+1;
+      for (j=v; j < m; j++)
+        *zp-- = *ap--;
+      ap -= v; na -= m;
+      z[1] = evalsigne(m!=v) | evallgefint(m-v+2);
+    } else
+      z[1] = evalsigne(0) | evallgefint(2);
+    gel(c, i) = z;
+  }
+  return c;
+}
+
+static GEN
+muliifft_unspliti(GEN V, long bs, long len)
+{
+  long s, i, j, l = lg(V);
+  GEN a = cgeti(len);
+  a[1] = evalsigne(1)|evallgefint(len);
+  for(i=2;i<len;i++)
+    a[i] = 0;
+  for(i=1, s=0; i<l; i++, s+=bs)
+  {
+    GEN  u = gel(V,i);
+    if (signe(u))
+    {
+      GEN ap = int_W(a,s);
+      GEN up = int_LSW(u);
+      long lu = NLIMBS(u);
+      LOCAL_OVERFLOW;
+      *ap-- = addll(*ap, *up--);
+      for (j=1; j<lu; j++)
+        *ap-- = addllx(*ap, *up--);
+      while (overflow)
+        *ap-- = addll(*ap, 1);
+    }
+  }
+  return int_normalize(a,0);
+}
+
+static GEN
+sqrispec_fft(GEN a, long na)
+{
+  pari_sp av, ltop = avma;
+  long len = 2*na;
+  long k, mod, bs, n;
+  GEN  FFT, M;
+  long i;
+  ulong o, ord;
+
+  mulliifft_params(len,&k,&mod,&bs,&n,&ord);
+  o = ord>>k;
+  M = int2n(mod*BITS_IN_LONG);
+  M[2+mod] = 1;
+  FFT = muliifft_spliti(a, na, bs, n, mod);
+  muliifft_dit(o, ord, M, FFT, 0, n);
+  av = avma;
+  for(i=1; i<=n; i++)
+  {
+    affii(Zf_sqr(gel(FFT,i), M), gel(FFT,i));
+    avma=av;
+  }
+  muliifft_dis(ord-o, ord, M, FFT, 0, n);
+  for(i=1; i<=n; i++)
+  {
+    affii(Zf_shift(gel(FFT,i), (ord>>1)-k, M), gel(FFT,i));
+    avma=av;
+  }
+  return gerepileuptoint(ltop, muliifft_unspliti(FFT,bs,2+len));
+}
+
+static GEN
+muliispec_fft(GEN a, GEN b, long na, long nb)
+{
+  pari_sp av, av2, ltop = avma;
+  long len = na+nb;
+  long k, mod, bs, n;
+  GEN FFT, FFTb, M;
+  long i;
+  ulong o, ord;
+
+  mulliifft_params(len,&k,&mod,&bs,&n,&ord);
+  o = ord>>k;
+  M = int2n(mod*BITS_IN_LONG);
+  M[2+mod] = 1;
+  FFT = muliifft_spliti(a, na, bs, n, mod);
+  av=avma;
+  muliifft_dit(o, ord, M, FFT, 0, n);
+  FFTb = muliifft_spliti(b, nb, bs, n, mod);
+  av2 = avma;
+  muliifft_dit(o, ord, M, FFTb, 0, n);
+  for(i=1; i<=n; i++)
+  {
+    affii(Zf_mul(gel(FFT,i), gel(FFTb,i), M), gel(FFT,i));
+    avma=av2;
+  }
+  avma=av;
+  muliifft_dis(ord-o, ord, M, FFT, 0, n);
+  for(i=1; i<=n; i++)
+  {
+    affii(Zf_shift(gel(FFT,i),(ord>>1)-k,M), gel(FFT,i));
+    avma=av;
+  }
+  return gerepileuptoint(ltop, muliifft_unspliti(FFT,bs,2+len));
+}
+
+/********************************************************************/
+/**                                                                **/
+/**               INTEGER MULTIPLICATION (KARATSUBA)               **/
+/**                                                                **/
+/********************************************************************/
+
 /* return (x shifted left d words) + y. Assume d > 0, x > 0 and y >= 0 */
 static GEN
 addshiftw(GEN x, GEN y, long d)
@@ -1305,14 +1569,14 @@ addshiftw(GEN x, GEN y, long d)
     y0 = yd-d; while (yd > y0) *--zd = *--yd; /* copy last d words of y */
     a -= d;
     if (a)
-      z = addiispec(x+2, y+2, lgefint(x)-2, a);
+      z = addiispec(LIMBS(x), LIMBS(y), NLIMBS(x), a);
     else
       z = icopy(x);
   }
   else
   {
     y0 = yd-a; while (yd > y0) *--zd = *--yd; /* copy last a words of y */
-    while (zd >= z0) *--zd = 0;    /* complete with 0s */
+    while (zd > z0) *--zd = 0;    /* complete with 0s */
     z = icopy(x);
   }
   lz = lgefint(z)+d;
@@ -1331,12 +1595,11 @@ muliispec(GEN a, GEN b, long na, long nb)
   pari_sp av;
 
   if (na < nb) swapspec(a,b, na,nb);
-  if (nb == 1) return muluispec((ulong)*b, a, na);
-  if (nb == 0) return gen_0;
-  if (nb < KARATSUBA_MULI_LIMIT) return muliispec_basecase(a,b,na,nb);
+  if (nb < MULII_KARATSUBA_LIMIT) return muliispec_basecase(a,b,na,nb);
+  if (nb >= MULII_FFT_LIMIT)      return muliispec_fft(a,b,na,nb);
   i=(na>>1); n0=na-i; na=i;
   av=avma; a0=a+na; n0a=n0;
-  while (!*a0 && n0a) { a0++; n0a--; }
+  while (n0a && !*a0) { a0++; n0a--; }
 
   if (n0a && nb > n0)
   { /* nb <= na <= n0 */
@@ -1346,17 +1609,17 @@ muliispec(GEN a, GEN b, long na, long nb)
     nb -= n0;
     c = muliispec(a,b,na,nb);
     b0 = b+nb; n0b = n0;
-    while (!*b0 && n0b) { b0++; n0b--; }
+    while (n0b && !*b0) { b0++; n0b--; }
     if (n0b)
     {
       c0 = muliispec(a0,b0, n0a,n0b);
 
       c2 = addiispec(a0,a, n0a,na);
       c1 = addiispec(b0,b, n0b,nb);
-      c1 = muliispec(c1+2,c2+2, lgefint(c1)-2,lgefint(c2)-2);
-      c2 = addiispec(c0+2, c+2, lgefint(c0)-2,lgefint(c) -2);
+      c1 = muliispec(LIMBS(c1),LIMBS(c2), NLIMBS(c1),NLIMBS(c2));
+      c2 = addiispec(LIMBS(c0),LIMBS(c),  NLIMBS(c0),NLIMBS(c));
 
-      c1 = subiispec(c1+2,c2+2, lgefint(c1)-2,lgefint(c2)-2);
+      c1 = subiispec(LIMBS(c1),LIMBS(c2), NLIMBS(c1),NLIMBS(c2));
     }
     else
     {
@@ -1375,15 +1638,14 @@ muliispec(GEN a, GEN b, long na, long nb)
 
 /* x % (2^n), assuming x, n >= 0 */
 GEN
-resmod2n(GEN x, long n)
+remi2n(GEN x, long n)
 {
   long hi,l,k,lx,ly;
   GEN z, xd, zd;
 
   if (!signe(x) || !n) return gen_0;
 
-  l = n & (BITS_IN_LONG-1);    /* n % BITS_IN_LONG */
-  k = n >> TWOPOTBITS_IN_LONG; /* n / BITS_IN_LONG */
+  k = dvmdsBIL(n, &l);
   lx = lgefint(x);
   if (lx < k+3) return icopy(x);
 
@@ -1414,10 +1676,11 @@ sqrispec(GEN a, long na)
   long n0, n0a, i;
   pari_sp av;
 
-  if (na < KARATSUBA_SQRI_LIMIT) return sqrispec_basecase(a,na);
+  if (na < SQRI_KARATSUBA_LIMIT) return sqrispec_basecase(a,na);
+  if (na >= SQRI_FFT_LIMIT) return sqrispec_fft(a,na);
   i=(na>>1); n0=na-i; na=i;
   av=avma; a0=a+na; n0a=n0;
-  while (!*a0 && n0a) { a0++; n0a--; }
+  while (n0a && !*a0) { a0++; n0a--; }
   c = sqrispec(a,na);
   if (n0a)
   {
@@ -1426,9 +1689,9 @@ sqrispec(GEN a, long na)
     c1 = shifti(muliispec(a0,a, n0a,na),1);
 #else /* faster */
     t = addiispec(a0,a,n0a,na);
-    t = sqrispec(t+2,lgefint(t)-2);
-    c1= addiispec(c0+2,c+2, lgefint(c0)-2, lgefint(c)-2);
-    c1= subiispec(t+2, c1+2, lgefint(t)-2, lgefint(c1)-2);
+    t = sqrispec(LIMBS(t),NLIMBS(t));
+    c1= addiispec(LIMBS(c0),LIMBS(c), NLIMBS(c0), NLIMBS(c));
+    c1= subiispec(LIMBS(t),LIMBS(c1), NLIMBS(t), NLIMBS(c1));
 #endif
     c = addshiftw(c,c1, n0);
     c = addshiftw(c,c0, n0);
@@ -1471,8 +1734,8 @@ p_sqrtu1(ulong *N, ulong *ps, ulong *pr)
 
   q = n0 >> (BITS_IN_LONG - 8);
   /* 2^6 = 64 <= q < 256 = 2^8 */
-  s = approx_tab[q - 64];				/* 128 <= s < 255 */
-  r = (n0 >> (BITS_IN_LONG - 16)) - s * s;		/* r <= 2*s */
+  s = approx_tab[q - 64];                                /* 128 <= s < 255 */
+  r = (n0 >> (BITS_IN_LONG - 16)) - s * s;                /* r <= 2*s */
   if (r > (s << 1)) { r -= (s << 1) | 1; s++; }
 
   /* 8-bit approximation from the high 8-bits of N[0] */
@@ -1547,23 +1810,14 @@ xmpn_copy(GEN z, GEN x, long n)
   long k = n;
   while (--k >= 0) z[k] = x[k];
 }
-static GEN
-cat1u(ulong d)
-{
-  GEN R = cgeti(4);
-  R[1] = evalsigne(1)|evallgefint(4);
-  R[2] = 1;
-  R[3] = d; return R;
-}
 /* a[0..la-1] * 2^(lb BIL) | b[0..lb-1] */
 static GEN
 catii(GEN a, long la, GEN b, long lb)
 {
   long l = la + lb + 2;
-  GEN z = cgeti(l);
-  z[1] = evalsigne(1) | evallgefint(l);
-  xmpn_copy(z + 2, a, la);
-  xmpn_copy(z + 2 + la, b, lb);
+  GEN z = cgetipos(l);
+  xmpn_copy(LIMBS(z), a, la);
+  xmpn_copy(LIMBS(z) + la, b, lb);
   return int_normalize(z, 0);
 }
 
@@ -1574,7 +1828,7 @@ sqrtispec2(GEN n, GEN *pr)
   ulong s, r;
   int hi = p_sqrtu2((ulong*)n, &s, &r);
   GEN S = utoi(s);
-  *pr = hi? cat1u(r): utoi(r);
+  *pr = hi? uutoi(1,r): utoi(r);
   return S;
 }
 
@@ -1592,7 +1846,7 @@ sqrtispec1_sh(GEN n, GEN *pr)
    * so 2^(2k) n = (S - s0)^2 + (2*S*s0 - s0^2 + R), s0 = S mod 2^k. */
   if (sh) {
     int k = sh >> 1;
-    ulong s0 = s & ((1<<k) - 1);
+    ulong s0 = s & ((1L<<k) - 1);
     r += s * (s0<<1);
     s >>= k;
     r >>= sh;
@@ -1620,7 +1874,7 @@ sqrtispec2_sh(GEN n, GEN *pr)
    * so 2^(2k) n = (S - s0)^2 + (2*S*s0 - s0^2 + R), s0 = S mod 2^k. */
   if (sh) {
     int k = sh >> 1;
-    ulong s0 = s & ((1<<k) - 1);
+    ulong s0 = s & ((1L<<k) - 1);
     LOCAL_HIREMAINDER;
     LOCAL_OVERFLOW;
     r = addll(r, mulll(s, (s0<<1)));
@@ -1628,10 +1882,10 @@ sqrtispec2_sh(GEN n, GEN *pr)
     hiremainder += hi; /* + 0 or 1 */
     s >>= k;
     r = (r>>sh) | (hiremainder << (BITS_IN_LONG-sh));
-    hi = (hiremainder & (1<<sh));
+    hi = (hiremainder & (1L<<sh));
   }
   S = utoi(s);
-  if (pr) *pr = hi? cat1u(r): utoi(r);
+  if (pr) *pr = hi? uutoi(1,r): utoi(r);
   return S;
 }
 
@@ -1648,16 +1902,16 @@ sqrtispec(GEN N, long n, GEN *r)
   h = n - l; /* N = a3(h) | a2(h) | a1(l) | a0(l words) */
   S = sqrtispec(N, h, &R); /* S^2 + R = a3|a2 */
 
-  z = catii(R+2, lgefint(R)-2, N + 2*h, l); /* = R | a1(l) */
+  z = catii(LIMBS(R), NLIMBS(R), N + 2*h, l); /* = R | a1(l) */
   q = dvmdii(z, shifti(S,1), &u);
-  z = catii(u+2, lgefint(u)-2, N + n + h, l); /* = u | a0(l) */
+  z = catii(LIMBS(u), NLIMBS(u), N + n + h, l); /* = u | a0(l) */
 
   S = addshiftw(S, q, l);
   R = subii(z, sqri(q));
   if (signe(R) < 0)
   {
     GEN S2 = shifti(S,1);
-    R = addis(subiispec(S2+2, R+2, lgefint(S2)-2,lgefint(R)-2), -1);
+    R = addis(subiispec(LIMBS(S2),LIMBS(R), NLIMBS(S2),NLIMBS(R)), -1);
     S = addis(S, -1);
   }
   *r = R; return S;
@@ -1671,7 +1925,7 @@ sqrtremi(GEN N, GEN *r)
 {
   pari_sp av;
   GEN S, R, n = N+2;
-  long k, l2, ln = lgefint(N) - 2;
+  long k, l2, ln = NLIMBS(N);
   int sh;
 
   if (ln <= 2)
@@ -1688,7 +1942,7 @@ sqrtremi(GEN N, GEN *r)
     GEN s0, t = new_chunk(ln + 1);
     t[ln] = 0;
     if (sh)
-    { shift_left(t, n, 0,ln-1, 0, (sh << 1)); }
+      shift_left(t, n, 0,ln-1, 0, sh << 1);
     else
       xmpn_copy(t, n, ln);
     S = sqrtispec(t, l2, &R); /* t normalized, 2 * l2 words */
@@ -1696,7 +1950,7 @@ sqrtremi(GEN N, GEN *r)
      * 2^(2k) n = S^2 + R, k = sh + (ln & 1)*BIL/2
      * so 2^(2k) n = (S - s0)^2 + (2*S*s0 - s0^2 + R), s0 = S mod 2^k. */
     k = sh + (ln & 1) * (BITS_IN_LONG/2);
-    s0 = resmod2n(S, k);
+    s0 = remi2n(S, k);
     R = addii(shifti(R,-1), mulii(s0, S));
     R = shifti(R, 1 - (k<<1));
     S = shifti(S, -k);
@@ -1780,16 +2034,16 @@ sqrtr_abs(GEN x)
   GEN a, t, y = cgetr(l);
   pari_sp av, av0 = avma;
 
-  a = cgetr(l+1); affrr(x,a);
+  a = rtor(x, l+1);
   t = cgetr(l+1);
   if (ex & 1) { /* odd exponent */
-    a[1] = evalsigne(1) | evalexpo(1);
+    a[1] = evalsigne(1) | _evalexpo(1);
     t[2] = (long)sqrtu2((ulong*)a + 2);
   } else { /* even exponent */
-    a[1] = evalsigne(1) | evalexpo(0);
+    a[1] = evalsigne(1) | _evalexpo(0);
     t[2] = (long)sqrtu2_1((ulong*)a + 2);
   }
-  t[1] = evalsigne(1) | evalexpo(0);
+  t[1] = evalsigne(1) | _evalexpo(0);
   for (i = 3; i <= l; i++) t[i] = 0;
 
   /* |x| = 2^(ex/2) a, t ~ sqrt(a) */
@@ -1806,3 +2060,42 @@ sqrtr_abs(GEN x)
 }
 
 #endif
+
+/*******************************************************************
+ *                                                                 *
+ *                           Base Conversion                       *
+ *                                                                 *
+ *******************************************************************/
+
+static void
+convi_dac(GEN x, ulong l, ulong *res)
+{
+  pari_sp ltop=avma;
+  ulong m;
+  GEN x1,x2;
+  if (l==1) { *res=itou(x); return; }
+  m=l>>1;
+  x1=dvmdii(x,powuu(1000000000UL,m),&x2);
+  convi_dac(x1,l-m,res+m);
+  convi_dac(x2,m,res);
+  avma=ltop;
+}
+
+/* convert integer --> base 10^9 [not memory clean] */
+ulong *
+convi(GEN x, long *l)
+{
+  long lz, lx = lgefint(x);
+  ulong *z;
+  if (lx == 3 && (ulong)x[2] < 1000000000UL) {
+    z = (ulong*)new_chunk(1);
+    *z = x[2];
+    *l = 1; return z+1;
+  }
+  lz = 1 + (long)bit_accuracy_mul(lx, LOG10_2/9);
+  z = (ulong*)new_chunk(lz);
+  convi_dac(x,(ulong)lz,z);
+  while (z[lz-1]==0) lz--;
+  *l=lz; return z+lz;
+}
+
