@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp_indep.c"
-/* $Id: mp_indep.c 12097 2010-01-29 11:57:49Z bill $
+/* $Id: mp_indep.c 13219 2011-06-09 12:53:12Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -16,7 +16,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 /* Find c such that 1=c*b mod 2^BITS_IN_LONG, assuming b odd (unchecked) */
 ulong
-invrev(ulong b)
+invmod2BIL(ulong b)
 {
   static int tab[] = { 0, 0, 0, 8, 0, 8, 0, 0 };
   ulong x = b + tab[b & 7]; /* b^(-1) mod 2^4 */
@@ -49,25 +49,22 @@ affrr(GEN x, GEN y)
 }
 
 GEN
-ishiftr(GEN x, long s)
+trunc2nr(GEN x, long n)
 {
-  long ex,lx,n;
+  long ex, lx;
   if (!signe(x)) return gen_0;
-  ex = expo(x) + s; if (ex < 0) return gen_0;
+  ex = expo(x) + n; if (ex < 0) return gen_0;
   lx = lg(x);
-  n=ex - bit_accuracy(lx) + 1;
-  return ishiftr_lg(x, lx, n);
+  return trunc2nr_lg(x, lx, ex - bit_accuracy(lx) + 1);
 }
 
+/* x a t_REAL, x = i/2^e, i a t_INT */
 GEN
-icopy_spec(GEN x, long nx)
+mantissa_real(GEN x, long *e)
 {
-  GEN z;
-  long i;
-  if (!nx) return gen_0;
-  z = cgeti(nx+2); z[1] = evalsigne(1)|evallgefint(nx+2);
-  for (i=0; i<nx; i++) z[i+2] = x[i];
-  return z;
+  long lx = lg(x);
+  *e = bit_accuracy(lx)-1-expo(x);
+  return trunc2nr_lg(x, lx, 0);
 }
 
 GEN
@@ -93,6 +90,51 @@ mulsi(long x, GEN y)
   setsigne(z,s); return z;
 }
 
+GEN
+mulss(long x, long y)
+{
+  long p1;
+  LOCAL_HIREMAINDER;
+
+  if (!x || !y) return gen_0;
+  if (x<0) {
+    x = -x;
+    if (y<0) { y = -y; p1 = mulll(x,y); return uutoi(hiremainder, p1); }
+    p1 = mulll(x,y); return uutoineg(hiremainder, p1);
+  } else {
+    if (y<0) { y = -y; p1 = mulll(x,y); return uutoineg(hiremainder, p1); }
+    p1 = mulll(x,y); return uutoi(hiremainder, p1);
+  }
+}
+GEN
+sqrs(long x)
+{
+  long p1;
+  LOCAL_HIREMAINDER;
+
+  if (!x) return gen_0;
+  if (x<0) x = -x;
+  p1 = mulll(x,x); return uutoi(hiremainder, p1);
+}
+GEN
+muluu(ulong x, ulong y)
+{
+  long p1;
+  LOCAL_HIREMAINDER;
+
+  if (!x || !y) return gen_0;
+  p1 = mulll(x,y); return uutoi(hiremainder, p1);
+}
+GEN
+sqru(ulong x)
+{
+  long p1;
+  LOCAL_HIREMAINDER;
+
+  if (!x) return gen_0;
+  p1 = mulll(x,x); return uutoi(hiremainder, p1);
+}
+
 /* assume x > 1, y != 0. Return u * y with sign s */
 static GEN
 mulur_2(ulong x, GEN y, long s)
@@ -107,22 +149,35 @@ mulur_2(ulong x, GEN y, long s)
   z[2]=hiremainder; /* != 0 since y normalized and |x| > 1 */
 
   sh = bfffo(hiremainder); m = BITS_IN_LONG-sh;
-  if (sh) shift_left2(z,z, 2,lx-1, garde,sh,m);
+  if (sh) shift_left(z,z, 2,lx-1, garde,sh);
   z[1] = evalsigne(s) | evalexpo(m+e); return z;
 }
 
+INLINE GEN
+mul0r(GEN x)
+{
+  long l = lg(x), e = expo(x);
+  e = (l > 2)? -bit_accuracy(l) + e: (e < 0? (e<<1): 0);
+  return real_0_bit(e);
+}
+/* lg(x) > 2 */
+INLINE GEN
+div0r(GEN x) {
+  long l = lg(x), e = expo(x);
+  return real_0_bit(-bit_accuracy(l) - e);
+}
+
 GEN
 mulsr(long x, GEN y)
 {
-  long s, e;
+  long s;
 
-  if (!x) return gen_0;
+  if (!x) return mul0r(y);
   s = signe(y);
   if (!s)
   {
     if (x < 0) x = -x;
-    e = expo(y) + (BITS_IN_LONG-1)-bfffo(x);
-    return real_0_bit(e);
+    return real_0_bit( expo(y) + expu(x) );
   }
   if (x==1)  return rcopy(y);
   if (x==-1) return negr(y);
@@ -135,15 +190,11 @@ mulsr(long x, GEN y)
 GEN
 mulur(ulong x, GEN y)
 {
-  long s, e;
+  long s;
 
-  if (!x) return gen_0;
+  if (!x) return mul0r(y);
   s = signe(y);
-  if (!s)
-  {
-    e = expo(y) + (BITS_IN_LONG-1)-bfffo(x);
-    return real_0_bit(e);
-  }
+  if (!s) return real_0_bit( expo(y) + expu(x) );
   if (x==1) return rcopy(y);
   return mulur_2(x, y, s);
 }
@@ -184,11 +235,11 @@ mulrrz_i(GEN z, GEN x, GEN y, long lz, long flag, long sz)
   LOCAL_HIREMAINDER;
   LOCAL_OVERFLOW;
 
-  if (lz > KARATSUBA_MULR_LIMIT) 
+  if (lz > MULRR_MULII_LIMIT)
   {
     pari_sp av = avma;
 #ifdef KARAMULR_VARIANT
-    GEN hi = karamulrr1(y+2, x+2, lz+flag-2, lz-2); 
+    GEN hi = karamulrr1(y+2, x+2, lz+flag-2, lz-2);
 #else
     GEN hi = muliispec_mirror(y+2, x+2, lz+flag-2, lz-2);
 #endif
@@ -205,19 +256,10 @@ mulrrz_i(GEN z, GEN x, GEN y, long lz, long flag, long sz)
     }
     if (garde & HIGHBIT)
     { /* round to nearest */
-      i = lz; do z[--i]++; while (z[i]==0 && i > 1);
-      if (i == 1) { z[2] = HIGHBIT; ez++; }
+      i = lz; do z[--i]++; while (i>1 && z[i]==0);
+      if (i == 1) { z[2] = (long)HIGHBIT; ez++; }
     }
     z[1] = evalsigne(sz)|evalexpo(ez);
-#if 0
-{
-GEN U;
-KARATSUBA_MULR_LIMIT = 100000;
-U = mulrr(x, y);
-KARATSUBA_MULR_LIMIT = 4;
-if (!gequal(U, z)) pari_err(talker,"toto");
-}
-#endif
     avma = av; return;
   }
   if (lz == 3)
@@ -267,7 +309,7 @@ if (!gequal(U, z)) pari_err(talker,"toto");
       for (i=lzz; i>j; i--)
       {
         hiremainder += overflow;
-	z[i] = addll(addmul(p1,y1[i]), z[i]);
+        z[i] = addll(addmul(p1,y1[i]), z[i]);
       }
       z[j] = hiremainder+overflow;
     }
@@ -291,7 +333,7 @@ if (!gequal(U, z)) pari_err(talker,"toto");
   }
   if (garde & HIGHBIT)
   { /* round to nearest */
-    i = lz; do z[--i]++; while (z[i]==0 && i > 2);
+    i = lz; do z[--i]++; while (i>2 && z[i]==0);
     if (z[i] == 0) { z[2] = (long)HIGHBIT; ez++; }
   }
   z[1] = evalsigne(sz) | evalexpo(ez);
@@ -313,14 +355,28 @@ mulrr(GEN x, GEN y)
   return z;
 }
 
+GEN
+sqrr(GEN x)
+{
+  long lz, sx = signe(x);
+  GEN z;
+
+  if (!sx) return real_0_bit(2*expo(x));
+  lz = lg(x); z = cgetr(lz);
+  mulrrz_i(z, x,x, lz,0, 1);
+  return z;
+}
+
 GEN
 mulir(GEN x, GEN y)
 {
   long sx = signe(x), sy, lz;
   GEN z;
 
-  if (!sx) return gen_0;
-  if (!is_bigint(x)) return mulsr(itos(x), y);
+  if (!sx) return mul0r(y);
+  if (lgefint(x) == 3) {
+    z = mulur((ulong)x[2], y); if (sx < 0) togglesign(z); return z;
+  }
   sy = signe(y);
   if (!sy) return real_0_bit(expi(x) + expo(y));
   if (sy < 0) sx = -sx;
@@ -371,46 +427,103 @@ GEN
 divir(GEN x, GEN y)
 {
   GEN z;
-  long ly;
+  long ly = lg(y), lx = lgefint(x);
   pari_sp av;
 
-  if (!signe(y)) pari_err(gdiver);
-  if (!signe(x)) return gen_0;
-  ly = lg(y); z = cgetr(ly); av = avma; 
+  if (ly == 2) pari_err(gdiver);
+  if (lx == 2) return div0r(y);
+  if (lx == 3) {
+    z = divur(x[2], y);
+    if (signe(x) < 0) togglesign(z);
+    return z;
+  }
+  z = cgetr(ly); av = avma;
   affrr(divrr(itor(x, ly+1), y), z);
   avma = av; return z;
 }
 
-void
-mpdivz(GEN x, GEN y, GEN z)
+GEN
+divur(ulong x, GEN y)
 {
-  pari_sp av = avma;
-  long tx = typ(x), ty = typ(y);
-  GEN q = tx==t_INT && ty==t_INT? divii(x,y): mpdiv(x,y);
+  pari_sp av;
+  long ly = lg(y);
+  GEN z;
 
-  if (typ(z) == t_REAL) affrr(q, z);
-  else
-  {
-    if (typ(q) == t_REAL) pari_err(gdiver);
-    affii(q, z); 
+  if (ly == 2) pari_err(gdiver);
+  if (!x) return div0r(y);
+  if (ly > INVNEWTON_LIMIT) {
+    av = avma; z = invr(y);
+    if (x == 1) return z;
+    return gerepileuptoleaf(av, mulur(x, z));
   }
-  avma = av;
+  z = cgetr(ly); av = avma;
+  affrr(divrr(utor(x,ly+1), y), z);
+  avma = av; return z;
 }
 
 GEN
 divsr(long x, GEN y)
 {
   pari_sp av;
-  long ly;
+  long ly = lg(y);
   GEN z;
 
-  if (!signe(y)) pari_err(gdiver);
-  if (!x) return gen_0;
-  ly = lg(y); z = cgetr(ly); av = avma;
+  if (ly == 2) pari_err(gdiver);
+  if (!x) return div0r(y);
+  if (ly > INVNEWTON_LIMIT) {
+    av = avma; z = invr(y);
+    if (x == 1) return z;
+    if (x ==-1) { togglesign(z); return z; }
+    return gerepileuptoleaf(av, mulsr(x, z));
+  }
+  z = cgetr(ly); av = avma;
   affrr(divrr(stor(x,ly+1), y), z);
   avma = av; return z;
 }
 
+/* returns 1/y, assume y != 0 */
+static GEN
+invr_basecase(GEN y)
+{
+  long ly = lg(y);
+  GEN z = cgetr(ly);
+  pari_sp av = avma;
+  affrr(divrr(real_1(ly+1), y), z);
+  avma = av; return z;
+}
+/* returns 1/b, Newton iteration */
+GEN
+invr(GEN b)
+{
+  const long s = 6;
+  long i, p, l = lg(b);
+  GEN x, a;
+  ulong mask;
+
+  if (l <= maxss(INVNEWTON_LIMIT, (1L<<s) + 2)) {
+    if (l == 2) pari_err(gdiver);
+    return invr_basecase(b);
+  }
+  mask = quadratic_prec_mask(l-2);
+  for(i=0, p=1; i<s; i++) { p <<= 1; if (mask & 1) p--; mask >>= 1; }
+  x = cgetr(l);
+  a = rcopy(b); a[1] = _evalexpo(0) | evalsigne(1);
+  affrr(invr_basecase(rtor(a, p+2)), x);
+  while (mask > 1)
+  {
+    p <<= 1; if (mask & 1) p--;
+    mask >>= 1;
+    setlg(a, p + 2);
+    setlg(x, p + 2);
+    /* TODO: mulrr(a,x) should be a half product (the higher half is known).
+     * mulrr(x, ) already is */
+    affrr(addrr(x, mulrr(x, subsr(1, mulrr(a,x)))), x);
+    avma = (pari_sp)a;
+  }
+  x[1] = (b[1] & SIGNBITS) | evalexpo(expo(x)-expo(b));
+  avma = (pari_sp)x; return x;
+}
+
 GEN
 modii(GEN x, GEN y)
 {
@@ -439,14 +552,38 @@ modiiz(GEN x, GEN y, GEN z)
 GEN
 divrs(GEN x, long y)
 {
-  long i,lx,garde,sh,s=signe(x);
+  long i, lx, garde, sh, s = signe(x);
   GEN z;
   LOCAL_HIREMAINDER;
 
   if (!y) pari_err(gdiver);
-  if (!s) return real_0_bit(expo(x) - (BITS_IN_LONG-1)+bfffo(y));
   if (y<0) { s = -s; y = -y; }
-  if (y==1) { z=rcopy(x); setsigne(z,s); return z; }
+  if (!s) return real_0_bit(expo(x) - expu(y));
+  if (y==1) { z = rcopy(x); setsigne(z,s); return z; }
+  if (y==2) { z = shiftr(x, -1); setsigne(z,s); return z; }
+
+  z=cgetr(lx=lg(x)); hiremainder=0;
+  for (i=2; i<lx; i++) z[i] = divll(x[i],y);
+
+  /* we may have hiremainder != 0 ==> garde */
+  garde=divll(0,y); sh=bfffo(z[2]);
+  if (sh) shift_left(z,z, 2,lx-1, garde,sh);
+  z[1] = evalsigne(s) | evalexpo(expo(x)-sh);
+  if ((garde << sh) & HIGHBIT) roundr_up_ip(z, lx);
+  return z;
+}
+
+GEN
+divru(GEN x, ulong y)
+{
+  long i, lx, garde, sh, s = signe(x);
+  GEN z;
+  LOCAL_HIREMAINDER;
+
+  if (!y) pari_err(gdiver);
+  if (!s) return real_0_bit(expo(x) - expu(y));
+  if (y==1) return rcopy(x);
+  if (y==2) return shiftr(x, -1);
 
   z=cgetr(lx=lg(x)); hiremainder=0;
   for (i=2; i<lx; i++) z[i] = divll(x[i],y);
@@ -469,11 +606,17 @@ truedvmdii(GEN x, GEN y, GEN *z)
   av = avma;
   q = dvmdii(x,y,&r); /* assume that r is last on stack */
 
-  if (signe(r)>=0)
+  switch(signe(r))
   {
-    if (z == ONLY_REM) return gerepileuptoint(av,r);
-    if (z) *z = r; else cgiv(r);
-    return q;
+    case 0:
+      if (z == ONLY_REM) { avma = av; return gen_0; }
+      if (z) *z = gen_0;
+      return q;
+    case 1:
+      if (z == ONLY_REM) return gerepileuptoint(av,r);
+      if (z) *z = r; else cgiv(r);
+      return q;
+    case -1: break;
   }
 
   if (z == ONLY_REM)
@@ -506,19 +649,35 @@ truedvmdis(GEN x, long y, GEN *z)
   if (z) *z = utoi(r + labs(y));
   return q;
 }
+GEN
+truedvmdsi(long x, GEN y, GEN *z)
+{
+  long q, r;
+  q = sdivsi_rem(x,y,&r);
+  if (r >= 0) {
+    if (z == ONLY_REM) return utoi(r);
+    if (z) *z = utoi(r);
+    return stoi(q);
+  }
+  if (z == ONLY_REM) /* r += sign(y) * y, that is |y| */
+    return subiuspec(y+2,(ulong)-r, lgefint(y)-2);
+  q = q - signe(y);
+  if (!z) return stoi(q);
+
+  *z = subiuspec(y+2,(ulong)-r, lgefint(y)-2);
+  return stoi(q);
+}
 
 /* 2^n = shifti(gen_1, n) */
 GEN
 int2n(long n) {
-  long i, m, d, l;
+  long i, m, l;
   GEN z;
   if (n < 0) return gen_0;
   if (n == 0) return gen_1;
 
-  d = n>>TWOPOTBITS_IN_LONG;
-  m = n & (BITS_IN_LONG-1);
-  l = d + 3; z = cgeti(l);
-  z[1] = evalsigne(1) | evallgefint(l);
+  l = dvmdsBIL(n, &m) + 3;
+  z = cgetipos(l);
   for (i = 2; i < l; i++) z[i] = 0;
   *int_MSW(z) = 1L << m; return z;
 }
@@ -526,14 +685,12 @@ int2n(long n) {
  * returns gen_0 */
 GEN
 int2u(ulong n) {
-  ulong i, m, d, l;
+  ulong i, m, l;
   GEN z;
   if (n == 0) return gen_1;
 
-  d = n>>TWOPOTBITS_IN_LONG;
-  m = n & (BITS_IN_LONG-1);
-  l = d + 3; z = cgeti(l);
-  z[1] = evalsigne(1) | evallgefint(l);
+  l = dvmduBIL(n, &m) + 3;
+  z = cgetipos(l);
   for (i = 2; i < l; i++) z[i] = 0;
   *int_MSW(z) = 1L << m; return z;
 }
@@ -552,52 +709,9 @@ mulii(GEN a,GEN b)
   setsigne(z,sa); return z;
 }
 
-GEN
-remiimul(GEN x, GEN sy)
-{
-  GEN r, q, y = (GEN)sy[1], invy;
-  long k;
-  pari_sp av = avma;
-
-  k = cmpii(x, y);
-  if (k <= 0) return k? icopy(x): gen_0;
-  invy = (GEN)sy[2];
-  q = mulir(x,invy);
-  q = truncr(q); /* differs from divii(x, y) at most by 1 */
-  r = subii(x, mulii(y,q));
-  if (signe(r) < 0)
-    r = subiispec(y+2,r+2, lgefint(y)-2, lgefint(r)-2); /* y - |r| */
-  else
-  {
-    /* remii(x,y) + y >= r >= remii(x,y) */
-    k = absi_cmp(r, y);
-    if (k >= 0)
-    {
-      if (k == 0) { avma = av; return gen_0; }
-      r = subiispec(r+2, y+2, lgefint(r)-2, lgefint(y)-2);
-    }
-  }
-#if 0
-  q = subii(r,remii(x,y));
-  if (signe(q))
-    pari_err(talker,"bug in remiimul: x = %Z\ny = %Z\ndif = %Z", x,y,q);
-#endif
-  return gerepileuptoint(av, r); /* = remii(x, y) */
-}
-
 GEN
 sqri(GEN a) { return sqrispec(a+2, lgefint(a)-2); }
 
-/* Old cgiv without reference count (which was not used anyway)
- * Should be a macro.
- */
-void
-cgiv(GEN x)
-{
-  if (x == (GEN) avma)
-    avma = (pari_sp) (x+lg(x));
-}
-
 /* sqrt()'s result may be off by 1 when a is not representable exactly as a
  * double [64bit machine] */
 ulong
@@ -610,101 +724,6 @@ usqrtsafe(ulong a)
   return x;
 }
 
-/********************************************************************/
-/**                                                                **/
-/**                         RANDOM INTEGERS                        **/
-/**                                                                **/
-/********************************************************************/
-static long pari_randseed = 1;
-
-/* BSD rand gives this: seed = 1103515245*seed + 12345 */
-/*Return 31 ``random'' bits.*/
-long
-pari_rand31(void)
-{
-  pari_randseed = (1000276549*pari_randseed + 12347) & 0x7fffffff;
-  return pari_randseed;
-}
-
-long
-setrand(long seed) { return (pari_randseed = seed); }
-
-long
-getrand(void) { return pari_randseed; }
-
-static ulong
-pari_rand(void)
-{
-#define GLUE2(hi, lo) (((hi) << BITS_IN_HALFULONG) | (lo))
-#if !defined(LONG_IS_64BIT)
-  return GLUE2((pari_rand31()>>12)&LOWMASK,
-               (pari_rand31()>>12)&LOWMASK);
-#else
-#define GLUE4(hi1,hi2, lo1,lo2) GLUE2(((hi1)<<16)|(hi2), ((lo1)<<16)|(lo2))
-#  define LOWMASK2 0xffffUL
-  return GLUE4((pari_rand31()>>12)&LOWMASK2,
-               (pari_rand31()>>12)&LOWMASK2,
-               (pari_rand31()>>12)&LOWMASK2,
-               (pari_rand31()>>12)&LOWMASK2);
-#endif
-}
-
-#if 1
-/* assume N > 0 */
-GEN
-randomi(GEN N)
-{
-  ulong n;
-  long lx, i;
-  GEN x, xMSW, xd, Nd;
-
-  lx = lgefint(N); x = cgeti(lx);
-  x[1] = evalsigne(1) | evallgefint(lx);
-  xMSW = xd = int_MSW(x);
-  for (i=2; i<lx; i++) { *xd = pari_rand(); xd = int_precW(xd); }
-
-  Nd = int_MSW(N); n = (ulong)*Nd;
-  xd = xMSW;
-  if (lx == 3) n--;
-  else
-    for (i=3; i<lx; i++)
-    {
-      xd = int_precW(xd);
-      Nd = int_precW(Nd);
-      if (*xd != *Nd) {
-        if ((ulong)*xd > (ulong)*Nd) n--;
-        break;
-      }
-    }
-  /* MSW needs to be generated between 0 and n */
-  if (n) {
-    LOCAL_HIREMAINDER;
-    (void)mulll((ulong)*xMSW, n + 1);
-    n = hiremainder;
-  }
-  *xMSW = (long)n;
-  if (!n) x = int_normalize(x, 1);
-  return x;
-}
-#else
-/* assume N > 0 */
-GEN
-randomi(GEN N)
-{
-  long i, lx = lgefint(N), shift = bfffo(*int_MSW(N));
-  GEN x = cgeti(lx), xMSW, xd;
-
-  x[1] = evalsigne(1) | evallgefint(lx);
-  xMSW = int_MSW(x);
-  for (xd = xMSW;;) {
-    for (i=2; i<lx; i++) { *xd = pari_rand(); xd = int_precW(xd); }
-    *xMSW = ((ulong)*xMSW) >> shift;
-    x = int_normalize(x, 0);
-    if (absi_cmp(x, N) < 0) return x;
-  }
-}
-#endif
-
 /********************************************************************/
 /**                                                                **/
 /**              EXPONENT / CONVERSION t_REAL --> double           **/
@@ -761,7 +780,7 @@ dbltor(double x)
     }
     else
       z[2] = HIGHBIT | A;
-    z[1] = evalexpo(e) | evalsigne(x<0? -1: 1);
+    z[1] = _evalexpo(e) | evalsigne(x<0? -1: 1);
   }
   return z;
 }
@@ -783,7 +802,7 @@ rtodbl(GEN x)
   /* start by rounding to closest */
   a = (x[2] & (HIGHBIT-1)) + 0x400;
   if (a & HIGHBIT) { ex++; a=0; }
-  if (ex >= exp_mid) pari_err(rtodber);
+  if (ex >= exp_mid) pari_err(overflower,"t_REAL->double conversion");
   fi.i = ((ex + exp_mid) << mant_len) | (a >> expo_len);
   if (s<0) fi.i |= HIGHBIT;
   return fi.f;
@@ -861,7 +880,7 @@ dbltor(double x)
         z[2] = (A << sh) | (B >> (32-sh));
         z[3] = B << sh;
       }
-      else 
+      else
       {
         sh = bfffo(B); /* B != 0 */
         e -= sh-1 + 32;
@@ -874,7 +893,7 @@ dbltor(double x)
       z[3] = B;
       z[2] = HIGHBIT | A;
     }
-    z[1] = evalexpo(e) | evalsigne(x<0? -1: 1);
+    z[1] = _evalexpo(e) | evalsigne(x<0? -1: 1);
   }
   return z;
 }
@@ -902,7 +921,7 @@ rtodbl(GEN x)
     if (a & HIGHBIT) { ex++; a=0; }
   }
   else b = 0;
-  if (ex >= exp_mid) pari_err(rtodber);
+  if (ex >= exp_mid) pari_err(overflower,"t_REAL->double conversion");
   ex += exp_mid;
   k = (a >> expo_len) | (ex << shift);
   if (s<0) k |= HIGHBIT;
