@@ -1,4 +1,4 @@
-/* $Id: asm0.h 7881 2006-04-18 17:23:00Z kb $ 
+/* $Id: asm0.h 12466 2010-06-24 14:53:41Z kb $
 Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
@@ -16,7 +16,7 @@ ASM divll
 */
 #ifdef ASMINLINE
 #define divll(a,b) \
-({ ulong __value, __arg1 = (a), __arg2 = (b), __tmp; \
+__extension__ ({ ulong __value, __arg1 = (a), __arg2 = (b), __tmp; \
   __asm__( "mov %1, %%y; nop;nop;nop;\n\t\
 udivcc  %3,%4,%0;\n\tumul    %0,%4,%2;\n\tsub     %3,%2,%1"\
         : "=&r" (__value), "=&r" (hiremainder), "=&r" (__tmp) \
