@@ -1,4 +1,4 @@
-/* $Id: default.c 7895 2006-04-19 20:34:12Z bill $
+/* $Id: default.c 13119 2011-04-16 21:17:42Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -14,11 +14,6 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "pari.h"
 #include "paripriv.h"
-#include "anal.h"
-
-#ifdef HAS_STRFTIME
-#  include <time.h>
-#endif
 
 /* Simple-minded parsing utilities. These are forbidden to use the GP stack
  * which may not exist at this point [e.g upon GP initialization]  */
@@ -43,12 +38,15 @@ get_sep(const char *t)
     switch(*s++ = *t++)
     {
       case '"':
-        if (outer || (s >= buf+2 && s[-2] != '\\')) outer = !outer;
-        break;
+        outer = !outer; break;
       case '\0':
         return buf;
       case ';':
-	if (outer) { s[-1] = 0; return buf; } break;
+        if (outer) { s[-1] = 0; return buf; }
+        break;
+      case '\\': /* gobble next char */
+        if (s == lim) break;
+        if (! (*s++ = *t++) ) return buf;
     }
     if (s == lim)
       pari_err(talker,"get_sep: argument too long (< %ld chars)", GET_SEP_SIZE);
@@ -71,12 +69,12 @@ my_int(char *s)
   ulong n = 0;
   char *p = s;
 
-  while (isdigit((int)*p)) { 
+  while (isdigit((int)*p)) {
     ulong m;
-    if (n > (~0UL / 10)) pari_err(talker2,"integer too large",s,s);
+    if (n > (~0UL / 10)) pari_err(syntaxer,"integer too large",s,s);
     n *= 10; m = n;
     n += *p++ - '0';
-    if (n < m) pari_err(talker2,"integer too large",s,s);
+    if (n < m) pari_err(syntaxer,"integer too large",s,s);
   }
   if (n)
   {
@@ -86,9 +84,9 @@ my_int(char *s)
       case 'm': case 'M': n = safe_mul(n,1000000UL);    p++; break;
       case 'g': case 'G': n = safe_mul(n,1000000000UL); p++; break;
     }
-    if (!n) pari_err(talker2,"integer too large",s,s);
+    if (!n) pari_err(syntaxer,"integer too large",s,s);
   }
-  if (*p) pari_err(talker2,"I was expecting an integer here", s, s);
+  if (*p) pari_err(syntaxer,"I was expecting an integer here", s, s);
   return n;
 }
 
@@ -98,12 +96,12 @@ get_int(const char *s, long dflt)
   char *p = get_sep(s);
   long n;
   int minus = 0;
-  
+
   if (*p == '-') { minus = 1; p++; }
   if (!isdigit((int)*p)) return dflt;
 
   n = (long)my_int(p);
-  if (n < 0) pari_err(talker2,"integer too large",s,s);
+  if (n < 0) pari_err(syntaxer,"integer too large",s,s);
   return minus? -n: n;
 }
 
@@ -111,7 +109,7 @@ ulong
 get_uint(const char *s)
 {
   char *p = get_sep(s);
-  if (*p == '-') pari_err(talker2,"arguments must be positive integers",s,s);
+  if (*p == '-') pari_err(syntaxer,"arguments must be positive integers",s,s);
   return my_int(p);
 }
 
@@ -120,67 +118,6 @@ get_uint(const char *s)
 /*                            DEFAULTS                              */
 /*                                                                  */
 /********************************************************************/
-static void
-init_hist(gp_data *D, size_t l, ulong total)
-{
-  gp_hist *H = D->hist;
-  H->total = total;
-  H->size = l;
-  H->res = (GEN *) gpmalloc(l * sizeof(GEN));
-  memset(H->res,0, l * sizeof(GEN));
-}
-
-static void
-init_path(gp_data *D)
-{
-  gp_path *path = D->path;
-  path->PATH = pari_strdup(pari_default_path());
-  path->dirs = NULL;
-}
-
-static void
-init_help(gp_data *D)
-{
-  char *h = os_getenv("GPHELP");
-# ifdef GPHELP
-  if (!h) h = GPHELP;
-# endif
-  if (h) h = pari_strdup(h);
-  D->help = h;
-}
-
-static void
-init_fmt(gp_data *D)
-{
-#ifdef LONG_IS_64BIT
-  static pariout_t DFLT_OUTPUT = { 'g', 0, 38, 1, f_PRETTYMAT, 0 };
-#else
-  static pariout_t DFLT_OUTPUT = { 'g', 0, 28, 1, f_PRETTYMAT, 0 };
-#endif
-  D->fmt = &DFLT_OUTPUT;
-}
-
-static char *DFT_PRETTYPRINTER = "tex2mail -TeX -noindent -ragged -by_par";
-static void
-init_pp(gp_data *D)
-{
-  gp_pp *p = D->pp;
-  p->cmd = pari_strdup(DFT_PRETTYPRINTER);
-  p->file = NULL;
-}
-
-static void
-do_strftime(const char *s, char *buf, long max)
-{
-#ifdef HAS_STRFTIME
-  time_t t = time(NULL);
-  (void)strftime(buf,max,s,localtime(&t));
-#else
-  strcpy(buf,s);
-#endif
-}
-
-/**************************************************************************/
 
 long
 getrealprecision(void)
@@ -191,16 +128,16 @@ getrealprecision(void)
 long
 setrealprecision(long n, long *prec)
 {
-  GP_DATA->fmt->sigd = n; 
+  GP_DATA->fmt->sigd = n;
   *prec = precreal = ndec2prec(n);
   return n;
 }
 
-static GEN
-sd_toggle(const char *v, long flag, char *s, int *ptn)
+GEN
+sd_toggle(const char *v, long flag, const char *s, int *ptn)
 {
   int state = *ptn;
-  if (*v)
+  if (v)
   {
     int n = (int)get_int(v,0);
     if (n == state) return gnil;
@@ -208,7 +145,7 @@ sd_toggle(const char *v, long flag, char *s, int *ptn)
     {
       char *t = stackmalloc(64 + strlen(s));
       (void)sprintf(t, "default: incorrect value for %s [0:off / 1:on]", s);
-      pari_err(talker2, t, v,v);
+      pari_err(syntaxer, t, v,v);
     }
     state = *ptn = n;
   }
@@ -216,30 +153,17 @@ sd_toggle(const char *v, long flag, char *s, int *ptn)
   {
     case d_RETURN: return utoi(state);
     case d_ACKNOWLEDGE:
-      if (state) pariprintf("   %s = 1 (on)\n", s);
-      else       pariprintf("   %s = 0 (off)\n", s);
+      if (state) pari_printf("   %s = 1 (on)\n", s);
+      else       pari_printf("   %s = 0 (off)\n", s);
       break;
   }
   return gnil;
 }
 
-static GEN
-sd_gptoggle(const char *v, long flag, char *s, ulong FLAG)
-{
-  int n = (GP_DATA->flags & FLAG)? 1: 0, old = n;
-  GEN z = sd_toggle(v, flag, s, &n);
-  if (n != old)
-  {
-    if (n) GP_DATA->flags |=  FLAG;
-    else   GP_DATA->flags &= ~FLAG;
-  }
-  return z;
-}
-
 static void
-sd_ulong_init(const char *v, char *s, ulong *ptn, ulong Min, ulong Max)
+sd_ulong_init(const char *v, const char *s, ulong *ptn, ulong Min, ulong Max)
 {
-  if (*v)
+  if (v)
   {
     ulong n = get_uint(v);
     if (n > Max || n < Min)
@@ -247,15 +171,15 @@ sd_ulong_init(const char *v, char *s, ulong *ptn, ulong Min, ulong Max)
       char *buf = stackmalloc(strlen(s) + 2 * 20 + 40);
       (void)sprintf(buf, "default: incorrect value for %s [%lu-%lu]",
                     s, Min, Max);
-      pari_err(talker2, buf, v,v);
+      pari_err(syntaxer, buf, v,v);
     }
     *ptn = n;
   }
 }
 
-static GEN
-sd_ulong(const char *v, long flag, char *s, ulong *ptn, ulong Min, ulong Max,
-           char **msg)
+GEN
+sd_ulong(const char *v, long flag, const char *s, ulong *ptn, ulong Min, ulong Max,
+         const char **msg)
 {
   ulong n = *ptn;
   sd_ulong_init(v, s, ptn, Min, Max);
@@ -264,15 +188,20 @@ sd_ulong(const char *v, long flag, char *s, ulong *ptn, ulong Min, ulong Max,
     case d_RETURN:
       return utoi(*ptn);
     case d_ACKNOWLEDGE:
-      if (!*v || *ptn != n) {
+      if (!v || *ptn != n) {
         if (msg)
         {
-          if (!*msg) msg++; /* single msg, always printed */
-          else       msg += *ptn; /* one per possible value */
-          pariprintf("   %s = %lu %s\n", s, *ptn, *msg);
+          ulong i;
+          for (i = 0; i < *ptn; i++) {
+            if (!msg[i]) { i--; break; }
+          }
+          if (i != (ulong)-1) {
+            if (i == *ptn) i = 0;
+            pari_printf("   %s = %lu %s\n", s, *ptn, msg[i]);
+            return gnil;
+          }
         }
-        else
-          pariprintf("   %s = %lu\n", s, *ptn);
+        pari_printf("   %s = %lu\n", s, *ptn);
       }
       break;
   }
@@ -283,22 +212,26 @@ GEN
 sd_realprecision(const char *v, long flag)
 {
   pariout_t *fmt = GP_DATA->fmt;
-  if (*v)
+  if (v)
   {
-    ulong newnb = fmt->sigd;
+    ulong newnb = fmt->sigd, prec;
     sd_ulong_init(v, "realprecision", &newnb, 1, prec2ndec(LGBITS));
     if (fmt->sigd == (long)newnb) return gnil;
-    fmt->sigd = newnb;
-    precreal = (ulong)ndec2prec(newnb);
+    if (fmt->sigd >= 0) fmt->sigd = newnb;
+    prec = (ulong)ndec2prec(newnb);
+    if (prec == precreal) return gnil;
+    precreal = prec;
   }
   if (flag == d_RETURN) return stoi(fmt->sigd);
   if (flag == d_ACKNOWLEDGE)
   {
     long n = prec2ndec(precreal);
-    pariprintf("   realprecision = %ld significant digits", n);
-    if (n != fmt->sigd)
-      pariprintf(" (%ld digits displayed)", fmt->sigd);
-    pariputc('\n');
+    pari_printf("   realprecision = %ld significant digits", n);
+    if (fmt->sigd < 0)
+      pari_puts(" (all digits displayed)");
+    else if (n != fmt->sigd)
+      pari_printf(" (%ld digits displayed)", fmt->sigd);
+    pari_putc('\n');
   }
   return gnil;
 }
@@ -306,41 +239,10 @@ sd_realprecision(const char *v, long flag)
 GEN
 sd_seriesprecision(const char *v, long flag)
 {
-  char *msg[] = {NULL, "significant terms"};
+  const char *msg[] = {"significant terms", NULL};
   return sd_ulong(v,flag,"seriesprecision",&precdl, 1,LGBITS,msg);
 }
 
-GEN
-sd_format(const char *v, long flag)
-{
-  pariout_t *fmt = GP_DATA->fmt;
-  if (*v)
-  {
-    char c = *v;
-    if (c!='e' && c!='f' && c!='g')
-      pari_err(talker2,"default: inexistent format",v,v);
-    fmt->format = c; v++;
-
-    if (isdigit((int)*v))
-      { fmt->fieldw=atol(v); while (isdigit((int)*v)) v++; }
-    if (*v++ == '.')
-    {
-      if (*v == '-') fmt->sigd = -1;
-      else
-	if (isdigit((int)*v)) fmt->sigd=atol(v);
-    }
-  }
-  if (flag == d_RETURN)
-  {
-    char *s = stackmalloc(64);
-    (void)sprintf(s, "%c%ld.%ld", fmt->format, fmt->fieldw, fmt->sigd);
-    return strtoGENstr(s);
-  }
-  if (flag == d_ACKNOWLEDGE)
-    pariprintf("   format = %c%ld.%ld\n", fmt->format, fmt->fieldw, fmt->sigd);
-  return gnil;
-}
-
 static long
 gp_get_color(char **st)
 {
@@ -353,11 +255,11 @@ gp_get_color(char **st)
   {
     if (*v == '[')
     {
-      char *a[3];
+      const char *a[3];
       long i = 0;
       for (a[0] = s = ++v; *s && *s != ']'; s++)
         if (*s == ',') { *s = 0; a[++i] = s+1; }
-      if (*s != ']') pari_err(talker2,"expected character: ']'",s, *st);
+      if (*s != ']') pari_err(syntaxer,"expected character: ']'",s, *st);
       *s = 0; for (i++; i<3; i++) a[i] = "";
       /*    properties    |   color    | background */
       c = (atoi(a[2])<<8) | atoi(a[0]) | (atoi(a[1])<<4);
@@ -366,7 +268,7 @@ gp_get_color(char **st)
     }
     else { c = c_NONE; trans = 0; }
   }
-  if (trans) c = c | (1<<12);
+  if (trans) c = c | (1L<<12);
   while (*v && *v++ != ',') /* empty */;
   if (c != c_NONE) disable_color = 0;
   *st = v; return c;
@@ -374,12 +276,12 @@ gp_get_color(char **st)
 
 /* 1: error, 2: history, 3: prompt, 4: input, 5: output, 6: help, 7: timer */
 GEN
-sd_colors(char *v, long flag)
+sd_colors(const char *v, long flag)
 {
   long c,l;
-  if (*v && !(GP_DATA->flags & (EMACS|TEXMACS)))
+  if (v && !(GP_DATA->flags & (gpd_EMACS|gpd_TEXMACS)))
   {
-    char *v0;
+    char *v0, *s;
     disable_color=1;
     l = strlen(v);
     if (l <= 2 && strncmp(v, "no", l) == 0)
@@ -388,12 +290,12 @@ sd_colors(char *v, long flag)
       v = "1, 5, 3, 7, 6, 2, 3"; /* Assume recent ReadLine. */
     if (l <= 7 && strncmp(v, "lightbg", l) == 0)
       v = "1, 6, 3, 4, 5, 2, 3"; /* Assume recent ReadLine. */
-    if (l <= 6 && strncmp(v, "boldfg", l) == 0)	/* Good for darkbg consoles */
+    if (l <= 6 && strncmp(v, "boldfg", l) == 0)        /* Good for darkbg consoles */
       v = "[1,,1], [5,,1], [3,,1], [7,,1], [6,,1], , [2,,1]";
-    v0 = v = filtre(v, 0);
+    v0 = s = filtre(v, 0);
     for (c=c_ERR; c < c_LAST; c++)
-      gp_colors[c] = gp_get_color(&v);
-    free(v0);
+      gp_colors[c] = gp_get_color(&s);
+    pari_free(v0);
   }
   if (flag == d_ACKNOWLEDGE || flag == d_RETURN)
   {
@@ -407,7 +309,7 @@ sd_colors(char *v, long flag)
       else
       {
         decode_color(n,col);
-        if (n & (1<<12))
+        if (n & (1L<<12))
         {
           if (col[0])
             sprintf(t,"[%ld,,%ld]",col[1],col[0]);
@@ -421,15 +323,46 @@ sd_colors(char *v, long flag)
       if (c < c_LAST - 1) { *t++=','; *t++=' '; }
     }
     if (flag==d_RETURN) return strtoGENstr(s);
-    pariprintf("   colors = \"%s\"\n",s);
+    pari_printf("   colors = \"%s\"\n",s);
   }
   return gnil;
 }
 
+GEN
+sd_format(const char *v, long flag)
+{
+  pariout_t *fmt = GP_DATA->fmt;
+  if (v)
+  {
+    char c = *v;
+    if (c!='e' && c!='f' && c!='g')
+      pari_err(syntaxer,"default: inexistent format",v,v);
+    fmt->format = c; v++;
+
+    if (isdigit((int)*v))
+      { while (isdigit((int)*v)) v++; } /* FIXME: skip obsolete field width */
+    if (*v++ == '.')
+    {
+      if (*v == '-') fmt->sigd = -1;
+      else
+        if (isdigit((int)*v)) fmt->sigd=atol(v);
+    }
+  }
+  if (flag == d_RETURN)
+  {
+    char *s = stackmalloc(64);
+    (void)sprintf(s, "%c.%ld", fmt->format, fmt->sigd);
+    return strtoGENstr(s);
+  }
+  if (flag == d_ACKNOWLEDGE)
+    pari_printf("   format = %c.%ld\n", fmt->format, fmt->sigd);
+  return gnil;
+}
+
 GEN
 sd_compatible(const char *v, long flag)
 {
-  char *msg[] = {
+  const char *msg[] = {
     "(no backward compatibility)",
     "(warn when using obsolete functions)",
     "(use old functions, don't ignore case)",
@@ -446,33 +379,21 @@ sd_compatible(const char *v, long flag)
 GEN
 sd_secure(const char *v, long flag)
 {
-  if (*v && (GP_DATA->flags & SECURE))
-  {
-    fprintferr("[secure mode]: Do you want to modify the 'secure' flag? (^C if not)\n");
-    hit_return();
-  }
-  return sd_gptoggle(v,flag,"secure", SECURE);
+  if (v && GP_DATA->secure)
+    pari_ask_confirm("[secure mode]: About to modify the 'secure' flag");
+  return sd_toggle(v,flag,"secure", &(GP_DATA->secure));
 }
 
+static THREAD long dbg = -1;
+void
+dbg_block(void) { if (DEBUGLEVEL) { dbg = DEBUGLEVEL; DEBUGLEVEL = 0; } }
+void
+dbg_release(void) { if (dbg >= 0) { DEBUGLEVEL = dbg; dbg = -1; } }
+
 GEN
 sd_debug(const char *v, long flag)
 { return sd_ulong(v,flag,"debug",&DEBUGLEVEL, 0,20,NULL); }
 
-ulong readline_state = DO_ARGS_COMPLETE;
-
-GEN
-sd_rl(const char *v, long flag)
-{
-  static const char * const msg[] = {NULL,
-	"(bits 0x2/0x4 control matched-insert/arg-complete)"};
-  ulong o_readline_state = readline_state;
-  GEN res = sd_ulong(v,flag,"readline", &readline_state, 0, 7, (char**)msg);
-
-  if (o_readline_state != readline_state)
-    (void)sd_gptoggle(readline_state? "1": "0", d_SILENT, "readline", USE_READLINE);
-  return res;
-}
-
 GEN
 sd_debugfiles(const char *v, long flag)
 { return sd_ulong(v,flag,"debugfiles",&DEBUGFILES, 0,20,NULL); }
@@ -481,21 +402,22 @@ GEN
 sd_debugmem(const char *v, long flag)
 { return sd_ulong(v,flag,"debugmem",&DEBUGMEM, 0,20,NULL); }
 
-GEN
-sd_echo(const char *v, long flag)
-{ return sd_gptoggle(v,flag,"echo", ECHO); }
-
-GEN
-sd_lines(const char *v, long flag)
-{ return sd_ulong(v,flag,"lines",&(GP_DATA->lim_lines), 0,VERYBIGINT,NULL); }
-
+/* set D->hist to size = s / total = t */
+static void
+init_hist(gp_data *D, size_t s, ulong t)
+{
+  gp_hist *H = D->hist;
+  H->total = t;
+  H->size = s;
+  H->res = (GEN *) pari_calloc(s * sizeof(GEN));
+}
 GEN
 sd_histsize(const char *v, long flag)
 {
   gp_hist *H = GP_DATA->hist;
   ulong n = H->size;
   GEN r = sd_ulong(v,flag,"histsize",&n, 1,
-                     (VERYBIGINT / sizeof(long)) - 1,NULL);
+                     (LONG_MAX / sizeof(long)) - 1,NULL);
   if (n != H->size)
   {
     const ulong total = H->total;
@@ -511,7 +433,7 @@ sd_histsize(const char *v, long flag)
     /* copy relevant history entries */
     g     = (total-1) % sG;
     h = k = (total-1) % sH;
-    kmin = k - min(sH, sG);
+    kmin = k - minss(sH, sG);
     for ( ; k > kmin; k--, g--, h--)
     {
       resH[h] = resG[g];
@@ -525,53 +447,57 @@ sd_histsize(const char *v, long flag)
       gunclone(resG[g]);
       if (!g) g = sG;
     }
-    free((void*)resG);
+    pari_free((void*)resG);
   }
   return r;
 }
 
 static void
 TeX_define(const char *s, const char *def) {
-  fprintf(logfile, "\\ifx\\%s\\undefined\n  \\def\\%s{%s}\\fi\n", s,s,def);
+  fprintf(pari_logfile, "\\ifx\\%s\\undefined\n  \\def\\%s{%s}\\fi\n", s,s,def);
 }
 static void
 TeX_define2(const char *s, const char *def) {
-  fprintf(logfile, "\\ifx\\%s\\undefined\n  \\def\\%s#1#2{%s}\\fi\n", s,s,def);
+  fprintf(pari_logfile, "\\ifx\\%s\\undefined\n  \\def\\%s#1#2{%s}\\fi\n", s,s,def);
+}
+
+static FILE *
+open_logfile(const char *s) {
+  FILE *log = fopen(s, "a");
+  if (!log) pari_err(openfiler,"logfile",s);
+#ifndef WINCE
+  setbuf(log,(char *)NULL);
+#endif
+  return log;
 }
 
 GEN
 sd_log(const char *v, long flag)
 {
-  static const char * const msg[] = {
+  const char *msg[] = {
       "(off)",
       "(on)",
       "(on with colors)",
       "(TeX output)", NULL
   };
   ulong oldstyle = logstyle;
-  GEN res = sd_ulong(v,flag,"log", &logstyle, 0, 3, (char**)msg);
+  GEN res = sd_ulong(v,flag,"log", &logstyle, 0, 3, msg);
 
-  if (!oldstyle != !logstyle)		/* Compare converts to boolean */
+  if (!oldstyle != !logstyle)                /* Compare converts to boolean */
   { /* toggled LOG */
     if (oldstyle)
     { /* close log */
       if (flag == d_ACKNOWLEDGE)
-        pariprintf("   [logfile was \"%s\"]\n", current_logfile);
-      fclose(logfile); logfile = NULL;
+        pari_printf("   [logfile was \"%s\"]\n", current_logfile);
+      fclose(pari_logfile); pari_logfile = NULL;
     }
     else
-    { /* open log */
-      logfile = fopen(current_logfile, "a");
-      if (!logfile) pari_err(openfiler,"logfile",current_logfile);
-#ifndef WINCE
-      setbuf(logfile,(char *)NULL);
-#endif
-    }
+      pari_logfile = open_logfile(current_logfile);
   }
-  if (logfile && oldstyle != logstyle && logstyle == logstyle_TeX)
+  if (pari_logfile && oldstyle != logstyle && logstyle == logstyle_TeX)
   {
-    TeX_define("PARIbreak", 
-               "\\hskip 0pt plus \\hsize\\relax\\discretionary{}{}{}}");
+    TeX_define("PARIbreak",
+               "\\hskip 0pt plus \\hsize\\relax\\discretionary{}{}{}");
     TeX_define("PARIpromptSTART", "\\vskip\\medskipamount\\bgroup\\bf");
     TeX_define("PARIpromptEND", "\\egroup\\bgroup\\tt");
     TeX_define("PARIinputEND", "\\egroup");
@@ -584,17 +510,17 @@ sd_log(const char *v, long flag)
 GEN
 sd_TeXstyle(const char *v, long flag)
 {
-  static const char * const msg[] = { NULL,
-	"(bits 0x2/0x4 control output of \\left/\\PARIbreak)"};
+  const char *msg[] = { "(bits 0x2/0x4 control output of \\left/\\PARIbreak)",
+                        NULL };
   ulong n = GP_DATA->fmt->TeXstyle;
-  GEN z = sd_ulong(v,flag,"TeXstyle", &n, 0, 7, (char**)msg);
+  GEN z = sd_ulong(v,flag,"TeXstyle", &n, 0, 7, msg);
   GP_DATA->fmt->TeXstyle = n; return z;
 }
 
 GEN
 sd_output(const char *v, long flag)
 {
-  char *msg[] = {"(raw)", "(prettymatrix)", "(prettyprint)",
+  const char *msg[] = {"(raw)", "(prettymatrix)", "(prettyprint)",
                  "(external prettyprint)", NULL};
   ulong n = GP_DATA->fmt->prettyp;
   GEN z = sd_ulong(v,flag,"output", &n, 0,3,msg);
@@ -606,16 +532,13 @@ sd_output(const char *v, long flag)
 GEN
 sd_parisize(const char *v, long flag)
 {
-  ulong oldn = top-bot, n = oldn;
-  GEN r = sd_ulong(v,flag,"parisize",&n, 10000,VERYBIGINT,NULL);
-  if (n != oldn)
-  {
-    if (!bot) top = (pari_sp)n; /* no stack allocated yet */
-    if (flag != d_INITRC) {
-      ulong R = (ulong)r[2];
-      allocatemoremem(n);
-      r = utoi(R);
-    }
+  ulong size = top - bot, n = size;
+  GEN r = sd_ulong(v,flag,"parisize",&n, 10000,LONG_MAX,NULL);
+  if (n != size) {
+    if (flag == d_INITRC)
+      pari_init_stack(n, size);
+    else
+      allocatemem(n);
   }
   return r;
 }
@@ -623,142 +546,119 @@ sd_parisize(const char *v, long flag)
 GEN
 sd_primelimit(const char *v, long flag)
 {
-  ulong n = GP_DATA->primelimit;
-  GEN r = sd_ulong(v,flag,"primelimit",&n, 0,2*(ulong)(VERYBIGINT-1024) + 1,NULL);
-  if (n != GP_DATA->primelimit)
+  ulong max = maxprime(), n = max;
+  GEN r = sd_ulong(v,flag,"primelimit",&n, 0,2*(ulong)(LONG_MAX-1024) + 1,NULL);
+  if (n != max)
   {
-    if (flag != d_INITRC)
-    {
-      byteptr ptr = initprimes(n);
-      free(diffptr); diffptr = ptr;
-    }
-    GP_DATA->primelimit = n;
+    byteptr ptr = initprimes(n);
+    pari_free(diffptr); diffptr = ptr;
   }
   return r;
 }
 
 GEN
 sd_simplify(const char *v, long flag)
-{ return sd_gptoggle(v,flag,"simplify", SIMPLIFY); }
+{ return sd_toggle(v,flag,"simplify", &(GP_DATA->simplify)); }
 
 GEN
 sd_strictmatch(const char *v, long flag)
-{ return sd_gptoggle(v,flag,"strictmatch", STRICTMATCH); }
+{ return sd_toggle(v,flag,"strictmatch", &(GP_DATA->strictmatch)); }
 
 GEN
-sd_timer(const char *v, long flag)
-{ return sd_gptoggle(v,flag,"timer", CHRONO); }
-
-GEN
-sd_filename(const char *v, long flag, char *s, char **f)
+sd_string(const char *v, long flag, const char *s, char **pstr)
 {
-  if (*v)
+  char *old = *pstr;
+  if (v)
   {
-    char *s, *old = *f;
-    long l;
-    char *ev = expand_tilde(v);
-    l = strlen(ev) + 256;
-    s = (char *) malloc(l);
-    do_strftime(ev,s, l-1); free(ev);
-    *f = pari_strdup(s); free(s); free(old);
+    char *str, *ev = path_expand(v);
+    long l = strlen(ev) + 256;
+    str = (char *) malloc(l);
+    strftime_expand(ev,str, l-1); pari_free(ev);
+    if (GP_DATA->secure)
+    {
+      char *msg=pari_sprintf("[secure mode]: About to change %s to '%s'",s,str);
+      pari_ask_confirm(msg);
+      pari_free(msg);
+    }
+    if (old) pari_free(old);
+    *pstr = old = pari_strdup(str);
+    pari_free(str);
   }
-  if (flag == d_RETURN) return strtoGENstr(*f);
-  if (flag == d_ACKNOWLEDGE) pariprintf("   %s = \"%s\"\n",s,*f);
+  else if (!old) old = (char*)"<undefined>";
+  if (flag == d_RETURN) return strtoGENstr(old);
+  if (flag == d_ACKNOWLEDGE) pari_printf("   %s = \"%s\"\n",s,old);
   return gnil;
 }
 
 GEN
 sd_logfile(const char *v, long flag)
 {
-  GEN r = sd_filename(v, flag, "logfile", &current_logfile);
-  if (*v && logfile)
+  GEN r = sd_string(v, flag, "logfile", &current_logfile);
+  if (v && pari_logfile)
   {
-    fclose(logfile);
-    logfile = fopen(current_logfile, "a");
-    if (!logfile) pari_err(openfiler,"logfile",current_logfile);
-#ifndef WINCE
-    setbuf(logfile,(char *)NULL);
-#endif
+    FILE *log = open_logfile(current_logfile);
+    fclose(pari_logfile); pari_logfile = log;
   }
   return r;
 }
 
 GEN
-sd_factor_add_primes(char *v, long flag)
+sd_factor_add_primes(const char *v, long flag)
 { return sd_toggle(v,flag,"factor_add_primes", &factor_add_primes); }
 
 GEN
-sd_new_galois_format(char *v, long flag)
+sd_factor_proven(const char *v, long flag)
+{ return sd_toggle(v,flag,"factor_proven", &factor_proven); }
+
+GEN
+sd_new_galois_format(const char *v, long flag)
 { return sd_toggle(v,flag,"new_galois_format", &new_galois_format); }
 
 GEN
-sd_psfile(const char *v, long flag)
-{ return sd_filename(v, flag, "psfile", &current_psfile); }
-
-static void
-err_secure(char *d, char *v)
-{ pari_err(talker,"[secure mode]: can't modify '%s' default (to %s)",d,v); }
-
-GEN
-sd_help(char *v, long flag)
+sd_datadir(const char *v, long flag)
 {
   const char *str;
-  if (*v)
+  if (v)
   {
-    if (GP_DATA->flags & SECURE) err_secure("help",v);
-    if (GP_DATA->help) free(GP_DATA->help);
-    GP_DATA->help = expand_tilde(v);
-  }
-  str = GP_DATA->help? GP_DATA->help: "none";
-  if (flag == d_RETURN) return strtoGENstr(str);
-  if (flag == d_ACKNOWLEDGE)
-    pariprintf("   help = \"%s\"\n", str);
-  return gnil;
-}
-
-GEN
-sd_datadir(char *v, long flag)
-{
-  const char *str;
-  if (*v)
-  {
-    if (pari_datadir) free(pari_datadir);
-    pari_datadir = expand_tilde(v);
+    if (pari_datadir) pari_free(pari_datadir);
+    pari_datadir = path_expand(v);
   }
   str = pari_datadir? pari_datadir: "none";
   if (flag == d_RETURN) return strtoGENstr(str);
   if (flag == d_ACKNOWLEDGE)
-    pariprintf("   datadir = \"%s\"\n", str);
+    pari_printf("   datadir = \"%s\"\n", str);
   return gnil;
 }
 
 GEN
-sd_path(char *v, long flag)
+sd_path(const char *v, long flag)
 {
   gp_path *p = GP_DATA->path;
-  if (*v)
+  if (v)
   {
-    free((void*)p->PATH);
+    pari_free((void*)p->PATH);
     p->PATH = pari_strdup(v);
     if (flag == d_INITRC) return gnil;
     gp_expand_path(p);
   }
   if (flag == d_RETURN) return strtoGENstr(p->PATH);
   if (flag == d_ACKNOWLEDGE)
-    pariprintf("   path = \"%s\"\n",p->PATH);
+    pari_printf("   path = \"%s\"\n",p->PATH);
   return gnil;
 }
 
+static const char *DFT_PRETTYPRINTER = "tex2mail -TeX -noindent -ragged -by_par";
 GEN
-sd_prettyprinter(char *v, long flag)
+sd_prettyprinter(const char *v, long flag)
 {
   gp_pp *pp = GP_DATA->pp;
-  if (*v && !(GP_DATA->flags & TEXMACS))
+  if (v && !(GP_DATA->flags & gpd_TEXMACS))
   {
     char *old = pp->cmd;
     int cancel = (!strcmp(v,"no"));
 
-    if (GP_DATA->flags & SECURE) err_secure("prettyprinter",v);
+    if (GP_DATA->secure)
+      pari_err(talker,"[secure mode]: can't modify 'prettyprinter' default (to %s)",v);
     if (!strcmp(v,"yes")) v = DFT_PRETTYPRINTER;
     if (old && strcmp(old,v) && pp->file)
     {
@@ -777,153 +677,132 @@ sd_prettyprinter(char *v, long flag)
       pp->file = f;
     }
     pp->cmd = cancel? NULL: pari_strdup(v);
-    if (old) free(old);
+    if (old) pari_free(old);
     if (flag == d_INITRC) return gnil;
   }
   if (flag == d_RETURN)
     return strtoGENstr(pp->cmd? pp->cmd: "");
   if (flag == d_ACKNOWLEDGE)
-    pariprintf("   prettyprinter = \"%s\"\n",pp->cmd? pp->cmd: "");
+    pari_printf("   prettyprinter = \"%s\"\n",pp->cmd? pp->cmd: "");
   return gnil;
 }
 
-static GEN
-sd_prompt_set(const char *v, long flag, char *how, char *p)
-{
-  if (*v)
-  {
-    strncpy(p,v,MAX_PROMPT_LEN);
-#ifdef macintosh
-    strcat(p,"\n");
-#endif
-  }
-  if (flag == d_RETURN) return strtoGENstr(p);
-  if (flag == d_ACKNOWLEDGE)
-    pariprintf("   prompt%s = \"%s\"\n", how, p);
-  return gnil;
+/* compare entrees s1 s2 according to the associated function name */
+static int
+compare_name(const void *s1, const void *s2) {
+  entree *e1 = *(entree**)s1, *e2 = *(entree**)s2;
+  return strcmp(e1->name, e2->name);
 }
-
-GEN
-sd_prompt(const char *v, long flag)
-{
-  return sd_prompt_set(v, flag, "", GP_DATA->prompt);
-}
-
-GEN
-sd_prompt_cont(const char *v, long flag)
-{
-  return sd_prompt_set(v, flag, "_cont", GP_DATA->prompt_cont);
-}
-
-char *
-expand_prompt(char *prompt, filtre_t *F)
-{
-  static char buf[MAX_PROMPT_LEN];
-  char *s = buf;
-  if (F->in_comment) return COMMENTPROMPT;
-  do_strftime(prompt, s, MAX_PROMPT_LEN-1);
-  return s;
-}
-
-default_type gp_default_list[] =
-{
-  {"colors",(void*)sd_colors},
-  {"compatible",(void*)sd_compatible},
-  {"datadir",(void*)sd_datadir},
-  {"debug",(void*)sd_debug},
-  {"debugfiles",(void*)sd_debugfiles},
-  {"debugmem",(void*)sd_debugmem},
-  {"echo",(void*)sd_echo},
-  {"factor_add_primes",(void*)sd_factor_add_primes},
-  {"format",(void*)sd_format},
-  {"help",(void*)sd_help},
-  {"histsize",(void*)sd_histsize},
-  {"lines",(void*)sd_lines},
-  {"log",(void*)sd_log},
-  {"logfile",(void*)sd_logfile},
-  {"new_galois_format",(void*)sd_new_galois_format},
-  {"output",(void*)sd_output},
-  {"parisize",(void*)sd_parisize},
-  {"path",(void*)sd_path},
-  {"primelimit",(void*)sd_primelimit},
-  {"prettyprinter",(void*)sd_prettyprinter},
-  {"prompt",(void*)sd_prompt},
-  {"prompt_cont",(void*)sd_prompt_cont},
-  {"psfile",(void*)sd_psfile},
-  {"realprecision",(void*)sd_realprecision},
-  {"readline",(void*)sd_rl},
-  {"secure",(void*)sd_secure},
-  {"seriesprecision",(void*)sd_seriesprecision},
-  {"simplify",(void*)sd_simplify},
-  {"strictmatch",(void*)sd_strictmatch},
-  {"TeXstyle",(void *)sd_TeXstyle},
-  {"timer",(void *)sd_timer},
-  {NULL,NULL} /* sentinel */
-};
-
+/* return all entries with class '16' */
 static void
-help_default(void)
+defaults_list(pari_stack *s)
 {
-  default_type *dft;
-
-  for (dft=gp_default_list; dft->fun; dft++)
-    ((void (*)(const char*,long)) dft->fun)("", d_ACKNOWLEDGE);
+  entree *ep;
+  long i;
+  for (i = 0; i < functions_tblsz; i++)
+    for (ep = defaults_hash[i]; ep; ep = ep->next)
+      if (ep->menu == 16) stack_pushp(s, ep);
 }
-
+/* ep associated to function f of arity 2. Call f(v,flag) */
+static GEN
+call_f2(entree *ep, const char *v, long flag)
+{ return ((GEN (*)(const char*,long))ep->value)(v, flag); }
 GEN
 setdefault(const char *s, const char *v, long flag)
 {
-  default_type *dft;
+  entree *ep;
+  if (!s)
+  { /* list all defaults */
+    pari_stack st;
+    entree **L;
+    long i;
+    stack_init(&st, sizeof(*L), (void**)&L);
+    defaults_list(&st);
+    qsort (L, st.n, sizeof(*L), compare_name);
+    for (i = 0; i < st.n; i++) (void)call_f2(L[i], NULL, d_ACKNOWLEDGE);
+    stack_delete(&st);
+    return gnil;
+  }
+  ep = is_entry_intern(s, defaults_hash, NULL);
+  if (!ep)
+  {
+    pari_err(talker,"unknown default: %s",s);
+    return NULL; /* not reached */
+  }
+  return call_f2(ep, v, flag);
+}
+int
+pari_is_default(const char *s)
+{ return !!is_entry_intern(s, defaults_hash, NULL); }
 
-  if (!*s) { help_default(); return gnil; }
-  for (dft=gp_default_list; dft->fun; dft++)
-    if (!strcmp(s,dft->name))
-    {
-      if (flag == d_EXISTS) return gen_1;
-      return ((GEN (*)(const char*,long)) dft->fun)(v,flag);
-    }
-  if (flag == d_EXISTS) return gen_0;
-  pari_err(talker,"unknown default: %s",s);
-  return NULL; /* not reached */
+GEN
+default0(const char *a, const char *b) { return setdefault(a,b, b? d_SILENT: d_RETURN); }
+
+/********************************************************************/
+/*                                                                  */
+/*                     INITIALIZE GP_DATA                           */
+/*                                                                  */
+/********************************************************************/
+/* initialize D->path */
+static void
+init_path(gp_data *D)
+{
+  gp_path *path = D->path;
+  path->PATH = pari_strdup(pari_default_path());
+  path->dirs = NULL;
 }
 
-GEN
-gp_default(char *a, char *b) { return setdefault(a,b, d_RETURN); }
-
-GEN
-default0(char *a, char *b, long flag)
+/* initialize D->fmt */
+static void
+init_fmt(gp_data *D)
 {
-  (void)flag; /* compatibility: to be deleted someday */
-  return setdefault(a,b, d_RETURN);
+#ifdef LONG_IS_64BIT
+  static pariout_t DFLT_OUTPUT = { 'g', 38, 1, f_PRETTYMAT, 0 };
+#else
+  static pariout_t DFLT_OUTPUT = { 'g', 28, 1, f_PRETTYMAT, 0 };
+#endif
+  D->fmt = &DFLT_OUTPUT;
+}
+
+/* initialize D->pp */
+static void
+init_pp(gp_data *D)
+{
+  gp_pp *p = D->pp;
+  p->cmd = pari_strdup(DFT_PRETTYPRINTER);
+  p->file = NULL;
 }
 
 gp_data *
 default_gp_data(void)
 {
-  static char Prompt[MAX_PROMPT_LEN], Prompt_cont[MAX_PROMPT_LEN];
   static gp_data __GPDATA, *D = &__GPDATA;
   static gp_hist __HIST;
   static gp_pp   __PP;
   static gp_path __PATH;
   static pari_timer __T;
 
-#ifdef READLINE
-  D->flags = (STRICTMATCH | SIMPLIFY | USE_READLINE);
-#else
-  D->flags = (STRICTMATCH | SIMPLIFY);
-#endif
-  D->primelimit = 500000;
-  D->lim_lines = 0;
+  D->flags       = 0;
+
+  /* GP-specific */
+  D->breakloop   = 1;
+  D->echo        = 0;
+  D->lim_lines   = 0;
+  D->recover     = 1;
+  D->chrono      = 0;
+
+  D->strictmatch = 1;
+  D->simplify    = 1;
+  D->secure      = 0;
+  D->use_readline= 0;
+  D->last_time   = 0;
   D->T    = &__T;
   D->hist = &__HIST;
   D->pp   = &__PP;
   D->path = &__PATH;
-  init_help(D);
   init_fmt(D);
   init_hist(D, 5000, 0);
   init_path(D);
   init_pp(D);
-  strcpy(Prompt,      DFT_PROMPT); D->prompt = Prompt;
-  strcpy(Prompt_cont, CONTPROMPT); D->prompt_cont = Prompt_cont;
   return D;
 }
