@@ -1,4 +1,4 @@
-/* $Id: members.c 9120 2007-10-29 10:33:14Z kb $
+/* $Id: members.c 12152 2010-02-16 22:18:31Z bill $
 
 Copyright (C) 2000-2003  The PARI group.
 
@@ -21,12 +21,26 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /**                          MEMBER FUNCTIONS                      **/
 /**                                                                **/
 /********************************************************************/
-#define is_ell(x) (typ(x) == t_VEC && lg(x)>=14)
-#define is_bigell(x) (typ(x) == t_VEC && lg(x)>=20)
+INLINE int is_ell5(GEN x) {
+  long lx = lg(x);
+  return (typ(x) == t_VEC && (lx == 6 || lx == 14 || lx == 20));
+}
+INLINE int is_smallell(GEN x) {
+  long lx = lg(x);
+  return (typ(x) == t_VEC && (lx == 14 || lx == 20));
+}
+INLINE int is_ell(GEN x) {
+  long lx = lg(x);
+  return (typ(x) == t_VEC && lx == 20);
+}
+
+static void
+member_err(const char *s) { pari_err(typeer,s); }
+
 GEN
 member_e(GEN x)
 {
-  x = get_primeid(x);
+  x = get_prid(x);
   if (!x) member_err("e");
   return gel(x,3);
 }
@@ -34,7 +48,7 @@ member_e(GEN x)
 GEN
 member_f(GEN x)
 {
-  x = get_primeid(x);
+  x = get_prid(x);
   if (!x) member_err("f");
   return gel(x,4);
 }
@@ -43,11 +57,18 @@ GEN
 member_p(GEN x)
 {
   long t; (void)get_nf(x,&t);
-  if (t == typ_GAL)
-    return gmael(x,2,1);
-  x = get_primeid(x);
-  if (!x) member_err("p");
-  return gel(x,1);
+  if (t == typ_GAL) return gal_get_p(x);
+  switch(typ(x)) {
+    case t_VEC:
+      x = get_prid(x); if (!x) member_err("p");
+      return pr_get_p(x);
+    case t_PADIC:
+      return gel(x,2);
+    case t_FFELT:
+      return FF_p_i(x);
+  }
+  member_err("p");
+  return NULL;
 }
 
 GEN
@@ -55,7 +76,7 @@ member_bid(GEN x)
 {
   long t; (void)get_nf(x,&t);
   switch(t) {
-    case typ_BNR: return gel(x,2);
+    case typ_BNR: return bnr_get_bid(x);
     case typ_BID: return x;
   }
   member_err("bid");
@@ -73,9 +94,12 @@ member_bnf(GEN x)
 GEN
 member_nf(GEN x)
 {
-  long t; x = get_nf(x,&t);
-  if (!x) member_err("nf");
-  return x;
+  long t; GEN y = get_nf(x,&t);
+  if (!y) {
+    if (t == typ_RNF) return gel(x,10);
+    member_err("nf");
+  }
+  return y;
 }
 
 /* integral basis */
@@ -87,12 +111,16 @@ member_zk(GEN x)
   {
     switch(t)
     {
-      case typ_CLA: return gmael(x,1,4);
-      case typ_Q: return mkvec2(gen_1, pol_x[varn(x[1])]);
+      case typ_Q:
+        y = cgetg(3,t_VEC);
+        gel(y,1) = gen_1;
+        gel(y,2) = pol_x(varn(x[1])); return y;
+      case typ_RNF:
+        return gel(x,7);
     }
     member_err("zk");
   }
-  return gel(y,7);
+  return nf_get_zk(y);
 }
 
 GEN
@@ -103,16 +131,12 @@ member_disc(GEN x) /* discriminant */
   {
     switch(t)
     {
-      case typ_Q  : return discsr(gel(x,1));
-      case typ_CLA:
-        x = gmael(x,1,3);
-        if (typ(x) != t_VEC || lg(x) != 3) break;
-        return gel(x,1);
-      case typ_ELL: return gel(x,12);
+      case typ_Q  : return quad_disc(x);
+      case typ_ELL: return ell_get_disc(x);
     }
     member_err("disc");
   }
-  return gel(y,3);
+  return nf_get_disc(y);
 }
 
 GEN
@@ -123,16 +147,16 @@ member_pol(GEN x) /* polynomial */
   {
     switch(t)
     {
-      case typ_CLA: return gmael(x,1,1);
       case typ_POL: return x;
       case typ_Q  : return gel(x,1);
-      case typ_GAL: return gel(x,1);
+      case typ_GAL: return gal_get_pol(x);
+      case typ_RNF: return gmael(x,11,1);
     }
     if (typ(x)==t_POLMOD) return gel(x,2);
-    if (typ(x)==t_VEC && lg(x) == 13) return gmael(x,11,1);
+    if (typ(x)==t_FFELT) return FF_to_FpXQ(x);
     member_err("pol");
   }
-  return gel(y,1);
+  return nf_get_pol(y);
 }
 
 GEN
@@ -140,13 +164,15 @@ member_mod(GEN x) /* modulus */
 {
   long t; (void)get_nf(x,&t);
   switch(t) {
-    case typ_GAL: return gmael(x,2,3);
-    case typ_BNR: x = gel(x,2); /* fall through */
+    case typ_GAL: return gal_get_mod(x);
+    case typ_BNR: return bnr_get_mod(x);
     case typ_BID: return gel(x,1);
   }
   switch(typ(x))
   {
     case t_INTMOD: case t_POLMOD: case t_QUAD: break;
+    case t_PADIC: return gel(x,3);
+    case t_FFELT: return FF_mod(x);
     default: member_err("mod");
   }
   return gel(x,1);
@@ -156,11 +182,7 @@ GEN
 member_sign(GEN x) /* signature */
 {
   long t; GEN y = get_nf(x,&t);
-  if (!y)
-  {
-    if (t == typ_CLA) return gmael(x,1,2);
-    member_err("sign");
-  }
+  if (!y) member_err("sign");
   return gel(y,2);
 }
 GEN
@@ -173,7 +195,7 @@ member_index(GEN x)
 {
   long t; GEN y = get_nf(x,&t);
   if (!y) member_err("index");
-  return gel(y,4);
+  return nf_get_index(y);
 }
 
 /* x assumed to be output by get_nf: ie a t_VEC with length 11 */
@@ -209,9 +231,9 @@ member_codiff(GEN x) /* codifferent */
   long t; GEN T, D, DinvT, nf = get_nf(x,&t), y = nfmats(nf);
   if (!y) member_err("codiff");
   T = gel(y,4);
-  D = absi(gel(nf,3));
+  D = absi(nf_get_disc(nf));
   DinvT = ZM_inv(T,D);
-  return gdiv(hnfmod(DinvT, D), D);
+  return gdiv(ZM_hnfmod(DinvT, D), D);
 }
 
 GEN
@@ -220,20 +242,19 @@ member_roots(GEN x) /* roots */
   long t; GEN y = get_nf(x,&t);
   if (!y)
   {
-    if (t == typ_ELL && is_bigell(x)) return gel(x,14);
-    if (t == typ_GAL) return gel(x,3);
+    if (t == typ_ELL && is_ell(x)) return ell_get_roots(x);
+    if (t == typ_GAL) return gal_get_roots(x);
     member_err("roots");
   }
-  return gel(y,6);
+  return nf_get_roots(y);
 }
 
 /* assume x output by get_bnf: ie a t_VEC with length 10 */
 static GEN
-check_RES(GEN x, char *s)
+check_RES(GEN x, const char *s)
 {
   GEN y = gel(x,8);
-  if (typ(y) != t_VEC || lg(y) < 4)
-    member_err(s);
+  if (typ(y) != t_VEC || lg(y) < 4) member_err(s);
   return y;
 }
 
@@ -246,8 +267,7 @@ member_clgp(GEN x) /* class group (3-component row vector) */
     switch(t)
     {
       case typ_QUA: return mkvec3(gel(x,1), gel(x,2), gel(x,3));
-      case typ_CLA: return gmael(x,1,5);
-      case typ_BID: return x = gel(x,2);
+      case typ_BID: return gel(x,2);
     }
     if (typ(x)==t_VEC)
       switch(lg(x))
@@ -268,11 +288,7 @@ member_reg(GEN x) /* regulator */
   long t; GEN y = get_bnf(x,&t);
   if (!y)
   {
-    switch(t)
-    {
-      case typ_CLA: return gmael(x,1,6);
-      case typ_QUA: return gel(x,4);
-    }
+    if (t == typ_QUA) return gel(x,4);
     member_err("reg");
   }
   if (t == typ_BNR) pari_err(impl,"ray regulator");
@@ -288,16 +304,14 @@ member_fu(GEN x) /* fundamental units */
   {
     switch(t)
     {
-      case typ_CLA: x = gel(x,1); if (lg(x) < 10) break;
-        return gel(x,9);
       case typ_Q:
-        x = discsr(gel(x,1));
-        return (signe(x)<0)? cgetg(1,t_VEC): fundunit(x);
+        x = quad_disc(x);
+        return (signe(x)<0)? cgetg(1,t_VEC): quadunit(x);
     }
     member_err("fu");
   }
   if (t == typ_BNR) pari_err(impl,"ray units");
-  return basistoalg(y, check_units(y,".fu"));
+  return matbasistoalg(y, bnf_get_fu(y));
 }
 
 /* torsion units. return [w,e] where w is the number of roots of 1, and e a
@@ -305,73 +319,52 @@ member_fu(GEN x) /* fundamental units */
 GEN
 member_tu(GEN x)
 {
-  long t; GEN y, bnf = get_bnf(x,&t), res = cgetg(3,t_VEC);
+  long t; GEN bnf = get_bnf(x,&t), res = cgetg(3,t_VEC);
   if (!bnf)
   {
-    switch(t)
-    {
-      case typ_Q:
-        y = discsr(gel(x,1));
-        if (signe(y)<0 && cmpiu(y,4)<=0) /* |y| <= 4 */
-          y = stoi((itos(y) == -4)? 4: 6);
-        else
-        { y = gen_2; x = gen_m1; }
-        gel(res,1) = y;
-        gel(res,2) = x; return res;
-      case typ_CLA:
-        x = gel(x,1);
-        if (lg(x) > 8)
-        {
-          y = gel(x,8);
-          if (typ(y) == t_VEC || lg(y) == 3) { res[2] = y[2]; break; }
-        }
-      default: member_err("tu");
-        return NULL; /* not reached */
-    }
+    GEN y;
+    if (t != typ_Q) member_err("tu");
+    y = quad_disc(x);
+    if (signe(y) > 0 || cmpiu(y,4) > 0) return mkvec2(gen_m1, gen_2);
+
+    gel(res,1) = utoipos((itos(y) == -4)? 4: 6);
+    gel(res,2) = gcopy(x);
   }
   else
   {
     if (t == typ_BNR) pari_err(impl,"ray torsion units");
-    x = gel(bnf,7);
-    y = gel(bnf,8);
-    if (typ(y) == t_VEC && lg(y) > 5) y = gel(y,4);
-    else
-    {
-      y = rootsof1(x);
-      gel(y,2) = gmul(gel(x,7), gel(y,2));
-    }
-    gel(res,2) = basistoalg(bnf, gel(y,2));
+    gel(res,1) = utoipos( bnf_get_tuN(bnf) );
+    gel(res,2) = basistoalg(bnf, bnf_get_tuU(bnf));
   }
-  res[1] = y[1]; return res;
+  return res;
 }
 
 GEN
 member_futu(GEN x) /*  concatenation of fu and tu, w is lost */
 {
-  GEN fuc = member_fu(x);
-  return shallowconcat(fuc, (GEN)member_tu(x)[2]);
+  return shallowconcat(member_fu(x), gel(member_tu(x),2));
 }
 
 GEN
 member_tufu(GEN x) /*  concatenation of tu and fu, w is lost */
 {
-  GEN fuc = member_fu(x);
-  return shallowconcat((GEN)member_tu(x)[2], fuc);
+  return shallowconcat(gel(member_tu(x),2), member_fu(x));
 }
 
+/* structure of (Z_K/m)^*, where x is an idealstarinit (with or without gen)
+ * or a bnrinit (with or without gen) */
 GEN
-member_zkst(GEN bid)
-/* structure of (Z_K/m)^*, where bid is an idealstarinit (with or without gen)
-   or a bnrinit (with or without gen) */
+member_zkst(GEN x)
 {
-  if (typ(bid)==t_VEC)
-    switch(lg(bid))
-    {
-      case 6: return gel(bid,2);   /* idealstarinit */
-      case 7: bid = gel(bid,2); /* bnrinit */
-        if (typ(bid) == t_VEC && lg(bid) > 2)
-          return gel(bid,2);
+  long t; (void)get_nf(x,&t);
+  switch(t)
+  {
+    case typ_BID: return gel(x,2);
+    case typ_BNR: {
+      GEN bid = bnr_get_bid(x);
+      if (typ(bid) == t_VEC && lg(bid) > 2) return gel(bid,2);
     }
+  }
   member_err("zkst");
   return NULL; /* not reached */
 }
@@ -379,36 +372,36 @@ member_zkst(GEN bid)
 GEN
 member_no(GEN clg) /* number of elements of a group (of type clgp) */
 {
+  pari_sp av = avma;
   clg = member_clgp(clg);
-  if (typ(clg)!=t_VEC  || (lg(clg)!=3 && lg(clg)!=4))
-    member_err("no");
-  return gel(clg,1);
+  if (typ(clg)!=t_VEC || (lg(clg)!=3 && lg(clg)!=4)) member_err("no");
+  avma = av; return gel(clg,1);
 }
 
 GEN
 member_cyc(GEN clg) /* cyclic decomposition (SNF) of a group (of type clgp) */
 {
+  pari_sp av = avma;
   clg = member_clgp(clg);
-  if (typ(clg)!=t_VEC  || (lg(clg)!=3 && lg(clg)!=4))
-    member_err("cyc");
-  return gel(clg,2);
+  if (typ(clg)!=t_VEC || (lg(clg)!=3 && lg(clg)!=4)) member_err("cyc");
+  avma = av; return gel(clg,2);
 }
 
 /* SNF generators of a group (of type clgp), or generators of a prime
- * ideal
- */
+ * ideal */
 GEN
 member_gen(GEN x)
 {
+  pari_sp av;
   long t;
-  GEN y = get_primeid(x);
+  GEN y = get_prid(x);
   if (y) return mkvec2(gel(y,1), gel(y,2));
   (void)get_nf(x,&t);
-  if (t == typ_GAL)
-    return gel(x,7);
+  if (t == typ_GAL) return gal_get_gen(x);
+  av = avma;
   x = member_clgp(x);
-  if (typ(x)!=t_VEC || lg(x)!=4)
-    member_err("gen");
+  if (typ(x)!=t_VEC || lg(x)!=4) member_err("gen");
+  avma = av;
   if (typ(x[1]) == t_COL) return gel(x,2); /* from bnfisprincipal */
   return gel(x,3);
 }
@@ -416,8 +409,7 @@ GEN
 member_group(GEN x)
 {
   long t; (void)get_nf(x,&t);
-  if (t == typ_GAL)
-    return gel(x,6);
+  if (t == typ_GAL) return gal_get_group(x);
   member_err("group");
   return NULL; /* not reached */
 }
@@ -425,8 +417,7 @@ GEN
 member_orders(GEN x)
 {
   long t; (void)get_nf(x,&t);
-  if (t == typ_GAL)
-    return gel(x,8);
+  if (t == typ_GAL) return gal_get_orders(x);
   member_err("orders");
   return NULL; /* not reached */
 }
@@ -434,123 +425,125 @@ member_orders(GEN x)
 GEN
 member_a1(GEN x)
 {
-  if (!is_ell(x)) member_err("a1");
-  return gel(x,1);
+  if (!is_ell5(x)) member_err("a1");
+  return ell_get_a1(x);
 }
 
 GEN
 member_a2(GEN x)
 {
-  if (!is_ell(x)) member_err("a2");
-  return gel(x,2);
+  if (!is_ell5(x)) member_err("a2");
+  return ell_get_a2(x);
 }
 
 GEN
 member_a3(GEN x)
 {
-  if (!is_ell(x)) member_err("a3");
-  return gel(x,3);
+  if (!is_ell5(x)) member_err("a3");
+  return ell_get_a3(x);
 }
 
 GEN
 member_a4(GEN x)
 {
-  if (!is_ell(x)) member_err("a4");
-  return gel(x,4);
+  if (!is_ell5(x)) member_err("a4");
+  return ell_get_a4(x);
 }
 
 GEN
 member_a6(GEN x)
 {
-  if (!is_ell(x)) member_err("a6");
-  return gel(x,5);
+  if (!is_ell5(x)) member_err("a6");
+  return ell_get_a6(x);
 }
 
 GEN
 member_b2(GEN x)
 {
-  if (!is_ell(x)) member_err("b2");
-  return gel(x,6);
+  if (!is_smallell(x)) member_err("b2");
+  return ell_get_b2(x);
 }
 
 GEN
 member_b4(GEN x)
 {
-  if (!is_ell(x)) member_err("b4");
-  return gel(x,7);
+  if (!is_smallell(x)) member_err("b4");
+  return ell_get_b4(x);
 }
 
 GEN
 member_b6(GEN x)
 {
-  if (!is_ell(x)) member_err("b6");
-  return gel(x,8);
+  if (!is_smallell(x)) member_err("b6");
+  return ell_get_b6(x);
 }
 
 GEN
 member_b8(GEN x)
 {
-  if (!is_ell(x)) member_err("b8");
-  return gel(x,9);
+  if (!is_smallell(x)) member_err("b8");
+  return ell_get_b8(x);
 }
 
 GEN
 member_c4(GEN x)
 {
-  if (!is_ell(x)) member_err("c4");
-  return gel(x,10);
+  if (!is_smallell(x)) member_err("c4");
+  return ell_get_c4(x);
 }
 
 GEN
 member_c6(GEN x)
 {
-  if (!is_ell(x)) member_err("c6");
-  return gel(x,11);
+  if (!is_smallell(x)) member_err("c6");
+  return ell_get_c6(x);
 }
 
 GEN
 member_j(GEN x)
 {
-  if (!is_ell(x)) member_err("j");
-  return gel(x,13);
+  if (!is_smallell(x)) member_err("j");
+  return ell_get_j(x);
 }
 
 GEN
 member_omega(GEN x)
 {
-  if (!is_bigell(x)) member_err("omega");
-  if (gcmp0(gel(x,19))) pari_err(talker,"curve not defined over R");
+  if (!is_ell(x)) member_err("omega");
+  if (!ell_is_real(x)) pari_err(talker,"curve not defined over R");
   return mkvec2(gel(x,15), gel(x,16));
 }
 
 GEN
 member_eta(GEN x)
 {
-  if (!is_bigell(x)) member_err("eta");
-  if (gcmp0(gel(x,19))) pari_err(talker,"curve not defined over R");
+  if (!is_ell(x)) member_err("eta");
+  if (!ell_is_real(x)) pari_err(talker,"curve not defined over R");
   return mkvec2(gel(x,17), gel(x,18));
 }
 
 GEN
 member_area(GEN x)
 {
-  if (!is_bigell(x)) member_err("area");
-  if (gcmp0(gel(x,19))) pari_err(talker,"curve not defined over R");
+  if (!is_ell(x)) member_err("area");
+  if (!ell_is_real(x)) pari_err(talker,"curve not defined over R");
   return gel(x,19);
 }
 
 GEN
 member_tate(GEN x)
 {
-  if (!is_bigell(x)) member_err("tate");
-  if (!gcmp0(gel(x,19))) pari_err(talker,"curve not defined over a p-adic field");
+  if (!is_ell(x)) member_err("tate");
+  if (!ell_is_padic(x))
+    pari_err(talker,"curve not defined over a p-adic field");
   return mkvec3(gel(x,15), gel(x,16), gel(x,17));
 }
 
 GEN
 member_w(GEN x)
 {
-  if (!is_bigell(x)) member_err("w");
-  if (!gcmp0(gel(x,19))) pari_err(talker,"curve not defined over a p-adic field");
+  if (!is_ell(x)) member_err("w");
+  if (!ell_is_padic(x))
+    pari_err(talker,"curve not defined over a p-adic field");
   return gel(x,18);
 }
