@@ -1,4 +1,4 @@
-/* $Id: aprcl.c 7857 2006-04-11 17:28:55Z kb $
+/* $Id: aprcl.c 13145 2011-04-20 21:56:40Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -36,7 +36,7 @@ typedef struct Red {
 
 typedef struct Cache { /* data associated to p^k */
   GEN aall, tall;
-  GEN cyc; 
+  GEN cyc;
   GEN E;
   GEN eta;
   GEN matvite, matinvvite;
@@ -71,21 +71,21 @@ red_cyclop(GEN T, long p)
   if (d <= -2) return T;
 
   /* reduce mod (x^p - 1) */
-  y = shallowcopy(T); z = (GEN*)(y+2);
+  y = leafcopy(T); z = (GEN*)(y+2);
   for (i = 0; i<=d; i++) z[i] = addii(z[i], z[i+p]);
 
   /* reduce mod x^(p-1) + ... + 1 */
   d = p-1;
   if (signe(z[d]))
     for (i=0; i<d; i++) z[i] = subii(z[i], z[d]);
-  return normalizepol_i(y, d+2);
+  return normalizepol_lg(y, d+2);
 }
 
 /* x t_VECSMALL, as red_cyclo2n_ip */
 static GEN
 u_red_cyclo2n_ip(GEN x, long n)
 {
-  long i, pow2 = 1<<(n-1);
+  long i, pow2 = 1L<<(n-1);
   GEN z;
 
   for (i = lg(x)-1; i>pow2; i--) x[i-pow2] -= x[i];
@@ -100,13 +100,13 @@ u_red_cyclo2n_ip(GEN x, long n)
 static GEN
 red_cyclo2n_ip(GEN x, long n)
 {
-  long i, pow2 = 1<<(n-1);
+  long i, pow2 = 1L<<(n-1);
   for (i = lg(x)-1; i>pow2+1; i--)
     if (signe(x[i])) gel(x,i-pow2) = subii(gel(x,i-pow2), gel(x,i));
-  return normalizepol_i(x, i+1);
+  return normalizepol_lg(x, i+1);
 }
 static GEN
-red_cyclo2n(GEN x, long n) { return red_cyclo2n_ip(shallowcopy(x), n); }
+red_cyclo2n(GEN x, long n) { return red_cyclo2n_ip(leafcopy(x), n); }
 
 /* x a non-zero VECSMALL */
 static GEN
@@ -125,7 +125,7 @@ smallpolrev(GEN x)
 /* x polynomial in t_VECSMALL form, T t_POL return x mod T */
 static GEN
 u_red(GEN x, GEN T) {
-  return grem(smallpolrev(x), T);
+  return RgX_rem(smallpolrev(x), T);
 }
 
 /* special case R->C = polcyclo(2^n) */
@@ -239,35 +239,40 @@ sqrmod5(GEN pol, Red *R)
 static GEN
 _mul(GEN x, GEN y, Red *R) { return R->red(gmul(x,y), R); }
 
-/* jac^floor(N/pk) mod (N, cyclo(pk)), flexible window */
+/* jac^floor(N/pk) mod (N, polcyclo(pk)), flexible window */
 static GEN
 _powpolmod(Cache *C, GEN jac, Red *R, GEN (*_sqr)(GEN, Red *))
 {
   const GEN taba = C->aall;
   const GEN tabt = C->tall;
   const long efin = lg(taba)-1, lv = R->lv;
-  GEN vz, res = jac, pol2 = _sqr(res, R);
+  GEN L, res = jac, pol2 = _sqr(res, R);
   long f;
-  pari_sp av;
+  pari_sp av0 = avma, av, lim;
 
-  vz = cgetg(lv+1, t_VEC); gel(vz,1) = res;
-  for (f=2; f<=lv; f++) gel(vz,f) = _mul(gel(vz,f-1), pol2, R);
-  av = avma;
+  L = cgetg(lv+1, t_VEC); gel(L,1) = res;
+  for (f=2; f<=lv; f++) gel(L,f) = _mul(gel(L,f-1), pol2, R);
+  av = avma; lim = stack_lim(av, 1);
   for (f = efin; f >= 1; f--)
   {
-    GEN t = (GEN)vz[taba[f]];
+    GEN t = gel(L, taba[f]);
     long tf = tabt[f];
-    res = f==efin ? t
-                  : _mul(t, res, R);
-    while (tf--) res = _sqr(res, R);
-    if ((f&7) == 0) res = gerepilecopy(av, res);
+    res = (f==efin)? t: _mul(t, res, R);
+    while (tf--) {
+      res = _sqr(res, R);
+      if (low_stack(lim, stack_lim(av,1))) {
+        res = gerepilecopy(av, res);
+        if(DEBUGMEM>1) pari_warn(warnmem,"powpolmod: f = %ld",f);
+      }
+    }
   }
-  return res;
+  return gerepilecopy(av0, res);
 }
 
 static GEN
 _powpolmodsimple(Cache *C, Red *R, GEN jac)
 {
+  pari_sp av = avma;
   GEN w = mulmat_pol(C->matvite, jac);
   long j, ph = lg(w);
 
@@ -275,6 +280,7 @@ _powpolmodsimple(Cache *C, Red *R, GEN jac)
   for (j=1; j<ph; j++)
     gel(w,j) = _powpolmod(C, centermodii(gel(w,j), R->N, R->N2), R, &sqrmod);
   w = centermod_i( gmul(C->matinvvite, w), R->N, R->N2 );
+  w = gerepileupto(av, w);
   return RgV_to_RgX(w, 0);
 }
 
@@ -317,7 +323,7 @@ e(ulong t, GEN *globfa)
   E = gel(fa,2); lfa = lg(P);
   nbd = 1;
   for (i=1; i<lfa; i++) { E[i]++; nbd *= E[i]; }
-  Primes = cget1(nbd + 1, t_VECSMALL);
+  Primes = vecsmalltrunc_init(nbd + 1);
   s = gen_2; /* nbd = number of divisors */
   for (k=0; k<nbd; k++)
   {
@@ -330,7 +336,8 @@ e(ulong t, GEN *globfa)
     /* d runs through the divisors of t */
     if (uisprime(++d))
     {
-      if (d != 2) appendL(Primes, (GEN)d);
+      if (d > 5000000) return gen_0;
+      if (d != 2) vecsmalltrunc_append(Primes, d);
       s = muliu(s, upowuu(d, 1 + u_lval(t,d)));
     }
   }
@@ -342,11 +349,9 @@ static ulong
 compt(GEN N)
 {
   pari_sp av0 = avma;
-  ulong C, t;
+  double Cd = 100 * rtodbl(logr_abs(itor(N,DEFAULTPREC))) / log(10.);
+  ulong t, C = (ulong)ceil(Cd);
   GEN B;
-
-  B = mulsr(100, divrr(glog(N,DEFAULTPREC), dbltor(log(10.))));
-  C = itos( gceil(B) );
   avma = av0;
   /* C < [200*log_10 e(t)] ==> return t. For e(t) < 10^529, N < 10^1058 */
   if (C <    540) return        6;
@@ -418,47 +423,50 @@ compt(GEN N)
   avma = av0; return t;
 }
 
-/* tabdl[i] = discrete log of i+1 in (Z/q)^*, q odd prime */
+/* T[i] = discrete log of i in (Z/q)^*, q odd prime
+ * To save on memory, compute half the table: T[q-x] = T[x] + (q-1)/2 */
 static GEN
 computetabdl(ulong q)
 {
-  GEN v = cgetg(q-1,t_VECSMALL), w = v-1; /* w[i] = dl(i) */
-  ulong g,qm3s2,qm1s2,a,i;
+  ulong g, a, i, qs2 = q>>1; /* (q-1)/2 */
+  GEN T = cgetg(qs2+2,t_VECSMALL);
 
-  g = gener_Fl(q);
-  qm3s2 = (q-3)>>1;
-  qm1s2 = qm3s2+1;
-  w[q-1] = qm1s2; a = 1;
-  for (i=1; i<=qm3s2; i++)
+  g = pgener_Fl(q); a = 1;
+  for (i=1; i < qs2; i++) /* g^((q-1)/2) = -1 */
   {
     a = Fl_mul(g,a,q);
-    w[a]   = i;
-    w[q-a] = i+qm1s2;
+    if (a > qs2) T[q-a] = i+qs2; else T[a] = i;
   }
-  return v;
+  T[qs2+1] = T[qs2] + qs2;
+  T[1] = 0; return T;
 }
 
-static void
-compute_fg(ulong q, long half, GEN *tabf, GEN *tabg)
+/* Return T: T[x] = dl of x(1-x) */
+static GEN
+compute_g(ulong q)
 {
-  const ulong qm3s2 = (q-3)>>1;
-  const ulong l = half? qm3s2: q-2;
-  ulong x;
-  *tabf = computetabdl(q);
-  *tabg = cgetg(l+1,t_VECSMALL);
-  for (x=1; x<=l; x++) (*tabg)[x] = (*tabf)[x] + (*tabf)[q-x-1];
+  const ulong qs2 = q>>1; /* (q-1)/2 */
+  ulong x, a;
+  GEN T = computetabdl(q); /* updated in place to save on memory */
+  a = 0; /* dl[1] */
+  for (x=2; x<=qs2+1; x++)
+  { /* a = dl(x) */
+    ulong b = T[x]; /* = dl(x) */
+    T[x] = b + a + qs2; /* dl(x) + dl(x-1) + dl(-1) */
+    a = b;
+  }
+  return T;
 }
 
 /* p odd prime */
 static GEN
-get_jac(Cache *C, ulong q, long pk, GEN tabf, GEN tabg)
+get_jac(Cache *C, ulong q, long pk, GEN tabg)
 {
-  ulong x, qm3s2;
+  ulong x, qs2 = q>>1; /* (q-1)/2 */
   GEN vpk = const_vecsmall(pk, 0);
 
-  qm3s2 = (q-3)>>1;
-  for (x=1; x<=qm3s2; x++) vpk[ tabg[x]%pk + 1 ] += 2;
-  vpk[ (2*tabf[qm3s2+1])%pk + 1 ]++;
+  for (x=2; x<=qs2; x++) vpk[ tabg[x]%pk + 1 ] += 2;
+  vpk[ tabg[x]%pk + 1 ]++; /* x = (q+1)/2 */
   return u_red(vpk, C->cyc);
 }
 
@@ -466,19 +474,21 @@ get_jac(Cache *C, ulong q, long pk, GEN tabf, GEN tabg)
 static GEN
 get_jac2(GEN N, ulong q, long k, GEN *j2q, GEN *j3q)
 {
-  GEN jpq, vpk, tabf, tabg;
-  ulong x, pk, i, qm3s2;
+  GEN jpq, vpk, T;
+  ulong x, pk, i, qs2;
 
   if (k == 1) return NULL;
 
-  compute_fg(q,0, &tabf,&tabg);
-
-  pk = 1 << k;;
+  T = computetabdl(q);
+  /* could store T[x+1] + T[x] + qs2 (cf compute_g).
+   * Recompute instead, saving half the memory. */
+  pk = 1UL << k;;
   vpk = const_vecsmall(pk, 0);
 
-  qm3s2 = (q-3)>>1;
-  for (x=1; x<=qm3s2; x++) vpk[ tabg[x]%pk + 1 ] += 2;
-  vpk[ (2*tabf[qm3s2+1])%pk + 1 ]++;
+  qs2 = q>>1; /* (q-1)/2 */
+
+  for (x=2; x<=qs2; x++) vpk[ (T[x]+T[x-1]+qs2)%pk + 1 ] += 2;
+  vpk[ (T[x]+T[x-1]+qs2)%pk + 1 ]++;
   jpq = u_red_cyclo2n_ip(vpk, k);
 
   if (k == 2) return jpq;
@@ -487,14 +497,16 @@ get_jac2(GEN N, ulong q, long k, GEN *j2q, GEN *j3q)
   {
     GEN v8 = cgetg(9,t_VECSMALL);
     for (x=1; x<=8; x++) v8[x] = 0;
-    for (x=1; x<=q-2; x++) v8[ ((2*tabf[x]+tabg[x])&7) + 1 ]++;
-    *j2q = polinflate(gsqr(u_red_cyclo2n_ip(v8,3)), pk>>3);
+    for (x=2; x<=qs2; x++) v8[ ((3*T[x]+T[x-1]+qs2)&7) + 1 ]++;
+    for (   ; x<=q-1; x++) v8[ ((3*T[q-x]+T[q-x+1]-3*qs2)&7) + 1 ]++;
+    *j2q = RgX_inflate(gsqr(u_red_cyclo2n_ip(v8,3)), pk>>3);
     *j2q = red_cyclo2n_ip(*j2q, k);
   }
   else *j2q = NULL;
 
   for (i=1; i<=pk; i++) vpk[i] = 0;
-  for (x=1; x<=q-2; x++) vpk[ (tabf[x]+tabg[x])%pk + 1 ]++;
+  for (x=2; x<=qs2; x++) vpk[ (2*T[x]+T[x-1]+qs2)%pk + 1 ]++;
+  for (   ; x<=q-1; x++) vpk[ (2*T[q-x]+T[q-x+1]-2*qs2)%pk + 1 ]++;
   *j3q = gmul(jpq, u_red_cyclo2n_ip(vpk,k));
   *j3q = red_cyclo2n_ip(*j3q, k);
   return jpq;
@@ -503,7 +515,7 @@ get_jac2(GEN N, ulong q, long k, GEN *j2q, GEN *j3q)
 static void
 calcjac(Cache **pC, GEN globfa, GEN *ptabfaq, GEN *ptabj)
 {
-  GEN J, tabf, tabg, faq, tabfaq, tabj, P, E, PE;
+  GEN J, tabg, faq, tabfaq, tabj, P, PE;
   long lfaq, j;
   ulong i, q, l;
   pari_sp av;
@@ -516,17 +528,16 @@ calcjac(Cache **pC, GEN globfa, GEN *ptabfaq, GEN *ptabj)
     q = globfa[i]; /* odd prime */
     faq = factoru_pow(q-1); gel(tabfaq,i) = faq;
     P = gel(faq,1); lfaq = lg(P);
-    E = gel(faq,2);
     PE= gel(faq,3);
     av = avma;
-    compute_fg(q, 1, &tabf, &tabg);
+    tabg = compute_g(q);
 
     J = cgetg(lfaq,t_VEC);
     gel(J,1) = cgetg(1,t_STR); /* dummy */
     for (j=2; j<lfaq; j++) /* skip p = P[1] = 2 */
     {
       long pe = PE[j];
-      gel(J,j) = get_jac(pC[pe], q, pe, tabf, tabg);
+      gel(J,j) = get_jac(pC[pe], q, pe, tabg);
     }
     gel(tabj,i) = gerepilecopy(av, J);
   }
@@ -546,14 +557,14 @@ finda(Cache *Cp, GEN N, long pk, long p)
     GEN ph, b, q;
     ulong u = 2;
     long v = Z_lvalrem(addis(N,-1), p, &q);
-    ph = powuu(p, v-1); pv = mulis(ph, p); /* N - 1 = p^v q */ 
+    ph = powuu(p, v-1); pv = muliu(ph, p); /* N - 1 = p^v q */
     if (p > 2)
     {
       for (;;u++)
       {
-	a = Fp_pow(utoipos(u), q, N);
-	b = Fp_pow(a, ph, N);
-	if (!gcmp1(b)) break;
+        a = Fp_pow(utoipos(u), q, N);
+        b = Fp_pow(a, ph, N);
+        if (!gequal1(b)) break;
       }
     }
     else
@@ -564,8 +575,8 @@ finda(Cache *Cp, GEN N, long pk, long p)
     }
     /* checking b^p = 1 mod N done economically in caller */
     b = gcdii(addis(b,-1), N);
-    if (!gcmp1(b)) return NULL;
-    
+    if (!gequal1(b)) return NULL;
+
     if (Cp) {
       Cp->avite  = a; /* a has order p^v */
       Cp->pkvite = pv;
@@ -583,7 +594,7 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
   long e;
   GEN tabt, taba, m;
 
-  C->cyc = cyclo(pk,0);
+  C->cyc = polcyclo(pk,0);
 
   if (p > 2)
   {
@@ -596,7 +607,7 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
     for (i=1; i<=pk; i++)
     {
       GEN z = FpX_rem(monomial(gen_1, i-1, 0), C->cyc, R->N);
-      gel(eta,i) = centermod_i(z, R->N, R->N2);
+      gel(eta,i) = FpX_center(z, R->N, R->N2);
     }
     C->eta = eta;
   }
@@ -613,18 +624,18 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
   {
     GEN vpa, p1, p2, p3, a2 = NULL, a = finda(Cp, R->N, pk, p);
     long jj, ph;
-    
+
     if (!a) return 0;
     ph = pk - pk/p;
     vpa = cgetg(ph+1,t_COL); gel(vpa,1) = a;
     if (pk > p) a2 = centermodii(sqri(a), R->N, R->N2);
     jj = 1;
     for (i=2; i<pk; i++) /* vpa = { a^i, (i,p) = 1 } */
-      if (i%p) { 
+      if (i%p) {
         GEN z = mulii((i%p==1) ? a2 : a, gel(vpa,jj));
         gel(vpa,++jj) = centermodii(z , R->N, R->N2);
       }
-    if (!gcmp1( centermodii( mulii(a, gel(vpa,ph)), R->N, R->N2) )) return 0;
+    if (!gequal1( centermodii( mulii(a, gel(vpa,ph)), R->N, R->N2) )) return 0;
     p1 = cgetg(ph+1,t_MAT);
     p2 = cgetg(ph+1,t_COL); gel(p1,1) = p2;
     for (i=1; i<=ph; i++) gel(p2,i) = gen_1;
@@ -632,7 +643,7 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
     for (j++; j <= ph; j++)
     {
       p2 = cgetg(ph+1,t_COL); gel(p1,j) = p2;
-      for (i=1; i<=ph; i++) 
+      for (i=1; i<=ph; i++)
         gel(p2,i) = centermodii(mulii(gel(vpa,i),gel(p3,i)), R->N, R->N2);
       p3 = p2;
     }
@@ -647,7 +658,7 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
   {
     long s = vali(m); m = shifti(m,-s);
     tabt[e] = e==1? s: s + R->k;
-    taba[e] = signe(m)? ((modBIL(m) & R->mask)+1)>>1: 0;
+    taba[e] = signe(m)? ((mod2BIL(m) & R->mask)+1)>>1: 0;
     m = shifti(m, -R->k);
   }
   setlg(taba, e); C->aall = taba;
@@ -656,11 +667,11 @@ filltabs(Cache *C, Cache *Cp, Red *R, long p, long pk, long ltab)
 }
 
 static Cache *
-alloc_cache()
+alloc_cache(void)
 {
   Cache *C = (Cache*)new_chunk(sizeof(Cache) / sizeof(long));
   C->matvite = NULL;
-  C->avite   = NULL; 
+  C->avite   = NULL;
   C->ctsgt = 0; return C;
 }
 
@@ -679,7 +690,7 @@ calcglobs(Red *R, ulong t, long *pltab, GEN *pP)
   k = 3; while (((k+1)*(k+2) << (k-1)) < b) k++;
   *pltab = ltab = (b/k) + 2;
   R->k  = k;
-  R->lv = 1 << (k-1);
+  R->lv = 1L << (k-1);
   R->mask = (1UL << k) - 1;
 
   fat = factoru_pow(t);
@@ -705,20 +716,29 @@ calcglobs(Red *R, ulong t, long *pltab, GEN *pP)
       if (!filltabs(pC[pk], pC[p], R, p,pk, ltab)) return NULL;
     }
   }
-  if (DEBUGLEVEL) fprintferr("\n");
+  if (DEBUGLEVEL) err_printf("\n");
   *pP = P;
   return pC;
 }
 
-/* sig_a^{-1}(z) for z in Q(ze) and sig_a: ze -> ze^a */
+/* sig_a^{-1}(z) for z in Q(zeta_pk) and sig_a: zeta -> zeta^a. Assume
+ * a reduced mod pk := p^k*/
 static GEN
 aut(long pk, GEN z, long a)
 {
-  GEN v = cgetg(pk+1,t_VEC);
-  long i;
-  for (i=1; i<=pk; i++)
-    gel(v,i) = polcoeff0(z, (a*(i-1))%pk, 0);
-  return gtopolyrev(v,0);
+  GEN v;
+  long b, i, dz = degpol(z);
+  if (a == 1 || dz < 0) return z;
+  v = cgetg(pk+2,t_POL);
+  v[1] = evalvarn(0);
+  b = 0;
+  gel(v,2) = gel(z,2); /* i = 0 */
+  for (i = 1; i < pk; i++)
+  {
+    b += a; if (b > pk) b -= pk; /* b = (a*i) % pk */
+    gel(v,i+2) = b > dz? gen_0: gel(z,b+2);
+  }
+  return normalizepol_lg(v, pk+2);
 }
 
 /* z^v for v in Z[G], represented by couples [sig_x^{-1},x] */
@@ -726,11 +746,11 @@ static GEN
 autvec_TH(long pk, GEN z, GEN v, GEN C)
 {
   long i, lv = lg(v);
-  GEN s = pol_1[varn(C)];
+  GEN s = pol_1(varn(C));
   for (i=1; i<lv; i++)
   {
     long y = v[i];
-    if (y) s = RgXQ_mul(s, RgXQ_u_pow(aut(pk,z, y), y, C), C);
+    if (y) s = RgXQ_mul(s, RgXQ_powu(aut(pk,z, y), y, C), C);
   }
   return s;
 }
@@ -739,23 +759,23 @@ static GEN
 autvec_AL(long pk, GEN z, GEN v, Red *R)
 {
   const long r = umodiu(R->N, pk);
-  GEN s = pol_1[varn(R->C)];
+  GEN s = pol_1(varn(R->C));
   long i, lv = lg(v);
   for (i=1; i<lv; i++)
   {
     long y = (r*v[i]) / pk;
-    if (y) s = RgXQ_mul(s, RgXQ_u_pow(aut(pk,z, v[i]), y, R->C), R->C);
+    if (y) s = RgXQ_mul(s, RgXQ_powu(aut(pk,z, v[i]), y, R->C), R->C);
   }
   return s;
 }
 
-/* 0 <= i < pk, such that x^i = z mod cyclo(pk),  -1 if no such i exist */
+/* 0 <= i < pk, such that x^i = z mod polcyclo(pk),  -1 if no such i exist */
 static long
 look_eta(GEN eta, long pk, GEN z)
 {
   long i;
   for (i=1; i<=pk; i++)
-    if (gequal(z, gel(eta,i))) return i-1;
+    if (ZX_equal(z, gel(eta,i))) return i-1;
   return -1;
 }
 /* same pk = 2^k */
@@ -765,14 +785,14 @@ look_eta2(long k, GEN z)
   long d, s;
   if (typ(z) != t_POL) d = 0; /* t_INT */
   else
-  { 
-    if (!ismonome(z)) return -1;
+  {
+    if (!RgX_is_monomial(z)) return -1;
     d = degpol(z);
     z = gel(z,d+2); /* leading term */
   }
   s = signe(z);
   if (!s || !is_pm1(z)) return -1;
-  return (s > 0)? d: d + (1<<(k-1));
+  return (s > 0)? d: d + (1L<<(k-1));
 }
 
 static long
@@ -781,13 +801,8 @@ step4a(Cache *C, Red *R, ulong q, long p, long k, GEN jpq)
   const long pk = upowuu(p,k);
   long ind;
   GEN s1, s2, s3;
-  
-  if (!jpq)
-  {
-    GEN tabf, tabg;
-    compute_fg(q,1, &tabf,&tabg);
-    jpq = get_jac(C, q, pk, tabf, tabg);
-  }
+
+  if (!jpq) jpq = get_jac(C, q, pk, compute_g(q));
   s1 = autvec_TH(pk, jpq, C->E, C->cyc);
   s2 = powpolmod(C,R, p,k, s1);
   s3 = autvec_AL(pk, jpq, C->E, R);
@@ -809,7 +824,7 @@ is_m1(GEN x, GEN N)
 static long
 step4b(Cache *C, Red *R, ulong q, long k)
 {
-  const long pk = 1 << k;
+  const long pk = 1L << k;
   long ind;
   GEN s1, s2, s3, j2q, j3q;
 
@@ -853,7 +868,7 @@ step4c(Cache *C, Red *R, ulong q)
 static long
 step4d(Cache *C, Red *R, ulong q)
 {
-  GEN s1 = Fp_pow(utoipos(q), R->N2, R->N); 
+  GEN s1 = Fp_pow(utoipos(q), R->N2, R->N);
   if (DEBUGLEVEL>2) C->ctsgt++;
   if (is_pm1(s1)) return 0;
   if (is_m1(s1, R->N)) return (mod4(R->N) == 1);
@@ -864,6 +879,7 @@ step4d(Cache *C, Red *R, ulong q)
 static long
 step5(Cache **pC, Red *R, long p, GEN et, ulong ltab)
 {
+  pari_sp av;
   ulong ct = 1, q;
   long pk, k, fl = -1;
   byteptr d = diffptr+2;
@@ -881,15 +897,16 @@ step5(Cache **pC, Red *R, long p, GEN et, ulong ltab)
       C = pC[1]; C->matvite = NULL; /* re-init */
       Cp = NULL;
     }
-    
-    if (!filltabs(C, Cp, R, p, pk, ltab)) return 0; 
-    R->C = C->cyc;
+
+    if (!filltabs(C, Cp, R, p, pk, ltab)) return 0;
+    av = avma; R->C = C->cyc;
     if (p >= 3)      fl = step4a(C,R, q,p,k, NULL);
     else if (k >= 3) fl = step4b(C,R, q,k);
     else if (k == 2) fl = step4c(C,R, q);
     else             fl = step4d(C,R, q);
     if (fl == -1) return (long)(-q);
     if (fl == 1) return ct;
+    avma = av;
     ct++;
    repeat:
     NEXT_PRIME_VIADIFF(q,d);
@@ -909,7 +926,7 @@ step6(GEN N, ulong t, GEN et)
   for (i=1; i<t; i++)
   {
     r = remii(mulii(r,N1), et);
-    if (gcmp1(r)) break;
+    if (gequal1(r)) break;
     if (!signe(remii(N,r)) && !equalii(r,N)) return mkvec2(r, gen_0);
     if ((i & 0x1f) == 0) r = gerepileuptoint(av, r);
   }
@@ -919,16 +936,17 @@ step6(GEN N, ulong t, GEN et)
 static GEN
 _res(long a, long b) { return b? mkvec2s(a, b): mkvecs(a); }
 
-GEN
+static GEN
 aprcl(GEN N)
 {
   GEN et, fat, flaglp, tabfaq, tabj, res, globfa;
   long i, j, l, ltab, lfat, fl, ctglob = 0;
   ulong p, q, t;
-  pari_sp av, av2;
+  pari_sp av;
   Red R;
   Cache **pC;
 
+  if (typ(N) != t_INT) pari_err(arither1);
   if (cmpis(N,12) <= 0)
     switch(itos(N))
     {
@@ -937,14 +955,14 @@ aprcl(GEN N)
     }
   if (Z_issquare(N)) return _res(0,0);
   t = compt(N);
-  if (DEBUGLEVEL) fprintferr("Choosing t = %ld\n",t);
+  if (DEBUGLEVEL) err_printf("Starting APRCL: Choosing t = %ld\n",t);
   et = e(t, &globfa);
   if (cmpii(sqri(et),N) < 0) pari_err(bugparier,"aprcl: e(t) too small");
-  if (!gcmp1(gcdii(N,mulsi(t,et)))) return _res(1,0);
+  if (!gequal1(gcdii(N,mului(t,et)))) return _res(1,0);
 
   R.N = N;
   R.N2= shifti(N, -1);
-  pC = calcglobs(&R, t, &ltab, &fat); 
+  pC = calcglobs(&R, t, &ltab, &fat);
   if (!pC) return _res(1,0);
   lfat = lg(fat); p = fat[lfat-1]; /* largest p | t */
   flaglp = cgetg(p+1, t_VECSMALL);
@@ -952,24 +970,22 @@ aprcl(GEN N)
   for (i=2; i<lfat; i++)
   {
     p = fat[i]; q = p*p;
-    flaglp[p] = (Fl_pow(umodiu(N,q),p-1,q) != 1);
+    flaglp[p] = (Fl_powu(umodiu(N,q),p-1,q) != 1);
   }
   calcjac(pC, globfa, &tabfaq, &tabj);
-  
+
   av = avma; l = lg(globfa);
   if (DEBUGLEVEL>2)
   {
-    fprintferr("Jacobi sums and tables computed\n");
-    fprintferr("Step4: q-values (# = %ld, largest = %ld): ", l-1, globfa[l-1]);
+    err_printf("Jacobi sums and tables computed\n");
+    err_printf("Step4: q-values (# = %ld): ", l-1);
   }
-  for (i=1; i<l; i++)
+  for (i=l-1; i>0; i--)
   {
     GEN faq = gel(tabfaq,i), P = gel(faq,1), E = gel(faq,2), PE = gel(faq,3);
     long lfaq = lg(P);
-    avma = av;
-    q = globfa[i]; if (DEBUGLEVEL>2) fprintferr("%ld ",q);
-    av2 = avma;
-    for (j=1; j<lfaq; j++, avma = av2)
+    q = globfa[i]; if (DEBUGLEVEL>2) err_printf("%ld ",q);
+    for (j=1; j<lfaq; j++, avma = av)
     {
       Cache *C;
       long pe, e;
@@ -985,29 +1001,29 @@ aprcl(GEN N)
       if (fl == 1) flaglp[p] = 1;
     }
   }
-  if (DEBUGLEVEL>2) fprintferr("\nStep5: testing conditions lp\n");
+  if (DEBUGLEVEL>2) err_printf("\nStep5: testing conditions lp\n");
   for (i=1; i<lfat; i++)
   {
     p = fat[i]; if (flaglp[p]) continue;
 
     fl = step5(pC, &R, p, et, ltab);
     if (!fl) return _res(1,0);
-    if (fl < 0) return _res(fl,p); 
+    if (fl < 0) return _res(fl,p);
     if (fl > ctglob) ctglob = fl; /* DEBUG */
   }
-  if (DEBUGLEVEL>2) fprintferr("Step6: testing potential divisors\n");
+  if (DEBUGLEVEL>2) err_printf("Step6: testing potential divisors\n");
   res = step6(N, t, et);
   if (DEBUGLEVEL>2)
   {
     ulong sc = pC[1]->ctsgt;
-    fprintferr("Individual Fermat powerings:\n");
-    for (i=2; i<lg(pC); i++) 
+    err_printf("Individual Fermat powerings:\n");
+    for (i=2; i<lg(pC); i++)
       if (pC[i]) {
-        fprintferr("  %-3ld: %3ld\n", i, pC[i]->ctsgt);
+        err_printf("  %-3ld: %3ld\n", i, pC[i]->ctsgt);
         sc += pC[i]->ctsgt;
       }
-    fprintferr("Number of Fermat powerings = %lu\n",sc);
-    fprintferr("Maximal number of nondeterministic steps = %lu\n",ctglob);
+    err_printf("Number of Fermat powerings = %lu\n",sc);
+    err_printf("Maximal number of nondeterministic steps = %lu\n",ctglob);
   }
   return res;
 }
