@@ -1,4 +1,4 @@
-/* $Id: elldata.c 7791 2006-03-22 16:12:26Z bill $
+/* $Id: elldata.c 12888 2011-01-31 17:48:11Z bill $
 
 Copyright (C) 2005  The PARI group.
 
@@ -20,7 +20,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /********************************************************************/
 #include "pari.h"
 #include "paripriv.h"
-#include "../language/anal.h"
 
 static long
 strtoclass(const char *s)
@@ -28,8 +27,7 @@ strtoclass(const char *s)
   long c=0;
   while (*s && *s<='9') s++;
   if (!*s) return -1;
-  while ('a'<=*s && *s<='z')
-    c=26*c+*(s++)-'a';
+  while ('a'<=*s && *s<='z') c = 26*c + *(s++)-'a';
   return c;
 }
 
@@ -37,14 +35,13 @@ strtoclass(const char *s)
  * f to the conductor, (100)
  * c to the isogeny class (in base 26), ("a" or 0)
  * i to the curve index (2).
- * return 0 if garbage is found at the end.
- */
+ * return 0 if parse error. */
 static int
 ellparsename(const char *s, long *f, long *c, long *i)
 {
   long j;
   *f=-1; *c=-1; *i=-1;
-  if (*s<'0' || *s>'9') return !*s;
+  if (*s<'0' || *s>'9') return 0;
   *f=0;
   for (j=0;j<10 && '0'<=*s && *s<='9';j++)
     *f=10**f+*(s++)-'0';
@@ -69,24 +66,15 @@ ellrecode(long x)
   GEN str;
   char *s;
   long d = 0, n = x;
-  do
-  { 
-    d++;
-    n/=26;
-  } while(n);
+  do { d++; n /= 26; } while (n);
   str = cgetg(nchar2nlong(d+1)+1, t_STR);
-  s = GSTR(str);
-  s[d] = 0;
+  s = GSTR(str); s[d] = 0;
   n = x;
-  do
-  { 
-    s[--d] = n%26 + 'a';
-    n/=26;
-  } while(n);
+  do { s[--d] = n%26 + 'a'; n /= 26; } while (n);
   return str;
 }
 
-GEN 
+GEN
 ellconvertname(GEN n)
 {
   switch(typ(n))
@@ -96,6 +84,8 @@ ellconvertname(GEN n)
       long f,i,c;
       if (!ellparsename(GSTR(n),&f,&c,&i))
         pari_err(talker,"Incorrect curve name in ellconvertname");
+      if (f<0 || c<0 || i<0)
+        pari_err(talker,"Incomplete curve name in ellconvertname");
       return mkvec3s(f,c,i);
     }
   case t_VEC:
@@ -103,61 +93,59 @@ ellconvertname(GEN n)
       pari_err(talker,"Incorrect vector in ellconvertname");
     else
     {
-      pari_sp ltop=avma;
-      GEN f=gel(n, 1), c=gel(n, 2), s=gel(n, 3);
-      if (typ(f)!=t_INT && typ(c)!=t_INT && typ(s)!=t_INT)
+      pari_sp av = avma;
+      GEN f=gel(n,1), c=gel(n,2), s=gel(n,3);
+      if (typ(f)!=t_INT || typ(c)!=t_INT || typ(s)!=t_INT)
         pari_err(typeer,"ellconvertname");
-      return gerepileupto(ltop, concat(concat(f,ellrecode(itos(c))),s));
+      return gerepilecopy(av, shallowconcat1(mkvec3(f, ellrecode(itos(c)), s)));
     }
   }
   pari_err(typeer,"ellconvertname");
   return NULL; /*Not reached*/
 }
 
-GEN
+static GEN
 ellcondfile(long f)
 {
-  long n=f/1000;
-  char *s = gpmalloc(strlen(pari_datadir) + 13 + 20);
-  FILE *stream;
+  pari_sp av = avma;
+  long n = f / 1000;
+  char *s = stackmalloc(strlen(pari_datadir) + 12 + 20 + 1);
+  pariFILE *F;
   GEN V;
   sprintf(s, "%s/elldata/ell%ld", pari_datadir, n);
-  stream = fopen(s,"r");
-  if (!stream) 
-    pari_err(talker,"Elliptic curves files not available for conductor %ld\n"
-               "[missing %s]",f,s);
-  V = gp_read_stream(stream);
-  if (!V || typ(V)!=t_VEC )
-    pari_err(talker,"Elliptic files %s not compatible\n",s);
-  fclose(stream);
-  free(s); 
-  return V;
+  F = pari_fopengz(s); avma = av;
+  if (!F) pari_err(talker,"Missing elldata for conductor %ld\n[need %s]",f,s);
+  V = gp_read_stream(F->file);
+  if (!V || typ(V)!=t_VEC ) pari_err(talker,"Incompatible elldata file %s\n",s);
+  pari_fclose(F); return V;
 }
 
-GEN
+/* FIXME: could use a binary search */
+static GEN
 ellcondlist(long f)
 {
-  pari_sp ltop=avma;
-  GEN  V=ellcondfile(f);
+  GEN  v, V = ellcondfile(f);
   long i;
   for (i=1; i<lg(V); i++)
-    if (cmpis(gmael(V,i,1), f)>=0)
-      break;
-  if (i==lg(V) || !equalis(gmael(V,i,1), f))
   {
-    avma=ltop; 
-    return cgetg(1,t_VEC);
+    int cmp  = cmpis(gmael(V,i,1), f);
+    if (cmp >= 0) {
+      if (cmp) break;
+      v = gel(V,i); return vecslice(v,2, lg(v)-1);
+    }
   }
-  return gerepilecopy(ltop, vecslice(gel(V,i),2, lg(gel(V,i))-1)); 
+  return cgetg(1,t_VEC);
 }
 
-static GEN 
-ellsearchbyname(GEN V, GEN name)
+static GEN
+ellsearchbyname(GEN V, char *name)
 {
   long j;
   for (j=1; j<lg(V); j++)
-    if (gequal(gmael(V,j,1), name))
-      return gel(V,j);
+  {
+    GEN v = gel(V,j);
+    if (!strcmp(GSTR(gel(v,1)), name)) return v;
+  }
   pari_err(talker,"No such elliptic curve");
   return NULL;
 }
@@ -168,39 +156,32 @@ ellsearchbyclass(GEN V, long c)
   long i,j,n;
   GEN res;
   for (n=0,j=1; j<lg(V); j++)
-    if (strtoclass(GSTR(gmael(V,j,1)))==c)
-      n++;
-  res=cgetg(n+1,t_VEC);
+    if (strtoclass(GSTR(gmael(V,j,1)))==c) n++;
+  res = cgetg(n+1,t_VEC);
   for (i=1,j=1; j<lg(V); j++)
-    if (strtoclass(GSTR(gmael(V,j,1)))==c)
-      res[i++]=V[j];
+    if (strtoclass(GSTR(gmael(V,j,1)))==c) res[i++] = V[j];
   return res;
 }
 
 GEN
 ellsearch(GEN A)
 {
-  pari_sp ltop=avma;
+  pari_sp av = avma;
   long f, c, i;
   GEN V;
-  if (typ(A)==t_INT)
-  {
-    f=itos(A); 
-    c=-1; 
-    i=-1;
-  } else if (typ(A)==t_STR) {
+  if      (typ(A)==t_INT) { f = itos(A); c = i = -1; }
+  else if (typ(A)==t_STR) {
     if (!ellparsename(GSTR(A),&f,&c,&i))
       pari_err(talker,"Incorrect curve name in ellsearch");
   } else {
     pari_err(typeer,"ellsearch");
     return NULL;
   }
-  V=ellcondlist(f);
-  if (c<0) 
-    return V;
-  if (i<0) 
-    return gerepilecopy(ltop, ellsearchbyclass(V,c));
-  return gerepilecopy(ltop, ellsearchbyname(V,A));
+  if (f <= 0) pari_err(talker,"Non-positive conductor in ellsearch");
+  V = ellcondlist(f);
+  if (c >= 0)
+    V = (i < 0)? ellsearchbyclass(V,c): ellsearchbyname(V, GSTR(A));
+  return gerepilecopy(av, V);
 }
 
 GEN
@@ -210,9 +191,8 @@ ellsearchcurve(GEN name)
   long f, c, i;
   if (!ellparsename(GSTR(name),&f,&c,&i))
     pari_err(talker,"Incorrect curve name in ellsearch");
-  if (f<0 || c<0 || i<0)
-    pari_err(talker,"Incomplete curve name in ellsearch");
-  return gerepilecopy(ltop, ellsearchbyname(ellcondlist(f), name));
+  if (f<0 || c<0 || i<0) pari_err(talker,"Incomplete curve name in ellsearch");
+  return gerepilecopy(ltop, ellsearchbyname(ellcondlist(f), GSTR(name)));
 }
 
 GEN
@@ -222,9 +202,9 @@ ellidentify(GEN E)
   GEN V, M, G = ellglobalred(E);
   long j;
   V = ellcondlist(itos(gel(G,1)));
-  M = coordch(vecslice(E,1,5),gel(G,2));
+  M = ellchangecurve(vecslice(E,1,5),gel(G,2));
   for (j=1; j<lg(V); j++)
-    if (gequal(gmael(V,j,2), M))
+    if (ZV_equal(gmael(V,j,2), M))
       return gerepilecopy(ltop, mkvec2(gel(V,j),gel(G,2)));
   pari_err(talker,"No such elliptic curve in database");
   return NULL;
@@ -236,17 +216,17 @@ ellgenerators(GEN E)
   pari_sp ltop=avma;
   GEN V=ellidentify(E);
   GEN gens=gmael(V,1,3);
-  GEN W=pointchinv(gens,gel(V,2));
+  GEN W=ellchangepointinv(gens,gel(V,2));
   return gerepileupto(ltop,W);
 }
 
 void
-forell(entree *ep, long a, long b, char *ch)
+forell(void *E, long call(void*, GEN), long a, long b)
 {
   long ca=a/1000, cb=b/1000;
   long i, j, k;
 
-  push_val(ep, NULL);
+  if (ca < 0) ca = 0;
   for(i=ca; i<=cb; i++)
   {
     pari_sp ltop=avma;
@@ -255,18 +235,22 @@ forell(entree *ep, long a, long b, char *ch)
     {
       GEN ells = gel(V,j);
       long cond= itos(gel(ells,1));
-      
+
       if (i==ca && cond<a) continue;
       if (i==cb && cond>b) break;
       for(k=2; k<lg(ells); k++)
       {
-        ep->value = (void*)gel(ells, k);
-        readseq_void(ch); 
-        if (loop_break()) goto forell_end;
+        if (call(E, gel(ells, k))) return;
       }
     }
     avma = ltop;
   }
-  forell_end:
-  pop_val(ep);
+}
+
+void
+forell0(long a, long b, GEN code)
+{
+  push_lex(gen_0, code);
+  forell((void*)code, &gp_evalvoid, a, b);
+  pop_lex(1);
 }
