@@ -1,4 +1,4 @@
-/* $Id: subfield.c 10283 2008-06-09 11:31:42Z kb $
+/* $Id: subfield.c 13145 2011-04-20 21:56:40Z kb $
 
 Copyright (C) 2000-2004  The PARI group.
 
@@ -55,57 +55,6 @@ typedef struct _blockdata {
 static GEN print_block_system(blockdata *B, GEN Y, GEN BS);
 static GEN test_block(blockdata *B, GEN L, GEN D);
 
-/* return P(X + c) using destructive Horner, optimize for c = 1,-1 */
-GEN
-translate_pol(GEN P, GEN c)
-{
-  pari_sp av = avma, lim;
-  GEN Q, *R;
-  long i, k, n;
-
-  if (!signe(P) || gcmp0(c)) return gcopy(P);
-  Q = shallowcopy(P);
-  R = (GEN*)(Q+2); n = degpol(P);
-  lim = stack_lim(av, 2);
-  if (gcmp1(c))
-  {
-    for (i=1; i<=n; i++)
-    {
-      for (k=n-i; k<n; k++) R[k] = gadd(R[k], R[k+1]);
-      if (low_stack(lim, stack_lim(av,2)))
-      {
-        if(DEBUGMEM>1) pari_warn(warnmem,"TR_POL(1), i = %ld/%ld", i,n);
-        Q = gerepilecopy(av, Q); R = (GEN*)Q+2;
-      }
-    }
-  }
-  else if (gcmp_1(c))
-  {
-    for (i=1; i<=n; i++)
-    {
-      for (k=n-i; k<n; k++) R[k] = gsub(R[k], R[k+1]);
-      if (low_stack(lim, stack_lim(av,2)))
-      {
-        if(DEBUGMEM>1) pari_warn(warnmem,"TR_POL(-1), i = %ld/%ld", i,n);
-        Q = gerepilecopy(av, Q); R = (GEN*)Q+2;
-      }
-    }
-  }
-  else
-  {
-    for (i=1; i<=n; i++)
-    {
-      for (k=n-i; k<n; k++) R[k] = gadd(R[k], gmul(c, R[k+1]));
-      if (low_stack(lim, stack_lim(av,2)))
-      {
-        if(DEBUGMEM>1) pari_warn(warnmem,"TR_POL, i = %ld/%ld", i,n);
-        Q = gerepilecopy(av, Q); R = (GEN*)Q+2;
-      }
-    }
-  }
-  return gerepilecopy(av, Q);
-}
-
 /* COMBINATORIAL PART: generate potential block systems */
 
 #define BIL 32 /* for 64bit machines also */
@@ -122,14 +71,14 @@ calc_block(blockdata *B, GEN Z, GEN Y, GEN SB)
 
   if (DEBUGLEVEL>3)
   {
-    fprintferr("lg(Z) = %ld, lg(Y) = %ld\n", r,lg(Y));
+    err_printf("lg(Z) = %ld, lg(Y) = %ld\n", r,lg(Y));
     if (DEBUGLEVEL > 5)
     {
-      fprintferr("Z = %Z\n",Z);
-      fprintferr("Y = %Z\n",Y);
+      err_printf("Z = %Ps\n",Z);
+      err_printf("Y = %Ps\n",Y);
     }
   }
-  lnon = min(BIL, r);
+  lnon = minss(BIL, r);
   e    = new_chunk(BIL);
   n    = new_chunk(r);
   non  = new_chunk(lnon);
@@ -145,7 +94,7 @@ calc_block(blockdata *B, GEN Z, GEN Y, GEN SB)
   {
     pari_sp av = avma;
     long k = nn / B->size;
-    for (j = 1; j < r; j++) 
+    for (j = 1; j < r; j++)
       if (n[j] % k) break;
     if (j == r)
     {
@@ -165,10 +114,10 @@ calc_block(blockdata *B, GEN Z, GEN Y, GEN SB)
       {
         if (++lpn >= BIL) pari_err(talker,"overflow in calc_block");
         pn[lpn] = n[j]; pnon[lpn] = j;
-        ngcd = cgcd(ngcd, n[j]);
+        ngcd = ugcd(ngcd, n[j]);
       }
     if (dk % ngcd) continue;
-    T = 1<<lpn;
+    T = 1L<<lpn;
     if (lpn == r-2)
     {
       T--; /* done already above --> print_block_system */
@@ -244,16 +193,9 @@ cycle_power_to_perm(GEN perm,GEN cy,long l)
 static GEN
 im_block_by_perm(GEN D,GEN perm)
 {
-  long i,j,lb,lcy;
-  GEN Dn,cy,p1;
-
-  lb=lg(D); Dn=cgetg(lb,t_VEC);
-  for (i=1; i<lb; i++)
-  {
-    cy=gel(D,i); lcy=lg(cy);
-    gel(Dn,i) = cgetg(lcy,t_VECSMALL); p1=gel(Dn,i);
-    for (j=1; j<lcy; j++) p1[j] = perm[cy[j]];
-  }
+  long i, lb = lg(D);
+  GEN Dn = cgetg(lb,t_VEC);
+  for (i=1; i<lb; i++) gel(Dn,i) = vecpermute(perm, gel(D,i));
   return Dn;
 }
 
@@ -273,9 +215,9 @@ print_block_system(blockdata *B, GEN Y, GEN SB)
   long *k, *n, **e, *t;
   GEN D, De, Z, cyperm, perm, VOID = cgetg(1, t_VECSMALL);
 
-  if (DEBUGLEVEL>5) fprintferr("Y = %Z\n",Y);
+  if (DEBUGLEVEL>5) err_printf("Y = %Ps\n",Y);
   n = new_chunk(N+1);
-  D = cget1(N+1, t_VEC);
+  D = vectrunc_init(N+1);
   t = new_chunk(r+1);
   k = new_chunk(r+1);
   Z = cgetg(r+1, t_VEC);
@@ -308,7 +250,7 @@ print_block_system(blockdata *B, GEN Y, GEN SB)
       gel(Z,ns) = p1;
     }
   }
-  if (DEBUGLEVEL>2) fprintferr("\nns = %ld\n",ns);
+  if (DEBUGLEVEL>2) err_printf("\nns = %ld\n",ns);
   if (!ns) return test_block(B, SB, D);
 
   setlg(Z, ns+1);
@@ -326,7 +268,7 @@ print_block_system(blockdata *B, GEN Y, GEN SB)
     for (u=1; u<=N; u++) perm[u] = u;
     for (u=1; u<=ns; u++)
       for (v=1; v<=t[u]; v++)
-	perm_mul_i(perm, cycle_power_to_perm(cyperm, gmael(Z,u,v), e[u][v]));
+        perm_mul_i(perm, cycle_power_to_perm(cyperm, gmael(Z,u,v), e[u][v]));
     SB = test_block(B, SB, im_block_by_perm(D,perm));
     avma = av;
 
@@ -339,21 +281,21 @@ print_block_system(blockdata *B, GEN Y, GEN SB)
       if (j >= 1) { e[ns][j]++; for (l=j+1; l<=t[ns]; l++) e[ns][l] = 0; }
       else
       {
-	i = ns-1;
-	while (i>=1)
-	{
-	  j = t[i];
-	  while (j>=1 && e[i][j] == k[i]) j--;
-	  if (j<1) i--;
+        i = ns-1;
+        while (i>=1)
+        {
+          j = t[i];
+          while (j>=1 && e[i][j] == k[i]) j--;
+          if (j<1) i--;
           else
-	  {
-	    e[i][j]++;
-	    for (l=j+1; l<=t[i]; l++) e[i][l] = 0;
-	    for (ll=i+1; ll<=ns; ll++)
+          {
+            e[i][j]++;
+            for (l=j+1; l<=t[i]; l++) e[i][l] = 0;
+            for (ll=i+1; ll<=ns; ll++)
               for (l=1; l<=t[ll]; l++) e[ll][l] = 0;
             break;
-	  }
-	}
+          }
+        }
       }
     }
   }
@@ -388,7 +330,7 @@ trace(GEN x, GEN Trq, GEN p)
 {
   long i, l;
   GEN s;
-  if (typ(x) == t_INT) return modii(mulii(x, gel(Trq,1)), p);
+  if (typ(x) == t_INT) return Fp_mul(x, gel(Trq,1), p);
   l = lg(x)-1; if (l == 1) return gen_0;
   x++; s = mulii(gel(x,1), gel(Trq,1));
   for (i=2; i<l; i++)
@@ -403,9 +345,9 @@ poltrace(GEN x, GEN Trq, GEN p)
   long i,l;
   GEN y;
   if (typ(x) == t_INT || varn(x) != 0) return trace(x, Trq, p);
-  l = lg(x); y = cgetg(l,t_POL); y[1]=x[1];
+  y = cgetg_copy(x, &l); y[1] = x[1];
   for (i=2; i<l; i++) gel(y,i) = trace(gel(x,i),Trq,p);
-  return y;
+  return normalizepol_lg(y, l);
 }
 
 /* Find h in Fp[X] such that h(a[i]) = listdelta[i] for all modular factors
@@ -416,7 +358,7 @@ poltrace(GEN x, GEN Trq, GEN p)
 static GEN
 chinese_retrieve_pol(GEN DATA, primedata *S, GEN listdelta)
 {
-  GEN interp, bezoutC, h, pol = FpX_red(gel(DATA,1), S->p);
+  GEN interp, bezoutC, h, p = S->p, pol = FpX_red(gel(DATA,1), p);
   long i, l;
   interp = gel(DATA,9);
   bezoutC= gel(DATA,6);
@@ -424,12 +366,12 @@ chinese_retrieve_pol(GEN DATA, primedata *S, GEN listdelta)
   h = NULL; l = lg(interp);
   for (i=1; i<l; i++)
   { /* h(firstroot[i]) = listdelta[i] */
-    GEN t = FqX_Fq_mul(gel(interp,i), gel(listdelta,i), S->T,S->p);
-    t = poltrace(t, (GEN)S->Trk[i], S->p);
-    t = gmul(t, gel(bezoutC,i));
-    h = h? gadd(h,t): t;
+    GEN t = FqX_Fq_mul(gel(interp,i), gel(listdelta,i), S->T, p);
+    t = poltrace(t, gel(S->Trk,i), p);
+    t = FpX_mul(t, gel(bezoutC,i), p);
+    h = h? FpX_add(h,t,p): t;
   }
-  return FpX_rem(FpX_red(h, S->p), pol, S->p);
+  return FpX_rem(h, pol, p);
 }
 
 /* g in Z[X] potentially defines a subfield of Q[X]/f. It is a subfield iff A
@@ -446,36 +388,36 @@ embedding(GEN g, GEN DATA, primedata *S, GEN den, GEN listdelta)
 
   T   = gel(DATA,1); rt = brent_kung_optpow(degpol(T), 2);
   maxp= gel(DATA,7);
-  gp = derivpol(g); av = avma;
+  gp = RgX_deriv(g); av = avma;
   w0 = chinese_retrieve_pol(DATA, S, listdelta);
   w0_Q = centermod(gmul(w0,den), p);
-  h0 = FpXQ_inv(FpX_FpXQ_compo(gp,w0, T,p), T,p); /* = 1/g'(w0) mod (T,p) */
+  h0 = FpXQ_inv(FpX_FpXQ_eval(gp,w0, T,p), T,p); /* = 1/g'(w0) mod (T,p) */
   wpow = NULL; q = sqri(p);
   for(;;)
   {/* Given g,w0,h0 in Z[x], s.t. h0.g'(w0) = 1 and g(w0) = 0 mod (T,p), find
     * [w1,h1] satisfying the same conditions mod p^2, [w1,h1] = [w0,h0] (mod p)
     * (cf. Dixon: J. Austral. Math. Soc., Series A, vol.49, 1990, p.445) */
     if (DEBUGLEVEL>1)
-      fprintferr("lifting embedding mod p^k = %Z^%ld\n",p, Z_pval(q,p));
+      err_printf("lifting embedding mod p^k = %Ps^%ld\n",S->p, Z_pval(q,S->p));
 
     /* w1 := w0 - h0 g(w0) mod (T,q) */
-    if (wpow)
-      a = FpX_FpXQV_compo(g,wpow, T,q);
-    else
-      a = FpX_FpXQ_compo(g,w0, T,q); /* first time */
+    if (wpow) a = FpX_FpXQV_eval(g,wpow, T,q);
+    else      a = FpX_FpXQ_eval(g,w0, T,q); /* first time */
     /* now, a = 0 (p) */
-    a = gmul(gneg(h0), gdivexact(a, p));
-    w1 = gadd(w0, gmul(p, FpX_rem(a, T,p)));
+    a = gmul(gneg(h0), ZX_Z_divexact(a, p));
+    w1 = gadd(w0, gmul(p, FpXQ_red(a, T,p)));
 
     w1_Q = centermod(gmul(w1, remii(den,q)), q);
-    if (gequal(w1_Q, w0_Q) || cmpii(q,maxp) > 0)
+    if (ZX_equal(w1_Q, w0_Q))
     {
-      GEN G = gcmp1(den)? g: RgX_rescale(g,den);
-      if (gcmp0(RgX_RgXQ_compo(G, w1_Q, T))) break;
+      GEN G = is_pm1(den)? g: RgX_rescale(g,den);
+      if (gequal0(RgX_RgXQ_eval(G, w1_Q, T))) break;
     }
-    if (cmpii(q, maxp) > 0)
+    else if (cmpii(q,maxp) > 0)
     {
-      if (DEBUGLEVEL) fprintferr("coeff too big for embedding\n");
+      GEN G = is_pm1(den)? g: RgX_rescale(g,den);
+      if (gequal0(RgX_RgXQ_eval(G, w1_Q, T))) break;
+      if (DEBUGLEVEL) err_printf("coeff too big for embedding\n");
       return NULL;
     }
     gerepileall(av, 5, &w1,&h0,&w1_Q,&q,&p);
@@ -483,14 +425,14 @@ embedding(GEN g, GEN DATA, primedata *S, GEN den, GEN listdelta)
     wpow = FpXQ_powers(w1, rt, T, q2);
     /* h0 := h0 * (2 - h0 g'(w1)) mod (T,q)
      *     = h0 + h0 * (1 - h0 g'(w1)) */
-    a = gmul(gneg(h0), FpX_FpXQV_compo(gp, FpXV_red(wpow,q),T,q));
+    a = gmul(gneg(h0), FpX_FpXQV_eval(gp, FpXV_red(wpow,q),T,q));
     a = ZX_Z_add(FpX_rem(a, T,q), gen_1); /* 1 - h0 g'(w1) = 0 (p) */
-    a = gmul(h0, gdivexact(a, p));
-    h0 = gadd(h0, gmul(p, FpX_rem(a, T,p)));
+    a = gmul(h0, ZX_Z_divexact(a, p));
+    h0 = gadd(h0, gmul(p, FpXQ_red(a, T,p)));
     w0 = w1; w0_Q = w1_Q; p = q; q = q2;
   }
   TR = gel(DATA,5);
-  if (!gcmp0(TR)) w1_Q = translate_pol(w1_Q, TR);
+  if (!gequal0(TR)) w1_Q = RgX_translate(w1_Q, TR);
   return gdiv(w1_Q,den);
 }
 
@@ -508,7 +450,7 @@ get_bezout(GEN pol, GEN fk, GEN p)
     d = FpX_extgcd(A,B,p, &u, &v);
     if (degpol(d) > 0) pari_err(talker, "relatively prime polynomials expected");
     d = constant_term(d);
-    if (!gcmp1(d)) v = FpX_Fp_mul(v, Fp_inv(d, p), p);
+    if (!gequal1(d)) v = FpX_Fp_mul(v, Fp_inv(d, p), p);
     gel(U,i) = FpX_mul(B,v, p);
   }
   return U;
@@ -518,10 +460,10 @@ static GEN
 init_traces(GEN ff, GEN T, GEN p)
 {
   long N = degpol(T),i,j,k, r = lg(ff);
-  GEN Frob = FpXQ_matrix_pow(FpXQ_pow(pol_x[varn(T)],p, T,p), N,N, T,p);
+  GEN Frob = FpXQ_matrix_pow(FpXQ_pow(pol_x(varn(T)),p, T,p), N,N, T,p);
   GEN y,p1,p2,Trk,pow,pow1;
 
-  k = degpol(ff[r-1]); /* largest degree in modular factorization */
+  k = degpol(gel(ff,r-1)); /* largest degree in modular factorization */
   pow = cgetg(k+1, t_VEC);
   gel(pow,1) = gen_0; /* dummy */
   gel(pow,2) = Frob;
@@ -535,14 +477,14 @@ init_traces(GEN ff, GEN T, GEN p)
     gel(pow1,i) = p1; p2 = gel(pow,i);
     for (j=1; j<=N; j++) gel(p1,j) = gcoeff(p2,1,j);
   }
-  
+
   /* Trk[i] = line 1 of x -> x + x^p + ... + x^{p^(i-1)} */
   Trk = pow; /* re-use (destroy) pow */
   gel(Trk,1) = vec_ei(N,1);
   for (i=2; i<=k; i++)
     gel(Trk,i) = gadd(gel(Trk,i-1), gel(pow1,i));
   y = cgetg(r, t_VEC);
-  for (i=1; i<r; i++) y[i] = Trk[degpol(ff[i])];
+  for (i=1; i<r; i++) y[i] = Trk[degpol(gel(ff,i))];
   return y;
 }
 
@@ -552,9 +494,9 @@ static GEN
 interpol(GEN H, GEN T, GEN p)
 {
   long i, m = lg(H);
-  GEN X = pol_x[0],d,p1,p2,a;
+  GEN X = pol_x(0),d,p1,p2,a;
 
-  p1=pol_1[0]; p2=gen_1; a = gneg(constant_term(gel(H,1))); /* = D[1] */
+  p1=pol_1(0); p2=gen_1; a = gneg(constant_term(gel(H,1))); /* = D[1] */
   for (i=2; i<m; i++)
   {
     d = constant_term(gel(H,i)); /* -D[i] */
@@ -570,9 +512,9 @@ init_primedata(primedata *S)
   long i, j, l, lff = lg(S->ff), v = fetch_var(), N = degpol(S->pol);
   GEN T, p = S->p;
 
-  if (S->lcm == degpol(S->ff[lff-1]))
+  if (S->lcm == degpol(gel(S->ff,lff-1)))
   {
-    T = shallowcopy((GEN)S->ff[lff-1]);
+    T = leafcopy(gel(S->ff,lff-1));
     setvarn(T, v);
   }
   else
@@ -601,7 +543,6 @@ choose_prime(primedata *S, GEN pol, GEN dpol)
   GEN Z, p, ff, oldff, n, oldn;
   pari_sp av;
 
-  if (DEBUGLEVEL) (void)timer2();
   p = utoipos(2);
   while (p[2] <= minp) NEXT_PRIME_VIADIFF(p[2], di);
   oldlcm = 0;
@@ -611,22 +552,22 @@ choose_prime(primedata *S, GEN pol, GEN dpol)
   {
     do NEXT_PRIME_VIADIFF(p[2], di); while (!smodis(dpol, p[2]));
     if (k > 5 * N) pari_err(talker,"sorry, too many block systems in nfsubfields");
-    ff = (GEN)FpX_factor(pol, p)[1];
+    ff = gel(FpX_factor(pol, p), 1);
     r = lg(ff)-1;
     if (r == N || r >= BIL) continue;
 
-    n = cgetg(r+1, t_VECSMALL); lcm = n[1] = degpol(ff[1]);
-    for (j=2; j<=r; j++) { n[j] = degpol(ff[j]); lcm = clcm(lcm, n[j]); }
+    n = cgetg(r+1, t_VECSMALL); lcm = n[1] = degpol(gel(ff,1));
+    for (j=2; j<=r; j++) { n[j] = degpol(gel(ff,j)); lcm = clcm(lcm, n[j]); }
     if (lcm <= oldlcm) continue; /* false when oldlcm = 0 */
 
-    if (DEBUGLEVEL) fprintferr("p = %ld,\tlcm = %ld,\torbits: %Z\n",p[2],lcm,n);
+    if (DEBUGLEVEL) err_printf("p = %ld,\tlcm = %ld,\torbits: %Ps\n",p[2],lcm,n);
     pp = p[2];
     oldn = n;
     oldff = ff;
     oldlcm = lcm; if (r == 1) break;
     av = avma;
   }
-  if (DEBUGLEVEL) fprintferr("Chosen prime: p = %ld\n", pp);
+  if (DEBUGLEVEL) err_printf("Chosen prime: p = %ld\n", pp);
   S->ff = oldff;
   S->lcm= oldlcm;
   S->p  = utoipos(pp);
@@ -641,6 +582,7 @@ choose_prime(primedata *S, GEN pol, GEN dpol)
   S->Z = Z;
 }
 
+/* maxroot t_REAL */
 static GEN
 bound_for_coeff(long m, GEN rr, GEN *maxroot)
 {
@@ -687,18 +629,20 @@ compute_data(blockdata *B)
   GEN p = S->p, T = S->T, ff = S->ff, DATA = B->DATA;
   long i, j, l, e, N, lff = lg(ff);
 
-  if (DEBUGLEVEL>1) fprintferr("Entering compute_data()\n\n");
+  if (DEBUGLEVEL>1) err_printf("Entering compute_data()\n\n");
   pol = B->PD->pol; N = degpol(pol);
   roo = B->PD->roo;
-  if (DATA) /* update (translate) an existing DATA */
+  if (DATA)
   {
-    GEN Xm1 = gsub(pol_x[varn(pol)], gen_1);
+    GEN Xm1 = gsub(pol_x(varn(pol)), gen_1);
     GEN TR = addis(gel(DATA,5), 1);
     GEN mTR = negi(TR), interp, bezoutC;
 
+    if (DEBUGLEVEL>1) err_printf("... update (translate) an existing DATA\n\n");
+
     gel(DATA,5) = TR;
-    pol = translate_pol(gel(DATA,1), gen_m1);
-    l = lg(roo); p1 = cgetg(l, t_VEC);
+    pol = RgX_translate(gel(DATA,1), gen_m1);
+    p1 = cgetg_copy(roo, &l);
     for (i=1; i<l; i++) gel(p1,i) = gadd(TR, gel(roo,i));
     roo = p1;
 
@@ -709,28 +653,28 @@ compute_data(blockdata *B)
     interp  = gel(DATA,9);
     for (i=1; i<l; i++)
     {
-      if (degpol(interp[i]) > 0) /* do not turn pol_1[0] into gen_1 */
+      if (degpol(gel(interp,i)) > 0) /* do not turn pol_1(0) into gen_1 */
       {
-        p1 = translate_pol(gel(interp,i), gen_m1);
+        p1 = RgX_translate(gel(interp,i), gen_m1);
         gel(interp,i) = FpXX_red(p1, p);
       }
-      if (degpol(bezoutC[i]) > 0)
+      if (degpol(gel(bezoutC,i)) > 0)
       {
-        p1 = translate_pol(gel(bezoutC,i), gen_m1);
+        p1 = RgX_translate(gel(bezoutC,i), gen_m1);
         gel(bezoutC,i) = FpXX_red(p1, p);
       }
     }
-    ff = cgetg(lff, t_VEC); /* copy, don't overwrite! */
+    ff = cgetg(lff, t_VEC); /* copy, do not overwrite! */
     for (i=1; i<lff; i++)
-      gel(ff,i) = FpX_red(translate_pol((GEN)S->ff[i], mTR), p);
+      gel(ff,i) = FpX_red(RgX_translate(gel(S->ff,i), mTR), p);
   }
   else
   {
     DATA = cgetg(10,t_VEC);
     fk = S->fk;
     gel(DATA,5) = gen_0;
-    gel(DATA,6) = shallowcopy(S->bezoutC);
-    gel(DATA,9) = shallowcopy(S->interp);
+    gel(DATA,6) = leafcopy(S->bezoutC);
+    gel(DATA,9) = leafcopy(S->interp);
   }
   gel(DATA,1) = pol;
   MM = gmul2n(bound_for_coeff(B->d, roo, &maxroot), 1);
@@ -739,9 +683,9 @@ compute_data(blockdata *B)
   gel(DATA,2) = pe;
   gel(DATA,4) = roots_from_deg1(fk);
 
-  /* compute fhk = hensel_lift_fact(pol,fk,T,p,pe,e) in 2 steps
+  /* compute fhk = ZpX_liftfact(pol,fk,T,p,e,pe) in 2 steps
    * 1) lift in Zp to precision p^e */
-  ffL = hensel_lift_fact(pol, ff, NULL, p, pe, e);
+  ffL = ZpX_liftfact(pol, ff, NULL, p, e, pe);
   fhk = NULL;
   for (l=i=1; i<lff; i++)
   { /* 2) lift factorization of ff[i] in Qp[X] / T */
@@ -749,21 +693,21 @@ compute_data(blockdata *B)
     long di = degpol(L);
     F = cgetg(di+1, t_VEC);
     for (j=1; j<=di; j++) F[j] = fk[l++];
-    L = hensel_lift_fact(L, F, T, p, pe, e);
+    L = ZpX_liftfact(L, F, T, p, e, pe);
     fhk = fhk? shallowconcat(fhk, L): L;
   }
   gel(DATA,3) = roots_from_deg1(fhk);
 
-  p1 = mulsr(N, gsqrt(gpowgs(utoipos(N-1),N-1),DEFAULTPREC));
-  p2 = gpowgs(maxroot, B->size + N*(N-1)/2);
-  p1 = gdiv(gmul(p1,p2), gsqrt(B->PD->dis,DEFAULTPREC));
+  p1 = mulur(N, powruhalf(stor(N-1,DEFAULTPREC), N-1));
+  p2 = powru(maxroot, B->size + N*(N-1)/2);
+  p1 = divrr(mulrr(p1,p2), gsqrt(B->PD->dis,DEFAULTPREC));
   gel(DATA,7) = mulii(shifti(ceil_safe(p1), 1), B->PD->den);
 
   if (DEBUGLEVEL>1) {
-    fprintferr("f = %Z\n",DATA[1]);
-    fprintferr("p = %Z, lift to p^%ld\n", p, e);
-    fprintferr("2 * Hadamard bound * ind = %Z\n",DATA[7]);
-    fprintferr("2 * M = %Z\n",DATA[8]);
+    err_printf("f = %Ps\n",DATA[1]);
+    err_printf("p = %Ps, lift to p^%ld\n", p, e);
+    err_printf("2 * Hadamard bound * ind = %Ps\n",DATA[7]);
+    err_printf("2 * M = %Ps\n",DATA[8]);
   }
   if (B->DATA) {
     DATA = gclone(DATA);
@@ -784,21 +728,21 @@ subfield(GEN A, blockdata *B)
   GEN M, pe, pol, fhk, g, e, d_1_term, delta, listdelta, whichdelta;
   GEN T = B->S->T, p = B->S->p, firstroot = B->S->firstroot;
 
-  pol= (GEN)B->DATA[1]; N = degpol(pol); d = N/m; /* m | N */
-  pe = (GEN)B->DATA[2];
-  fhk= (GEN)B->DATA[3];
-  M  = (GEN)B->DATA[8];
+  pol= gel(B->DATA,1); N = degpol(pol); d = N/m; /* m | N */
+  pe = gel(B->DATA,2);
+  fhk= gel(B->DATA,3);
+  M  = gel(B->DATA,8);
 
   delta = cgetg(m+1,t_VEC);
   whichdelta = cgetg(N+1, t_VECSMALL);
   d_1_term = gen_0;
   for (i=1; i<=m; i++)
   {
-    GEN Ai = gel(A,i), p1 = (GEN)fhk[Ai[1]];
+    GEN Ai = gel(A,i), p1 = gel(fhk,Ai[1]);
     for (j=2; j<=d; j++)
-      p1 = Fq_mul(p1, (GEN)fhk[Ai[j]], T, pe);
+      p1 = Fq_mul(p1, gel(fhk,Ai[j]), T, pe);
     gel(delta,i) = p1;
-    if (DEBUGLEVEL>2) fprintferr("delta[%ld] = %Z\n",i,p1);
+    if (DEBUGLEVEL>5) err_printf("delta[%ld] = %Ps\n",i,p1);
     /* g = prod (X - delta[i])
      * if g o h = 0 (pol), we'll have h(Ai[j]) = delta[i] for all j */
     /* fk[k] belongs to block number whichdelta[k] */
@@ -808,28 +752,29 @@ subfield(GEN A, blockdata *B)
   }
   d_1_term = centermod(d_1_term, pe); /* Tr(g) */
   if (absi_cmp(d_1_term, gel(M,3)) > 0) {
-    if (DEBUGLEVEL>1) fprintferr("d-1 test failed\n");
+    if (DEBUGLEVEL>1) err_printf("d-1 test failed\n");
     return NULL;
   }
   g = FqV_roots_to_pol(delta, T, pe, 0);
   g = centermod(polsimplify(g), pe); /* assume g in Z[X] */
-  if (DEBUGLEVEL>2) fprintferr("pol. found = %Z\n",g);
   if (!ok_coeffs(g,M)) {
-    if (DEBUGLEVEL>1) fprintferr("coeff too big for pol g(x)\n");
+    if (DEBUGLEVEL>2) err_printf("pol. found = %Ps\n",g);
+    if (DEBUGLEVEL>1) err_printf("coeff too big for pol g(x)\n");
     return NULL;
   }
   if (!FpX_is_squarefree(g, p)) {
-    if (DEBUGLEVEL>1) fprintferr("changing f(x): p divides disc(g)\n");
+    if (DEBUGLEVEL>2) err_printf("pol. found = %Ps\n",g);
+    if (DEBUGLEVEL>1) err_printf("changing f(x): p divides disc(g)\n");
     compute_data(B);
     return subfield(A, B);
   }
 
   lf = lg(firstroot); listdelta = cgetg(lf, t_VEC);
   for (i=1; i<lf; i++) listdelta[i] = delta[whichdelta[firstroot[i]]];
-  if (DEBUGLEVEL) fprintferr("candidate = %Z\n", g);
+  if (DEBUGLEVEL) err_printf("candidate = %Ps\n", g);
   e = embedding(g, B->DATA, B->S, B->PD->den, listdelta);
   if (!e) return NULL;
-  if (DEBUGLEVEL) fprintferr("embedding = %Z\n", e);
+  if (DEBUGLEVEL) err_printf("... OK!\n");
   return _subfield(g, e);
 }
 
@@ -855,10 +800,11 @@ subfields_of_given_degree(blockdata *B)
   pari_sp av = avma;
   GEN L;
 
-  if (DEBUGLEVEL) fprintferr("\n* Look for subfields of degree %ld\n\n", B->d);
+  if (DEBUGLEVEL) err_printf("\n* Look for subfields of degree %ld\n\n", B->d);
   B->DATA = NULL; compute_data(B);
   L = calc_block(B, B->S->Z, cgetg(1,t_VEC), NULL);
-  if (DEBUGLEVEL) fprintferr("\nSubfields of degree %ld: %Z\n", B->d, L);
+  if (DEBUGLEVEL>9)
+    err_printf("\nSubfields of degree %ld: %Ps\n", B->d, L? L: cgetg(1,t_VEC));
   if (isclone(B->DATA)) gunclone(B->DATA);
   avma = av; return L;
 }
@@ -877,41 +823,99 @@ subfields_poldata(GEN T, poldata *PD)
 {
   GEN  nf,L,dis;
 
-  T = shallowcopy(get_nfpol(T, &nf));
+  T = leafcopy(get_nfpol(T, &nf));
   PD->pol = T; setvarn(T, 0);
   if (nf)
   {
-    PD->den = Q_denom(gel(nf,7));
-    PD->roo = gel(nf,6);
-    PD->dis = mulii(absi(gel(nf,3)), sqri(gel(nf,4)));
+    PD->den = Q_denom(nf_get_zk(nf));
+    PD->roo = nf_get_roots(nf);
+    PD->dis = mulii(absi(nf_get_disc(nf)), sqri(nf_get_index(nf)));
   }
   else
   {
-    PD->den = initgaloisborne(T,NULL,ZX_get_prec(T), &L,NULL,&dis);
+    PD->den = initgaloisborne(T,NULL,ZX_max_lg(T), &L,NULL,&dis);
     PD->roo = L;
     PD->dis = absi(dis);
   }
 }
 
-GEN
-subfields(GEN nf, GEN d0)
+static GEN
+subfieldsall(GEN nf)
 {
   pari_sp av = avma;
-  long N, v0, d = itos(d0);
+  long N, ld, i, v0;
+  GEN G, pol, dg, LSB, NLSB;
+  poldata PD;
+  primedata S;
+  blockdata B;
+
+  /* much easier if nf is Galois (WSS) */
+  G = galoisinit(nf, NULL);
+  if (G != gen_0)
+  {
+    GEN L, S, p;
+    long l;
+
+    pol = get_nfpol(nf, &nf);
+    L = lift_intern( galoissubfields(G, 0, varn(pol)) );
+    l = lg(L);
+    S = cgetg(l, t_VECSMALL);
+    for (i=1; i<l; i++) S[i] = lg(gmael(L,i,1));
+    p = vecsmall_indexsort(S);
+    return gerepilecopy(av,  vecpermute(L, p));
+  }
+
+  subfields_poldata(nf, &PD);
+  pol = PD.pol;
+
+  v0 = varn(pol); N = degpol(pol);
+  dg = divisors(utoipos(N)); ld = lg(dg)-1;
+  if (DEBUGLEVEL) err_printf("\n***** Entering subfields\n\npol = %Ps\n",pol);
+
+
+  LSB = _subfield(pol_x(0), gen_0);
+  if (ld > 2)
+  {
+    B.PD = &PD;
+    B.S  = &S;
+    B.N  = N;
+    choose_prime(&S, PD.pol, PD.dis);
+    for (i=ld-1; i>1; i--)
+    {
+      B.size  = itos(gel(dg,i));
+      B.d = N / B.size;
+      NLSB = subfields_of_given_degree(&B);
+      if (NLSB) { LSB = concat(LSB, NLSB); gunclone(NLSB); }
+    }
+    (void)delete_var(); /* from choose_prime */
+  }
+  LSB = shallowconcat(LSB, _subfield(pol, pol_x(0)));
+  if (DEBUGLEVEL) err_printf("\n***** Leaving subfields\n\n");
+  return fix_var(gerepilecopy(av, LSB), v0);
+}
+
+GEN
+nfsubfields(GEN nf, long d)
+{
+  pari_sp av = avma;
+  long N, v0;
   GEN LSB, pol, G;
   poldata PD;
   primedata S;
   blockdata B;
 
+  if (!d) return subfieldsall(nf);
+
   pol = get_nfpol(nf, &nf); /* in order to treat trivial cases */
+  RgX_check_ZX(pol,"nfsubfields");
   v0 = varn(pol); N = degpol(pol);
-  if (d == N) return gerepilecopy(av, _subfield(pol, pol_x[v0]));
-  if (d == 1) return gerepilecopy(av, _subfield(pol_x[v0], pol));
+  if (d == N) return gerepilecopy(av, _subfield(pol, pol_x(v0)));
+  if (d == 1) return gerepilecopy(av, _subfield(pol_x(v0), pol));
   if (d < 1 || d > N || N % d) return cgetg(1,t_VEC);
 
   /* much easier if nf is Galois (WSS) */
-  G = galoisconj4(nf? nf: pol, NULL, 1);
-  if (typ(G) != t_INT)
+  G = galoisinit(nf? nf: pol, NULL);
+  if (G != gen_0)
   { /* Bingo */
     GEN L = galoissubgroups(G), F;
     long k,i, l = lg(L), o = N/d;
@@ -943,63 +947,3 @@ subfields(GEN nf, GEN d0)
   G = gcopy(LSB); gunclone(LSB);
   return fix_var(G, v0);
 }
-
-static GEN
-subfieldsall(GEN nf)
-{
-  pari_sp av = avma;
-  long N, ld, i, v0;
-  GEN G, pol, dg, LSB, NLSB;
-  poldata PD;
-  primedata S;
-  blockdata B;
-
-  /* much easier if nf is Galois (WSS) */
-  G = galoisconj4(nf, NULL, 1);
-  if (typ(G) != t_INT)
-  {
-    GEN L, S, p;
-    long l;
-
-    pol = get_nfpol(nf, &nf);
-    L = lift_intern( galoissubfields(G, 0, varn(pol)) );
-    l = lg(L);
-    S = cgetg(l, t_VECSMALL);
-    for (i=1; i<l; i++) S[i] = lg(gmael(L,i,1));
-    p = vecsmall_indexsort(S);
-    return gerepilecopy(av,  vecpermute(L, p));
-  }
-
-  subfields_poldata(nf, &PD);
-  pol = PD.pol;
-
-  v0 = varn(pol); N = degpol(pol);
-  dg = divisors(utoipos(N)); ld = lg(dg)-1;
-  if (DEBUGLEVEL) fprintferr("\n***** Entering subfields\n\npol = %Z\n",pol);
-
-  LSB = _subfield(pol, pol_x[0]);
-  if (ld > 2)
-  {
-    B.PD = &PD;
-    B.S  = &S;
-    B.N  = N;
-    choose_prime(&S, PD.pol, PD.dis);
-    for (i=2; i<ld; i++)
-    {
-      B.size  = itos(gel(dg,i));
-      B.d = N / B.size;
-      NLSB = subfields_of_given_degree(&B);
-      if (NLSB) { LSB = concat(LSB, NLSB); gunclone(NLSB); }
-    }
-    (void)delete_var(); /* from choose_prime */
-  }
-  LSB = shallowconcat(LSB, _subfield(pol_x[0], pol));
-  if (DEBUGLEVEL) fprintferr("\n***** Leaving subfields\n\n");
-  return fix_var(gerepilecopy(av, LSB), v0);
-}
-
-GEN
-subfields0(GEN nf,GEN d)
-{
-  return d? subfields(nf,d): subfieldsall(nf);
-}
