@@ -1,4 +1,4 @@
-/* $Id: darwin.c 7520 2005-12-09 18:03:55Z kb $ */
+/* $Id: darwin.c 11744 2009-06-03 12:39:15Z bill $ */
 /*
 Copyright (c) 2002 Peter O'Gorman <ogorman@users.sourceforge.net>
 
@@ -200,7 +200,7 @@ void *dlsym(void *handle, const char *symbol)
     {
       sprintf(malloc_sym, "_%s", symbol);
       value = dlsymIntern(handle, malloc_sym);
-      free(malloc_sym);
+      pari_free(malloc_sym);
     }
     else
       error(0, "Unable to allocate memory");
