@@ -1,4 +1,4 @@
-/* $Id: tune.c 7522 2005-12-09 18:14:24Z kb $
+/* $Id: tune.c 13149 2011-04-21 22:22:41Z bill $
 
 Copyright (C) 2001  The PARI group.
 
@@ -25,12 +25,25 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 int option_trace = 0;
 double Step_Factor = .01; /* small steps by default */
-ulong DFLT_mod = 17UL;
+ulong DFLT_mod1, DFLT_hmod, DFLT_mod2;
+GEN LARGE_mod;
+
+#ifdef LONG_IS_64BIT
+#  define DFLT_mod DFLT_mod1
+#  define Fmod_MUL_MULII_LIMIT Flx_MUL_MULII_LIMIT
+#  define Fmod_SQR_SQRI_LIMIT  Flx_SQR_SQRI_LIMIT
+#  else
+#  define DFLT_mod DFLT_mod2
+#  define Fmod_MUL_MULII_LIMIT Flx_MUL_MULII2_LIMIT
+#  define Fmod_SQR_SQRI_LIMIT  Flx_SQR_SQRI2_LIMIT
+#endif
 
 typedef struct {
   ulong reps, type;
-  long *var, size;
+  long *var, *var_disable, size, enabled;
   GEN x, y;
+  ulong l;
+  GEN p;
 } speed_param;
 
 typedef double (*speed_function_t)(speed_param *s);
@@ -42,10 +55,10 @@ typedef struct {
   int               type; /* t_INT or t_REAL */
   long              min_size;
   long              max_size;
-  speed_function_t  fun1;
-  speed_function_t  fun2;
+  speed_function_t  fun;
   double            step_factor; /* how much to step sizes (rounded down) */
   double            stop_factor;
+  long              *var_disable;
 } tune_param;
 
 /* ========================================================== */
@@ -61,45 +74,66 @@ double speed_endtime(void);
 static pari_timer __T;
 static double speed_unittime = 1e-4;
 static int    speed_precision= 1000;
-static void speed_starttime() { TIMERstart(&__T); }
-static double speed_endtime() { return (double)TIMER(&__T)/1000.; }
+static void speed_starttime() { timer_start(&__T); }
+static double speed_endtime() { return (double)timer_delay(&__T)/1000.; }
 #endif
 
 /* ========================================================== */
-/* int, n words */
+/* int, n words, odd */
 static GEN
 rand_INT(long n)
 {
   pari_sp av = avma;
   GEN x, N = int2n(n*BITS_IN_LONG);
-  do x = genrand(N); while (lgefint(x) != n+2);
+  do x = randomi(N); while (lgefint(x) != n+2);
+  if (!mpodd(x)) x = addis(x,1); /*For Montgomery REDC */
   return gerepileuptoint(av, x);
 }
 /* real, n words */
 static GEN
 rand_REAL(long n) { return gmul2n(itor(rand_INT(n), n+2),-BITS_IN_LONG*n); }
 
+static GEN
+rand_FpX(long n)
+{
+  GEN x;
+  do x = random_FpX(n+1, 0, LARGE_mod); while (degpol(x) < n);
+  return x;
+}
 /* Flx, degree n */
 static GEN
-rand_Flx(long n)
+rand_Flx(long n, ulong l)
+{
+  GEN x;
+  do x = random_Flx(n+1, 0, l); while (degpol(x) < n);
+  return x;
+}
+
+/* normalized Fpx, degree n */
+static GEN
+rand_NFpX(long n)
 {
   pari_sp av = avma;
-  GEN x;
-  do x = FpX_rand(n+1, 0, utoipos(DFLT_mod)); while (degpol(x) < n);
-  return gerepileuptoleaf(av, ZX_to_Flx(x, DFLT_mod));
+  GEN x = gadd(monomial(gen_1,n,0), random_FpX(n, 0, LARGE_mod));
+  return gerepileupto(av, x);
 }
 
 /* normalized Flx, degree n */
 static GEN
-rand_NFlx(long n)
+rand_NFlx(long n, ulong l)
 {
   pari_sp av = avma;
-  GEN x = gadd(monomial(gen_1,n,0), FpX_rand(n, 0, utoipos(DFLT_mod)));
-  return gerepileuptoleaf(av, ZX_to_Flx(x, DFLT_mod));
+  GEN x = Flx_add(Flx_shift(pol1_Flx(0),n), random_Flx(n, 0, l), l);
+  return gerepileuptoleaf(av, x);
 }
 
+#define t_Fhx   99
 #define t_Flx  100
-#define t_NFlx 101
+#define t_Fl1x 101
+#define t_Fl2x 102
+#define t_NFlx 103
+#define t_FpX  104
+#define t_NFpX 105
 
 static GEN
 rand_g(long n, long type)
@@ -107,12 +141,31 @@ rand_g(long n, long type)
   switch (type) {
     case t_INT:  return rand_INT(n);
     case t_REAL: return rand_REAL(n);
-    case t_Flx:  return rand_Flx(n);
-    case t_NFlx: return rand_NFlx(n);
+    case t_Fhx:  return rand_Flx(n,DFLT_hmod);
+    case t_Flx:  return rand_Flx(n,DFLT_mod);
+    case t_Fl1x: return rand_Flx(n,DFLT_mod1);
+    case t_Fl2x: return rand_Flx(n,DFLT_mod2);
+    case t_NFlx: return rand_NFlx(n,DFLT_mod);
+    case t_FpX:  return rand_FpX(n);
+    case t_NFpX: return rand_NFpX(n);
   }
   return NULL;
 }
 
+static void
+dftmod(speed_param *s)
+{
+  switch (s->type) {
+    case t_Fhx:  s->l=DFLT_hmod; return;
+    case t_Flx:  s->l=DFLT_mod;  return;
+    case t_Fl1x: s->l=DFLT_mod1; return;
+    case t_Fl2x: s->l=DFLT_mod2; return;
+    case t_NFlx: s->l=DFLT_mod;  return;
+    case t_FpX:  s->p=LARGE_mod; return;
+    case t_NFpX: s->p=LARGE_mod; return;
+  }
+}
+
 /* ========================================================== */
 #define TIME_FUN(call) {\
   {                                      \
@@ -125,107 +178,113 @@ rand_g(long n, long type)
   return speed_endtime();                \
 }
 
-#define  enable(s) (*(s->var)=lg(s->x)-2)/* enable  asymptotically fastest */
-#define disable(s) (*(s->var)=lg(s->x)+1)/* disable asymptotically fastest */
+#define  m_enable(s,var) (*(s->var)=lg(s->x)-2)/* enable  asymptotically fastest */
+#define m_disable(s,var) (*(s->var)=lg(s->x)+1)/* disable asymptotically fastest */
+
+static void enable(speed_param *s)
+{
+  m_enable(s,var); s->enabled = 1;
+  if (s->var_disable) m_disable(s,var_disable);
+}
+
+static void disable(speed_param *s)
+{
+  m_disable(s,var); s->enabled = 0;
+  if (s->var_disable) m_disable(s,var_disable);
+}
 
 static double speed_mulrr(speed_param *s)
-{ disable(s); TIME_FUN(mulrr(s->x, s->y)); }
-static double speed_karamulrr(speed_param *s)
-{ enable(s);  TIME_FUN(mulrr(s->x, s->y)); }
+{ TIME_FUN(mulrr(s->x, s->y)); }
 
 static double speed_mulii(speed_param *s)
-{ disable(s); TIME_FUN(mulii(s->x, s->y)); }
-static double speed_karamulii(speed_param *s)
-{ enable(s); TIME_FUN(mulii(s->x, s->y)); }
+{ TIME_FUN(mulii(s->x, s->y)); }
 
 static double speed_exp(speed_param *s)
-{ disable(s); TIME_FUN(mpexp(s->x)); }
-static double speed_expnewton(speed_param *s)
-{ enable(s);  mplog2(lg(s->x)+2); TIME_FUN(mpexp(s->x)); }
+{ TIME_FUN(mpexp(s->x)); }
+
+static double speed_inv(speed_param *s)
+{ TIME_FUN(invr(s->x)); }
 
 static double speed_log(speed_param *s)
-{ disable(s); TIME_FUN(mplog(s->x)); }
-static double speed_logagm(speed_param *s)
-{ enable(s);  TIME_FUN(mplog(s->x)); }
+{ TIME_FUN(mplog(s->x)); }
 
 static double speed_logcx(speed_param *s)
-{ GEN z; setexpo(s->x,0); z = gadd(gen_1, gmul(gi, s->x));
+{ GEN z; setexpo(s->x,0); z = mkcomplex(gen_1, s->x);
   glog(z,s->size);
-  disable(s); TIME_FUN(glog(z,s->size)); }
-static double speed_logcxagm(speed_param *s)
-{ GEN z; setexpo(s->x,0); z = gadd(gen_1, gmul(gi, s->x));
-  glog(z,s->size);
-  enable(s); TIME_FUN(glog(z,s->size)); }
+  TIME_FUN(glog(z,s->size)); }
 
 static double speed_atan(speed_param *s)
-{ setexpo(s->x, 0); disable(s);
-  gatan(s->x, 0);
-  TIME_FUN(gatan(s->x, 0)); }
-static double speed_atanagm(speed_param *s)
-{ setexpo(s->x, 0); enable(s);
+{ setexpo(s->x, 0);
   gatan(s->x, 0);
   TIME_FUN(gatan(s->x, 0)); }
 
 static double speed_sqri (speed_param *s)
-{ disable(s); TIME_FUN(sqri(s->x)); }
-static double speed_karasqri (speed_param *s)
-{ enable(s);  TIME_FUN(sqri(s->x)); }
+{ TIME_FUN(sqri(s->x)); }
+
+static double speed_Fp_pow(speed_param *s)
+{ TIME_FUN( Fp_pow(s->x, subis(s->y,1), s->y)); }
 
 static double speed_divrr(speed_param *s)
-{ disable(s); TIME_FUN(divrr(s->x, s->y)); }
-static double speed_divrrgmp(speed_param *s)
-{ enable(s); TIME_FUN(divrr(s->x, s->y)); }
+{ TIME_FUN(divrr(s->x, s->y)); }
 
 static double speed_invmod(speed_param *s)
-{ GEN T; disable(s); TIME_FUN(invmod(s->x, s->y, &T)); }
-static double speed_invmodgmp(speed_param *s)
-{ GEN T; enable(s); TIME_FUN(invmod(s->x, s->y, &T)); }
+{ GEN T; TIME_FUN(invmod(s->x, s->y, &T)); }
 
 static double speed_Flx_sqr(speed_param *s)
-{ ulong p = DFLT_mod; disable(s); TIME_FUN(Flx_sqr(s->x, p)); }
-static double speed_Flx_karasqr(speed_param *s)
-{ ulong p = DFLT_mod; enable(s); TIME_FUN(Flx_sqr(s->x, p)); }
+{ TIME_FUN(Flx_sqr(s->x, s->l)); }
 
 static double speed_Flx_inv(speed_param *s)
-{ ulong p = DFLT_mod; disable(s);
-  TIME_FUN(Flx_invmontgomery(s->x, p)); }
-static double speed_Flx_invnewton(speed_param *s)
-{ ulong p = DFLT_mod; enable(s);
-  TIME_FUN(Flx_invmontgomery(s->x, p)); }
+{ TIME_FUN(Flx_invMontgomery(s->x, s->l)); }
 
 static double speed_Flx_mul(speed_param *s)
-{ ulong p = DFLT_mod; disable(s); TIME_FUN(Flx_mul(s->x, s->y, p)); }
-static double speed_Flx_karamul(speed_param *s)
-{ ulong p = DFLT_mod; enable(s); TIME_FUN(Flx_mul(s->x, s->y, p)); }
+{ TIME_FUN(Flx_mul(s->x, s->y, s->l)); }
 
-#define INIT_RED(s, op)                                 \
-  long i, lx = lg(s->x);                                \
-  op = cgeti(2*lx - 2);                                 \
-  op[1] = evallgefint(2*lx - 2) | evalsigne(1);         \
-  for (i=2; i<lx; i++) op[i]      = s->x[i];            \
-  for (i=2; i<lx; i++) op[lx-2+i] = s->x[i];            \
-  modBIL(s->y) |= 1; /* make sure modulus is odd */
-static double speed_redc(speed_param *s) {
-  ulong inv = (ulong)-invrev(modBIL(s->y));
-  GEN op; INIT_RED(s, op);
-  TIME_FUN( red_montgomery(op, s->y, inv) ); };
-static double speed_modii(speed_param *s) {
-  GEN op; INIT_RED(s, op);
-  TIME_FUN( remii(op, s->y) ); };
-static double speed_remiimul(speed_param *s) {
-  GEN sM = init_remiimul(s->y);
-  GEN op; INIT_RED(s, op);
-  TIME_FUN( remiimul(op, sM) ); }
+static double speed_Flx_rem(speed_param *s) {
+  GEN x = rand_NFlx((degpol(s->x)-1)*2, s->l);
+  TIME_FUN(Flx_rem(x, s->x, s->l));
+}
 
-static double speed_Flxq_pow_redc(speed_param *s) {
-  ulong p = DFLT_mod;
-  enable(s); TIME_FUN( Flxq_pow(polx_Flx(0), utoipos(p), s->y, p) );
+static double speed_Flxq_pow(speed_param *s) {
+  GEN x = rand_Flx(degpol(s->x)-1, s->l);
+  TIME_FUN( Flxq_pow(x, utoipos(s->l), s->x, s->l) );
 }
-static double speed_Flxq_pow_mod(speed_param *s) {
-  ulong p = DFLT_mod;
-  disable(s); TIME_FUN( Flxq_pow(polx_Flx(0), utoipos(p), s->y, p) );
+
+static double speed_Flx_halfgcd(speed_param *s)
+{ TIME_FUN(Flx_halfgcd(s->x, s->y, s->l)); }
+
+static double speed_Flx_gcd(speed_param *s)
+{ TIME_FUN(Flx_gcd(s->x, s->y, s->l)); }
+
+static double speed_Flx_extgcd(speed_param *s)
+{ GEN u,v; TIME_FUN(Flx_extgcd(s->x, s->y, s->l, &u, &v)); }
+
+static double speed_FpX_inv(speed_param *s)
+{ TIME_FUN(FpX_invMontgomery(s->x, s->p)); }
+
+static double speed_FpX_rem(speed_param *s)
+{
+  GEN x = rand_NFpX((degpol(s->x)-1)*2);
+  TIME_FUN(FpX_rem(x, s->x, s->p));
 }
 
+static double speed_FpXQ_pow(speed_param *s) {
+  GEN x = rand_NFpX(degpol(s->x)-1);
+  TIME_FUN( FpXQ_pow(x, s->p, s->x, s->p) );
+}
+
+static double speed_FpX_halfgcd(speed_param *s)
+{ TIME_FUN(FpX_halfgcd(s->x, s->y, s->p)); }
+static double speed_FpX_gcd(speed_param *s)
+{ TIME_FUN(FpX_gcd(s->x, s->y, s->p)); }
+static double speed_FpX_extgcd(speed_param *s)
+{ GEN u,v; TIME_FUN(FpX_extgcd(s->x, s->y, s->p, &u, &v)); }
+
+/* small coeffs: earlier thresholds for more complicated rings */
+static double speed_RgX_sqr(speed_param *s)
+{ TIME_FUN(RgX_sqr(s->x)); }
+static double speed_RgX_mul(speed_param *s)
+{ TIME_FUN(RgX_mul(s->x, s->y)); }
+
 enum { PARI = 1, GMP = 2 };
 #ifdef PARI_KERNEL_GMP
 #  define AVOID PARI
@@ -237,23 +296,42 @@ enum { PARI = 1, GMP = 2 };
  * occur first */
 #define var(a) # a, &a
 static tune_param param[] = {
-{PARI,var(KARATSUBA_MULI_LIMIT),   t_INT, 4,0, speed_mulii,speed_karamulii},
-{PARI,var(KARATSUBA_SQRI_LIMIT),   t_INT, 4,0, speed_sqri,speed_karasqri},
-{0,   var(KARATSUBA_MULR_LIMIT),   t_REAL,4,0, speed_mulrr,speed_karamulrr},
-{PARI,var(MONTGOMERY_LIMIT),       t_INT, 3,0, speed_redc,speed_modii},
-{0,   var(REMIIMUL_LIMIT),         t_INT, 3,0, speed_modii,speed_remiimul},
-{GMP, var(DIVRR_GMP_LIMIT),        t_REAL,4,0, speed_divrr,speed_divrrgmp},
-{0,   var(EXPNEWTON_LIMIT),        t_REAL,64,0, speed_exp,speed_expnewton},
-{0,   var(LOGAGM_LIMIT),           t_REAL,4,0, speed_log,speed_logagm},
-{0,   var(LOGAGMCX_LIMIT),         t_REAL,3,0, speed_logcx,speed_logcxagm,0.05},
-{0,   var(AGM_ATAN_LIMIT),         t_REAL,20,0, speed_atan,speed_atanagm,0.05},
-{GMP, var(INVMOD_GMP_LIMIT),       t_INT, 3,0, speed_invmod,speed_invmodgmp},
-{0,   var(Flx_MUL_LIMIT),          t_Flx, 4,0, speed_Flx_mul,speed_Flx_karamul},
-{0,   var(Flx_SQR_LIMIT),          t_Flx, 4,0, speed_Flx_sqr,speed_Flx_karasqr},
-{0,   var(Flx_INVMONTGOMERY_LIMIT),t_NFlx,10,30000,
-                                   speed_Flx_inv,speed_Flx_invnewton,0.3},
-{0,  var(Flx_POW_MONTGOMERY_LIMIT),t_NFlx,1,0,
-                                   speed_Flxq_pow_redc,speed_Flxq_pow_mod}
+{PARI,var(MULII_KARATSUBA_LIMIT),  t_INT, 4,0, speed_mulii,0,0,&MULII_FFT_LIMIT},
+{PARI,var(SQRI_KARATSUBA_LIMIT),   t_INT, 4,0, speed_sqri,0,0,&SQRI_FFT_LIMIT},
+{PARI,var(MULII_FFT_LIMIT),        t_INT, 500,0, speed_mulii,0.02},
+{PARI,var(SQRI_FFT_LIMIT),         t_INT, 500,0, speed_sqri,0.02},
+{0,   var(MULRR_MULII_LIMIT),      t_REAL,4,0, speed_mulrr},
+{0,   var(Fp_POW_REDC_LIMIT),      t_INT, 3,100, speed_Fp_pow,0,0,&Fp_POW_BARRETT_LIMIT},
+{0,   var(Fp_POW_BARRETT_LIMIT),   t_INT, 3,0, speed_Fp_pow},
+{0,   var(INVNEWTON_LIMIT),        t_REAL,66,0, speed_inv,0.03},
+{GMP, var(DIVRR_GMP_LIMIT),        t_REAL,4,0, speed_divrr},
+{0,   var(EXPNEWTON_LIMIT),        t_REAL,66,0, speed_exp},
+{0,   var(LOGAGM_LIMIT),           t_REAL,4,0, speed_log},
+{0,   var(LOGAGMCX_LIMIT),         t_REAL,3,0, speed_logcx,0.05},
+{0,   var(AGM_ATAN_LIMIT),         t_REAL,20,0, speed_atan,0.05},
+{GMP, var(INVMOD_GMP_LIMIT),       t_INT, 3,0, speed_invmod},
+{0,   var(Flx_MUL_KARATSUBA_LIMIT),t_Flx,5,0, speed_Flx_mul,0,0,&Fmod_MUL_MULII_LIMIT},
+{0,   var(Flx_SQR_KARATSUBA_LIMIT),t_Flx,5,0, speed_Flx_sqr,0,0,&Fmod_SQR_SQRI_LIMIT},
+{0,   var(Flx_MUL_HALFMULII_LIMIT),t_Fhx,3,0, speed_Flx_mul},
+{0,   var(Flx_SQR_HALFSQRI_LIMIT), t_Fhx,3,0, speed_Flx_sqr},
+{0,   var(Flx_MUL_MULII_LIMIT),    t_Fl1x,5,0, speed_Flx_mul},
+{0,   var(Flx_SQR_SQRI_LIMIT),     t_Fl1x,5,0, speed_Flx_sqr},
+{0,   var(Flx_MUL_MULII2_LIMIT),   t_Fl2x,5,20000, speed_Flx_mul,0.05},
+{0,   var(Flx_SQR_SQRI2_LIMIT),    t_Fl2x,5,20000, speed_Flx_sqr,0.05},
+{0,   var(Flx_INVMONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flx_inv,0.05},
+{0,  var(Flx_REM_MONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flx_rem,0.05},
+{0,  var(Flx_POW_MONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flxq_pow},
+{0,  var(Flx_HALFGCD_LIMIT),       t_Flx,10,0, speed_Flx_halfgcd},
+{0,  var(Flx_GCD_LIMIT),           t_Flx,10,0, speed_Flx_gcd,0.1},
+{0,  var(Flx_EXTGCD_LIMIT),        t_Flx,10,0, speed_Flx_extgcd},
+{0,  var(FpX_INVMONTGOMERY_LIMIT), t_NFpX,10,0, speed_FpX_inv,0.05},
+{0,  var(FpX_REM_MONTGOMERY_LIMIT),t_NFpX,10,0, speed_FpX_rem,0.05},
+{0,  var(FpX_POW_MONTGOMERY_LIMIT),t_NFpX,10,0, speed_FpXQ_pow},
+{0,  var(FpX_HALFGCD_LIMIT),       t_FpX,10,0, speed_FpX_halfgcd},
+{0,  var(FpX_GCD_LIMIT),           t_FpX,10,0, speed_FpX_gcd,0.1},
+{0,  var(FpX_EXTGCD_LIMIT),        t_FpX,10,0, speed_FpX_extgcd},
+{0,  var(RgX_MUL_LIMIT),           t_FpX, 4,0, speed_RgX_mul},
+{0,  var(RgX_SQR_LIMIT),           t_FpX, 4,0, speed_RgX_sqr},
 };
 
 /* ========================================================== */
@@ -267,15 +345,15 @@ int
 double_cmp_ptr(double *x, double *y) { return (int)(*x - *y); }
 
 double
-time_fun(speed_function_t fun, speed_param *s)
+time_fun(speed_function_t fun, speed_param *s, long enabled)
 {
   const double TOLERANCE = 1.005; /* 0.5% */
   pari_sp av = avma;
   double t[30];
   ulong i, j, e;
 
-  s->x = rand_g(s->size, s->type);
-  s->y = rand_g(s->size, s->type); s->reps = 1;
+  s->reps = 1;
+  if (enabled) enable(s); else disable(s);
   for (i = 0; i < numberof(t); i++)
   {
     for (;;)
@@ -288,7 +366,7 @@ time_fun(speed_function_t fun, speed_param *s)
       /* go to a value of reps to make t[i] >= precision */
       reps_d = ceil (1.1 * s->reps
                      * speed_unittime * speed_precision
-                     / max(t[i], speed_unittime));
+                     / maxdd(t[i], speed_unittime));
       if (reps_d > 2e9 || reps_d < 1.0)
         pari_err(talker, "Fatal error: new reps bad: %.2f\n", reps_d);
 
@@ -317,13 +395,24 @@ add_dat(long size, double d)
 {
   if (ndat == allocdat)
   {
-    allocdat += max(allocdat, 100);
-    dat = (struct dat_t*) gprealloc((void*)dat, allocdat * sizeof(dat[0]));
+    allocdat += maxss(allocdat, 100);
+    dat = (struct dat_t*) pari_realloc((void*)dat, allocdat * sizeof(dat[0]));
   }
   dat[ndat].size = size;
   dat[ndat].d    = d; ndat++;
 }
 
+void
+diag(const char *format, ...)
+{
+  va_list ap;
+  va_start(ap, format);
+  vfprintf(stderr, format, ap);
+}
+void
+print_define(const char *name, long value)
+{ printf("#define __%-25s  %5ld\n\n", name, value); }
+
 long
 analyze_dat(int final)
 {
@@ -337,10 +426,10 @@ analyze_dat(int final)
 
   if (final && option_trace >= 3)
   {
-    printf("\n");
-    printf("x is the sum of the badness from setting thresh at given size\n");
-    printf("  (minimum x is sought)\n");
-    printf("size=%ld  first x=%.4f\n", dat[j].size, x);
+    diag("\n");
+    diag("x is the sum of the badness from setting thresh at given size\n");
+    diag("  (minimum x is sought)\n");
+    diag("size=%ld  first x=%.4f\n", dat[j].size, x);
   }
 
   min_x = x;
@@ -352,7 +441,7 @@ analyze_dat(int final)
   for (j = 0; j < ndat; j++)
   {
     if (final && option_trace >= 3)
-      printf ("size=%ld  x=%.4f\n", dat[j].size, x);
+      diag ("size=%ld  x=%.4f\n", dat[j].size, x);
 
     if (x < min_x) { min_x = x; min_j = j; }
     x -= dat[j].d;
@@ -360,41 +449,44 @@ analyze_dat(int final)
   return min_j;
 }
 
-void
-print_define(const char *name, long value)
-{ printf("#define __%-25s  %5ld\n\n", name, value); }
-
 void
 Test(tune_param *param)
 {
   int since_positive, since_change, thresh, new_thresh;
   speed_param s;
+  long save_var_disable = -1;
 
   if (param->kernel == AVOID) { print_define(param->name, -1); return; }
 
 #define DEFAULT(x,n)  if (! (param->x))  param->x = (n);
-  DEFAULT(fun2, param->fun1);
   DEFAULT(step_factor, Step_Factor);
   DEFAULT(stop_factor, 1.2);
   DEFAULT(max_size, 10000);
+  if (param->var_disable) save_var_disable = *(param->var_disable);
 
   s.type = param->type;
   s.size = param->min_size;
   s.var  = param->var;
+  s.var_disable  = param->var_disable;
+  dftmod(&s);
   ndat = since_positive = since_change = thresh = 0;
   if (option_trace >= 1)
-    printf("Setting %s... (default %ld)\n", param->name, *(param->var));
+    diag("Setting %s... (default %ld)\n", param->name, *(param->var));
   if (option_trace >= 2)
   {
-    printf("              algorithm-A  algorithm-B   ratio  possible\n");
-    printf("               (seconds)    (seconds)    diff    thresh\n");
+    diag("              algorithm-A  algorithm-B   ratio  possible\n");
+    diag("               (seconds)    (seconds)    diff    thresh\n");
   }
 
   for(;;)
   {
+    pari_sp av=avma;
     double t1, t2, d;
-    t1 = time_fun(param->fun1, &s);
-    t2 = time_fun(param->fun2, &s);
+    s.x = rand_g(s.size, s.type);
+    s.y = rand_g(s.size, s.type);
+    t1 = time_fun(param->fun, &s, 0);
+    t2 = time_fun(param->fun, &s, 1);
+    avma = av;
     if (t2 >= t1) d = (t2 - t1) / t2;
     else          d = (t2 - t1) / t1;
 
@@ -402,8 +494,8 @@ Test(tune_param *param)
     new_thresh = analyze_dat(0);
 
     if (option_trace >= 2)
-      printf ("size =%4ld     %.8f   %.8f  % .4f %c  %ld\n",
-               s.size, t1,t2, d, d < 0? '#': ' ', dat[new_thresh].size);
+      diag ("size =%4ld     %.8f   %.8f  % .4f %c  %ld\n",
+            s.size, t1,t2, d, d < 0? '#': ' ', dat[new_thresh].size);
 
 #define SINCE_POSITIVE 20
 #define SINCE_CHANGE 50
@@ -414,14 +506,14 @@ Test(tune_param *param)
       if (++since_positive > SINCE_POSITIVE)
       {
         if (option_trace >= 1)
-          printf ("Stop: since_positive (%d)\n", SINCE_POSITIVE);
+          diag("Stop: since_positive (%d)\n", SINCE_POSITIVE);
         break;
       }
     /* Stop if method A has become slower by a certain factor */
     if (t1 >= t2 * param->stop_factor)
     {
       if (option_trace >= 1)
-        printf ("Stop: t1 >= t2 * factor (%.1f)\n", param->stop_factor);
+        diag("Stop: t1 >= t2 * factor (%.1f)\n", param->stop_factor);
       break;
     }
     /* Stop if threshold implied hasn't changed for a while */
@@ -431,28 +523,29 @@ Test(tune_param *param)
       if (++since_change > SINCE_CHANGE)
       {
         if (option_trace >= 1)
-          printf ("Stop: since_change (%d)\n", SINCE_CHANGE);
+          diag("Stop: since_change (%d)\n", SINCE_CHANGE);
         break;
       }
-    s.size += max((long)floor(s.size * param->step_factor), 1);
+    s.size += maxss((long)floor(s.size * param->step_factor), 1);
     if (s.size >= param->max_size)
     {
       if (option_trace >= 1)
-        printf ("Stop: max_size (%ld). Disable Algorithm B?\n",param->max_size);
+        diag("Stop: max_size (%ld). Disable Algorithm B?\n",param->max_size);
       break;
     }
   }
   thresh = dat[analyze_dat(1)].size;
   print_define(param->name, thresh);
   *(param->var) = thresh; /* set to optimal value for next tests */
+  if (param->var_disable) *(param->var_disable) = save_var_disable;
 }
 
 void error(char **argv) {
   long i;
-  printf("usage: tune [-t] [-s step_factor] [-p mod] [-u unittime] var1 var2 ...\n");
-  printf("Tunable variables: (omitting variable indices tunes everybody)\n");
+  diag("usage: tune [-t] [-s step_factor] [-p mod] [-u unittime] var1 var2 ...\n");
+  diag("Tunable variables: (omitting variable indices tunes everybody)\n");
   for (i = 0; i < (long)numberof(param); i++)
-    printf("  %2ld: %-25s (default %4ld)\n", i, param[i].name, *(param[i].var));
+    diag("  %2ld: %-25s (default %4ld)\n", i, param[i].name, *(param[i].var));
   exit(1);
 }
 
@@ -461,7 +554,16 @@ main(int argc, char **argv)
 {
   int i, r, n = 0;
   GEN v;
-  pari_init(4000000, 2);
+  pari_init(8000000, 2);
+  (void) init_modular(&DFLT_mod);
+  LARGE_mod=subis(powuu(3,128),62);
+#ifdef LONG_IS_64BIT
+  DFLT_hmod = 257;
+  DFLT_mod2 = 281474976710677UL;
+#else
+  DFLT_hmod = 17;
+  DFLT_mod1 = 1031UL;
+#endif
   v = new_chunk(argc);
   for (i = 1; i < argc; i++)
   {
